<xsl:stylesheet 
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="text" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>

    <xsl:param name="OUTPUT_DIR"    required="yes"/>
    <xsl:param name="NETWORK"        required="yes"/>
    <xsl:param name="ANALYSIS"        required="yes"/>
    <xsl:param name="SOLVER"         required="yes"/>
    <xsl:param name="USER"             required="yes"/>
    <xsl:param name="VERSION"        required="yes"/>
    <xsl:param name="TARGETS"       required="yes"/>
    <xsl:param name="TARGETS_OUTDIR"      required="yes"/>
    <xsl:param name="REWRITE"        required="no"/>
    <xsl:param name="TYPE"              required="no"/>
    <xsl:param name="START"              required="no"/>
    <xsl:param name="NEXT"              required="no"/>
    <xsl:param name="PWD"              required="no"/>


    <xsl:variable name="TARGET_CONSTRAINTS_LIST">
      <xsl:copy-of select="document(concat('../',$OUTPUT_DIR,'/',
			   $ANALYSIS,'/constraints-targets-list.xml'))"/>
    </xsl:variable>

    <xsl:variable name="TARGETS_LIST_INCLUDE">
      <xsl:copy-of select="document(concat('../',$OUTPUT_DIR,'/',
			   $ANALYSIS,'/targets-list-include.xml'))"/>
    </xsl:variable>

    <xsl:variable name="PARAMS_INCLUDE">
      <xsl:copy-of select="document(concat('../',$OUTPUT_DIR,'/',
			   $ANALYSIS,'/params-include.xml'))"/>
    </xsl:variable>


    <xsl:variable name="NETWORK-LATEX-FIX">
      <xsl:value-of select="replace($NETWORK,'_','-')"/>
    </xsl:variable>


    <xsl:variable name="TARGETS-LATEX-FIX">
      <xsl:value-of select="replace($TARGETS,'_','\_')"/>
    </xsl:variable>

    <xsl:variable name="SOLVER-LATEX-FIX">
      <xsl:value-of select="replace($SOLVER,'_','-')"/>
    </xsl:variable>
    
    <xsl:variable name="NETWORK_ANALYSIS">
      <xsl:value-of select="$NETWORK"/><text>/</text><xsl:value-of select="$ANALYSIS"/>
    </xsl:variable>

    <xsl:variable name="NETWORK_ANALYSIS-LATEX-FIX">
      <xsl:value-of select="replace($NETWORK_ANALYSIS,'_','-')"/>
    </xsl:variable>

   
    <xsl:template match="/">

      <text xml:space="preserve">      

%%%%%%%%%%%%%%%%%%%
% header 
%%%%%%%%%%%%%%%%%%

\documentclass{article}

\usepackage[a2paper]{geometry}
\usepackage[colorlinks]{hyperref}

\usepackage{longtable}
\usepackage{capt-of}
\usepackage{amsmath}
\usepackage{amssymb}
\usepackage{natbib}
\usepackage{stmaryrd}
\usepackage{mathrsfs}
\usepackage{bussproofs}
\usepackage{graphicx}
\usepackage{color}
\usepackage{multirow}
\usepackage[T1]{fontenc}
\usepackage{Graph/Latex/inference}
\usepackage{xspace}
\usepackage{listings}
\usepackage{array,multirow,makecell}
\usepackage{sverb}
\usepackage[version=3]{mhchem}
\usepackage{etoolbox} % needed for ifnumber
\usepackage{Graph/Latex/ifnumber}
\usepackage{Graph/Latex/mycommand}

\usepackage{pdftexcmds} % for switch

\usepackage{Graph/Latex/macros}
      </text>
      
      <xsl:if test="not(//*[@latex-look])">
	<text>
	  \usepackage{Graph/Latex/macros-species}
	  \usepackage{Graph/Latex/switch-macros}
	</text>
      </xsl:if>
<!--
      <xsl:if test="//*[@latex-look]">
	<text>\usepackage{</text>
	<xsl:value-of select="$OUTPUT_DIR"/>
	<text>/</text>
	<xsl:value-of select="$ANALYSIS"/>
	<text>/Targets/</text>
	<xsl:value-of select="$COMPLETE_TARGETS_LIST//difference-constraints[1]/@id"/>
	<text>/Knockouts-Graph/latex-macros}</text>
      </xsl:if>
-->
      <xsl:if test="//*[@latex-look]">
	<text>\usepackage{</text>
	<xsl:value-of select="$OUTPUT_DIR"/>
	<text>/Equations/latex-macros}</text>
      </xsl:if>

      <text>
\usepackage[usenames,dvipsnames,x11names]{xcolor}
\usepackage{tikz}
  \usetikzlibrary{math} %needed tikz library
  \definecolor{lightblue}{HTML}{92dcec}

\usepackage{Graph/Latex/tikz-macros}
    
\usetikzlibrary{decorations.pathmorphing}

\EnableBpAbbreviations

\newcommand\OutputFile[1]{\input{</text>
      <xsl:value-of select="$OUTPUT_DIR"/>
      <text>/#1}}

\newcommand\VerbOutputFile[1]{\verbinput{</text>
      <xsl:value-of select="$OUTPUT_DIR"/>
      <text>/#1}}


\newcommand\OutputFileIfExists[1]{\IfFileExists{</text>
<xsl:value-of  select="$OUTPUT_DIR"/><text>/#1}
{\OutputFile{#1}}
}




      
\begin{document}

%%%%%%%%%%%%%%%%%%%
% title page
%%%%%%%%%%%%%%%%%%

      </text>
      <xsl:choose>
	<xsl:when test="$TYPE='Bool'">
	  \title{Abstract Simulation of \\
	         Reaction Networks via Boolean Networks}
	  \author{Joachim Niehren,  Cristian Versari, Athénaïs Vaginay\\
BioComputing Team of Université de Lille and Loria}
	</xsl:when>
	<xsl:otherwise>
	  <text>
\title{Change Prediction for Reaction Networks with Partial Kinetic
Information\\
BioComputing's Reaction-network Tool: Version 
	  </text>
	  <xsl:copy-of select="$VERSION"/>}
\author{Joachim Niehren,  Cristian Versari\\ 
BioComputing Team, University of Lille}
	</xsl:otherwise>
      </xsl:choose>
    <text>
\maketitle

% $\mycommand{-1}$

% $\mycommand{22}$

% $\mycommand{2x}$

% %%%%%%%%%%%%%%%%%%
%  shared part
% %%%%%%%%%%%%%%%%%%


<xsl:choose>
  <xsl:when  test="$TYPE='Bool'">
  <text>
  \section*{Task Specification (file
  Makefile-</text>
  <xsl:value-of select="$USER"/>
  <text>.mk)}

  \paragraph{Reaction Network: </text>
      <xsl:value-of select="$NETWORK-LATEX-FIX"/>
      <text>}
      
      \paragraph{Parameters}
      \mbox{ }
      
      \medskip
      
      START = </text>
  <xsl:value-of select="$START"/>
  <text> \qquad  start with boolean abstraction of initial concentrations
    \\
    
  NEXT= </text>
  <xsl:value-of select="$NEXT"/>
  <text> \qquad\qquad compute transition to next boolean state
  </text>
  </xsl:when>
  <xsl:otherwise>
    <text>
      \section*{Constraint Solving Tasks}
    </text>
      <text>\paragraph{Tool's User: </text>
      <xsl:value-of select="$USER"/>
<text>} </text>
  </xsl:otherwise>
</xsl:choose>



\subsection*{Target Specification (file param-</text>
<xsl:value-of select="$USER"/>
<text>.xml)}



\begin{verbatim}
</text>
<xsl:value-of select="$PARAMS_INCLUDE"/>
<!--<xsl:message> 
  +++ PARAMS = <xsl:copy-of select="$PARAMS_INCLUDE"/>
</xsl:message>-->
<text>
\end{verbatim}

\paragraph{Target List}

\begin{verbatim}
</text>
<xsl:value-of select="$TARGETS_LIST_INCLUDE"/>
<text>
  \end{verbatim}

\subsection*{Parameter Specification}

\paragraph{Makefile-</text>
      <xsl:value-of select="$USER"/>
<text>.mk}

      \verbinput{Makefile-</text>
      <xsl:value-of select="$USER"/>
      <xsl:text>.mk}


\subsection*{Solution statistics}


\begin{center}

\OutputFile{/</xsl:text>
<xsl:value-of select="$ANALYSIS"/> <xsl:text>/statistics.tex}

\captionof{figure}{Solution statistics.}

\addtocounter{table}{-1}
\end{center}

\tableofcontents

\section{The Reaction Network:
      </xsl:text>
      <xsl:value-of select="$NETWORK-LATEX-FIX"/>
      <text>}

      The network is specified in XML format. 
      From
this an equivalent graph is generated by BioCompting's Network-Graph tool.
It is presented below.

\OutputFile{Graph/graph.tex} 




\begin{center}
\OutputFile{Graph/molecules.tex}</text>

      <text>\captionof{figure}{Molecules of </text>
      <xsl:value-of select="$NETWORK-LATEX-FIX"/>
      <text>.}
\addtocounter{table}{-1}
\end{center}

\begin{center}
\OutputFile{Graph/reactions.tex}</text>
    <text>\captionof{figure}{Reactions of </text>
    <xsl:value-of select="$NETWORK-LATEX-FIX"/>
    <text>.}
\addtocounter{table}{-1}

\end{center}

    \section{Equations for the Network}

\subsection{Ordinary Differential Equations (ODEs)}

The following ODE's are inferred from the reaction network and the knockout
candidates. When the kinetics is known only partially the kinetic
expressions are to be taken up to similarity.

\noindent\OutputFile{Equations/ODEs.tex}

    </text>
    
    <xsl:if test="$TYPE='Bool'">
      <text>
	\subsection{From ODEs to Positive Equations}
	\label{positive-equations}

We now interpret the variables of the ODE equations
over the reals (instead the over real functions).
We the rewrite the ODE equations, so are
interpreted over positive reals, while using
the signature of the boolean algebra
only (addition=or and
multiplication=and). The real typed
variables $\dot{\z{A}}$ are then replaced
by positive-real typed variables $\posdot{A}$
and $\negdot{A}$. Division and substraction
are rewritting into multiplication and addition
respectively.

\subsection{Polynomial Dot Equations}
\subsubsection{Positive Equations}
\medskip
\noindent\OutputFile{Equations/positive-equations.tex}

\subsubsection{Flat Positive Equations}
\label{flat-positive-equations}

\OutputFile{Equations/flat-positive-equations.tex}

\subsubsection{Non-Linear Constraints with Dots}

\OutputFile{Equations/non-linear-constraints.tex}

\subsection{Linear Equations with Dots} 

\OutputFile{Equations/linear-equations.tex}

\subsubsection{Minsupp: Minimal Support Linear Consequences of Linear
Dot Equations}
\label{sec:minimal-support-linear-equations-EFM}

\OutputFileIfExists{Equations/minsupp/linear-equations/equations.tex}

\subsubsection{Exact Boolean Rewriting of Linear Dot Equations}

\OutputFileIfExists{Equations/exact/linear-equations/equations.tex}
    </text>
    </xsl:if>

    <xsl:if test="not($TYPE='Bool')">
      <text>
\subsection{Steady State Equations}
\label{steady-state}

When imposing the steady state assumptions (concentrations, reaction speeds,
and the speeds of input/output fluxes have become stable, then we obtain the
following arithmetic equations, but with kinetic functions known only modulo
similarity $\sim$:

\OutputFile{Equations/steady-state.tex}

\subsubsection{Flat Steady State Equations}
\label{flat-steady-state}

\OutputFile{Equations/flat-steady-state.tex}

\subsubsection{Minsupp: Minimal Support Linear Consequence of Steady
State Equations}
\label{sec:minimal-support-EFM}

We compute the set of linear consequence
of steady state equations with minimal support
based on elementary flux modes (EFMs).

\OutputFileIfExists{Equations/minsupp/steady-state/equations.tex}

\subsubsection{Exact Boolean Rewriting of Steady State Equations}
\label{sec:minimal-support-exact}

The exact boolean rewriting yields the following equatios.


\OutputFileIfExists{Equations/exact/steady-state/equations.tex}

</text>
    </xsl:if>

<xsl:if test="$REWRITE='yes'">
<text>
  \subsection{Simplified Steady State Equations (Rewrite)}


%We need to remove nested expressions before applying the
%abstract interpretation.

\OutputFile{Equations/simplified-steady-state.tex}
</text>
</xsl:if>

<text>
% %%%%%%%%%%%%%%%%%%
% individual parts for the elements of the targets-list
% %%%%%%%%%%%%%%%%%%%
</text>
      
      <xsl:apply-templates select="$TARGET_CONSTRAINTS_LIST/targets-list/difference-constraints"/>

      <text>
%%%%%%%%%%%%%%%%%
% footer
%%%%%%%%%%%%%%%%%%
      </text>
      <xsl:apply-templates select="/" mode="footer"/>
    </xsl:template>
   
    <xsl:template match="difference-constraints">
      <xsl:variable name="targets-id">
	<xsl:value-of select="@id"/>
      </xsl:variable>
      <xsl:variable name="fix-targets-id">
	<xsl:value-of select="replace($targets-id,'_','\\_')"/>
      </xsl:variable>
      
      <text>\</text>
      <xsl:if test="preceding::difference-constraints">
	<text>re</text>
      </xsl:if>
      <text>newcommand\OutputTargetFile[1]{\input{</text>
      <xsl:value-of select="concat($OUTPUT_DIR,'/',$ANALYSIS,'/',$TARGETS_OUTDIR,'/',$targets-id)"/>
      <text>/#1}}
      </text>

      <text>\</text>
      <xsl:if test="preceding::difference-constraints">
	<text>re</text>
      </xsl:if>
      <text>newcommand\GraphicsOutputTargetFile[1]{

      \noindent\includegraphics[width=\textwidth]{</text>
      <xsl:value-of select="concat($OUTPUT_DIR,'/',$ANALYSIS,'/',$TARGETS_OUTDIR,'/',$targets-id)"/>
      <text>/#1}}

      <text>\</text>
      <xsl:if test="preceding::difference-constraints">
	<text>re</text>
      </xsl:if>
      <text>newcommand\VerbOutputTargetFile[1]{\verbinput{</text>
      <xsl:value-of select="concat($OUTPUT_DIR,'/',$ANALYSIS,'/',$TARGETS_OUTDIR,'/',$targets-id)"/>
      <text>/#1}}
      </text>

      <xsl:variable name="full-id">
	<xsl:value-of select="network/@id"/>
	<xsl:if test="network/@kind">
	  <text>-</text>
	  <xsl:value-of select="network/@kind"/>
	</xsl:if>
      </xsl:variable>

      <text>\renewcommand\TIKZID[1]{</text>
      <xsl:value-of select="$full-id"/>
      <text xml:space="preserve">}

      \section{Constraints for Target "</text>
      <xsl:value-of select="$fix-targets-id"/>
      <text>"}
      \label{sec:</text>
      <xsl:value-of select="$targets-id"/>
      <text>}
\begin{center}

\OutputTargetFile{Knockouts-Graph/graph.tex} 

\smallskip
\hrule
      \captionof{figure}{\label{fig:</text>
      <xsl:value-of select="$NETWORK_ANALYSIS-LATEX-FIX"/>
      <text>}
The knockout graph of </text>
     <xsl:value-of select="$NETWORK_ANALYSIS-LATEX-FIX"/>
     <text>.}
\addtocounter{table}{-1}
\end{center}
     </text>

<text> \IGNORE{ \subsection{Network: </text>
<xsl:value-of select="$NETWORK-LATEX-FIX"/>
<text>}

See the graph in  Figure \ref{fig:</text>
<xsl:value-of select="$NETWORK_ANALYSIS-LATEX-FIX"/>
<text>}, generated from the file  \texttt{Networks/</text> 
<xsl:copy-of select="$NETWORK-LATEX-FIX"/>
<text xml:space="preserve">.xml}. </text>}

<text>In the knockout graph the reactions in
medium orange
are predected for knockout but not safely. Those in light orange
are candidates but not predicted, and those in
dark orange are safe predictions. 

\subsection{Targets}

The complete set of target constraints is the following:

\OutputTargetFile{constraints-targets.tex} 

We use \textbf{</text>
<xsl:value-of select="$SOLVER-LATEX-FIX"/>
<text>} for constraint solving. 
</text>

<text>
  \subsection{Statistics of Solution Set}
  
  \OutputTargetFile{Solutions/statistics.tex}
  
  % \subsection{Difference Constraints}
%
% By abstract interpreation we obtain the difference constraints from
% the flattened simplified steady state equations. Additional difference
% constraints are added from the modification targets.
%
% \OutputTargetFile{Solutions/difference-constraints.tex}
%
% \subsection{Simplified Difference Constraints}
%
% We again apply algebraic rewriting rules in order to simplify the
% difference constraints. This does performs equivalence transforamtions,
% so the solution set is not changed, but may speed up the constraint
% solver.
%
%\OutputTargetFile{Solutions/difference-constraints.tex}
%
%\subsection{Flat Simplified Difference Constraints}
%
% We flatten the simplified difference constraints in order to translate
% them to scala.
%
%\OutputTargetFile{Solutions/flat-constraints.tex}
</text>

<xsl:variable name="statistics-path">
  <xsl:value-of
      select="concat('../',$OUTPUT_DIR,'/',$ANALYSIS,'/',$TARGETS_OUTDIR,'/',$targets-id,
	      '/Solutions/statistics.xml')"/>
</xsl:variable>
<text>
  %%%%
  
  \subsection{Solutions}
  
  
  
  \begin{center}
  \OutputTargetFile{Solutions/solutions.tex}
  \end{center}

</text>
    
<!-- <text> -->
<!-- \subsubsection{Safe Solutions} -->


<!-- </text> -->

<!-- <xsl:choose> -->
<!--   <xsl:when test="document($statistics-path)// -->
<!-- 		  number-of[@kind='added safe solutions'][text()='-']"> -->
<!--     \paragraph{Initial} -->
    
<!--     \begin{center} -->
<!--     \OutputTargetFile{Solutions/added-safe-solutions.tex} -->
<!--     \end{center} -->
    
<!--   </xsl:when> -->
<!--   <xsl:when test="document($statistics-path)// -->
<!-- 		  number-of[@kind='added safe solutions'][text()='0']"> -->
    
<!--     \paragraph{No safe solutions were added} compared to the previous target. -->
    
<!--   </xsl:when> -->
<!--   <xsl:otherwise> -->

<!--     <xsl:text> We now present the changes of the set of safe solution, -->
<!--     added and lost compared to the previous target. -->

<!--   \subsubsection{Added} -->

<!--   \begin{center} -->
<!--   \OutputTargetFile{Solutions/added-safe-solutions.tex} -->
<!--   \end{center} -->
<!--     </xsl:text> -->
<!--   </xsl:otherwise> -->
<!-- </xsl:choose> -->

<!-- <xsl:choose> -->
<!--   <xsl:when test="document($statistics-path)// -->
<!-- 		  number-of[@kind='lost safe solutions'][text()='-']"> -->

<!--   </xsl:when> -->
<!--   <xsl:when test="document($statistics-path)// -->
<!-- 		  number-of[@kind='lost safe solutions'][text()='0']"> -->

<!-- \paragraph{No safe solutions were lost} compared to the previous target. -->

<!--   </xsl:when> -->
  
<!--   <xsl:otherwise> -->

<!--   \subsubsection{Lost} -->

<!-- 	\begin{center} -->
<!-- 	\OutputTargetFile{Solutions/lost-safe-solutions.tex} -->
<!-- 	\end{center} -->

<!--   </xsl:otherwise> -->
<!-- </xsl:choose> -->

<!-- %%%% -->


<!-- \subsubsection{Unsafe Solutions} -->



<!-- <xsl:choose> -->
<!--   <xsl:when test="document($statistics-path)// -->
<!-- 		  number-of[@kind='added unsafe solutions'][text()='-']"> -->
<!--     \subsubsection{Initial} -->
    
<!--     \begin{center} -->
<!--     \OutputTargetFile{Solutions/added-unsafe-solutions.tex} -->
<!--     \end{center} -->
    
<!--   </xsl:when> -->
<!--   <xsl:when test="document($statistics-path)// -->
<!-- 		  number-of[@kind='added unsafe solutions'][text()='0']"> -->
    
<!--     \paragraph{No unsafe solutions were added} compared to the previous target. -->
    
<!--   </xsl:when> -->
<!--   <xsl:otherwise> -->

<!--     We now present the changes of the set of unsafe solution, -->
<!--     added and lost compared to the previous target. -->

<!--   \paragraph{Added} -->

<!--   \begin{center} -->
<!--   \OutputTargetFile{Solutions/added-unsafe-solutions.tex} -->
<!--   \end{center} -->
	
<!--   </xsl:otherwise> -->
<!-- </xsl:choose> -->
  
<!-- <xsl:choose> -->
<!--   <xsl:when test="document($statistics-path)// -->
<!-- 		  number-of[@kind='lost unsafe solutions'][text()='-']"> -->

<!--   </xsl:when> -->
<!--   <xsl:when test="document($statistics-path)// -->
<!-- 		  number-of[@kind='lost unsafe solutions'][text()='0']"> -->

<!-- \paragraph{No unsafe solutions were lost} compared to the previous target. -->

<!--   </xsl:when> -->
  
<!--   <xsl:otherwise> -->

<!--   \paragraph{Lost} -->

<!-- 	\begin{center} -->
<!-- 	\OutputTargetFile{Solutions/lost-unsafe-solutions.tex} -->
<!-- 	\end{center} -->

<!--   </xsl:otherwise> -->
<!-- </xsl:choose> -->

<text> 
  \subsubsection{Pure Solutions}

  Pure solutions are solutions without the  variable
  \texttt{b\_target}. Therefore they cannot be checked for safety or unsafety.

<xsl:choose>
  <xsl:when test="document($statistics-path)//
		  number-of[@kind='added pure solutions'][text()='-']">
    \paragraph{Initial}
    
    \begin{center}
    \OutputTargetFile{Solutions/added-pure-solutions.tex}
    \end{center}
    
  </xsl:when>
  <xsl:when test="document($statistics-path)//
		  number-of[@kind='added pure solutions'][text()='0']">
    
    \paragraph{No pure solutions were added} compared to the previous target.
    
  </xsl:when>
  <xsl:otherwise>

    We now present the changes of the set of pure solution,
    added and lost compared to the previous target.

  \paragraph{Added}

  \begin{center}
  \OutputTargetFile{Solutions/added-pure-solutions.tex}
  \end{center}
	
  </xsl:otherwise>
</xsl:choose>
  
<xsl:choose>
  <xsl:when test="document($statistics-path)//
		  number-of[@kind='lost pure solutions'][text()='-']">

  </xsl:when>
  <xsl:when test="document($statistics-path)//
		  number-of[@kind='lost pure solutions'][text()='0']">

\paragraph{No pure solutions were lost} compared to the previous target.

  </xsl:when>
  
  <xsl:otherwise>

  \paragraph{Lost}

	\begin{center}
	\OutputTargetFile{Solutions/lost-pure-solutions.tex}
	\end{center}

  </xsl:otherwise>
</xsl:choose>

  %%%%%%
</text>
  <xsl:if test="$TYPE='Bool'">
     <text>
       \subsection{Transition Graph of Boolean Network}
        \IfFileExists{</text>
<xsl:value-of
    select="concat($OUTPUT_DIR,'/',$ANALYSIS,'/',$TARGETS_OUTDIR,'/',$targets-id,'/Solutions/boolean-network-graph.png')"/>
     <text>}{File at \texttt{</text>

	 <xsl:value-of select="concat($OUTPUT_DIR,'/',$ANALYSIS,'/',$TARGETS_OUTDIR,'/',$targets-id,'/Solutions/boolean-network-graph.png')"/>
	 <text>}

	 \medskip
	 
	 \GraphicsOutputTargetFile{Solutions/boolean-network-graph.png}
}
       {Graph png-file not present at \texttt{</text>

	 <xsl:value-of select="concat($OUTPUT_DIR,'/',$ANALYSIS,'/',$TARGETS_OUTDIR,'/',$targets-id,'/Solutions/boolean-network-graph.png')"/>
	 <text>}. That may be due to a timeout of the call of \texttt{dot -Tpng }...}
	 </text>
     </xsl:if>

  <text>
\subsection{Graphs of the n-first Solutions (pure) } %Initial Safe of Unsafe, or Added Unsafe)}

\OutputTargetFile{Solutions-Graph/graph.tex}

\subsection{Change Predictions}

Our algorithm now selects solutions that correspond
to the prediction of knockout, knockups, and inflow
changes.

Any solution with single change raises a single change
prediction. A knock change of reaction $r$ requires
$\ko{r} \in\{\DoN,\Do,\Up,\Up\}$. A speed change of
reacion $r$ requires $v_r\in\{\DoN,\Do,\Up,\Up\}$.

A double-knock prediction for $(r,r')$
requires that $r$ is a single knock prediction
and that there exists a solution in which reaction
$r$ is deleted ($o_r=\NoN$) and reaction $r'$ 
is knocked out or knocked up ($o_{r'}=\DoN$ or $o_{r'}=\Up$). 

\OutputTargetFile{Solutions/changes.tex}

\subsection{Substitution On Difference Variables}

The simplification procedure on difference constraints has performed the following
substitutions on difference variables  (Applied in Solution Graphs):

\OutputTargetFile{substitution-constraints.tex}
  </text>
</text>
</xsl:template>

 <xsl:template match="*" mode="footer">
   <text>\section{The elemenary flux modes of the metabolic subnetwork}

\OutputFile{Equations/Flux-Graph/graph.tex}

\newpage
   </text>

   
 <text>

   \section{Minizinc Constraints for the last target}

   <xsl:if test="$TYPE='bool'">

     \subsection{Constraints in file: boolean-constraints.mzn}

   \VerbOutputTargetFile{Solutions/boolean-constraints.mzn}

\subsection{Network constraints in file: boolean-network-constraints.mzn}
   
   \VerbOutputTargetFile{Solutions/boolean-network-constraints.mzn}

\subsection{Network constraints in file: next-species.mzn}
   
   \VerbOutputTargetFile{Solutions/next-species.mzn}
   </xsl:if>

   <xsl:if test="not($TYPE='bool')">

\subsection{new-constraints.mzn}

  \VerbOutputTargetFile{Solutions/new-constraints.mzn}
   </xsl:if>
    
\end{document}</text>
 </xsl:template>
</xsl:stylesheet>
