<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
     version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:variable name="EMPTY">
      <text>Targets/empty.xml</text>    
    </xsl:variable>

    <xsl:param name="EQUATIONS"  select="$EMPTY" required="no"/>		

    <xsl:variable name="simplified">
      <xsl:copy-of select="document(concat('../',$EQUATIONS,
			   '/flat-simplified-steady-state.xml'))"/>
    </xsl:variable>

 

    <xsl:variable name="positive-equations">
      <comment txt="flat-positive-constraints"/>
      <xsl:copy-of select="document(concat('../',$EQUATIONS,
			   '/flat-positive-equations.xml'))"/>
    </xsl:variable>

    <!-- main target -->

    <xsl:template match="difference-constraints|equations">
      <difference-constraints>
	<xsl:apply-templates select="*" mode="cons"/>
      </difference-constraints>
    </xsl:template>

    <!-- minsupp equations -->
    
    <xsl:variable name="minsupp-equations">
       <xsl:copy-of select="document(concat('../',$EQUATIONS,
			   '/minsupp/steady-state/flat-equations.xml'))"/>
    </xsl:variable>

    <xsl:template match="minsupp-equations" mode="cons">
      <xsl:comment> minsupp/steady-state/flat-equations.xml </xsl:comment>
      <xsl:apply-templates select=
			   "$minsupp-equations/(equations|steady-state)/*" mode="cons"/>
    </xsl:template>
    
    <xsl:variable name="minsupp-dot-equations">
      <xsl:copy-of select="document(concat('../',$EQUATIONS,
			   '/minsupp/linear-equations/flat-minimal-support-consequences.xml'))"/>
    </xsl:variable>
    
    <xsl:template match="minsupp-dot-equations" mode="cons">
      <xsl:comment> minsupp/steady-state/flat-equations.xml </xsl:comment>
      <xsl:apply-templates select=
			   "$minsupp-dot-equations/(steady-state|equations)/*" mode="cons"/>
    </xsl:template>

    <!-- exact equations -->

    <xsl:variable name="exact-equations">
      <xsl:copy-of select="document(concat('../',$EQUATIONS,
			   '/exact/steady-state/flat-equations.xml'))"/>
    </xsl:variable>
    
    <xsl:template match="exact-equations" mode="cons">
      <xsl:comment> exact/linear-equations/flat-equations.xml </xsl:comment>
      <xsl:apply-templates select=
			   "$exact-equations/(steady-state|equations)/*" mode="cons"/>
    </xsl:template>
    
    <xsl:variable name="exact-dot-equations">
      <xsl:copy-of select="document(concat('../',$EQUATIONS,
			   '/exact/linear-equations/flat-equations.xml'))"/>
    </xsl:variable>
    
    <xsl:template match="exact-dot-equations" mode="cons">
      <xsl:comment> exact/linear-equations/flat-equations.xml </xsl:comment>
      <xsl:apply-templates select=
			   "$exact-dot-equations/(steady-state|equations)/*" mode="cons"/>
    </xsl:template>

    <!-- steady-state equations -->

    <xsl:variable name="steady-state">
      <xsl:copy-of select="document(concat('../',$EQUATIONS,
			   '/flat-steady-state.xml'))"/>
    </xsl:variable>

    <xsl:template match="steady-state" mode="cons">
      <xsl:message>
	difference-constraints.xsl accesses: flat-steady-state.xml
	- number of equations <xsl:value-of select="count($steady-state/steady-state/*)"/>
      </xsl:message>
      <xsl:comment> steady state equations </xsl:comment>
      <xsl:apply-templates select=
			   "$steady-state/steady-state/*" mode="cons"/>
    </xsl:template>

    <!-- conservation laws -->
    
    <xsl:variable name="cons-equations">
       <xsl:copy-of select="document(concat('../',$EQUATIONS,
			   '/ConsLaws/flat-CONS-equations.xml'))"/>
    </xsl:variable>

    <xsl:template match="cons-equations" mode="cons">
      <xsl:apply-templates select=
			   "$cons-equations/steady-state/child::*" mode="cons"/>
    </xsl:template>

    

    <!-- function cons -->
    <xsl:template match="rewrite-equations" mode="cons">
      <xsl:apply-templates select=
			   "$simplified/steady-state/child::*" mode="cons"/>
    </xsl:template>
    
    <xsl:template match="comment" mode="cons">
      <xsl:copy-of select="."/>
    </xsl:template>

 

    <xsl:template match="positive-equations" mode="cons">
      <xsl:message>
	difference-constraints.xsl accesses: flat-positive-equations.xml
	- number of equations <xsl:value-of
	select="count($positive-equations/equations/*)"/>
      </xsl:message>
       <xsl:comment> positive equations and constraints </xsl:comment>
       <xsl:apply-templates select=
			    "$positive-equations/equations/*" mode="cons"/>
    </xsl:template>

   

    <xsl:template match="context-mass-loss[@min]" mode="cons">
      <mass-loss>
	<xsl:copy-of select="@min"/>
	<in>
	  <xsl:comment> inflows </xsl:comment>
	</in>
	<out>
	  <xsl:comment> outflows </xsl:comment>
	</out>
      </mass-loss>
    </xsl:template>


    <xsl:template match="equal
			 [child::*[1][self::sum]]
			 [child::*[2][self::sum]]"              mode="cons">
     <equal>
        <xsl:apply-templates select="child::*"/>
     </equal>
   </xsl:template>

   <xsl:template match="equal[child::sum]
			[not(child::*[1][self::sum])]" mode="cons">
      <sum>
	<xsl:apply-templates select="child::sum/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </sum>
   </xsl:template>

   <!-- may be not right? -->
   <xsl:template match="equal[*[2][name()='mult']]" mode="cons">
      <mult>
	<xsl:apply-templates select="child::*[2]/*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </mult>
   </xsl:template>

   <xsl:template match="equal[intsum]" mode="cons">
      <intsum>
	<xsl:apply-templates select="child::intsum/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </intsum>
   </xsl:template>

   <xsl:template match="equal[power]" mode="cons">
      <power>
	<xsl:apply-templates select="child::power/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
	</power>
   </xsl:template>

   <xsl:template match="equal[inh]" mode="cons">
     <inh>
       <xsl:apply-templates select="child::inh/child::*"/>
       <xsl:apply-templates select="child::*[1]"/>
     </inh>
   </xsl:template>

   <xsl:template match="equal[acc]" mode="cons">
      <acc>
	<xsl:apply-templates select="child::acc/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </acc>
   </xsl:template>
   
   <!-- hmmm quick hack? -->
    <xsl:template match="equal[*[2][self::var]]" mode="cons">
      <equal>
	<xsl:copy-of select="*"/>
      </equal>
   </xsl:template>
   
    <xsl:template match="equal[apply]" mode="cons">
      <apply>
	<xsl:apply-templates select="apply/@kinetics"/>
	<xsl:apply-templates select="apply/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </apply>
    </xsl:template>

    <xsl:template match="similar[child::*[1][self::sum]]" mode="cons">
      <sum>
	<xsl:apply-templates select="child::*[1]/self::sum/child::*"/>
	<xsl:apply-templates select="child::*[2]"/>
      </sum>
    </xsl:template>

    <xsl:template match="similar[child::*[2][self::sum]]" mode="cons">
      <sum>
	<xsl:apply-templates select="child::*[2]/self::sum/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </sum>
    </xsl:template>

   <xsl:template match="similar[count(var|speed|knockout|conc)=2]" mode="cons">
      <equal>
	<xsl:apply-templates select="child::*"/>
      </equal>
   </xsl:template>
   
   <xsl:template match="nonzero" mode="cons">
     <xsl:copy-of select="."/>
   </xsl:template>

   <!--
   <xsl:template match="nonzero" mode="cons">
     <member>
       <xsl:apply-templates select="child::*[1]"/>
       <set>
	 <diff value="No"/>
	 <diff value="Do"/>
	  <diff value="Up"/>
       </set>
     </member>
   </xsl:template>
    -->

    <xsl:template match="similar[mult][not(sum)]
			 [not(binary-minus)]" mode="cons">
<!--      <xsl:if test=".[child::*[1]] 
		    [child::*[2]/self::mult[child::*[1]/
		    self::knockout]]">

	<bound max="1">
	 <member>
	    <xsl:apply-templates select="child::*[2]/
					 self::mult/child::*[1]"/>
	    <set>
	      <diff value="DoN"/>
	    </set>
	 </member>
	 <member>
	    <xsl:apply-templates select="child::*[2]/self::mult/
					 child::*[2]"/>
	    <set>
	      <diff value="NoN"/>
	      <diff value="UpN"/>
	    </set>
	 </member>
	</bound> 
      </xsl:if> -->
      <mult>
	<xsl:apply-templates select="child::mult/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </mult>
    </xsl:template>

    <xsl:template match="similar[apply]" mode="cons">
      <apply>
	<xsl:copy-of select="apply/@kinetics"/>
	<xsl:copy-of select="apply/child::*"/>
	<xsl:copy-of select="child::*[1]"/>
      </apply>
    </xsl:template>

    <xsl:template match="similar" mode="cons">
      <equal>
	<xsl:apply-templates select="child::*"/>
      </equal>
    </xsl:template>

    <xsl:template match="constant" mode="cons">
      <diff-equal>
	  <xsl:copy-of select="@value"/>
      </diff-equal>
    </xsl:template>

      <!-- otherwise -->
 
    <xsl:template match="*" mode="cons">  
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <!--<xsl:message>	difference-constraints.xsl: cons otherwise <xsl:copy-of select="$tag"/>
      </xsl:message>-->
      <xsl:element name="{$tag}"> 
	<xsl:apply-templates select="@*|child::*"/>
      </xsl:element>
    </xsl:template>
    
    <xsl:template match="@*" mode="cons">
	<xsl:copy-of select="."/>
    </xsl:template>
    
    <!-- otherwise needed -->
    <xsl:template match="*">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <!--<xsl:message>	difference-constraints.xsl: otherwise <xsl:copy-of select="$tag"/>
      </xsl:message>-->
      <xsl:element name="{$tag}"> 
	<xsl:apply-templates select="@*|child::*"/>
      </xsl:element>
    </xsl:template>
    <xsl:template match="@*">
	<xsl:copy-of select="."/>
    </xsl:template>

</xsl:stylesheet>
