<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

  <xsl:output   
      method="xml" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>


  <xsl:param name="params"/>

  
  <xsl:variable name="params-doc">
    <xsl:copy-of select="document($params)"/>
  </xsl:variable>
  <xsl:variable name="specification-doc">
    <xsl:copy-of select="document(concat(
			 '../Targets/',
			 $params-doc/params/@specification,
			 '.xml'))"/>
  </xsl:variable>
  <xsl:variable name="targets-spec">
      <xsl:copy-of
	  select="$specification-doc/specification/
		  targets[@id=$params-doc/params/@targets]"/>
  </xsl:variable>
    
  <xsl:template match="/">
    <xsl:message>parameters
         - <xsl:copy-of select="$params-doc"/>
    </xsl:message>
    <xsl:message>specification file: Targets/<xsl:value-of
      select="$params-doc/params/@specification"/>.xml
    </xsl:message>
      <xsl:variable name="targets">
      <xsl:apply-templates select="$targets-spec" mode="resolve"/>
    </xsl:variable>
    <!--
	<xsl:message>selected targets
	- <xsl:copy-of select="$targets"/>
	</xsl:message>
	-->
    <xsl:copy-of select="$targets"/>
  </xsl:template>
  
  <xsl:template match="target-ref" mode="resolve">
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    <xsl:variable name="target">
      <xsl:apply-templates
	  select="$specification-doc/
		  specification/target[@id=$id]"   mode="resolve-and-integrate"/>
    </xsl:variable>
    <xsl:copy-of select="$target"/>
    </xsl:template>

    <xsl:template match="target-all" mode="resolve">
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    <xsl:variable name="target">
      <xsl:apply-templates
	  select="$specification-doc/
		  specification/target[@id=$id]"   mode="resolve-and-integrate"/>
    </xsl:variable>
    <xsl:copy-of select="$target/target/*"/>
    </xsl:template>

    <xsl:template match="*" mode="resolve-and-integrate">
      <xsl:variable name="pretarget">
	<xsl:apply-templates select="." mode="resolve"/>
	</xsl:variable> 
	<xsl:message>
	  select-argets.xsl
	  - pretarget
	  - <xsl:copy-of select="$pretarget"/>
	</xsl:message>
	<xsl:variable name="target">
	  <xsl:apply-templates select="$pretarget"
			       mode="integrate-attributes"/>
	</xsl:variable>
	<xsl:message>
	  target <xsl:copy-of select="$target"/>
	</xsl:message>
	<xsl:copy-of select="$target"/>
    </xsl:template>

    <xsl:template match="param" mode="resolve">
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    <xsl:variable name="name">
      <xsl:value-of select="@name"/>
    </xsl:variable>
    <!--<xsl:message>resolve
      param <xsl:copy-of select="."/>"
      name  "<xsl:value-of select="$name"/>"
      id    "<xsl:value-of select="$id"/>"
      </xsl:message>-->
    <attribute>
      <xsl:if test="not($params-doc/params/@*[name()=$id]='')">
	<xsl:attribute name="{$name}">
	  <xsl:value-of select="$params-doc/params/@*[name()=$id]"/>
	</xsl:attribute>
      </xsl:if>
    </attribute>
     </xsl:template>

    <xsl:template match="*" mode="resolve">
      <!--<xsl:message>
	default <xsl:value-of select="name()"/>
      </xsl:message>-->
    <xsl:variable name="name">
      <xsl:value-of select="name()"/>
    </xsl:variable>
    <xsl:element name="{$name}">
      <xsl:apply-templates select="@*|*|text()|comment()" mode="resolve"/>
    </xsl:element>
  </xsl:template>
   <xsl:template match="@*|text()|comment()" mode="resolve">
    <xsl:copy-of select="."/>
   </xsl:template>

   <xsl:template match="*[attribute]" mode="integrate-attributes">
 
     <!--<xsl:message>
       +++ do integrate-attribute <xsl:value-of select="name()"/>
       <xsl:copy-of select="."/>
       </xsl:message>
 -->
       
     <xsl:variable name="name">
      <xsl:value-of select="name()"/>
     </xsl:variable>
     <xsl:variable name="int">
       <xsl:element name="{$name}">
	 <xsl:copy-of select="attribute/@*"/>
       </xsl:element>
     </xsl:variable>
     <xsl:message>
       int=<xsl:copy-of select="$int"/>
     </xsl:message>
     <xsl:copy-of select="$int"/>
   </xsl:template>

   <xsl:template match="*" mode="integrate-attributes">
     <!--<xsl:message>
       integrate-attributes <xsl:value-of select="name()"/>
     </xsl:message>-->
    <xsl:variable name="name">
      <xsl:value-of select="name()"/>
    </xsl:variable>
    <xsl:element name="{$name}">
      <xsl:apply-templates select="@*|*|text()|comment()"
			   mode="integrate-attributes"/>
    </xsl:element>
  </xsl:template>
   <xsl:template match="@*|text()|comment()" mode="integrate-attributes">
    <xsl:copy-of select="."/>
   </xsl:template>
  
</xsl:stylesheet>
