<xsl:stylesheet 
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="text" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>

    <xsl:param name="OUTPUT_DIR"    required="yes"/>
    <xsl:param name="NETWORK"       required="yes"/>
    <xsl:param name="ANALYSIS"      required="yes"/>
    <xsl:param name="GRAPHPATH"     required="yes"/>
    <xsl:param name="TYPE"     required="yes"/>

    <xsl:variable name="NETWORK-LATEX-FIX">
      <xsl:value-of select="replace($NETWORK,'_','-')"/>
    </xsl:variable>
    
    <xsl:variable name="OUTPUT_DIR-LATEX-FIX">
      <xsl:value-of select="replace($OUTPUT_DIR,'_','-')"/>
    </xsl:variable>
    
    <xsl:template match="/">

      <xsl:variable name="full-id">
	<xsl:value-of select="network/@id"/>
	<xsl:if test="network/@kind">
	  <text>-</text>
	  <xsl:value-of select="network/@kind"/>
	</xsl:if>
      </xsl:variable>

      <xsl:variable name="id-latex-fix">
	  <xsl:value-of select="replace(network/@id,'_','-')"/>
      </xsl:variable>


      <text xml:space="preserve">      
\documentclass{</text>
      <xsl:copy-of select="$GRAPHPATH"/>
	<text>/Latex/llncs}

\usepackage[a3paper]{geometry}
\usepackage[colorlinks]{hyperref}

\usepackage{longtable}
\usepackage{capt-of}

\usepackage{amsmath}
\usepackage{amssymb}
\usepackage{natbib}
\usepackage{stmaryrd}
\usepackage{mathrsfs}
\usepackage{bussproofs}
\usepackage{graphicx}
\usepackage{color}
\usepackage{multirow}
\usepackage[mathletters]{ucs}
\usepackage[utf8x]{inputenc}
%\usepackage[T1]{fontenc}
\usepackage{</text>
      <xsl:copy-of select="$GRAPHPATH"/>
	<text>/Latex/inference}
\usepackage{xspace}
\usepackage{listings}
\usepackage[version=3]{mhchem}
\usepackage{pdftexcmds} % for switch-macros
\usepackage{</text>
      <xsl:copy-of select="$GRAPHPATH"/>
	<text>/Latex/macros}
\usepackage{etoolbox} % needed for ifnumber
\usepackage{</text>
      <xsl:copy-of select="$GRAPHPATH"/>
	<text>/Latex/ifnumber}
      </text>
      
      <xsl:if test="not(//*[@latex-look])">
	<text>
	  \usepackage{../Graph/Latex/macros-species}
	  \usepackage{../Graph/Latex/switch-macros}
	</text>
      </xsl:if>

      <xsl:if test="//*[@latex-look]">
	<text>
	\usepackage{</text>
	<xsl:value-of select="$OUTPUT_DIR"/><text>/</text>
	<text>/Graph/latex-macros}</text>
      </xsl:if>


  <text>\usepackage[usenames,dvipsnames,x11names]{xcolor}
\usepackage{tikz}
\usepackage{</text>
      <xsl:copy-of select="$GRAPHPATH"/>
	<text>/Latex/tikz-macros}

\usetikzlibrary{decorations.pathmorphing}


\EnableBpAbbreviations

\renewcommand\TIKZID[1]{</text>
      <xsl:value-of select="$full-id"/>
      <text xml:space="preserve">}

\newcommand\OutputFile[1]{\input{</text>
      <xsl:value-of select="$OUTPUT_DIR"/>
      <text>/#1}}

\newcommand\VerbOutputFile[1]{\verbinput{</text>
      <xsl:value-of select="$OUTPUT_DIR"/>
      <text>/#1}}


\newcommand\OutputFileIfExists[1]{\IfFileExists{</text>
<xsl:value-of  select="$OUTPUT_DIR"/><text>/#1}
{\OutputFile{#1}}
}

      \begin{document}



\begin{center}\huge\bf
Reaction Network </text> 
<xsl:value-of select="$NETWORK-LATEX-FIX"/>
<text>\\ Equations TYPE=</text>
      <xsl:value-of select="$TYPE"/>
<text>


  \end{center}

%\tableofcontents 

 

\section{Network</text>
<text>}

\paragraph{Reaction Network.}

See file: \texttt{Network/</text>
<xsl:copy-of select="$OUTPUT_DIR-LATEX-FIX"/>
<text xml:space="preserve">/</text>
<xsl:value-of select="NETWORK-LATEX-FIX"/>
<text xml:space="preserve">network.xml}
\begin{center}
\input{</text>
<xsl:value-of select="$OUTPUT_DIR"/>
<text>/Graph/graph-tikz.tex}</text>

<text>
\smallskip
\hrule
\captionof{figure}{\label{fig:</text><xsl:value-of select="$NETWORK-LATEX-FIX"/><text>}
The graph of the network in </text><xsl:value-of select="$OUTPUT_DIR-LATEX-FIX"/><text>/Graph/graph.tex}
\addtocounter{table}{-1}
\end{center}

\paragraph{Underlying Metabolic Network and its Elementary Flux Modes.}

\begin{center}
\input{</text>
<xsl:value-of select="$OUTPUT_DIR"/><text>/</text>
<text>Flux-Graph/graph-tikz.tex}</text>

<text>
\smallskip
\hrule
\captionof{figure}{\label{fig:</text><xsl:value-of select="$NETWORK-LATEX-FIX"/><text>}
The flux graph of the metabolic part of network in </text><xsl:value-of select="$OUTPUT_DIR-LATEX-FIX"/><text>/Flux-Graph/old-graph.xml.}
\addtocounter{table}{-1}
\end{center}

\section{Ordinary Differential Equations (ODEs)}

The following ODE's are inferred from the reaction network and the knockout
candidates. Since some kinetic functions are known only up to similarity, we
often have to weaken equality into $\sim$.

\medskip

\noindent\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/ODEs.tex}
</text>

<xsl:if test="$TYPE='Bool'">
<text>
\section{Polynomial Dot Equations}

\subsection{Positive Equations}

We now consider consider ODEs as positive arithmetic equations.
Derivatives $dS/dt$ yield real-valued variables $\dot{S}$ at any time point.
These reals are split to their positive and negative parts. So for each
real variable $\dot{S}$ we have two positive real variables
$\posdot{S}$ and $\negdot{S}$ such that $\dot{S}=\posdot{S}-\negdot{S}$.
In addition,
we consider partial kinetic infomation as complete, i.e., we
produce expressions modulo equality instead of similarity.

\medskip

\noindent
\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/positive-equations.tex}


\subsection{Flattened Positive Equations}

\noindent\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/flat-positive-equations.tex}

\subsection{Non-Linear Constraints}

\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/non-linear-constraints.tex}

\section{Linear Dot Equations}

\subsection{Linear Equations}

\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/linear-equations.tex}

\subsection{Minimal Support Consequences}

\OutputFileIfExists{minsupp/linear-equations/equations.tex}

\subsection{Exact Rewriting}

\OutputFileIfExists{exact/linear-equations/equations.tex}

</text>
</xsl:if>

  <xsl:if test="not($TYPE='Bool')">
<text>
\section{Steady State Equations}

When imposing the steady state assumptions (concentrations, reaction speeds,
and the speeds of input/output fluxes have become stable, then we obtain the
following arithmetic equations, but with kinetic functions known only modulo
similarity $\sim$:

\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/steady-state.tex}


\subsection{Flattened Steady State Equations}

We remove nested expressions before applying the
abstract interpretation.


\noindent\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/flat-steady-state.tex}

\subsection{Minimal Support Consequences}

\OutputFileIfExists{minsupp/steady-state/equations.tex}


\subsection{Exact Rewriting}

\OutputFileIfExists{exact/steady-state/equations.tex}

</text>

  </xsl:if>

<text>
\end{document}</text>
    </xsl:template>


</xsl:stylesheet>

