<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:mathml="http://www.w3.org/1998/Math/MathML"
    version="2.0"
    exclude-result-prefixes="xs rdf xsl mathml"
    >

  <!--xpath-default-namespace="http://www.sbml.org/sbml/level2/version4"-->


  <xsl:output   
      method="xml" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>
  <xsl:param name="NETWORK"      required="no"/>
  <xsl:variable name="DEBUG">false</xsl:variable>

  
  <xsl:variable name="model">
    <xsl:copy-of select="/"/>
  </xsl:variable>

  
  <xsl:template match="/">
    <xsl:if test="$DEBUG='true'">
      <xsl:message>°°°° Core SBML: eliminate macros </xsl:message>
    </xsl:if>
    <xsl:variable name="network">
      <xsl:apply-templates select="*" mode="elim-macros"/>
    </xsl:variable>
    
    <xsl:copy-of select="$network"/>
  </xsl:template>

    
   <!-- function apply-->

  <xsl:template match="*:lambda[*[1][local-name()='bvar']]"  mode="apply">
    <xsl:param name="args"/>
    <xsl:if test="$DEBUG='true'">
    <xsl:message>   apply function: lambda
    - function = <xsl:copy-of select="."/>
    - args = <xsl:copy-of select="$args/*"/>
    - firstarg = <xsl:copy-of select="$args/*:args/*[1]"/>
    </xsl:message>
   </xsl:if> 
    <xsl:variable name="argument">
	 <xsl:copy-of select="$args/*:args/*[1]"/>
    </xsl:variable>
    <!--<xsl:message>
      variable =  <xsl:copy-of select="*[1]"/>
    </xsl:message>-->
     <xsl:variable name="nextfun">
       <lambda>
	 <xsl:apply-templates select="*[position()&gt;1]"  mode="substitute">
	   <xsl:with-param name="variable">
	      <xsl:copy-of select="*[1]"/>
	   </xsl:with-param>
	   <xsl:with-param name="argument">
	     <xsl:copy-of select="$argument"/>
	   </xsl:with-param>
	   <xsl:with-param name="reaction">
	     <xsl:copy-of select="ancestor::reaction"/>
	  </xsl:with-param>
	 </xsl:apply-templates>
       </lambda>
     </xsl:variable>
    <xsl:variable name="nextargs">
      <args>
	<xsl:copy-of select="$args/*:args/*[position() &gt; 1]"/>
      </args>
    </xsl:variable>
    <xsl:if test="$DEBUG='true'">
      <xsl:message>   - nextargs = <xsl:copy-of select="$nextargs"/>
    </xsl:message></xsl:if>
   <xsl:apply-templates select="$nextfun"    mode="apply">
      <xsl:with-param name="args">
	<xsl:copy-of select="$nextargs"/>
      </xsl:with-param>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="*:lambda[*[1][not(local-name()='bvar')]]"
		mode="apply">
    <!--<xsl:message>
      end-of-apply: <xsl:copy-of select="*"/>
    </xsl:message>-->
    <xsl:copy-of select="*"/>    
  </xsl:template>

  <!-- function substitute -->
  <xsl:template match="var"  mode="substitute">
    <xsl:param name="variable"/>
    <xsl:param name="argument"/>
    <xsl:param name="reaction"/>
   <!-- <xsl:message> substitute 
    var=<xsl:copy-of select="."/>
    variable=<xsl:copy-of select="$variable"/>
    argument=<xsl:copy-of select="$argument"/>
    </xsl:message>-->
    <xsl:choose>
      <xsl:when test="$variable/bvar/@id=@id">
	<xsl:if test="$DEBUG='true'">
	  <xsl:message> successful [<xsl:copy-of
	  select="$variable"/>/<xsl:copy-of select="$argument"/>]
	  </xsl:message>
	</xsl:if>
	<xsl:copy-of select="$argument"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:copy-of select="."/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="*"  mode="substitute">
    <xsl:param name="variable"/>
    <xsl:param name="argument"/>
    <xsl:param name="reaction"/>

    <!-- <xsl:message> substitute <xsl:copy-of select="$variable"/> in <xsl:copy-of select="."/> 
    </xsl:message>-->

    <xsl:variable name="name">
      <xsl:value-of select="name()"/>
    </xsl:variable>
      
    <xsl:element name="{$name}">
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates select="*|text()" mode="substitute">
	<xsl:with-param name="variable">
	  <xsl:copy-of select="$variable"/>
	</xsl:with-param>
	<xsl:with-param name="argument">
	  <xsl:copy-of select="$argument"/>
	</xsl:with-param>
	<xsl:with-param name="reaction">
	  <xsl:copy-of select="$reaction"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </xsl:element>
  </xsl:template>

    <xsl:template match="text()"  mode="substitute">
      <xsl:copy-of select="."/>
    </xsl:template>


     
   <!-- function elim-macros -->

   <xsl:template match="speed" mode="elim-macros">
     <xsl:param name="reaction"/>
     <xsl:variable name="id">
       <xsl:value-of select="@reaction"/>
     </xsl:variable>
     <!--<xsl:message>
       speed of reaction
       <xsl:copy-of
	   select="$model//reaction[@id=$id]/kinetic-expression/*"/>
     </xsl:message>-->
     <xsl:choose>
       <xsl:when test="count($model//reaction[@id=$id])=1">
	 <xsl:comment> speed of reaction ++ </xsl:comment>
	 <xsl:apply-templates
		select="$model//reaction[@id=$id]/kinetic-expression/*"
		mode="elim-macros">
	    <xsl:with-param name="reaction">
	      <xsl:copy-of select="$model//reaction[@id=$id] "/>
	    </xsl:with-param>
	  </xsl:apply-templates>
	  <xsl:comment> end speed of reaction ++ </xsl:comment>
		 
	    <!-- <xsl:copy-of
		 select="$model//reaction[@id=$id]/kinetic-expression/*"/>-->
       </xsl:when>
       <xsl:otherwise>
       <xsl:message>
	 ERROR: unknown or nonunique reaction to resolve <xsl:copy-of
	 select="."/>
       </xsl:message>
     </xsl:otherwise>
     </xsl:choose>
     
   </xsl:template>
   
   <xsl:template match="expr" mode="elim-macros">
     <xsl:param name="reaction"/>
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:if test="$DEBUG='true'">
       <xsl:message>
       +++++
       <xsl:copy-of select="."/>
       - reaction <xsl:copy-of select="$reaction"/>
       - local expression <xsl:copy-of
       select="$reaction/reaction/expression[@id=$id]"/>
       - global expression<xsl:copy-of
       select="$model//network/expression[@id=$id]"/>
       </xsl:message>
     </xsl:if>
<!--        <xsl:when test="not($reaction='') and
     count($reaction/reaction/expression[@id=$id])=1">-->
     <xsl:choose>
       <xsl:when test="count($reaction/reaction/expression[@id=$id])=1">
	  <xsl:variable name="result">
	    <xsl:if test="$DEBUG='true'">
	      <xsl:message>    - local expression chosen +++++++++++++
	      <xsl:copy-of
		  select="$reaction/reaction/expression[@id=$id]/*"/>
	      </xsl:message>
	    </xsl:if>
	   <xsl:apply-templates
		select="$reaction/reaction/expression[@id=$id]/*"
		mode="elim-macros"/>
	 </xsl:variable>
	 
	 <xsl:apply-templates select="$result" mode="elim-macros">
	   <xsl:with-param name="reaction">
	     <xsl:copy-of select="$reaction"/>
	   </xsl:with-param>
	 </xsl:apply-templates>
       </xsl:when>

       <xsl:when test="count($model//network
		       /expression[@id=$id])=1">
	  <xsl:variable name="result">
	    <xsl:apply-templates
		select="$model//network
			/expression[@id=$id]/*"
		mode="elim-macros"/>
	 </xsl:variable>
	 
	 <xsl:apply-templates select="$result" mode="elim-macros">
	   <xsl:with-param name="reaction">
	     <xsl:copy-of select="$reaction"/>
	   </xsl:with-param>
	 </xsl:apply-templates>
       </xsl:when>
       
       <xsl:when
	   test="count($reaction/reaction/expression[@id=$id])&gt;1
		 or count($model//network/expression[@id=$id])&gt;1">
	 <xsl:message>
	   ++++++ ERROR expr <xsl:value-of select="@id"/> defined more than  once  +++++++++++++++++++++++++++++++++++++++++

	   -local <xsl:copy-of
	   select="$reaction/reaction/expression[@id=$id]"/>
	   -global <xsl:copy-of
	   select="$model//network/expression[@id=$id]"/>
	 </xsl:message>
       </xsl:when>
       
       <xsl:otherwise>
	 <xsl:message>
	   ++++++ expr <xsl:value-of select="@id"/> in reaction
	   <xsl:copy-of select="$reaction"/>
	 </xsl:message>
	 <xsl:message>
	   Error: missing parameter: <xsl:value-of select="$id"/>
	 </xsl:message>
	 <constant value="dummy-for-missing-parameter-may-be-local"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:template>

   <xsl:template match="reaction" mode="elim-macros">
     <reaction>
       <xsl:apply-templates select="@*|*|text()|comment()"
			    mode="elim-macros">
	   <xsl:with-param name="reaction">
	     <xsl:copy-of select="."/>
	   </xsl:with-param>
       </xsl:apply-templates>
     </reaction>
   </xsl:template>

   <xsl:template match="apply" mode="elim-macros">
     <xsl:param name="reaction"/>
     <xsl:variable name="id">
       <xsl:value-of select="@fun"/>
     </xsl:variable>
      <xsl:variable name="function">
       <xsl:copy-of select="$model//function[@id=$id]/lambda"/>
      </xsl:variable>
      
      <xsl:variable name="formal-arguments">
       <args>
	 <xsl:copy-of select="$function//lambda/bvar"/>
       </args>
     </xsl:variable>
     <xsl:choose>
       <xsl:when test="count($formal-arguments/args/*:bvar)
		       =count(*)">
	 <xsl:if test="$DEBUG='true'">
	   <xsl:message> function type correct
	   function <xsl:copy-of select="$function"/>
	   arguments <xsl:copy-of select="*"/>
	   </xsl:message>
	 </xsl:if>
         <xsl:variable name="apply-result">
	   <xsl:apply-templates select="$function//lambda" mode="apply">
	     <xsl:with-param name="args">
	       <args>
		 <xsl:copy-of select="*"/>
	       </args>
	     </xsl:with-param>
	   </xsl:apply-templates>
	 </xsl:variable>
	<xsl:if test="$DEBUG='true'">
	   <xsl:message>
	     +++ apply-result=<xsl:copy-of select="$apply-result"/> +++++
	   </xsl:message>
       </xsl:if>
       <xsl:variable name="translation">
	 <xsl:apply-templates select="$apply-result"
			      mode="elim-macros">
	   <xsl:with-param name="reaction">
	     <xsl:copy-of select="$reaction"/>
	   </xsl:with-param>
	 </xsl:apply-templates>
       </xsl:variable>
       <xsl:if test="$DEBUG='true'">
       <xsl:message>
         translation=<xsl:copy-of select="$translation"/>
       </xsl:message>
       </xsl:if>
       <xsl:copy-of select="$translation"/>
        </xsl:when>
       <xsl:otherwise>
	 <xsl:message> type error in function application
	 <xsl:copy-of select="."/>
	 - number of formal arguments = <xsl:value-of
	 select="count($formal-arguments/args/*:bvar)"/>
	 - number of passed arguments = <xsl:value-of
	 select="count(*)"/>
	 function name <xsl:value-of select="$id"/>
	 function definition:  <xsl:copy-of select="$function"/>
	 </xsl:message>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:template>
   
     
   <xsl:template match="*" mode="elim-macros">
     <xsl:param name="reaction"/>
     <xsl:variable name="tag">
       <xsl:value-of select="name(.)"/>
     </xsl:variable>
     <xsl:if test="$DEBUG='true'">
     <xsl:message>
       elim-macros otherwise <xsl:value-of select="$tag"/>
     </xsl:message>
     </xsl:if>
     <xsl:element name="{$tag}">
       <xsl:apply-templates select="@*|*|comment()|text()"
			    mode="elim-macros">
	 <xsl:with-param name="reaction">
	     <xsl:copy-of select="$reaction"/>
	   </xsl:with-param>
	 </xsl:apply-templates>
      </xsl:element>
   </xsl:template>

   <xsl:template match="@*|comment()|text()" mode="elim-macros">
     <xsl:copy-of select="."/>
   </xsl:template>

   </xsl:stylesheet>
