<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:template match="network">
      <network kind="elim-edge-cluster">
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates select="child::*[not(self::edgecluster)] " mode="elim"/>
				     
      </network>

    </xsl:template>
    
    <!-- function elim -->
    <xsl:template match="reaction" mode="elim">
      <reaction> 
	<xsl:apply-templates select="@*|*" mode="elim"/>
      </reaction>
    </xsl:template>

    <xsl:template match="context" mode="elim">
      <context> 
	<xsl:copy-of select="@*|*"/>
      </context>
    </xsl:template>
    
    <xsl:template match="reactant[@edgecluster]  | product[@edgecluster]      | 
			 activator[@edgecluster] | accelerator[@edgecluster] |
			 inhibitor[@edgecluster] | prod-inh[@edgecluster] "
		  mode="elim">
      <xsl:variable name="tag">
	  <xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:apply-templates select="." mode="sources">
	<xsl:with-param name="tag">
	  <xsl:copy-of select="$tag"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="*|@*" mode="elim">
      <xsl:copy-of select="."/>
    </xsl:template>

    <!-- recursive function sources -->

    <xsl:template match="*[@edgecluster]" mode="sources">
      <xsl:param name="tag"/>
      <xsl:variable name="here" select="."/>
     <xsl:for-each select="//network/edgecluster[@id=$here/@edgecluster]/source">
       <xsl:choose>
	 <xsl:when test="@edgecluster">
	   <xsl:apply-templates select="." mode="sources">
	     <xsl:with-param name="tag">
	       <xsl:copy-of select="$tag"/>
	     </xsl:with-param>
	   </xsl:apply-templates>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:element name="{$tag}">
	     <xsl:copy-of select="@*"/>
	   </xsl:element>  
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:for-each>
   </xsl:template>


   <!-- function dist 
   <xsl:template match="network" mode="dist"> 
     <network>
       <xsl:copy-of         select="@*"/>
       <xsl:copy-of         select="metabolite|protein|actor|comment"/>
       <xsl:apply-templates select="reaction|context" mode="dist"/> 
     </network>
   </xsl:template>


   <xsl:template match="context" mode="dist">
 	<xsl:copy-of select="."/>
   </xsl:template>

    <xsl:template match="reaction" mode="dist">
      <reaction> 
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates select="*" mode="dist"/>
      </reaction>
    </xsl:template>

    <xsl:template match="kinetics|comment" mode="dist">
      <xsl:copy-of select="."/>
    </xsl:template>
    
    <xsl:template match="*[@spec][not(spec)]" mode="dist">
      <xsl:copy-of select="."/>
    </xsl:template>

    <xsl:template match="*[child::spec]" mode="dist">
     <xsl:variable name="tag">
	  <xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:for-each select="spec">
	<xsl:element name="{$tag}">
	  <xsl:attribute name="spec">
	    <xsl:value-of select="."/>
	  </xsl:attribute>
	</xsl:element>
      </xsl:for-each>
    </xsl:template>

-->

</xsl:stylesheet>
