<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 

xmlns=""
xmlns:xs="http://www.w3.org/2001/XMLSchema"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
version="2.0"
exclude-result-prefixes="xs">

  <xsl:output   
      method="xml" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>


  <xsl:template match="network">
    <network>
      <xsl:copy-of select="@*|metabolite|protein|species|actor|edgecluster"/>
      <xsl:apply-templates select="(metabolite|protein|species|actor)
				   [parent::network][@epsilon]"/>
      <xsl:copy-of select="reaction|context|expression|parameter"/>
      <xsl:apply-templates select="(metabolite|protein|species|actor)
				   [parent::network][@epsilon]" mode="add-epsilon"/>
    </network>
  </xsl:template>

  <xsl:template match="(metabolite|protein|species|actor)[@epsilon]">
    <xsl:variable name="name">
      <xsl:value-of select="name()"/>
    </xsl:variable>
    <actor>
      <xsl:attribute name="id">
	<xsl:value-of select="concat(@id,'_eps')"/>
      </xsl:attribute>
      <xsl:attribute name="latex-look">
	<xsl:value-of select="concat('\ensuremath{',@id,'{_\epsilon}}')"/>
      </xsl:attribute>
      <xsl:attribute name="x">
	<xsl:value-of select="@x+11"/>
      </xsl:attribute>
      <xsl:attribute name="y">
	<xsl:value-of select="@y+11"/>
      </xsl:attribute>
      <xsl:attribute name="aux">yes</xsl:attribute>
      <xsl:copy-of select="@*[not(name()='x')]
			   [not(name()='y')][not(name()='id')]
			   [not(name()='aux')]
			   [not(name()='initialConcentration')]
			   [not(name()='epsilon')]
			   [not(name()='type')]
			       |
			       *[not(name()='kinetic-expression')]
			       |
			       text() | comment()"/>
    </actor>
  </xsl:template>

  <xsl:template match="(metabolite|protein|species|actor)
				 [parent::network][@epsilon]" mode="add-epsilon">
    <reaction>
      <xsl:attribute name="x">
	<xsl:value-of select="@x+10"/>
      </xsl:attribute>
      <xsl:attribute name="y">
	<xsl:value-of select="@y+10"/>
      </xsl:attribute>
      <xsl:attribute name="id">
	<xsl:value-of select="concat(@id,'-prod-eps')"/>
      </xsl:attribute>
      <xsl:attribute name="latex-look">
	<xsl:value-of select="concat('\ensuremath{',@id,'_\epsilon^{prod}}')"/>
      </xsl:attribute>
      <kinetic-expression angle="-90">
	<conc>
	  <xsl:attribute name="spec">
	    <xsl:value-of select="@id"/>
	  </xsl:attribute>
	</conc>
      </kinetic-expression>
      <product>
	<xsl:attribute name="spec">
	  <xsl:value-of select="concat(@id,'_eps')"/>
	    </xsl:attribute>
      </product>
    </reaction>
    <reaction>
      <xsl:attribute name="x">
	<xsl:value-of select="@x+12"/>
      </xsl:attribute>
      <xsl:attribute name="y">
	<xsl:value-of select="@y+12"/>
      </xsl:attribute>
      <xsl:attribute name="id">
	<xsl:value-of select="concat(@id,'-cons-eps')"/>
      </xsl:attribute>
      <xsl:attribute name="latex-look">
	<xsl:value-of select="concat('\ensuremath{',@id,'^{cons}_\epsilon}')"/>
      </xsl:attribute>
      <kinetic-expression angle="-90">
	<constant id="eps">
	  <xsl:attribute name="value">
	    <xsl:value-of select="@epsilon"/>
	  </xsl:attribute>
	</constant>
      </kinetic-expression>
      <reactant>
	<xsl:attribute name="spec">
	  <xsl:value-of select="concat(@id,'_eps')"/>
	</xsl:attribute>
      </reactant>
    </reaction>
  </xsl:template>

  <xsl:template match="*">
    <xsl:variable name="name">
      <xsl:value-of select="name()"/>
    </xsl:variable>
    <xsl:element name="{$name}">
      <xsl:apply-templates select="@*|*|text()|comment()"/>
    </xsl:element>
  </xsl:template>
  <xsl:template match="@*|text()|comment()">
    <xsl:copy-of select="."/>
  </xsl:template>
  

</xsl:stylesheet>
