<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:strip-space elements="*" />

     
    <xsl:template match="network"> 
	<network>
	  <xsl:copy-of select="@*"/>
	  <xsl:apply-templates select="child::*"/>
	</network>
    </xsl:template>
    

    <xsl:template match="reaction">
      <xsl:apply-templates select="." mode="remove-product-inh"/>
      <xsl:apply-templates select="." mode="degradation"/>
    </xsl:template>

    <xsl:template match="*|@*">
      <xsl:copy-of select="."/>
    </xsl:template>

	  
    <!-- pure expression -->

    <xsl:template match="reaction" mode="remove-product-inh">
      <reaction>
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates select="child::*" mode="remove-product-inh"/>
      </reaction>
    </xsl:template>

    <xsl:template match="product-inh" mode="remove-product-inh">
      <product>
	<xsl:copy-of select="@spec"/>
      </product>
    </xsl:template>

    <xsl:template match="*|@*" mode="remove-product-inh">
      <xsl:copy-of select="."/>
    </xsl:template>

    <!-- implicit degradation -->

    <xsl:template match="reaction" mode="degradation">
      <xsl:variable name="id">
	<xsl:value-of select="normalize-space(@id)"/>
      </xsl:variable>
      <xsl:variable name="x">
	<xsl:value-of select="for $xx in @x return $xx -1"/>
      </xsl:variable>
      <xsl:variable name="y">
	<xsl:value-of select="@y"/>
      </xsl:variable>
      <xsl:for-each select="product-inh">
	<reaction>
	  <xsl:attribute name="id">
          <text>DEG</text>
	  <xsl:value-of select="$id"/>
	  </xsl:attribute>
	  <xsl:attribute name="latex-look">
          <text>\DEG{</text>
	    <xsl:value-of select="$id"/>
	    <text>}</text>
	  </xsl:attribute>
	  <xsl:attribute name="x">
	    <xsl:value-of select="$x"/>
	  </xsl:attribute>
	  <xsl:attribute name="y">
	    <xsl:value-of select="$y"/>
	  </xsl:attribute>
	  <xsl:attribute name="comment">implicit degradation</xsl:attribute>
	  <kinetics id="ma" angle="30"/>
	  <reactant>
	    <xsl:copy-of select="@spec"/>
	  </reactant>
	</reaction>
      </xsl:for-each>
    </xsl:template>
      
    <xsl:template match="*|@*" mode="degradation">
      <xsl:copy-of select="."/>
    </xsl:template>



</xsl:stylesheet>
