<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>

	<xsl:template match="equations | steady-state">
		<xsl:message>
			found tag equations or steady state
		</xsl:message>
      <equations>
	<xsl:apply-templates select="."  mode="filter-linear"/>
      </equations>
      <xsl:variable name="non-linear-constraints">
	<equations>
	  <xsl:apply-templates select="."  mode="filter-non-linear"/>
	</equations>
      </xsl:variable>
      <xsl:result-document href="non-linear-constraints.xml">
	<xsl:copy-of select="$non-linear-constraints"/>
      </xsl:result-document>
    </xsl:template>

    <xsl:template match="equal" mode="filter-linear">
      <xsl:variable name="children-is-linear-sum">
	<list>
	  <xsl:apply-templates select="*" mode="is-linear-sum"/>
	</list>
      </xsl:variable>
      <xsl:if test="count($children-is-linear-sum/list/true)=2 ">
	<xsl:copy-of select="."/>
      </xsl:if>
    </xsl:template>

    <xsl:template match="equal" mode="filter-non-linear">
      <xsl:variable name="children-is-linear-sum">
	<list>
	  <xsl:apply-templates select="*" mode="is-linear-sum"/>
	</list>
      </xsl:variable>
      <xsl:if test="$children-is-linear-sum/list/false">
	<xsl:copy-of select="."/>
      </xsl:if>
    </xsl:template>
    <xsl:template match="similar|nonzero" mode="filter-non-linear">
      <xsl:copy-of select="."/>
    </xsl:template>

    <xsl:template match="*" mode="is-variable">
      <xsl:choose>
	  <xsl:when test="self::speed
		    or self::conc
		    or self::knockout
		    or self::var
		    or self::posdot
		    or self::negdot
		    ">
	    <true/>
	  </xsl:when>
	  <xsl:otherwise>
	    <false/>
	  </xsl:otherwise>
      </xsl:choose>
    </xsl:template>
    
    <xsl:template match="*" mode="is-linear-product">
      <xsl:variable name="is-variable">
	<xsl:apply-templates select="." mode="is-variable"/>
      </xsl:variable>
      <xsl:choose>
	<xsl:when test="($is-variable/true) 
		    | self::mult[count(conc|speed|var|knockout|var|posdot|negdot)=1]
                                       [count( inh| acc | mult | sum| power )=0]
		    ">
	<true/>
	</xsl:when>
	<xsl:otherwise>
	  <false/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

     <xsl:template match="*" mode="is-linear-sum">
      <xsl:variable name="is-linear-product">
	<xsl:apply-templates select="." mode="is-linear-product"/>
      </xsl:variable>
      <xsl:variable name="children-is-linear-product">
	<list>
	  <xsl:apply-templates select="*" mode="is-linear-product"/>
	</list>
      </xsl:variable>
      <!--<xsl:message>
	children-is-linear-product <xsl:copy-of select="$children-is-linear-product"/>
      </xsl:message>-->
      <xsl:choose>
	<xsl:when test="($is-linear-product/true) 
			| self::sum[not($children-is-linear-product/list/false)]
			">
	  <true/>
	</xsl:when>
	<xsl:otherwise>
	  <false/>
	</xsl:otherwise>
      </xsl:choose>
  </xsl:template>

</xsl:stylesheet>
