<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>

    
  
    <xsl:template match="equations | steady-state">
      <!--<xsl:message>
	+++ hi <xsl:value-of select="name()"/>
      </xsl:message>-->
      <xsl:apply-templates select="."  mode="positive"/>
    </xsl:template>

    <!-- function positive -->
    
    <xsl:template match="equal[sum]" mode="positive">
      <equal>
	<xsl:apply-templates select="comment()"/>
	<xsl:apply-templates select="." mode="left"/>
	<xsl:apply-templates select="." mode="right"/>
      </equal>
      <xsl:for-each select="sum//divide | sum//power | sum//minus[count(*)=2]">
	<xsl:apply-templates select="." mode="positive-equation"/>
      </xsl:for-each>
       </xsl:template>

       <xsl:template match="equal[not(sum)]" mode="positive">
	 <equal>
	   <xsl:copy-of select="*[1]"/>
	   <xsl:apply-templates select="*[2]" mode="positive-term"/>
	 </equal>
	 <xsl:for-each select=".//divide | .//power | .//minus">
	   <xsl:apply-templates select="." mode="positive-equation"/>
	 </xsl:for-each>
       </xsl:template>    
  
       <xsl:template match="similar|nonzero"  mode="positive">
	 <xsl:copy-of select="."/>
       </xsl:template>

       <xsl:template match="equations|steady-state" mode="positive">
	 <xsl:variable name="name">
	   <xsl:value-of select="name()"/>
	 </xsl:variable>
	 <xsl:element name="{$name}">
	   <xsl:apply-templates select="*" mode="positive"/>
       </xsl:element>
       </xsl:template>
       
       <xsl:template match="*" mode="positive">
	 <xsl:message>
	   positive-equation warning
	   - else in mode="positive" <xsl:copy-of select="."/>
	 </xsl:message>
	 <xsl:variable name="name">
	 <xsl:value-of select="name()"/>
	 </xsl:variable>
	 <xsl:element name="{$name}">
	   <xsl:apply-templates select="*" mode="positive"/>
       </xsl:element>
       </xsl:template>

       <!-- function left and right -->
       <xsl:template match="equal" mode="left">
       <sum>
	 <xsl:copy-of select="(child::*[1])[
			      name()='var' or
			      name()='conc' or
			      name()='speed' or
			      name()='constant'] "/>
	 <xsl:apply-templates
	     select="child::*[1]/child::*[not(self::minus)]"
	     mode="positive-term"/>
	 <xsl:apply-templates
	     select="child::*[2]/child::minus/child::*"
	     mode="positive-term"/>
       </sum>
     </xsl:template>

     <xsl:template match="equal" mode="right">
       <sum>
	 <xsl:apply-templates
	     select="child::*[1]/child::minus/child::*" mode="positive-term"/>
	 <xsl:apply-templates
	     select="child::*[2]/(self::constant| self::var | child::*[not(self::minus)])" mode="positive-term"/>
       </sum>
     </xsl:template>

     <xsl:variable name="PREFIX">poseq-</xsl:variable>
     <!-- function positive-equation -->
     <xsl:template match="minus[count(*)=1]"  mode="positive-equation">
       <equal>
	 <sum>
	   <var>
	     <xsl:attribute name="id">
	       <xsl:value-of select="concat($PREFIX,generate-id(.))"/>
	     </xsl:attribute>
	   </var>
	   <xsl:apply-templates select="*" mode="positive-term"/>
	 </sum>
	 <constant value="0"/>
       </equal>
     </xsl:template>
     <xsl:template match="minus[count(*)=2]"  mode="positive-equation">
       <equal>
	 <sum>
	   <var>
	     <xsl:attribute name="id">
	       <xsl:value-of select="concat($PREFIX,generate-id(.))"/>
	     </xsl:attribute>
	   </var>
	   <xsl:apply-templates select="*[2]" mode="positive-term"/>
	 </sum>
	   <xsl:apply-templates select="*[1]" mode="positive-term"/>
       </equal>
     </xsl:template>
     <xsl:template match="divide"  mode="positive-equation">
       <equal>
	 <mult>
	   <var>
	     <xsl:attribute name="id">
	       <xsl:value-of select="concat($PREFIX,generate-id(.))"/>
	     </xsl:attribute>
	   </var>
	   <xsl:apply-templates select="*[2]" mode="positive-term"/>
	 </mult>
	   <xsl:apply-templates select="*[1]" mode="positive-term"/>
       </equal>
     </xsl:template>
      <xsl:template match="power[*[2][self::constant[@value&lt;1]]]"
		    mode="positive-equation">
	<xsl:variable name="degree">
	  <xsl:value-of select="*[2]/@value"/>
	</xsl:variable>
	<xsl:variable name="divdegree">
	  <xsl:value-of select="1 div $degree"/>
	</xsl:variable>
       <equal>
	 <power>
	   <var>
	     <xsl:attribute name="id">
	       <xsl:value-of select="concat($PREFIX,generate-id(.))"/>
	     </xsl:attribute>
	   </var>
	   <constant id="1div2">
	     <xsl:attribute name="value">
	       <xsl:copy-of select="$divdegree"/>
	     </xsl:attribute>
	   </constant>
	 </power>
	 <xsl:apply-templates select="*[1]" mode="positive-term"/>
       </equal>
     </xsl:template>
     <xsl:template match="*"  mode="positive-equation">
       <equal>
	 <var>
	   <xsl:attribute name="id">
	     <xsl:value-of select="concat($PREFIX,generate-id(.))"/>
	   </xsl:attribute>
	 </var>
	 <xsl:variable name="name">
	   <xsl:value-of select="name()"/>
	 </xsl:variable>
	 <xsl:element name="{$name}">
	   <xsl:apply-templates select="*|@*|text()|comment()" mode="positive-term"/>
	 </xsl:element>
       </equal>
     </xsl:template>
     
     <!-- function positive-term -->

     <xsl:template match=
		   "minus|divide|
		    power[*[2][self::constant[@value&lt;1]]]"  mode="positive-term">
       <var>
	 <xsl:attribute name="id">
	   <xsl:value-of select="concat($PREFIX,generate-id(.))"/>
	 </xsl:attribute>
       </var>
     </xsl:template>
     
     <xsl:template match="*"  mode="positive-term">
       <xsl:variable name="name">
	 <xsl:value-of select="name()"/>
       </xsl:variable>
       <!--<xsl:message>
	 ++ else positive-term: <xsl:copy-of select="."/>
       </xsl:message>-->
       <xsl:element name="{$name}">
	 <xsl:apply-templates select="*|@*|text()|comment()" mode="positive-term"/>
       </xsl:element>
     </xsl:template>

     <xsl:template match="@*|text()|comment()" mode="positive-term">
       <xsl:copy-of select="."/>
     </xsl:template>
     
     <!-- end positive-term -->

     
     <xsl:template match="*|text()" mode="consumed">
       <xsl:copy-of select="."/>
     </xsl:template>
     

     <xsl:template match="sum" mode="consumed">
       <sum> 
	 <xsl:copy-of select="child::minus/child::*"/>
       </sum>
     </xsl:template>
     <xsl:template match="*|text()" mode="consumed">
       <xsl:copy-of select="."/>
     </xsl:template>
  
     <xsl:template match="sum" mode="produced">
       <sum>
	 <xsl:copy-of select="child::*[not(self::minus)]"/>
       </sum>
     </xsl:template>
     <xsl:template match="*|text()" mode="produced">
	 <xsl:copy-of select="."/>
     </xsl:template>


</xsl:stylesheet>
