<!-- computes stoichiometry matrix for a metabolic network with stoichiometries -->

<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>

	<xsl:param name="MAX"     required="yes"/>
	
	<xsl:template match="network">
	<xsl:variable name="reactions">
	<reactions>
	  <xsl:for-each select="//reaction">
	    <reaction>
	      <xsl:copy-of select="@id"/>
	    </reaction>
	  </xsl:for-each>
	</reactions>
      </xsl:variable>

      <xsl:variable name="matrix">
	<matrix>
	  <xsl:copy-of select="@*"/>
	  <xsl:apply-templates select="metabolite" mode="extract"/>
	</matrix>
      </xsl:variable>

      
      <xsl:variable name="efmtool-matrix-matlab">
	<xsl:apply-templates select="$matrix" mode="efmtool"/>
      </xsl:variable>
      
      <xsl:result-document
	  href="stoichiometry-matrix.matlab" method="text"> 
	<xsl:value-of select="$efmtool-matrix-matlab"/>
      </xsl:result-document>

      <xsl:variable name="efmtool-stoichiometry-java">
	<xsl:apply-templates select="$matrix" mode="efmtool-java">
		<xsl:with-param name="network" select="."/>
	</xsl:apply-templates>
      </xsl:variable>

       <xsl:variable name="efmtool-stoichiometry-xml">
		<xsl:apply-templates select="$matrix" mode="efmtool-xml">
			<xsl:with-param name="network" select="."/>
		</xsl:apply-templates>
      </xsl:variable>
      
      <xsl:result-document href="Stoichiometry.java" method="text">
	<xsl:value-of select="$efmtool-stoichiometry-java"/>
		</xsl:result-document>

		<xsl:result-document href="Stoichiometry.xml" method="text">
			<xsl:value-of select="$efmtool-stoichiometry-xml"/>
		</xsl:result-document>

      <xsl:copy-of select="$matrix"/>
    </xsl:template>

    <!-- function extract:
	 converts metabolic-network into stoichiometry matrix -->
    
    <xsl:template match="metabolite" mode="extract">
      <metabolite>
	<xsl:copy-of select="@*"/>
	<xsl:variable name="id">
	  <xsl:value-of select="@id"/>
	</xsl:variable>
	<xsl:for-each select="//reaction">
	  <reaction>
	    <xsl:copy-of select="@*"/>
	    <xsl:attribute name="stoichiometry">
	      <xsl:choose>
	      <xsl:when test="child::*[@spec=$id and @stoichiometry]">
		<xsl:copy-of
		    select="child::*[
			    @spec=$id and @stoichiometry
			    ]/@stoichiometry"/>
	      </xsl:when>
	      <xsl:otherwise>0</xsl:otherwise>
	      </xsl:choose>
	    </xsl:attribute>
	  </reaction>
	</xsl:for-each>
      </metabolite>
    </xsl:template>

     <!-- function efmtool for matlab -->
     <xsl:template match="matrix" mode="efmtool">
       <text xml:space="preserve">[
</text>      
       <xsl:apply-templates select="*" mode="efmtool"/>
       <text>];</text>
     </xsl:template>
    <xsl:template match="metabolite" mode="efmtool">
      <xsl:apply-templates select="reaction" mode="efmtool"/>
    </xsl:template>
    <xsl:template match="reaction" mode="efmtool">
      <xsl:value-of select="@stoichiometry"/>
      <xsl:choose>
	<xsl:when test="following-sibling::reaction">
	  <text>,</text>
	</xsl:when>
	<xsl:otherwise>
	  <text xml:space="preserve">
</text>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <!-- function efmtool-java -->
    
    <xsl:template match="matrix" mode="efmtool-java">
      <xsl:param name="network"/>
      <text xml:space="preserve">import java.util.Arrays;
import java.util.HashSet;
</text>
      <text xml:space="preserve">public interface Stoichiometry {
    /**
     * The stoichiometric matrix
     */
    int[][] MATRIX = new int[][] {
</text>      
      <xsl:apply-templates select="metabolite" mode="efmtool-java"/>
<text>    };
    /**
     * Reaction reversibilities, true for reversible. Correspond to stoichiometry 
     * matrix columns.
     */
     boolean[] REAC_REVERSIBILITIES = new boolean[] {
</text>
      <xsl:for-each select="metabolite[1]/reaction">
	<xsl:choose>
	  <xsl:when test=".[@reversible]">
	    <text>true</text>
	  </xsl:when>
	  <xsl:otherwise>
	    <text>false</text>
	  </xsl:otherwise>
	</xsl:choose>
	<xsl:if test="following-sibling::reaction">
	  <text>, </text>
	</xsl:if>
      </xsl:for-each>
      <text>};
    
    /**
    * context obsolet?
    
    HashSet&lt;Integer&gt; CONTEXT = new HashSet&lt;Integer&gt;(Arrays.asList(</text>
	<text>)); 
	*/
/**			
*	<xsl:for-each select="$network//context">
*     		<xsl:value-of select="@id"/>
*       		 <xsl:if test="following-sibling::context">
*        			<text>, </text>
*        		</xsl:if>
*      	</xsl:for-each>		
*/	

    /**
     * Names of the metabolites, corresponding to stoichiometry matrix rows
     */
     String[] METABOLITE_NAMES = {</text>
     <xsl:for-each select="metabolite">
       <text>"</text>
       <xsl:value-of select="@id"/>
       <text>"</text>
       <xsl:choose>
	 <xsl:when test="following-sibling::metabolite">
	   <text>, </text>
	</xsl:when>
	<xsl:otherwise/>
      </xsl:choose>
     </xsl:for-each>
     <text>};
    /**
     * Names of the reactions, corresponding to stoichiometry matrix columns
     */
     String[] REACTION_NAMES = {</text>
     <xsl:for-each select="metabolite[1]/reaction">
       <text>"</text>
       <xsl:value-of select="@id"/>
       <text>"</text>
       <xsl:if test="following-sibling::reaction">
	 <text>, </text>
       </xsl:if>
     </xsl:for-each>
     <text>};
    /**
     * Formulas for all reactions, corresponding to stoichiometry matrix columns.
     * Letters used in the formulas are the metabolites, corresponding to the
     * stoichiometry matrix rows.
     */
}</text>
    </xsl:template>

    <xsl:template match="metabolite" mode="efmtool-java">
      <text xml:space="preserve">{ </text>
      <xsl:apply-templates select="reaction" mode="efmtool-java"/>
      <text xml:space="preserve">  }</text>
      <xsl:choose>
	<xsl:when test="following-sibling::metabolite">
	  <text>, //</text>
	  <xsl:value-of select="@id"/>
	  <text xml:space="preserve">
</text>	    
	</xsl:when>
	<xsl:otherwise>
	  <text>  //</text>
	  <xsl:value-of select="@id"/>
	  <text xml:space="preserve">
</text>
	</xsl:otherwise>
      </xsl:choose>

    </xsl:template>

    <xsl:template match="metabolite" mode="efmtool-xml">
		<xsl:apply-templates select="reaction" mode="efmtool-xml"/>
		<xsl:choose>
				<xsl:when test="following-sibling::metabolite">
					<text xml:space="preserve">|
					</text>
				</xsl:when>
				<xsl:otherwise>
						<text xml:space="preserve"> 
						</text>
				</xsl:otherwise>	
		</xsl:choose>
	</xsl:template>


    <xsl:template match="reaction" mode="efmtool-java">
      <xsl:value-of select="@stoichiometry"/>
      <xsl:choose>
	<xsl:when test="following-sibling::reaction">
	  <text xml:space="preserve">, </text>
	</xsl:when>
	<xsl:otherwise>
	  <text xml:space="preserve"> </text>
	</xsl:otherwise>
      </xsl:choose>

    </xsl:template>

	<xsl:template match="matrix" mode="efmtool-xml">
			<xsl:param name="network"/>
      	
	  	<text xml:space="preserve">constraint
		</text>
		<text xml:space="preserve">	linearEquationsMatrix(
		</text>
		<text>%Names of the reactions, corresponding to stoichiometry matrix columns
				[</text>
		<xsl:for-each select="metabolite[1]/reaction">
			<text> var_<xsl:value-of select="@id"/></text>
			<xsl:if test="following-sibling::reaction">
				<text>, </text>
			</xsl:if>
		</xsl:for-each>
		<text>],
			%Stoichiometry matrix
				[|</text>
		<xsl:apply-templates select="metabolite" mode="efmtool-xml"/>
		<text> |],</text>
		<text><xsl:value-of select="$MAX"/></text>
		<text> );</text>

		<text>	int: No = 1;
			int: Up = 2;
			int: Do = 3;
			int: NoN = 4;
			int: UpN = 5;
			int: DoN = 6;	
		</text>
		<text> set of int: AbstractDomain = {No, Do, Up, NoN, DoN, UpN};</text>
		<xsl:for-each select="metabolite[1]/reaction">
			<text> var Directions: var_<xsl:value-of select="@id"/>; </text>
		</xsl:for-each>
	</xsl:template>

     <xsl:template match="reaction" mode="efmtool-xml">
      <xsl:value-of select="@stoichiometry"/>
      <xsl:choose>
	<xsl:when test="following-sibling::reaction">
	  <text xml:space="preserve">, </text>
	</xsl:when>
	<xsl:otherwise>
	</xsl:otherwise>
      </xsl:choose>

    </xsl:template>

   
</xsl:stylesheet>
