<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns=""
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    version="2.0"
    exclude-result-prefixes="xs">

  <xsl:output   
      method="xml" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>

  <xsl:variable name="DEBUG">false</xsl:variable>
  

  <xsl:template match="reaction|variable|parameter|expression|context|
		       species|metabolite|protein|actor|function">
    <xsl:variable name="name">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>
    <xsl:element name="{$name}">
      <xsl:attribute name="id">
	<xsl:apply-templates select="@id" mode="latex-id"/>
      </xsl:attribute>
      <xsl:attribute name="latex-look">
	<xsl:choose>
	  <xsl:when test="@latex-look">
	    <xsl:apply-templates select="@latex-look" mode="latex-look"/>
	  </xsl:when>
	<xsl:otherwise>
	  <xsl:apply-templates select="@id" mode="latex-look"/>
	</xsl:otherwise>
	</xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="@*[not(name()='id')]
				   [not(name()='latex-look')]
				   |*|comment()|text()"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="*[@id]
		       [not(name()='reaction'  or name()='function'
		       or name()='variable'      or name()='parameter'
		       or name()='expression' or name()='species'
		       or name()='metabolite'  or name()='context'
		       or name()='protein'        or name()='actor')]">
    <xsl:variable name="name">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>
    <xsl:element name="{$name}">
      <xsl:attribute name="id">
	<xsl:apply-templates select="@id" mode="latex-id"/>
      </xsl:attribute>
      <xsl:apply-templates select="@*[not(name()='id')]
				   [not(name()='latex-look')]
				   |*|comment()|text()"/>
    </xsl:element>
  </xsl:template>


  <xsl:template match="@spec">
    <xsl:attribute name="spec">
      <xsl:apply-templates select="." mode="latex-id"/>
    </xsl:attribute>
  </xsl:template>

  <xsl:template match="@compartment">
    <xsl:attribute name="compartment">
      <xsl:apply-templates select="." mode="latex-id"/>
    </xsl:attribute>
  </xsl:template>
  
  <xsl:template match="@initial-expression">
    <xsl:attribute name="initial-expression">
      <xsl:apply-templates select="." mode="latex-id"/>
    </xsl:attribute>
  </xsl:template>

  <xsl:template match="@fun">
     <xsl:attribute name="fun">
       <xsl:apply-templates select="." mode="latex-id"/>
     </xsl:attribute>
     <xsl:attribute name="latex-look">
       <xsl:choose>
	 <xsl:when test="parent::*/@latex-look">
	   <xsl:apply-templates select="parent::*/@latex-look"
				mode="latex-look"/>
	 </xsl:when>
	 <xsl:otherwise>
	   	   <xsl:apply-templates select="."  mode="latex-look"/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:attribute>
  </xsl:template>
  
  <xsl:template match="@concentration">
    <xsl:attribute name="concentration">
      <xsl:apply-templates select="." mode="latex-id"/>
    </xsl:attribute>
  </xsl:template>
  
  <xsl:template match="*" mode="compile">
    <xsl:variable name="name">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>
    <xsl:element name="{$name}">
      <xsl:apply-templates select="@*|*|comment()|text()"/>
    </xsl:element>
  </xsl:template>
  
  
  <xsl:template match="*">
    <xsl:variable name="name">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>
    <xsl:element name="{$name}">
      <xsl:apply-templates select="@*|*|comment()|text()"/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="@*|text()|comment()">
    <xsl:copy-of select="."/>
  </xsl:template>

  <!-- function latex-id -->

  <xsl:variable name="hash">#</xsl:variable>
  <xsl:variable name="percent">%</xsl:variable>
  <xsl:variable name="underscore">UU</xsl:variable>
  <xsl:variable name="slashslashu">\\\\_</xsl:variable>

  
  <xsl:template match="*|text()|@*" mode="latex-id">
    <xsl:if test="$DEBUG='true'">
      <xsl:message> latex-id <xsl:value-of select="."/>    to
      <xsl:value-of
	  select="replace(replace(replace(.,'_',$underscore),$hash,'HASH'),$percent,'Percent')"/>
      </xsl:message>
    </xsl:if>
      <xsl:value-of select="replace(replace(.,'_',$underscore),$hash,'HASH')"/>
  </xsl:template>

  <xsl:template match="*|text()|@*" mode="latex-look">
    <xsl:variable name="value">
      <xsl:value-of select="."/>
    </xsl:variable>
     <xsl:value-of select="replace(replace(
			   replace(replace(replace( $value,
			   '-','{-}'),
			   '_','\\_'),
			   $slashslashu,'\\_'),
			   $hash,'\\#'),
			   $percent,'\\%')"/>
  </xsl:template>
 
</xsl:stylesheet>
