<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns=""
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    version="2.0"
    exclude-result-prefixes="xs">

  <xsl:output   
      method="text" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>

  <xsl:variable name="DEBUG">false</xsl:variable>
  <xsl:param name="SCALE"  required="yes"/>


  
  
  <xsl:template match="solution">
    <xsl:text>\begin{center} S</xsl:text>
    <xsl:value-of select="@position"/>
    <xsl:text>. Solution \end{center}</xsl:text>
  </xsl:template>

  
  
  <xsl:template match="graph">

    <xsl:variable name="full-id">
      <xsl:value-of select="@id"/>
      <xsl:if test="network/@kind">
	<text>-</text>
	<xsl:value-of select="network/@kind"/>
      </xsl:if>
    </xsl:variable>
  
    <text xml:space="preserve">\renewcommand\TIKZID[1]{</text>
    <xsl:value-of select="$full-id"/>
    <text xml:space="preserve">}
</text>

      <xsl:text xml:space="preserve">\renewcommand\TIKZREFID[1]{</xsl:text>
    <xsl:value-of select="$full-id"/>
    <text xml:space="preserve">}
</text>

 
<xsl:text>
  
 \pgfdeclaredecoration{simple line}{initial}{
    \state{initial}[width=\pgfdecoratedpathlength-1sp]{\pgfmoveto{\pgfpointorigin}}
    \state{final}{\pgflineto{\pgfpointorigin}}
    } 
    \tikzset{
    shift left/.style={decorate,decoration={simple line,raise=#1}},
    shift right/.style={decorate,decoration={simple line,raise=-1*#1}},
    }	  
</xsl:text>

    <xsl:text xml:space="preserve">\begin{tikzpicture}[scale=</xsl:text>
    <xsl:value-of select="$SCALE"/>
    <xsl:text>,concept color = gray!15]
</xsl:text>-->

    <xsl:apply-templates select="node"/>

    <xsl:text>
% edges
      
</xsl:text>

    <xsl:apply-templates select="edge"/>

    <xsl:text>
 % paths
      
</xsl:text>

    <xsl:apply-templates select="path"/>

    <xsl:text>
\end{tikzpicture}
    </xsl:text>

  </xsl:template>

  <xsl:template match="node">
    <xsl:text>  \node (</xsl:text>
    <xsl:value-of select="@id"/>
    <xsl:text>) at (</xsl:text>
    <xsl:value-of select="@x"/>
    <xsl:text>,</xsl:text>
    <xsl:value-of select="@y"/>
    <xsl:text>)</xsl:text>
    <xsl:text>[</xsl:text>
    <xsl:value-of select="@type"/>
    <xsl:if test="@annotation">
      <xsl:if test="$DEBUG='true'">
	<xsl:message> node annotation <xsl:value-of select="@annotation"/></xsl:message>
      </xsl:if>
      <xsl:if test="@type">
	<xsl:text>,</xsl:text>
      </xsl:if>
      <xsl:text>label={[label distance=-0.1]</xsl:text>
      <xsl:value-of select="@angle"/>
      <xsl:text>:</xsl:text>
      <xsl:value-of select="@annotation"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>]</xsl:text>
    <xsl:text>{$</xsl:text>
    <xsl:choose>
      <xsl:when test="@label">
	<xsl:value-of select="@label"/>
      </xsl:when>
      <xsl:otherwise>
      	<xsl:value-of select="@id"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>$};
    </xsl:text>

    <xsl:if test="@difference">
      <xsl:if test="$DEBUG='true'">
	<xsl:message> ++++++++ node[@difference] ++++++++++</xsl:message>
      </xsl:if>
      <xsl:text>\node [right] at (</xsl:text>
      <xsl:value-of select="@id"/>
      <xsl:text>.north east) {$</xsl:text>
      <xsl:value-of select="@difference"/>
      <xsl:text>$};
</xsl:text>
    </xsl:if>
  </xsl:template>


  <xsl:template match="edge">
    <xsl:if test="$DEBUG='true'">
      <xsl:message> <xsl:copy-of select="."/></xsl:message>
    </xsl:if>
    <xsl:text>  \draw</xsl:text>
    <xsl:if test="@type">
      <xsl:text>[</xsl:text>
      <xsl:value-of select="@type"/>
      <xsl:text>]</xsl:text>
    </xsl:if>
    <xsl:text>(</xsl:text>
    <xsl:value-of select="@from"/>
    <xsl:text>) -- (</xsl:text>
    <xsl:value-of select="@to"/>
    <xsl:text>)</xsl:text>
    <xsl:if test="@annotate">
      <xsl:value-of select="concat(' ',@annotate)"/>
    </xsl:if>
    <xsl:text>;
</xsl:text>
  </xsl:template>

  <xsl:template match="path[@from]">
    <xsl:text>\path </xsl:text>
    <xsl:if test="not(@operator)">
      <xsl:text>[-&gt;</xsl:text>
      <xsl:if test="@thickness">
	<xsl:text>, line width=</xsl:text>  <xsl:value-of select="@thickness"/>
	<xsl:text>pt, </xsl:text>
      </xsl:if>
      <xsl:value-of select="@color"/>
      <xsl:text>] </xsl:text>
    </xsl:if>
    <xsl:text>(</xsl:text>
    <xsl:value-of select="@from"/>
    <xsl:choose>
      <xsl:when test="@operator">
	<xsl:text>) </xsl:text>
	<xsl:value-of select="@operator"/>
	<xsl:text> (</xsl:text>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text>) edge[shift left=</xsl:text>
	<xsl:value-of select="@shift"/>
	<xsl:text>mm] (</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:value-of select="@to"/>
    <xsl:text>);
</xsl:text>       
  </xsl:template>

   <xsl:template match="path[not(@from)]"/>
  
  <xsl:template match="comment"/>

</xsl:stylesheet>
