<xsl:stylesheet 
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

  <xsl:output   
      method="text" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>
  
  <xsl:strip-space elements="*" />
  
  <xsl:template match="/">

    
    <xsl:variable name="full-id">
      <xsl:value-of select="network/@id"/>
      <xsl:if test="network/@kind">
	<text>-</text>
	<xsl:value-of select="network/@kind"/>
      </xsl:if>
    </xsl:variable>

    <text xml:space="preserve">\renewcommand\TIKZID[1]{</text>
    <xsl:value-of select="$full-id"/>
    <text xml:space="preserve">}
    \renewcommand\TIKZREFID[1]{</text>
    <xsl:value-of select="$full-id"/>
    <text>}

    \begin{longtable}[l]{||l@{\ensuremath{\ \EQU\ }}l|}

    \multicolumn{2}{l}{Macro definitions}
    \\\hline\hline
    </text>

    <xsl:apply-templates select="//network"/>

    <text xml:space="preserve">\hline\hline
    \end{longtable}
    </text>

    <xsl:if test="//network/species[@type='algebraic' or @type='differential']">
      <text>
	
     \medskip
	
	\begin{longtable}[l]{||l@{\ensuremath{\ \EQU\ }}l|}

	\multicolumn{2}{l}{Algebraic and differential variables}
	\\\hline\hline
      </text>

      <xsl:apply-templates select="//network" mode="alg-diff"/>

      <text xml:space="preserve">\hline\hline
      \end{longtable}
      </text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="//network"> 
    <xsl:variable name="expressions">
      <xsl:apply-templates select="expression"/>
    </xsl:variable>
    <xsl:value-of select="$expressions"/>
  </xsl:template>

  <xsl:template match="//network" mode="alg-diff"> 
    <xsl:variable name="expressions">
      <xsl:apply-templates select="species[@type='algebraic'
				   or @type='differential']"/>
    </xsl:variable>
    <xsl:value-of select="$expressions"/>
  </xsl:template>
  
  <xsl:template match="expression">

    <text xml:space="preserve"> 
      $\expr{<xsl:value-of select="normalize-space(@id)"/>}$ 
      &amp; $ <xsl:apply-templates select="*"
      mode="kinetic-expression"/>$
      \\
    </text>


    <xsl:for-each select="comment">
      <text xml:space="preserve"> &amp; </text>
      <xsl:value-of select="@latex-look"/>
      <text> &amp; </text>
      <xsl:value-of select="@initialConcentration"/>
      <text>\\
      </text>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="species[@type='algebraic']">

    <text xml:space="preserve"> 
      $\z{<xsl:value-of select="normalize-space(@id)"/>}$ 
      &amp; $ <xsl:apply-templates select="*"
      mode="kinetic-expression"/>$
      \\
    </text>


    <xsl:for-each select="comment">
      <text xml:space="preserve"> &amp; </text>
      <xsl:value-of select="@latex-look"/>
      <text> &amp; </text>
      <xsl:value-of select="@initialConcentration"/>
      <text>\\
      </text>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="species[@type='differential']">

    <text xml:space="preserve"> 
      $\dot{\z{<xsl:value-of select="normalize-space(@id)"/>}}$ 
      &amp; $ <xsl:apply-templates select="*"
      mode="kinetic-expression"/>$
      \\
    </text>


    <xsl:for-each select="comment">
      <text xml:space="preserve"> &amp; </text>
      <xsl:value-of select="@latex-look"/>
      <text> &amp; </text>
      <xsl:value-of select="@initialConcentration"/>
      <text>\\
      </text>
    </xsl:for-each>
  </xsl:template>


  <!-- function update -->

  <xsl:template match="update" mode="update">
    <xsl:message>
      update <xsl:copy-of select="."/>
    </xsl:message>
    <xsl:text> $ \z{</xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
    <xsl:text>}  := </xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
    <xsl:text> $ </xsl:text>
  </xsl:template>

  
  <!-- function condition -->

  <xsl:template match="condition" mode="condition">
    <xsl:message>
      event condition <xsl:copy-of select="."/>
    </xsl:message>
    <xsl:apply-templates select="*" mode="condition"/>
  </xsl:template>

  <xsl:template match="and" mode="condition">
    <xsl:message>
      event condition <xsl:copy-of select="."/>
    </xsl:message>
    <xsl:apply-templates select="*[1]" mode="condition"/>
    <xsl:text> $ \wedge $</xsl:text>
    <xsl:apply-templates select="*[2]" mode="condition"/>
  </xsl:template>

  <xsl:template match="leq" mode="condition">
    <xsl:message>
      event condition <xsl:copy-of select="."/>
    </xsl:message>
    <xsl:text> $ </xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
    <xsl:text> \leq </xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
    <xsl:text> $ </xsl:text>
  </xsl:template>

  <xsl:template match="lt" mode="condition">
    <xsl:message>
      event condition <xsl:copy-of select="."/>
    </xsl:message>
    <xsl:text> $ </xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
    <xsl:text> &lt; </xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
    <xsl:text> $ </xsl:text>
  </xsl:template>

  <xsl:template match="eq" mode="condition">
    <xsl:message>
      event condition <xsl:copy-of select="."/>
    </xsl:message>
    <xsl:text> $ </xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
    <xsl:text> = </xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
    <xsl:text> $ </xsl:text>
  </xsl:template>

  <!-- function kinetic-expresssion -->

  <xsl:template match="mult" mode="kinetic-expression">
    <xsl:for-each select="*">
      <xsl:apply-templates select="." mode="kinetic-expression"/>
      <xsl:if test="not(position()=last()) and
		    (self::conc or not(following-sibling::*[1][name()='conc']))">
	<xsl:text>\;</xsl:text>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>


  <xsl:template   match="apply[@fun]" mode="kinetic-expression">
    <xsl:text>\expr{</xsl:text>
    <xsl:choose>
      <xsl:when test="@latex-look">
	<xsl:value-of select="@latex-look"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:value-of select="@fun"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>}(</xsl:text>
    <xsl:for-each select="*">
      <xsl:apply-templates select="." mode="kinetic-expression"/>
      <xsl:if test="not(position()=last())">
	<text xml:space="preserve">,</text>
      </xsl:if>
    </xsl:for-each>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="sum" mode="kinetic-expression">
    <xsl:if test="parent::mult">
      <text> ( </text>
    </xsl:if>
    <xsl:for-each select="*">
      <xsl:apply-templates select="." mode="kinetic-expression"/>
      <xsl:if test="not(position()=last())">
	<xsl:text> + </xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:if test="parent::mult">
      <text> ) </text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="divide" mode="kinetic-expression">
    <xsl:text>\frac{ </xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
    <xsl:text>}{ </xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
    <xsl:text>} </xsl:text>
  </xsl:template>

  <xsl:template match="minus[count(*)=2]" mode="kinetic-expression">
    <xsl:text> (</xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
    <xsl:text>- </xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
    <xsl:text> )</xsl:text>
  </xsl:template>

  <xsl:template   match="power" mode="kinetic-expression">
    <xsl:choose>
      <xsl:when test="*[2][name()='constant'][@value='0.5']">
	<text> \sqrt{</text>
	<xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 	<text xml:space="preserve"> } </text>
      </xsl:when>
      <xsl:otherwise>
	<text> {(</text>
	<xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 
	<text xml:space="preserve"> )}^{ </text>
	<xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
	<text xml:space="preserve"> } </text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="minus[count(*)=1]" mode="kinetic-expression">
    <xsl:text>- </xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
    <xsl:text> </xsl:text>
  </xsl:template>

  <xsl:template match="knockout" mode="kinetic-expression">
    <xsl:text>\ko{</xsl:text> 
    <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="conc" mode="kinetic-expression">
    <xsl:text>\z{</xsl:text> <xsl:value-of select="@spec"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="delay" mode="kinetic-expression">
    <xsl:text> delay(</xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template   match="lt" mode="kinetic-expression">
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 
    <xsl:text xml:space="preserve"> &lt; </xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
  </xsl:template>

  <xsl:template   match="ifthenelse" mode="kinetic-expression">
    <xsl:text>\textbf{if }</xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 
    <xsl:text xml:space="preserve">\textbf{ then }</xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
    <xsl:text xml:space="preserve">\textbf{ else }</xsl:text>
    <xsl:apply-templates select="*[3]" mode="kinetic-expression"/>
    <xsl:text xml:space="preserve"> \textbf{ end}</xsl:text>
  </xsl:template>

  <xsl:template match="log|sin|cos|tan|cot|sinh|cosh|tanh|coth"
		mode="kinetic-expression">
    <xsl:variable name="name">
      <xsl:value-of select="name()"/>
    </xsl:variable> 
    <xsl:value-of select="$name"/>
    <xsl:text>(</xsl:text> 
    <xsl:apply-templates select="*" mode="kinetic-expression"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  
  <xsl:template match="var" mode="kinetic-expression">
    <xsl:text>\w{</xsl:text> <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>
  
  <xsl:template match="constant" mode="kinetic-expression">
    <xsl:value-of select="@value"/>
  </xsl:template>

  <xsl:template match="param|expr" mode="kinetic-expression">
    <xsl:text>\expr{</xsl:text>
    <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
  </xsl:template>


   <xsl:template match="time" mode="kinetic-expression">
    <xsl:text>{\color{blue}\textrm{time}}</xsl:text>
   </xsl:template>

   <xsl:template match="ceiling" mode="kinetic-expression">
     <xsl:text>\ceiling{ </xsl:text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
     <xsl:text>} </xsl:text>
    </xsl:template>

    <xsl:template match="floor" mode="kinetic-expression">
     <xsl:text>\floor{ </xsl:text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
     <xsl:text>} </xsl:text>
    </xsl:template>
    
  <xsl:template match="*" mode="kinetics">
    <xsl:variable name="name">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>
    <xsl:message> latex-expressions otherwise <xsl:value-of
    select="$name"/> </xsl:message>
    <xsl:element name="{$name}">
      <xsl:apply-templates select="@*|*|comment()|text()" mode="kinetics"/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="@*|text()|comment()" mode="kinetics">
    <xsl:copy-of select="."/>
  </xsl:template>
  
</xsl:stylesheet>
