<xsl:stylesheet 
    xmlns=""
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="text" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>


    <xsl:variable name="hash">#</xsl:variable>

    <xsl:variable name="network-id-or-name">
      <xsl:choose>
	<xsl:when test="//network/@id">
	  <xsl:value-of select="replace(//network/@id,' ','')"/>
	</xsl:when>
	<xsl:when test="//network/@name">
	  <xsl:value-of select="replace(//network/@name,' ','')"/>
	</xsl:when>
	<xsl:otherwise>NoName</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
   <xsl:variable name="underscore">UNDERSCORE</xsl:variable>
   <xsl:variable name="network-id-latex-fix">
     <xsl:apply-templates select="$network-id-or-name" mode="latex-special"/>
    </xsl:variable>
    
    <xsl:template match="//network">
     <!-- <xsl:message>
	network-id-latex-fix <xsl:copy-of select="$network-id-latex-fix"/>
      </xsl:message>
      -->
      <text xml:space="preserve">%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% network </text>
      <xsl:value-of select="@id"/>
      <text xml:space="preserve">
</text>
     <text xml:space="preserve">%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% species and their latex look 
</text>

<xsl:apply-templates select="expression|parameter|function|event|
			     reaction/expression |
			     metabolite|species|actor|protein|reaction|context" mode="node"/>
     
     <xsl:apply-templates select="." mode="switch-macros"/>
    </xsl:template>

    <xsl:template  match="context[input]" mode="node">
      <xsl:variable name="reaction">
	<reaction>
	  <xsl:copy-of select="@*"/>
	  <product>
	    <xsl:copy-of select="input/@*"/>
	  </product>
	</reaction>
      </xsl:variable>
      <xsl:apply-templates select="$reaction" mode="node"/>
    </xsl:template>
    
    <xsl:template  match="context[output]" mode="node">
      <xsl:variable name="reaction">
	<reaction>
	  <xsl:copy-of select="@*"/>
	  <reactant>
	    <xsl:copy-of select="output/@*"/>
	  </reactant>
	</reaction>
      </xsl:variable>
      <xsl:apply-templates select="$reaction" mode="node"/>
    </xsl:template>
      
    <xsl:template match="(expression|parameter|
			 metabolite|species|actor|protein|reaction)
			 [not(@id=preceding::expression/@id)]" mode="node">
      <!-- good debug
      <xsl:message>
	latex-macros id <xsl:value-of select="@id"/>
      </xsl:message>
      -->
      <xsl:variable name="slashslashu">\\\\_</xsl:variable>
      <xsl:variable name="Nid">
	<xsl:value-of select="$network-id-latex-fix"/>
      </xsl:variable>
      <text>\newcommand\</text>
      <xsl:value-of select="$Nid"/>
      <xsl:apply-templates select="@id" mode="latex-special"/>
      <text>{</text>
      <xsl:value-of select="replace(@id,'_','\\_')"/>
      <text xml:space="preserve">}
</text>      
<xsl:if test="@latex-look">
  <text>% here
  </text>
	<text>\newcommand\Look</text>
	<xsl:value-of select="$Nid"/>
	<xsl:apply-templates select="@id" mode="latex-special"/>
	<text>{</text>
	<xsl:value-of select="replace(@latex-look,$slashslashu,'\\_')"/>
	<text xml:space="preserve">}
</text>	
      </xsl:if>
    </xsl:template>

   <xsl:template match="function" mode="node">
     <xsl:variable name="slashslashu">\\\\_</xsl:variable>
      <xsl:variable name="Nid">
	<xsl:value-of select="$network-id-latex-fix"/>
      </xsl:variable>
      <text>\newcommand\</text>
      <xsl:value-of select="$Nid"/>
      <xsl:apply-templates select="@id" mode="latex-special"/>
      <text>{</text>
      <xsl:value-of select="replace(@id,'_','\\_')"/>
      <text xml:space="preserve">}
</text>      
<xsl:if test="@latex-look">
  <text>% here
  </text>
	<text>\newcommand\Look</text>
	<xsl:value-of select="$Nid"/>
	<xsl:apply-templates select="@id" mode="latex-special"/>
	<text>{</text>
	<xsl:value-of select="replace(@latex-look,$slashslashu,'\\_')"/>
	<text xml:space="preserve">}
</text>	
      </xsl:if>
    </xsl:template>
    
    
    <xsl:template match="network" mode="switch-macros">
      <text>%% switch macros for changing latex look
	
\renewcommand*{\LookSwitch}[1]{%
    \stringcases
    {#1}%
    {%
      </text>
      <xsl:apply-templates select="expression|parameter|function|
				   reaction/expression |
				   metabolite|species|actor|protein|reaction|context"
			   mode="switch-macros"/>
    <text xml:space="preserve">}%
    {#1}%
}
</text>
    </xsl:template>


    <xsl:template  match="context[input]" mode="switch-macros">
      <xsl:variable name="reaction">
	<reaction>
	  <xsl:copy-of select="@*"/>
	  <product>
	    <xsl:copy-of select="input/@*"/>
	  </product>
	</reaction>
      </xsl:variable>
      <xsl:apply-templates select="$reaction" mode="switch-macros"/>
    </xsl:template>
    
    <xsl:template  match="context[output]" mode="switch-macros">
      <xsl:variable name="reaction">
	<reaction>
	  <xsl:copy-of select="@*"/>
	  <reactant>
	    <xsl:copy-of select="output/@*"/>
	  </reactant>
	</reaction>
      </xsl:variable>
      <xsl:apply-templates select="$reaction" mode="switch-macros"/>
    </xsl:template>


    <xsl:template match="reaction|expression|parameter|function|
			 metabolite|species|actor|protein"
		  mode="switch-macros">
      
      <xsl:if test="@latex-look">
	<text>{</text>
	<xsl:value-of select="@id"/>
	<text>}{\Look</text>
	<xsl:value-of select="$network-id-latex-fix"/>
	<xsl:apply-templates select="@id" mode="latex-special"/>
	<text xml:space="preserve">}%
	</text>
      </xsl:if>

      <!--<xsl:if test="@latex-look">
	<text>\newcommand\Look</text>
	<xsl:value-of select="$Nid"/>
	<xsl:apply-templates select="@id" mode="latex-special"/>
	<text>{</text>
	<xsl:value-of select="replace(replace(replace(@latex-look,'_','\\_'),$slashslashu,'\\_'),$hash,'HASH')"/>
	<text xml:space="preserve">}
	</text>	
      </xsl:if>-->


      
      <xsl:if test="not(@latex-look)">
	<text>{</text>
	<xsl:value-of select="@id"/>
	<text>}{</text>
	 <xsl:value-of select="replace(@id,'_','\\_')"/>
	<text xml:space="preserve">}%
	</text>
      </xsl:if>
      
      <xsl:if test="@latex-look and not(name()='reaction')">
	<text>{\</text>
	<xsl:value-of select="@id"/>
	<text>}{\Look</text>
	<xsl:value-of select="$network-id-latex-fix"/>
	<xsl:apply-templates select="@id" mode="latex-special"/>
	<text xml:space="preserve">}%
	</text>
      </xsl:if>
      
      <xsl:if test="@latex-look">
	<text>{\</text>
	<xsl:value-of select="$network-id-latex-fix"/>
	<xsl:apply-templates select="@id" mode="latex-special"/>
	<text>}{\Look</text>
	<xsl:value-of select="$network-id-latex-fix"/>
	<xsl:apply-templates select="@id" mode="latex-special"/>
	<text xml:space="preserve">}%
	</text>
      </xsl:if>

    </xsl:template>

    <xsl:template match="@*|text()|*" mode="latex-special">
      <xsl:value-of select="replace(replace(replace(replace(replace(
			    replace(replace(replace(replace(replace(replace(
			    replace(replace(replace(.,
			      '0','Zero'),
			      '1','One'),
			      '2','Two'),
			      '3','Three'),
			      '4','Four'),
			      '5','Five'),
			      '6','Six'),
			      '7','Seven'),
			      '8','Eight'),
			      '9','Nine'),
			      '\+','Plus'),
			      $hash,'\\hash'),
			      '_',$underscore),
			      '-','')"/>
    </xsl:template>
    

</xsl:stylesheet>
