<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns="http://www.w3.org/1999/xhtml"   
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0">
  <xsl:output method="text" version="5.0"
	      encoding="UTF-8" indent="yes" />

  <xsl:param name="VERSION"     required="yes"/>
  <xsl:param name="DATE"          required="yes"/>

  <xsl:template match="document">
    
    <xsl:text>&lt;!--Version </xsl:text>
    <xsl:copy-of select="$VERSION"/>
    <xsl:text> from </xsl:text>
    <xsl:copy-of select="$DATE"/>
    <xsl:copy-of select="copyright"/>
    <xsl:text> --&gt;</xsl:text>
    
    <xsl:apply-templates select="*"/>
  </xsl:template>

  <xsl:template match="version">
    <xsl:value-of select="$VERSION"/>
  </xsl:template>
  
  <xsl:template match="author">
    <xsl:text>&lt;!-- </xsl:text>
    <xsl:apply-templates select="*|text()"/>
    <xsl:text> --&gt;</xsl:text>
  </xsl:template>

  <xsl:template match="copyright">
    <xsl:text>&lt;!--</xsl:text>   
    <xsl:apply-templates select="*|text()"/>
    <xsl:text> --&gt;
 </xsl:text>
  </xsl:template>


  <xsl:template match="title">
    <xsl:text>&lt;!--
   ++++++++++++++++++++++++++++++
    </xsl:text>
    	  <xsl:value-of select="text()"/>
    <xsl:text>
   ++++++++++++++++++++++++++++++ --&gt;</xsl:text>
      <xsl:apply-templates select="*"/>
  </xsl:template>
	
  <xsl:template match="section">
    <xsl:text>&lt;!--
    +++++++++++ Section ++++++++++++++
    </xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:text>
    +++++++++++++++++++++++++++++++ --&gt;</xsl:text>
    
    <xsl:apply-templates select="*"/>
  </xsl:template>
  
  <xsl:template match="subsection">
    <xsl:text>&lt;!--
    +++++++++++ Subsection ++++++++++
</xsl:text>
     <xsl:value-of select="@name"/>
     <xsl:text> --&gt;</xsl:text>
     <xsl:apply-templates select="*|text()"/>
  </xsl:template>

  <xsl:template match="a-link"/>
  <xsl:template match="list">
    <xsl:text>&lt;!-- </xsl:text>

    <xsl:if test="@name"> 
      <h3> <xsl:value-of select="@name"/></h3> 
    </xsl:if>
    <ul>
    <xsl:apply-templates select="*"/> </ul>
     <xsl:text>--&gt;</xsl:text>
 </xsl:template>
  
 <xsl:template match="item">
   <text> ° </text>
     <xsl:apply-templates select="node()"/>
     <text> </text>
 </xsl:template>

  <xsl:template match="reaction">
    <!--r<sub><xsl:value-of select="@id"/></sub>-->
    <xsl:value-of select="@id"/>
  </xsl:template>
  <xsl:template match="spec">
    <code><xsl:value-of select="@id"/></code>
  </xsl:template>
  <xsl:template match="BS">
    <code>BS<sub><xsl:value-of select="@spec"/></sub></code>
  </xsl:template>
  <xsl:template match="Prom">
    <code>P<sub><xsl:value-of select="@spec"/></sub></code>
  </xsl:template>
  
  <xsl:template match="article">
    <text>
    &lt;!--  - - - - - - - - - - - - - -  - - - - - - - - - - - - - - - -
    </text>
    <xsl:value-of select="@name"/>
    <text>
     - - - - - - - - - - - - - -  - - - - - - - - - - - - - - - - -  --&gt;</text>

    <xsl:apply-templates select="@*[not(name()='name')]|*|text()"/>
   
  </xsl:template>
  
  <xsl:template match="file">
    <xsl:variable name="filename">
      <xsl:value-of select="@name"/>
    </xsl:variable>
    <xsl:variable name="file">
      <xsl:copy-of select="document($filename)"/>
    </xsl:variable>
    <xsl:apply-templates select="$file//document/*"/>
  </xsl:template>

  <xsl:template match="xml-code">
    <xsl:variable name="filename">
      <xsl:value-of select="concat('Created/',@name,'/include.xml')"/>
    </xsl:variable>
    <xsl:variable name="file">
       <xsl:copy-of select="document($filename)"/>
    </xsl:variable>
    <xsl:apply-templates select="$file//xml"/>
  </xsl:template>
  
   <xsl:template match="declare">
     <xsl:text> &lt;!-- </xsl:text>
     <xsl:value-of select="@name"/>
     <xsl:text>  </xsl:text>
     <xsl:value-of select="@by"/>
     <xsl:apply-templates select="*|text()"/>
     <xsl:text> --&gt;
</xsl:text> 
   </xsl:template>
   <xsl:template match="a">
     <a>
       <xsl:apply-templates select="@*|*|text()"/>
     </a>
   </xsl:template>
    <xsl:template match="p">
      <xsl:text>&lt;!--
      </xsl:text>
      <xsl:if test="@name">
	<xsl:value-of select="@name"/> 
      </xsl:if>
      <xsl:apply-templates select="*|text()"/>
      <xsl:text>--&gt; </xsl:text>
    </xsl:template>
   <xsl:template match="div">
     <div>
       <xsl:apply-templates select="*|text()"/>
     </div>
   </xsl:template>
   <xsl:template match="dtd">
     <div class="dtd">
       <pre>
	 <code>
	   <xsl:apply-templates select="*|comment()|text()"/>
	 </code>
       </pre>
     </div>
   </xsl:template>
   <xsl:template match="comment()">
     &lt;!-- <xsl:value-of select="."/> --&gt;
   </xsl:template>
   <xsl:template match="xml">
     <div class="xml">
       <pre>
	   <xsl:value-of select="."/>
       </pre>		 
     </div>
   </xsl:template>   <xsl:template match="make">
     <div class="make">
       <pre>
	   <xsl:value-of select="."/>
       </pre>
     </div>
   </xsl:template>
   <xsl:template match="pre">
     <pre>
       <xsl:apply-templates select="@*|*"/>
      </pre>
   </xsl:template>
   <xsl:template match="code">
     <code>
       <xsl:value-of select="."/>
      </code>
   </xsl:template>
   <xsl:template match="center">
     <center> <xsl:apply-templates select="*|@*|text()"/> </center>
   </xsl:template>

   <xsl:template match="h2">
     <h2>
	 <xsl:value-of select="@name"/>
     </h2>
     <xsl:apply-templates select="*|text()"/> 
   </xsl:template>
  <xsl:template match="address">
    <xsl:apply-templates select="*"/>
  </xsl:template>
   <xsl:template match="h4">
     <h4>
       <xsl:value-of select="@name"/>
     </h4>
       <xsl:apply-templates select="*|text()"/>
   </xsl:template>
  <xsl:template match="address">
    <xsl:apply-templates select="*"/>
  </xsl:template>

  <xsl:template match="*">
    <xsl:variable name="tag">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>

    <xsl:element name="{$tag}">
        <xsl:apply-templates select="@*|*|text()"/>
    </xsl:element>
  </xsl:template>

  
  <xsl:template match="text()">
    <xsl:value-of select="."/>
  </xsl:template>  
  <xsl:template match="@*">
    <xsl:copy-of select="."/>
  </xsl:template>
  
</xsl:stylesheet>
