<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    version="2.0"
    exclude-result-prefixes="xs">

  <xsl:output   
      method="xml" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>

  <xsl:template match="network">
    <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}"> 
	<xsl:copy-of select="@*[not(name()='network-version')]"/>
	<xsl:apply-templates select="*"/>
      </xsl:element>
  </xsl:template>
  
  <xsl:template match="species[@type='metabolite' or @type='protein' or @type='actor']">
    <xsl:variable name="tag">
	<xsl:value-of select="@type"/>
    </xsl:variable>
    <xsl:element name="{$tag}">
      <xsl:apply-templates select="@*[not(name(.)='type')] | child::*"/>
    </xsl:element>
  </xsl:template>
  <xsl:template match="species[@type='external']">
    <species type='external'>
       <xsl:apply-templates select="@*[not(name(.)='type')] | child::*"/>
     </species>
  </xsl:template>

 
  <xsl:template match="species[not(@type='metabolite' or
		       @type='protein' or @type='actor' or @type='external')]">
    <!--
    <xsl:message>
      species
      type: <xsl:copy-of select="@type"/>
      id: <xsl:copy-of select="@id"/>
    </xsl:message>
-->
    <metabolite>
      <xsl:copy-of select="@*[not(name(.)='type')] | child::*"/>
    </metabolite>
  </xsl:template>
  <xsl:template match="reaction[not(@type='inflow' or @type='outflow')] | 
		       edgecluster|comment">
    <xsl:copy-of select="."/>
  </xsl:template>
  <xsl:template match="reaction[@type='inflow']">
    <context>
      <xsl:copy-of select="@*[not(name()='type')]"/>
      <input>
	<xsl:copy-of select="child::product[1]/@*"/>
      </input>
    </context>
  </xsl:template>
  <xsl:template match="reaction[@type='outflow']">
    <context>
      <xsl:copy-of select="@*[not(name()='type')]"/>
      <output>
	<xsl:copy-of select="child::reactant[1]/@*"/>
      </output>
    </context>
  </xsl:template>


  
  <xsl:template match="input">
    <xsl:attribute name="type">inflow</xsl:attribute>
    <product>
      <xsl:copy-of select="@*"/>
    </product>
  </xsl:template>
  <xsl:template match="output">
    <xsl:attribute name="type">outflow</xsl:attribute>
    <reactant>
      <xsl:copy-of select="@*"/>
    </reactant>
  </xsl:template>

</xsl:stylesheet>
