<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    version="2.0"
    exclude-result-prefixes="xs">

  <xsl:output   
      method="xml" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>

  <xsl:template match="network">
    <network kind="CoreSBML" source="http://researchers.lille.inria.fr/niehren/Core-SBML">
      <xsl:copy-of select="@*[not(name()='version')]"/>
      <xsl:apply-templates select="*"/>
    </network>
  </xsl:template>


  <xsl:template match="variable[@type='metabolite'
		       or @type='protein' or @type='actor']">
    <xsl:variable name="tag">
	<xsl:value-of select="@type"/>
    </xsl:variable>
    <xsl:element name="{$tag}">
      <xsl:apply-templates select="@*[not(name(.)='type')] | child::*"/>
    </xsl:element>
  </xsl:template>
  

    <xsl:template match="variable[@type='algebraic']">
    <species type='algebraic'>
       <xsl:apply-templates select="@*[not(name(.)='type')] | child::*"/>
     </species>
  </xsl:template>

  <xsl:template match="variable[@type='species']">
    <metabolite>
      <xsl:apply-templates select="@*[not(name(.)='type')] | child::*"/>
    </metabolite>
  </xsl:template>


  <xsl:template match="variable[@type='control']">
    <parameter>
      <xsl:apply-templates select="@*[not(name(.)='type')]"/>
    </parameter>
  </xsl:template>

   
  <xsl:template match="variable[
		       not(@type='metabolite') and
		       not(@type='protein') and
		       not(@type='actor')    and
		       not(@type='control')    and
		       not(@type='species')    and
		       not(@type='algebraic') and
		       not(@type='differential')
		       ]">
    <xsl:message>format-1.1-from-2.1: strange case <xsl:copy-of select="."/></xsl:message>
    <metabolite>
      <xsl:apply-templates select="@*[not(name(.)='type')] | child::*"/>
    </metabolite>
  </xsl:template>
  
  <xsl:template match="reaction[not(@type='inflow'
		       or @type='outflow')] | 
		       edgecluster|comment">
    <reaction>
      <xsl:apply-templates select="@*|*"/>
    </reaction>
    
  </xsl:template>
  <xsl:template match="reaction[@type='inflow']">
    <context>
      <xsl:copy-of select="@*[not(name()='type')]"/>
      <input>
	<xsl:copy-of select="child::product[1]/@*"/>
      </input>
    </context>
  </xsl:template>
  
  <xsl:template match="reaction[@type='outflow']">
    <context>
      <xsl:copy-of select="@*[not(name()='type')]"/>
      <output>
	<xsl:copy-of select="child::reactant[1]/@*"/>
      </output>
    </context>
  </xsl:template>

  <xsl:template match="var[not(@type='control')]">
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$id=ancestor::function/lambda/bvar/@id">
	<xsl:copy-of select="."/>
      </xsl:when>
      <xsl:otherwise>
      	<conc>
	  <xsl:attribute name="spec">
	    <xsl:value-of select="@id"/>
	  </xsl:attribute>
	</conc>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="var[@type='control']">
    <param>
      <xsl:attribute name="id">
	<xsl:value-of select="@id"/>
      </xsl:attribute>
    </param>
  </xsl:template>
  
  <xsl:template match="times">
    <mult>
      <xsl:apply-templates select="@*|*|text()|comment()"/>
    </mult>
  </xsl:template>
  
  <xsl:template match="plus">
    <sum>
      <xsl:apply-templates select="@*|*|text()|comment()"/>
    </sum>
  </xsl:template>
  
  <xsl:template match="input">
    <xsl:attribute name="type">inflow</xsl:attribute>
    <product>
      <xsl:copy-of select="@*"/>
    </product>
  </xsl:template>
  <xsl:template match="output">
    <xsl:attribute name="type">outflow</xsl:attribute>
    <reactant>
      <xsl:copy-of select="@*"/>
    </reactant>
  </xsl:template>


  <xsl:template match="variable[@type='differential']">
   <!-- <xsl:variable name="reaction">
      <reaction>
	<xsl:attribute name="id">
	  <xsl:value-of select="concat('reaction_',@id)"/>
	</xsl:attribute>
	<xsl:copy-of select="@latex-look"/>
	<xsl:copy-of select="kinetic-expression"/>
	<product>
	  <xsl:attribute name="spec">
	    <xsl:value-of select="@id"/>
	  </xsl:attribute>
	</product>
      </reaction>
    </xsl:variable>-->
 
      <species type="differential"> 
	<xsl:copy-of select="@id|@latex-look"/>
	<xsl:attribute name="initialConcentration">
	  <xsl:value-of select="@initial"/>
	</xsl:attribute>
	<xsl:apply-templates select="kinetic-expression"/>
      </species>
      
      <!--<xsl:apply-templates select="$reaction/reaction"/>-->
  </xsl:template>

  
  
  <xsl:template match="*">
    <!--<xsl:message>
      otherwise <xsl:copy-of select="name()"/>
    </xsl:message>-->
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="@*|*|comment()|text()"/>
      </xsl:element>
  </xsl:template>
  
  <xsl:template match="@*|comment()|text()">
    <xsl:copy-of select="."/>
  </xsl:template>
  

</xsl:stylesheet>
