<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:variable name="EMPTY">
      <text> Candidates/empty.xml</text>    
    </xsl:variable>

    <xsl:variable name="EMPTY-LAWS">
      <text> Laws/empty.xml</text>    
    </xsl:variable>	    

    <xsl:param name="LAWS" select="$EMPTY-LAWS" required="no"/>
    <xsl:param name="reactions-only" select="'false'" required="no"/>

    <xsl:variable name="laws">
       <xsl:copy-of select="document(concat('../',$LAWS))"/>
    </xsl:variable>

    <xsl:variable name="network">
       <xsl:copy-of select="network"/>
    </xsl:variable>

    <xsl:variable name="DEBUG">
      <text>false</text>  <!--either 'true' or 'false' -->
    </xsl:variable>

    <xsl:template match="network">
      <xsl:message>
	conservation laws= <xsl:copy-of select="$laws"/>
	reactions-only=<xsl:copy-of select="$reactions-only"/>
      </xsl:message>

      <!--<xsl:message>
	network
	<xsl:copy-of select="$network"/>
      </xsl:message>-->

      <xsl:variable name="ODEs-with-expr">
	<equations>
	  <xsl:apply-templates select="*"/>
	  <xsl:comment>
	    conservation laws
	  </xsl:comment>
	  <xsl:copy-of select="$laws/equations/*"/>
	</equations>
      </xsl:variable>

       <xsl:variable name="ODEs-without-expr">
	<equations>
	  <xsl:apply-templates select="metabolite | protein | actor |
			 species[not(@type='algebraic' or @type='differential')]"/>
	  <xsl:comment>
	    conservation laws
	  </xsl:comment>

	  <xsl:copy-of select="$laws/equations/*"/>
	</equations>
       </xsl:variable>

       <xsl:message>
	 °°°°°°°°°°°° reactions-only=<xsl:value-of select="$reactions-only"/>
       </xsl:message>
       <xsl:choose>
	 <xsl:when test="$reactions-only='true'">
	   <xsl:apply-templates select="$ODEs-without-expr"
				mode="elim-expr"/>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:apply-templates select="$ODEs-with-expr"
				mode="elim-expr"/>
	 </xsl:otherwise>
       </xsl:choose>
    </xsl:template>

    <xsl:template match="metabolite | protein | actor |
			 species[not(kinetic-expression)]">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:comment>
	<xsl:value-of select="name()"/>
	<xsl:text> </xsl:text><xsl:value-of select="@id"/>
      </xsl:comment>
      <xsl:variable name="id">
	  <xsl:value-of select="@id"/>
      </xsl:variable>
      <equal>
	<dot>
	  <xsl:attribute name="spec">
	    <xsl:value-of select="@id"/>
	  </xsl:attribute>
	  </dot>
	  <sum>
	    <xsl:apply-templates
		select="//reaction/child::*[@spec=$id and @stoichiometry]" 
				 mode="the-speed"/>
	    <xsl:apply-templates
		select="//context/child::*[@spec=$id]"   mode="role"/>
	  </sum>
      </equal>
      <xsl:comment> essential species </xsl:comment>
      <!--
	  <xsl:if test="@essential='yes'">
	<nonzero>
	  <conc>
	    <xsl:attribute name="spec">
	      <xsl:value-of select="@id"/>
	    </xsl:attribute>
	  </conc>
	</nonzero>
      </xsl:if>-->
    </xsl:template>
    
    <xsl:template match="species[kinetic-expression][not(@type='differential')]">
      <xsl:comment> species <xsl:value-of select="@id"/> </xsl:comment>
      <equal>
	<conc>
	  <xsl:attribute name="spec">
	    <xsl:value-of select="@id"/>
	  </xsl:attribute>
	</conc>
	<xsl:copy-of select="kinetic-expression/*"/>
      </equal>
    </xsl:template>

     <xsl:template match="species[kinetic-expression][@type='differential']">
      <xsl:comment> species <xsl:value-of select="@id"/> </xsl:comment>
      <equal>
	<dot>
	  <xsl:attribute name="spec">
	    <xsl:value-of select="@id"/>
	  </xsl:attribute>
	</dot>
	<xsl:copy-of select="kinetic-expression/*"/>
      </equal>
     </xsl:template>

     <xsl:template match="expression ">
      <xsl:comment> expression </xsl:comment>
      <equal>
	<expr>
	    <xsl:copy-of select="@id"/>
	</expr>
	<xsl:copy-of select="*"/>
      </equal>
     </xsl:template>

     <xsl:template match="parameter">
       <xsl:if test="$DEBUG='true'">
	 <xsl:message> control parameter <xsl:copy-of select="."/>
	 </xsl:message>
       </xsl:if>
      <xsl:comment> control parameter </xsl:comment>
      <equal>
	<dot>
	  <xsl:attribute name="spec">
	    <xsl:copy-of select="@id"/>
	  </xsl:attribute>
	</dot>
	<constant value="0"/>
      </equal>
    </xsl:template>


    <xsl:template match="context[output]">
      <xsl:comment> outflow <xsl:value-of select="@id"/> </xsl:comment>
      <similar>
	<speed>
	  <xsl:attribute name="reaction">
	    <xsl:value-of select="@id"/>
	  </xsl:attribute>
	</speed>
	<conc>
	  <xsl:copy-of select="output/@spec"/>
	</conc>
      </similar>
   </xsl:template>

   <xsl:template match="context[input]"/>
   
   <xsl:template match="*" mode="the-speed">
     <xsl:if test="$DEBUG='true'"><xsl:message>
       ODEs
       - the-speed <xsl:copy-of select="."/>
     </xsl:message></xsl:if>
     <xsl:variable name="the-speed">
       <speed>
	 <xsl:attribute name="reaction"> 
	   <xsl:value-of select="parent::*/@id"/>
	 </xsl:attribute>
       </speed>
     </xsl:variable>
     
     <xsl:if test="$DEBUG='true'">
       <xsl:message>       - spec <xsl:value-of select="@spec"/>
       - reaction <xsl:value-of select="parent::*/@id"/>
       - stoichio <xsl:value-of select="@stoichiometry"/>
       </xsl:message>
     </xsl:if>
     
     <xsl:variable name="speed">
       <xsl:choose>
	 <xsl:when test="@stoichiometry=1">
	   <xsl:copy-of select="$the-speed"/>
	 </xsl:when>
	 <xsl:when test="@stoichiometry=0">
	   <!-- nothing -->
	 </xsl:when>
	 <xsl:when test="@stoichiometry=-1">
	   <minus>
	     <xsl:copy-of select="$the-speed"/>
	   </minus>
	 </xsl:when>
	 <xsl:when test="@stoichiometry &gt; 1">
	   <mult>
	     <constant>
	       <xsl:attribute name="value">
		 <xsl:copy-of select="@stoichiometry"/>
	       </xsl:attribute>
	     </constant>
	     <xsl:copy-of select="$the-speed"/>
	   </mult>
	 </xsl:when>
	 <xsl:when test="@stoichiometry &lt; 1">
	   <minus>
	     <mult>
	       <constant>
		 <xsl:attribute name="value">
		   <xsl:copy-of select="- @stoichiometry"/>
		 </xsl:attribute>
	       </constant>
	       <xsl:copy-of select="$the-speed"/>
	     </mult>
	   </minus>
	 </xsl:when>
       </xsl:choose>
     </xsl:variable>
     <xsl:copy-of select="$speed"/>
   </xsl:template>

   <xsl:template   match="input"  mode="role">
     <speed>
       <xsl:attribute name="reaction">
	 <xsl:value-of select="parent::*/@id"/>
       </xsl:attribute>
     </speed>
   </xsl:template>

   <xsl:template   match="output"  mode="role">
     <minus>
       <speed>
	 <xsl:attribute name="reaction">
	   <xsl:value-of select="parent::*/@id"/>
	 </xsl:attribute>
       </speed>
     </minus>
   </xsl:template>

   <xsl:template match="reaction[kinetics/@id='ma']">
     <xsl:if test="$DEBUG='true'"><xsl:message>
       reaction with kinetics <xsl:value-of select="@id"/>
     </xsl:message></xsl:if>
     <xsl:comment> reaction <xsl:value-of select="@id"/> </xsl:comment>
     <xsl:variable name="speed">
       <mult>
	 <xsl:if test="kinetics/@para">
	   <constant>
	     <xsl:attribute name="value">
	       <xsl:copy-of select="kinetics/@para"/>
	     </xsl:attribute>
	   </constant>
	 </xsl:if>
	 <xsl:apply-templates select="reactant|activator" mode="conc"/>
       </mult>
     </xsl:variable>
     <xsl:variable name="mode">
       <xsl:choose>
	 <xsl:when test="kinetics/@mode">
	   <xsl:value-of select="kinetics/@mode"/>
	 </xsl:when>
	 <xsl:otherwise>similar</xsl:otherwise>
       </xsl:choose>
     </xsl:variable>
     <xsl:element name="{$mode}">
       <speed>
	 <xsl:attribute name="reaction">
	   <xsl:value-of select="@id"/>
	 </xsl:attribute>
       </speed>
	 <xsl:choose>
	   <xsl:when test="@candidate">
	     <mult>
	       <knockout>
		 <xsl:copy-of select="@id"/>
	       </knockout>
	       <xsl:copy-of select="$speed"/>
	     </mult>
	   </xsl:when>
	   <xsl:otherwise>
	     <xsl:copy-of select="$speed"/>
	   </xsl:otherwise>
	 </xsl:choose>
     </xsl:element>
   </xsl:template>

   <!-- reaction with kinetic expressions -->

    <xsl:template match="reaction[kinetic-expression]">
      <xsl:if test="$DEBUG='true'">
	<xsl:message>
	reaction with kinetic expression <xsl:value-of select="@id"/>
      </xsl:message>
      </xsl:if>
      <xsl:variable name="speed">
	<xsl:copy-of select="kinetic-expression/*"/>
      </xsl:variable>
     <xsl:if test="$DEBUG='true'"><xsl:message>
       +++ speed <xsl:copy-of select="$speed"/>
     </xsl:message></xsl:if>
     
     <equal>
       <speed>
	 <xsl:attribute name="reaction">
	   <xsl:value-of select="@id"/>
       </xsl:attribute>
     </speed>
       <xsl:choose>
	 <xsl:when test="@candidate">
	   <mult>
	     <knockout>
	       <xsl:copy-of select="@id"/>
	     </knockout>
	     <xsl:copy-of select="$speed"/>
	   </mult>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="$speed"/>
	 </xsl:otherwise>
       </xsl:choose>
     </equal>

 </xsl:template>

   
   <xsl:template match="reaction[
			(not(kinetic-expression) and not(kinetics))
			or kinetics/@id='exp'
			or kinetics/@id='deac'  ]"  >
     <xsl:if test="$DEBUG='true'"><xsl:message>
       reaction <xsl:value-of select="@id"/>
     </xsl:message></xsl:if>
     <xsl:comment> reaction <xsl:value-of select="@id"/> </xsl:comment>      
     <xsl:variable name="positiv"> 
       <mult>
	 <xsl:apply-templates select="activator|reactant" mode="conc"/>
	 <xsl:if test="inhibitor">
	   <inh>
	     <sum>
	       <xsl:apply-templates select="inhibitor" mode="conc"/>
	     </sum>
	   </inh>
	 </xsl:if>
	 <xsl:if test="accelerator">
	   <acc>
	     <sum>
	       <xsl:apply-templates select="accelerator" mode="conc"/>
	     </sum>
	   </acc>
	 </xsl:if>
	 </mult>
     </xsl:variable>
     
     <xsl:variable name="speed">
       <xsl:choose>
	 <xsl:when test="product-inh">
	   <binary-minus>
	     <xsl:copy-of select="$positiv"/>
	     <sum>
	       <xsl:apply-templates select="product-inh"/>
	     </sum>
	   </binary-minus>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="$positiv"/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:variable>
       
     <similar>
       <speed>
	 <xsl:attribute name="reaction">
	   <xsl:value-of select="@id"/>
	 </xsl:attribute>
       </speed>
       <xsl:choose>
	 <xsl:when test="@candidate">
	   <mult>
	     <knockout>
	       <xsl:copy-of select="@id"/>
	     </knockout>
	     <xsl:copy-of select="$speed"/>
	   </mult>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="$speed"/>
	 </xsl:otherwise>
       </xsl:choose>
     </similar>
   </xsl:template>

   
   <xsl:template match="reaction[kinetics]
			[not(kinetics/@expr)]
			[not(kinetics/@id='ma')]
			[not(kinetics/@id='exp')]">
     <xsl:message>
       reaction with other kinetics
     </xsl:message>
     <xsl:variable name="speed">
       <apply>
	 <xsl:copy-of select="kinetics/@para"/>
	 <xsl:attribute name="kinetics">
	   <xsl:choose>
	     <xsl:when test="kinetics/@id">
	       <xsl:value-of select="kinetics/@id"/>
	     </xsl:when>
	     <xsl:otherwise>exp</xsl:otherwise>
	   </xsl:choose>
	 </xsl:attribute>
	 <xsl:copy-of select="reactant|accelerator|activator|inhibitor"/>
	 <xsl:apply-templates select="product-inh" mode="speed"/>
       </apply>
      </xsl:variable>
     <reaction>
       <xsl:copy-of select="@id|child::*"/>
       <speed>
	 <xsl:copy-of select="kinetics/@mode"/>
	 <xsl:choose>
	   <xsl:when test="@candidate">
	     <mult>
	       <knockout>
		 <xsl:copy-of select="@id"/>
	       </knockout>
	       <xsl:copy-of select="$speed"/>
	     </mult>
	   </xsl:when>
	   <xsl:otherwise>
	     <xsl:copy-of select="$speed"/>
	   </xsl:otherwise>
	 </xsl:choose>
       </speed>
     </reaction>
   </xsl:template>

   <xsl:template match="*" mode="conc">
     <conc>
       <xsl:copy-of select="@spec"/>
      </conc>
   </xsl:template>
   
   <xsl:template match="reaction[kinetics/@expr]">
     <xsl:variable name="speed">
       <expr>
	 <xsl:value-of select="kinetics/@expr"/>
       </expr>
     </xsl:variable>
     <reaction>
       <xsl:copy-of select="@id|child::*"/>
       <speed>
	 <xsl:copy-of select="kinetics/@mode"/>
	 <xsl:choose>
	   <xsl:when test="@candidate">
	     <mult>
	       <knockout>
		 <xsl:copy-of select="@id"/>
	       </knockout>
	       <xsl:copy-of select="$speed"/>
	     </mult>
	   </xsl:when>
	   <xsl:otherwise>
	     <xsl:copy-of select="$speed"/>
	   </xsl:otherwise>
	 </xsl:choose>
       </speed>
     </reaction>
   </xsl:template>

   <xsl:template match="product-inh" mode="speed">
     <product-inh>
       <xsl:copy-of select="@spec"/>
     </product-inh>
   </xsl:template>

   <xsl:template match="equal" mode="speed">
     <xsl:copy-of select="."/>
   </xsl:template>


    <!-- function elim-expr -->
    <xsl:template match="expr|param" mode="elim-expr">
      <!--<xsl:variable name="id">
	<xsl:value-of select="@id"/>
      </xsl:variable>
      <xsl:variable name="expression">
	<xsl:copy-of select="$network/network/expression[@id=$id]"/>
      </xsl:variable>
      <xsl:message>
	- - expression <xsl:value-of select="$id"/>=<xsl:copy-of select="$expression"/>
      </xsl:message>-->
      <var>
	<!--<xsl:copy-of select="@* | $expression/expression/@latex-look"/>-->
	<xsl:copy-of select="@id" />
      </var>
  </xsl:template>

  <xsl:template match="*" mode="elim-expr">
    <xsl:variable name="tag">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>
    <xsl:element name="{$tag}">
      <xsl:apply-templates select="@*|*|comment()|text()" mode="elim-expr"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="@*|
		       comment()|text()" mode="elim-expr">
    <xsl:copy-of select="."/>
  </xsl:template>

    
</xsl:stylesheet>
