<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>


    <!-- flatten positive equations -->
    
    <xsl:variable name="DEBUG">false</xsl:variable>

    <xsl:param name="PREFIX" required="yes"/>     

    <xsl:template match="steady-state|equations">
      <xsl:if test="$DEBUG='true'">
	<xsl:message>
	****flatten.xsl **********************************
	PREFIX <xsl:value-of select="$PREFIX"/>
       </xsl:message>
      </xsl:if>

      <xsl:variable name="simplify">
	<xsl:apply-templates select="." mode="simplify"/>
      </xsl:variable>

      <xsl:if test="$DEBUG='true'">
       <xsl:message>
	 program: flatten.xsl
	 message: simplify
	 <xsl:copy-of select="$simplify"/>
       </xsl:message>
     </xsl:if>

     <xsl:variable name="left-flat">
	  <xsl:apply-templates select="$simplify" mode="left-flat"/>
      </xsl:variable>
      
      <xsl:if test="$DEBUG='true'">
       <xsl:message>
	 program: flatten.xsl
	 message: left-flat
	 <xsl:copy-of select="$left-flat"/>
       </xsl:message>
     </xsl:if>

      <xsl:variable name="id-equations">
	<xsl:apply-templates select="$left-flat"
			     mode="add-id"/>
      </xsl:variable>

     <xsl:if test="$DEBUG='true'">
       <xsl:message>
	 program: flatten.xsl
	 message: id-equations
	 <xsl:copy-of select="$id-equations"/>
       </xsl:message>
     </xsl:if>
     
      <xsl:variable name="flatten">
	<xsl:apply-templates select="$id-equations"  mode="flatten"/>
      </xsl:variable>

      <xsl:if test="$DEBUG='true'">
	<xsl:message>
	  program: flatten.xsl
	  message: result of flatten applied to id-equations
	  <xsl:copy-of select="$flatten"/>
	</xsl:message>
      </xsl:if>

      <xsl:copy-of select="$flatten"/>

    </xsl:template>

    <!-- function is-flat -->
    <xsl:template match=
		  "constant | speed | conc | var |
		   knockout | posdot | negdot "   mode="is-flat">
      <true/>
    </xsl:template>
    <xsl:template match="*" mode="is-flat">
      <false/>
    </xsl:template>

    <!--function left-flat -->
    <xsl:template match="equal" mode="left-flat">
      <xsl:variable name="left-is-flat">
	<xsl:apply-templates select="*[1]" mode="is-flat"/>
      </xsl:variable>
      <xsl:variable name="right-is-flat">
	<xsl:apply-templates select="*[2]" mode="is-flat"/>
      </xsl:variable>
      <xsl:choose>
	<xsl:when test="$left-is-flat/true">
	  <xsl:copy-of select="."/>
	</xsl:when>
	<xsl:when test="$right-is-flat/true">
	  <xsl:comment> flipped equation </xsl:comment>
	  <equal>
	    <xsl:copy-of select="child::*[2]"/>
	    <xsl:copy-of select="child::*[1]"/>
	  </equal>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:comment> left flattening </xsl:comment>
	  <equal>
	    <var>
	      <xsl:attribute name="id">
		<xsl:value-of select="concat('left--',$PREFIX,'--',generate-id(.))"/>
	      </xsl:attribute>
	    </var>
	    <xsl:copy-of select="child::*[1]"/>
	  </equal>
	  <equal>
	    <var>
	      <xsl:attribute name="id">
		<xsl:value-of select="concat('left--',$PREFIX,'--',generate-id(.))"/>
	      </xsl:attribute>
	    </var>
	    <xsl:copy-of select="child::*[2]"/>
	  </equal>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>
    
    <xsl:template match="@*|text()| comment()| similar|nonzero"       mode="left-flat">
      <xsl:copy-of select="."/>
    </xsl:template>

    <xsl:template match="steady-state| equations"               mode="left-flat">
      <xsl:variable name="name">
	<xsl:value-of select="name()"/>
      </xsl:variable>
      <xsl:element name="{$name}">
	<xsl:apply-templates select="*|@*|text()| comment" mode="left-flat"/>
      </xsl:element>
    </xsl:template>

    <xsl:template match="global-vars" mode="left-flat">
      <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*"               mode="left-flat">
      <xsl:message>
	Error left-flat
	equation=<xsl:copy-of select="."/>
      </xsl:message>
    </xsl:template>
    
    <!-- function add-id -->
    <xsl:template match="steady-state" mode="add-id">
      <steady-state debug="with-ids">
	<xsl:apply-templates select="child::*" mode="add-id"/>
      </steady-state>
    </xsl:template>

      <xsl:template match="equations" mode="add-id">
      <equations>
	<xsl:apply-templates select="child::*" mode="add-id"/>
      </equations>
    </xsl:template>

    <xsl:template match="*" mode="add-id">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="child::*" mode="put-id"/>
      </xsl:element>
    </xsl:template>

    <!-- function put-id -->
    <xsl:template match="sum|mult|inh|acc|power"
		  mode="put-id">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:attribute name="var-id">
	  <xsl:value-of select="concat($PREFIX,'-',generate-id(.))"/>
	</xsl:attribute>
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates select="child::*" mode="put-id"/>
      </xsl:element>
    </xsl:template>

    <xsl:template match=
		  "speed|conc|var|knockout|posdot|negdot|constant" mode="put-id">
	<xsl:copy-of select="."/>
    </xsl:template>

    <xsl:template match="steady-state| equations" mode="flatten">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates
	    select=
	    "//equal   |
	     //similar  |
	     //nonzero  | 
	     //mult/       (mult | sum | power | inh |acc| inv)    |
	     //sum/       (mult | sum | power  | inh |acc| inv)    |
	     //power/     (mult | sum | power | inh |acc| inv)    | 
	     //inh/         (mult  | sum | power | inh |acc| inv)    | 
	     //acc/         (mult  | sum | power | inh |acc| inv)    "      mode="constraint"/>
      </xsl:element>
    </xsl:template>
     
    <xsl:template match="equal|similar|nonzero" mode="constraint">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="child::*[1]" mode="variable"/>
	<xsl:apply-templates select="child::*[2]" mode="term"/>
      </xsl:element>
    </xsl:template>

    <xsl:template match="sum|mult|inh|acc|power" mode="constraint">
      <equal>
	<xsl:apply-templates select="." mode="variable"/>
	<xsl:apply-templates select="." mode="term"/>
      </equal>
    </xsl:template>
    
    <xsl:template match="sum|mult|power|inh|acc|inv" mode="variable">
      <var>
	<xsl:attribute name="id">
	  <xsl:copy-of select="concat($PREFIX,'--',./@var-id)"/>
	</xsl:attribute>
      </var>
    </xsl:template>

  <!--  <xsl:template match="inh|acc|inv" mode="variable">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="child::*" mode="variable"/>
      </xsl:element>
    </xsl:template>-->
    
    <xsl:template match="conc|speed|var|constant|knockout|posdot|negdot" mode="variable">
      <xsl:copy-of select="."/>
    </xsl:template>

     <!-- function term -->
    
    <xsl:template match="sum|mult|power| acc|inh|inv" mode="term">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:copy-of select="@kinetics|@value"/>
	<xsl:apply-templates select="child::*" mode="variable"/>
      </xsl:element>	   
    </xsl:template>

    <xsl:template match=
		  "conc| speed | var| knockout |
		   posdot| negdot|constant    "        mode="term">
      <xsl:copy-of select="."/>
    </xsl:template>

    <!-- function simplify -->
    <xsl:template match="sum[count(*)=0]" mode="simplify">
      <constant id="empty-sum" value="0"/>
    </xsl:template>
    <xsl:template match="sum[count(*)=1]" mode="simplify">
      <xsl:apply-templates select="*" mode="simplify"/>
    </xsl:template>
    <xsl:template match="mult[count(*)=0]" mode="simplify">
      <constant id="empty-mult" value="1"/>
    </xsl:template>
    <xsl:template match="mult[count(*)=1]" mode="simplify">
      <xsl:apply-templates select="*" mode="simplify"/>
    </xsl:template>
    <!-- else -->
    <xsl:template match="*" mode="simplify">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="@*|*|text()|comment()" mode="simplify"/>
      </xsl:element>	   
    </xsl:template>
    <xsl:template match="@*|text()|comment()" mode="simplify">
      <xsl:copy-of select="."/>
    </xsl:template>
</xsl:stylesheet>
