<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="text" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>

    <!-- default parameters -->
    <xsl:variable name="EMPTY-NETWORK">
      <xsl:text>Graph/empty.xml</xsl:text>    
    </xsl:variable>

    <!-- parameters -->
    <xsl:param name="NETWORK"         select="$EMPTY-NETWORK"     required="no"/>
    
    <!-- chose parameters file names or else the defaults -->

    <xsl:variable name="network">
    <xsl:choose>
      <xsl:when test="doc-available(concat('../',$NETWORK))">
	<xsl:message>NETWORK=<xsl:value-of select="$NETWORK"/>
	</xsl:message>
	<xsl:copy-of select="doc(concat('../',$NETWORK))"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:message> ++++ network NOT found at <xsl:value-of select="$NETWORK"/>
	</xsl:message>
	<text>Graph/empty.xml</text>
      </xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    
    <xsl:template match="//steady-state">
      <text>\textbf{</text> <xsl:value-of select="@msg"/> 

      <text xml:space="preserve">}

\begin{longtable}{ll} 
</text>
      <xsl:apply-templates select="similar|equal|nonzero|output-model"/>
      <text xml:space="preserve">\end{longtable}
</text>
    </xsl:template>

    <xsl:template match="equal|similar">
      <xsl:variable name="relation">
	<xsl:choose>
	  <xsl:when test="name()='equal'">
	    <text> $\ensuremath{\EQU}$ </text>
	  </xsl:when>
	  <xsl:when test="name()='similar'">
	    <text> $\sim$ </text>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:value-of select="name()"/>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:variable>
      <text> $ </text>
      <xsl:apply-templates select="child::*[1]"/>
      <text> $  </text> 
      <xsl:value-of select="$relation"/>
      <text>  $</text>
      <xsl:apply-templates select="child::*[2]"/>
      <text xml:space="preserve">$\\
</text>
    </xsl:template>

    <xsl:template match="nonzero">
      <text>$</text>
      <xsl:apply-templates select="child::*[1]"/>
      <text> \not= 0$ \\
</text>
    </xsl:template>
   <xsl:template match="sum">
     <xsl:if test="parent::mult">
       <text> ( </text>
     </xsl:if>
     <xsl:choose>
       <xsl:when test="count(child::*)=0">
	 <xsl:text>0</xsl:text>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:apply-templates select="child::*" mode="plus"/>
       </xsl:otherwise>
     </xsl:choose>
     <xsl:if test="parent::mult">
       <text> ) </text>
     </xsl:if>
   </xsl:template>

   <xsl:template match="output-model">
     <xsl:apply-templates select="child::*"/>
   </xsl:template>

   <xsl:template match="binary-minus">
     <xsl:apply-templates select="child::*[1]"/>
     <text> - </text>
     <xsl:if test="not(child::*[2][self::var or self::conc or self::input or self::output])">
       <text> ( </text>
     </xsl:if>
     <xsl:apply-templates select="child::*[2]"/>
      <xsl:if test="not(child::*[2][self::var or self::conc or self::input or self::output])">
       <text> ) </text>
     </xsl:if>
   </xsl:template>


   <xsl:template match="divide">
     <xsl:choose>
       <xsl:when test="count(child::*) =2">
	 <text> \frac{ </text>
	 <xsl:apply-templates select="*[1]"/>
	 <text> }{ </text>
	 <xsl:apply-templates select="*[2]"/>
	 <text> } </text>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:for-each select="child::*[following-sibling::*]">
	   <xsl:apply-templates select="."/>
	   <text xml:space="preserve">\  </text>
	 </xsl:for-each>
	 <xsl:apply-templates select="child::*[not(following-sibling::*)]"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:template>

   <xsl:template   match="apply[@fun]">
      <xsl:value-of select="@fun"/>
	<xsl:text>(</xsl:text>
	<xsl:for-each select="*">
	  <xsl:apply-templates select="."/>
	  <xsl:if test="not(position()=last())">
	    <text xml:space="preserve">,</text>
	  </xsl:if>
     </xsl:for-each>
	  <xsl:text>)</xsl:text>
   </xsl:template>
   
   <xsl:template match="mult">
     <xsl:choose>
       <xsl:when test="count(child::*) =0">
	 <text>1</text>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:for-each select="child::*[following-sibling::*]">
	   <xsl:apply-templates select="."/>
	    <xsl:if test="not(self::constant and following-sibling::*[1][name()='conc'])">
	      <xsl:text>\;</xsl:text>
	    </xsl:if>
	 </xsl:for-each>
	 <xsl:apply-templates select="child::*[not(following-sibling::*)]"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:template>

   <xsl:template match="apply[@kinetics]">
     <xsl:value-of select="@kinetics"/><text>(</text>
     <xsl:apply-templates select="child::*"/>
     <text xml:space="preserve">)</text>
   </xsl:template>

   <xsl:template match="expr">
     <xsl:value-of select="text()"/>
   </xsl:template>

   <xsl:template match="inh">
     <text>\inh(</text>
     <xsl:apply-templates select="child::*"/>
     <text xml:space="preserve">)</text>
   </xsl:template>

   <xsl:template match="acc">
     <text>\acc(</text>
     <xsl:apply-templates select="child::*"/>
     <text xml:space="preserve">)</text>
   </xsl:template>

   <xsl:template match="constant">
     <xsl:if test="not(parent::mult and (@value='1' or @value='1.0'))">
       <xsl:value-of select="@value"/>
     </xsl:if>
   </xsl:template>

     <xsl:template   match="power" >
    <xsl:choose>
      <xsl:when test="*[2][name()='constant'][@value='0.5']">
	<text> \sqrt{</text>
	<xsl:apply-templates select="*[1]" />
	<text xml:space="preserve"> } </text>
      </xsl:when>
      <xsl:otherwise>
	<text> {(</text>
     <xsl:apply-templates select="*[1]" /> 
     <text xml:space="preserve">)}^{ </text>
     <xsl:apply-templates select="*[2]" />
     <text xml:space="preserve">} </text>
      </xsl:otherwise>
    </xsl:choose>
     </xsl:template>

     <xsl:template match="delay">
    <xsl:text> delay(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
   <xsl:text>,</xsl:text>
    <xsl:apply-templates select="*[2]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>
     
   <xsl:template   match="minus[count(*)=1]">
     <text xml:space="preserve"> - </text> 
     <xsl:apply-templates select="*"/> 
     <xsl:apply-templates select="." mode="komma"/>
   </xsl:template>

   <xsl:template   match="minus[count(*)=2]">
     <text xml:space="preserve"> ( </text> 
      <xsl:apply-templates select="*[1]"/>
      <text xml:space="preserve"> - </text> 
      <xsl:apply-templates select="*[2]"/>
      <text xml:space="preserve"> ) </text> 
   </xsl:template>

   <xsl:template   match="inv">
     <text xml:space="preserve"> \frac{1}{ </text> 
     <xsl:apply-templates select="*"/> 
     <text>}</text>
     <xsl:apply-templates select="." mode="komma"/>
   </xsl:template>

   <xsl:template match="speed">
     <text xml:space="preserve"> \ensuremath{\v{</text>
     <xsl:value-of select="@reaction"/> 
     <text xml:space="preserve">}} </text>
     <xsl:apply-templates select="." mode="komma"/>
   </xsl:template>

 
  <xsl:template   match="output">
    <text>\y{</text><xsl:value-of select="@spec"/><text>}</text>
     <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template   match="input">
    <text>\x{</text><xsl:value-of select="@spec"/><text>}</text>
     <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="conc">
    <text>\z{</text> <xsl:value-of select="@spec"/>
    <text space="preserve">}</text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

   <xsl:template match="dot">
     <text xml:space="preserve"> \ensuremath{\Dot{\z{</text>
     <xsl:value-of select="@spec"/> 
     <text xml:space="preserve">}}} </text>
     <xsl:apply-templates select="." mode="komma"/>
   </xsl:template>

   <xsl:template match="posdot">
    <text>\ensuremath{\posdot{</text> <xsl:value-of select="@spec"/>
    <text space="preserve">}}</text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>


    <xsl:template match="negdot">
    <text>\ensuremath{\negdot{</text> <xsl:value-of select="@spec"/>
    <text space="preserve">}}</text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="knockout">
    <text>\ko{</text> 
    <xsl:value-of select="@id"/>
    <text>}</text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="var">
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    <xsl:variable name="node">
      <xsl:copy-of select="$network/*/*[@id=$id]"/>
    </xsl:variable>
    <xsl:if test="not($node/*/@latex-look)">
      <xsl:message> --- latex-look <xsl:value-of select="$id"/>=<xsl:value-of select="$node/*/@latex-look"/>
      </xsl:message>
    </xsl:if>
    <xsl:choose>
      <xsl:when test="$node/*">
	<text>\ensuremath{\expr{</text> <xsl:value-of select="$node/*/@latex-look"/>
	<text space="preserve">}}</text>
	<xsl:apply-templates select="." mode="komma"/>
      </xsl:when>
      <xsl:otherwise>
      <text>\ensuremath{\w{</text> <xsl:value-of select="@id"/>
      <text space="preserve">}}</text>
      <xsl:apply-templates select="." mode="komma"/>
    </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
      
    <xsl:template match="*[following-sibling::* 
			and not(
			parent::power or
			parent::equal or
			parent::similar or 
			parent::sum or
			parent::mult or
			parent::minus or 
			parent::inh or
			parent::acc or 
			parent::divide or 
			parent::binary-minus)]"             mode="komma">
       <text xml:space="preserve">, </text> 
    </xsl:template>
    
    <xsl:template match="*" mode="komma"/>

   <xsl:template match="*[following-sibling::*[position()=1 
			and not(self::minus)] 
			and not(parent::mult)]" mode="plus">
     <xsl:apply-templates select="."/>
     <text> + </text> 
   </xsl:template>
   
   <xsl:template match="*" mode="plus">
     <xsl:apply-templates select="."/>
   </xsl:template>

</xsl:stylesheet>

