<!-- filter the metabolic part of a reaction network -->
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>

    <xsl:template match="network">
      <xsl:variable name="pre-metabolic-network">
	<xsl:apply-templates select="." mode="filter"/>
      </xsl:variable>
      <!--<xsl:message>
	pre-metabolic-network
	<xsl:copy-of select="$pre-metabolic-network"/>
      </xsl:message>-->
      <xsl:result-document href="$pre-metabolit-network" method="xml"/> 
      <xsl:apply-templates select="$pre-metabolic-network" mode="clean"/>
    </xsl:template>
    
    <!-- function filter -->
    <xsl:template match="  network
			 | reaction
			 | product[@spec=//metabolite/@id]
			 | reactant[@spec=//metabolite/@id]
			 "
		  mode="filter">
      <xsl:variable name="tag">
	<xsl:value-of select="name()"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="@*|*" mode="filter"/>
      </xsl:element>
    </xsl:template>

    <!-- copy -->
    <xsl:template match="metabolite" mode="filter">
      <metabolite>
	<xsl:apply-templates select="@*|*" mode="filter"/>
      </metabolite>
    </xsl:template>

    <!-- rewrite -->
    <xsl:template match="context[*/@spec=//metabolite/@id]" mode="filter">
      <context>
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates select="@x|@y|input|output|in-out" mode="filter"/>
      </context>
    </xsl:template>
    <xsl:template match="input" mode="filter">
      <input>
	  <xsl:copy-of select="@spec"/>
      </input>
    </xsl:template>
    <xsl:template match="output" mode="filter">
      <output>
	<xsl:copy-of select="@spec"/>
	</output>
      </xsl:template>
    <xsl:template match="in-out" mode="filter">
      <reactant>
	<xsl:copy-of select="@spec"/>
      </reactant>
      <product>
	<xsl:copy-of select="spec"/>
      </product>
    </xsl:template>
    
    <!--ignore -->
    <xsl:template match="  protein | actor
			 | comment | @comment 
			 | @candidate | @essential | @stoichiometry
			 | prod-inh
			 | product[not(@spec=//metabolite/@id)]
			 | reactant[not(@spec=//metabolite/@id)]
			 | inhibitor | accelerator | activator
			 | context[not(child::*/@spec=//metabolite/@id)]
			 | @kind "
		  mode="filter"/>
    <!-- copy -->
    <xsl:template match="@*|text()" mode="filter">
      <xsl:copy-of select="."/>
    </xsl:template>

    <!-- function clean -->
    <xsl:template match="reaction[not(child::*)]" mode="clean"/>

    <xsl:template match="network" mode="clean">
      <xsl:variable name="tag">
	<xsl:value-of select="name()"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:attribute name="kind">metabolic-network</xsl:attribute>
	<xsl:apply-templates select="@*|*" mode="clean"/>
      </xsl:element>
    </xsl:template>
    <xsl:template match="*" mode="clean">
      <xsl:variable name="tag">
	<xsl:value-of select="name()"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="@*|*" mode="clean"/>
      </xsl:element>
    </xsl:template>

    <xsl:template match="@*" mode="clean">
      <xsl:copy-of select="."/>
    </xsl:template>
</xsl:stylesheet>
