<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <!-- user parameters -->

    <xsl:variable name="debug">no</xsl:variable>
    <!-- mode debug either 'yes' or 'no' -->
    

    <xsl:template match="steady-state[//expr]">
      <steady-state msg="these steady state equations cannot be simplified  since they contain latex expressions"/>
    </xsl:template>
		 

    <xsl:template match="steady-state[not(//expr)]">
      <!--
	  <xsl:message>
	  simplified-steady-state debug "<xsl:copy-of select="$debug"/>"
	  </xsl:message>
      -->
      <xsl:variable name="base">
	<xsl:apply-templates select="." mode="base"/>
      </xsl:variable>
  
      <xsl:variable name="start">
	<steady-state change="no">
	  <xsl:copy-of select="$base/steady-state/child::*"/>
	</steady-state>
      </xsl:variable>
      
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  start after base simplifications <xsl:copy-of select="$start"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="next">
	<xsl:apply-templates select="$start/steady-state" mode="simplify"/>
      </xsl:variable>

      <!-- repeat while changes -->

      <xsl:if test="$debug='yes'">
	<xsl:message>
	  next <xsl:copy-of select="$next"/>
	</xsl:message>
      </xsl:if>
      
      <xsl:if test="$debug='yes'">
	<xsl:message> 
	  any changes? <xsl:value-of select="$next/steady-state/@change"/>
	  still nonzero? "<xsl:value-of select="$next//nonzero"/>"
	</xsl:message>
      </xsl:if>
      <xsl:choose>
	<xsl:when test="$next/steady-state/@change='yes' 
			and count($next/steady-state)=1">
	   <xsl:apply-templates select="$next/steady-state"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy-of select="$next/steady-state"/>
	</xsl:otherwise>
      </xsl:choose>
 
    </xsl:template>
    
    <xsl:template match="steady-state" mode="simplify">

      <xsl:variable name="elim-equal">
	<xsl:apply-templates select="." mode="elim-equal"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  elim-equal A <xsl:copy-of select="$elim-equal"/>
	</xsl:message>
      </xsl:if>
      
      <xsl:variable name="linear-equations">
	<xsl:apply-templates select="$elim-equal" mode="linear-equations"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  linear-equations <xsl:copy-of select="$linear-equations"/>
	</xsl:message>
      </xsl:if>
      
      <xsl:variable name="sum-equal-sum">
	<xsl:apply-templates select="$linear-equations" mode="sum-equal-sum"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  sum-equal-sum <xsl:copy-of select="$sum-equal-sum"/>
	</xsl:message>
      </xsl:if>

      <xsl:variable name="sum-equal-sum-same">
	<xsl:apply-templates select="$sum-equal-sum" mode="sum-equal-sum-same"/>
      </xsl:variable>

      <xsl:variable name="sum-sum-to-sum">
	<xsl:apply-templates select="$sum-equal-sum-same" mode="sum-sum-to-sum"/>
      </xsl:variable>

      <xsl:variable name="equal-binary-minus">
	<xsl:apply-templates select="$sum-sum-to-sum" mode="equal-binary-minus"/>
      </xsl:variable>
      <xsl:copy-of select="$equal-binary-minus"/>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  equal-binary-minus <xsl:copy-of select="$equal-binary-minus"/>
	</xsl:message>
      </xsl:if>
    </xsl:template>

    <!-- base simplifications -->
    <xsl:template match="steady-state" mode="base">

      <xsl:variable name="neutral-element-simplified-equations">
	<xsl:apply-templates select="." mode="neutral-element"/>
      </xsl:variable>

      <xsl:variable name="arity-simplified-equations">
	<xsl:apply-templates select="$neutral-element-simplified-equations"
			     mode="arity-simplify"/>
      </xsl:variable>

      <xsl:if test="$debug='yes'">
	<xsl:message>
	  arity-simplified-equations
	  <xsl:copy-of select="$arity-simplified-equations"/>
	</xsl:message>
      </xsl:if>
      
      <xsl:variable name="zero-sum-simplified-equations">
	<xsl:apply-templates select="$arity-simplified-equations" 
			     mode="zero-equal-sum-simplify"/>
      </xsl:variable>

      <xsl:variable name="elim-zero-equal-zero">
	<xsl:apply-templates select="$zero-sum-simplified-equations"
			     mode="elim-zero-equal-zero"/>
      </xsl:variable>

      <xsl:copy-of select="$elim-zero-equal-zero"/>
    </xsl:template>

    <!-- neutral elements -->

    <xsl:template match="mult" mode="neutral-element">
      <mult>
	<xsl:apply-templates select="child::*[not(self::constant[@value='1'])]" mode="neutral-element"/>
      </mult>
    </xsl:template>
    
    <xsl:template match="sum" mode="neutral-element">
      <sum>
	<xsl:apply-templates select="child::*[not(self::constant[@value='0'])]" mode="neutral-element"/>
      </sum>
    </xsl:template>

    <xsl:template match="*" mode="neutral-element">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates select="child::*"      mode="neutral-element"/>
      </xsl:element>
    </xsl:template>
    

    <!-- arity-simplify -->

    <xsl:template match="mult[count(child::*) = 1]" mode="arity-simplify">
      <xsl:apply-templates select="child::*"        mode="arity-simplify"/>
    </xsl:template>
     
    <xsl:template match="mult[count(child::*) = 0]" mode="arity-simplify">
      <constant value="1"/>
    </xsl:template>
    
    <xsl:template match="sum[count(child::*) = 1]"  mode="arity-simplify">
      <xsl:apply-templates select="child::*"        mode="arity-simplify"/>
    </xsl:template>
    
    <xsl:template match="sum[count(child::*) = 0]"  mode="arity-simplify">
      <constant value="0"/>
    </xsl:template>
    
    <xsl:template match="inh[count(child::*) = 0]"  mode="arity-simplify"/>
    
    <xsl:template match="acc[count(child::*) = 0]"  mode="arity-simplify"/>

    <xsl:template match="inv[count(child::*) = 0]"  mode="arity-simplify"/>

    <xsl:template match="*" mode="arity-simplify">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates select="child::*"      mode="arity-simplify"/>
      </xsl:element>
    </xsl:template>
     

     <!-- function zero-equal-sum-simplify -->
     
     <xsl:template match="steady-state" mode="zero-equal-sum-simplify">
       <steady-state>
	 <xsl:apply-templates select="equal[sum and constant[@value='0']]" 
			      mode="zero-equal-sum-simplify"/>
	 <xsl:copy-of select="*[not(self::equal[sum and constant[@value='0']])]"/> 
       </steady-state>
     </xsl:template>
     
     <xsl:template match="equal[sum and constant[@value='0']]" 
		   mode="zero-equal-sum-simplify">
       <xsl:if test="$debug='yes'">
	 <xsl:message>
	   zero-equal-sum-simplify <xsl:copy-of select="."/>
	   into 
	 <xsl:for-each select="child::sum/child::*">
	   <equal>
	     <constant value="0"/>
	     <xsl:copy-of select="."/>
	   </equal>
	 </xsl:for-each>
	 </xsl:message>
       </xsl:if>
       <xsl:for-each select="child::sum/child::*">
	 <equal>
	   <constant value="0"/>
	   <xsl:copy-of select="."/>
	 </equal>
       </xsl:for-each>
     </xsl:template>

     <xsl:template match="*" mode="zero-equal-sum-simplify">
       <xsl:copy-of select="."/>
     </xsl:template>
     
     <!-- function elim-zero-equal-zero -->

     <xsl:template match="steady-state" mode="elim-zero-equal-zero">
        <xsl:choose>
	 <xsl:when test="equal[constant[1]/@value=constant[2]/@value]">
	   <steady-state>
	     <xsl:copy-of select="child::*[not(self::equal[constant[1]/@value=constant[2]/@value])]"/>
	   </steady-state>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
	</xsl:choose>
     </xsl:template>

     <!-- sum-sum-to-sum -->

    <xsl:template match="steady-state" mode="sum-sum-to-sum">
      <xsl:choose>
	<xsl:when test=".//sum[sum]">
	  <steady-state change="yes">
	    <!--
	    <xsl:message>
	      sum-sum-to-sum successful
	    </xsl:message>
	    -->
	    <xsl:apply-templates select="child::*" mode="sum-sum-to-sum"/>
	  </steady-state>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy-of select="."/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

     <xsl:template match="sum[sum]" mode="sum-sum-to-sum">
       <sum>
	 <xsl:apply-templates 
	     select="child::*[not(self::sum)] | child::sum/child::*"
	     mode="sum-sum-to-sum"/>
       </sum>
     </xsl:template>

     <!-- s(b(x,y),z) = b(s(x,z),y) --> 
     <xsl:template match="sum[binary-minus][not(sum)]" mode="sum-sum-to-sum">
       <binary-minus>
	 <sum>
	   <xsl:apply-templates
	       select="child::*[not(self::binary-minus)] | child::binary-minus/child::*[1]"
	       mode="sum-sum-to-sum"/>
	 </sum>
	 <xsl:apply-templates select="child::binary-minus/child::*[2]"
			      mode="sum-sum-to-sum"/>
       </binary-minus>
     </xsl:template>

     
     <!-- b(x,b(y,z)) = b(s(x,z),y) --> 
     <xsl:template match="binary-minus[child::*[2][self::binary-minus]]" mode="sum-sum-to-sum">
       <binary-minus>
	 <sum>
	   <xsl:apply-templates
	       select="child::*[1] | child::*[2]/child::*[2]"
	       mode="sum-sum-to-sum"/>
	 </sum>
	 <xsl:apply-templates select="child::*[2]/child::*[1]"
			      mode="sum-sum-to-sum"/>
       </binary-minus>
     </xsl:template>

      
     <!-- b(b(x,y),z) = b(x,s(y,z)) --> 
     <xsl:template match="binary-minus[child::*[1][self::binary-minus]]
			  [not(child::*[2][self::binary-minus])]"
		   mode="sum-sum-to-sum">
       <binary-minus>
	 <xsl:apply-templates select="child::*[1]/child::*[1]"
			      mode="sum-sum-to-sum"/>
	 <sum>
	   <xsl:apply-templates
	       select="child::*[1]/child::*[2] | child::*[2]"
	       mode="sum-sum-to-sum"/>
	 </sum>
       </binary-minus>
     </xsl:template>

     <xsl:template match="*" mode="sum-sum-to-sum">
       <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="child::*|@*" mode="sum-sum-to-sum"/>
      </xsl:element>
     </xsl:template>

     <xsl:template match="@*" mode="sum-sum-to-sum">
       <xsl:copy-of select="."/>
     </xsl:template>

     <!-- equal-binary-minus -->

    <xsl:template match="steady-state" mode="equal-binary-minus">
      <xsl:choose>
	<xsl:when test="equal[child::binary-minus]">
	  <steady-state change="yes">
	    <!--
	    <xsl:message>
	      equal-binary-minus successful
	    </xsl:message>
	    -->
	    <xsl:apply-templates select="child::*" mode="equal-binary-minus"/>
	  </steady-state>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy-of select="."/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

     <xsl:template match="equal[child::*[2]/self::binary-minus]" mode="equal-binary-minus">
       <equal>
	 <sum>
	   <xsl:apply-templates select="child::*[1]" mode="equal-binary-minus"/>
	   <xsl:apply-templates select="child::*[2]/child::*[2]" mode="equal-binary-minus"/>
	 </sum>
	 <xsl:apply-templates select="child::*[2]/child::*[1]" mode="equal-binary-minus"/>
       </equal>
     </xsl:template>

     <xsl:template match="equal[child::*[1]/self::binary-minus]" mode="equal-binary-minus">
       <equal>
	 <sum>
	   <xsl:apply-templates select="child::*[2]" mode="equal-binary-minus"/>
	   <xsl:apply-templates select="child::*[1]/child::*[2]" mode="equal-binary-minus"/>
	 </sum>
	 <xsl:apply-templates select="child::*[1]/child::*[1]" mode="equal-binary-minus"/>
       </equal>
     </xsl:template>

     <xsl:template match="*" mode="equal-binary-minus">
       <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="child::*|@*" mode="equal-binary-minus"/>
      </xsl:element>
     </xsl:template>

     <xsl:template match="@*" mode="equal-binary-minus">
       <xsl:copy-of select="."/>
     </xsl:template>






     <!-- sum-equal-sum-same -->
    <xsl:template match="steady-state" mode="sum-equal-sum-same">
      <xsl:choose>
	<xsl:when test="equal[child::sum[1]/child::speed/@reaction
			= child::sum[2]/child::speed/@reaction] |
			equal[child::sum[1]/child::input/@spec
			= child::sum[2]/child::input/@spec]     | 
			equal[child::sum[1]/child::output/@spec
			= child::sum[2]/child::output/@spec]    |
			//binary-minus[child::sum[1]/child::speed/@reaction
			= child::sum[2]/child::speed/@reaction] |
			//binary-minus[child::sum[1]/child::input/@spec
			= child::sum[2]/child::input/@spec]     |
			//binary-minus[child::sum[1]/child::output/@spec
			= child::sum[2]/child::output/@spec] 
			"> 
	  <xsl:if test="$debug='yes'">
	    <xsl:message>
	      sum-equal-sum-same applicable
	      <xsl:copy-of select="//binary-minus[child::sum[1]/child::output/@spec
				   = child::sum[2]/child::output/@spec]"/>
	    </xsl:message>
	  </xsl:if>
	    <steady-state change="yes">
	    <xsl:apply-templates select="child::*" mode="sum-equal-sum-same"/>
	  </steady-state>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy-of select="."/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <xsl:template match="equal[child::sum[1]/speed/@reaction=child::sum[2]/speed/@reaction][not(child::sum[1]/output/@spec=child::sum[2]/output/@spec)]" 
		  mode="sum-equal-sum-same">
 
      <xsl:variable name="output">
	<equal>
	  <sum>
	    <xsl:copy-of select="child::sum[1]/child::speed[
				 @reaction=preceding-sibling::speed/@reaction or
				 not(@reaction=ancestor::equal/sum[2]/child::speed/@reaction)]"/>
	    <xsl:copy-of select="child::sum[1]/child::*[not(self::speed)]"/>
	  </sum>
	  <sum>
	    <xsl:copy-of select="child::sum[2]/child::speed[
				 @reaction=preceding-sibling::speed/@reaction or
				 not(@reaction=ancestor::equal/sum[1]/child::speed/@reaction)]"/>
	    <xsl:copy-of select="child::sum[2]/child::*[not(self::speed)]"/>
	  </sum>
	</equal>
      </xsl:variable>
      <!--
      <xsl:message>
	sum-equal-sum-same-speed 

            <xsl:copy-of select="."/>    
	to  <xsl:copy-of select="$output"/>
      </xsl:message>
      -->
      <xsl:copy-of select="$output"/>
    </xsl:template>

    <!-- JJJJJ -->
    <xsl:template match="binary-minus[child::sum[1]/speed/@reaction=child::sum[2]/speed/@reaction]" 
		  mode="sum-equal-sum-same">
      <xsl:variable name="output">
	<binary-minus>
	  <sum>
	    <xsl:copy-of select="child::sum[1]/child::speed[
				 @reaction=preceding-sibling::speed/@reaction or
				 not(@reaction=ancestor::binary-minus/sum[2]/child::speed/@reaction)]"/>
	    <xsl:copy-of select="child::sum[1]/child::*[not(self::speed)]"/>
	  </sum>
	  <sum>
	    <xsl:copy-of select="child::sum[2]/child::speed[
				 @reaction=preceding-sibling::speed/@reaction or
				 not(@reaction=ancestor::binary-minus/sum[1]/child::speed/@reaction)]"/>
	    <xsl:copy-of select="child::sum[2]/child::*[not(self::speed)]"/>
	  </sum>
	</binary-minus>
      </xsl:variable>
      <!--
      <xsl:message>
	binary-minus-same-speed 

	<xsl:copy-of select="."/>    
	to <xsl:copy-of select="$output"/>
      </xsl:message>
      -->
      <xsl:copy-of select="$output"/>
    </xsl:template>

    <xsl:template match="equal[child::sum[1]/input/@spec=child::sum[2]/input/@spec]
			 [not(child::sum[1]/speed/@reaction=child::sum[2]/speed/@reaction)]" 
		  mode="sum-equal-sum-same">
      <xsl:variable name="output">
	<equal>
	  <sum>
	    <xsl:copy-of select="child::sum[1]/child::input[
				 @spec=preceding-sibling::input/@spec or
				 not(@spec=ancestor::equal/sum[2]/child::input/@spec)]"/>
	    <xsl:copy-of select="child::sum[1]/child::*[not(self::input)]"/>
	  </sum>
	  <sum>
	    <xsl:copy-of select="child::sum[2]/child::input[
				 @spec=preceding-sibling::input/@spec or
				 not(@spec=ancestor::equal/sum[1]/child::input/@spec)]"/>
	    <xsl:copy-of select="child::sum[2]/child::*[not(self::input)]"/>
	  </sum>
	</equal>
      </xsl:variable>
      <!--
      <xsl:message>
	sum-equal-sum-same-input 

           <xsl:copy-of select="."/>    
	to <xsl:copy-of select="$output"/>
      </xsl:message>
      -->
      <xsl:copy-of select="$output"/>
    </xsl:template>   

    <!--JJJJJJ-->
    <xsl:template match="binary-minus[child::sum[1]/input/@spec=child::sum[2]/input/@spec]
			 [not(child::sum[1]/speed/@reaction=child::sum[2]/speed/@reaction)]" 
		  mode="sum-equal-sum-same">
      
      <xsl:variable name="output">
	<binary-minus>
	  <sum>
	    <xsl:copy-of select="child::sum[1]/child::input[
				 @spec=preceding-sibling::input/@spec or
				 not(@spec=ancestor::binary-minus/sum[2]/child::input/@spec)]"/>
	    <xsl:copy-of select="child::sum[1]/child::*[not(self::input)]"/>
	  </sum>
	  <sum>
	    <xsl:copy-of select="child::sum[2]/child::input[
				 @spec=preceding-sibling::input/@spec or
				 not(@spec=ancestor::binary-minus/sum[1]/child::input/@spec)]"/>
	    <xsl:copy-of select="child::sum[2]/child::*[not(self::input)]"/>
	  </sum>
	</binary-minus>
      </xsl:variable>
      <!--
      <xsl:message>
	binary-minus-same-input 

           <xsl:copy-of select="."/>    
	to <xsl:copy-of select="$output"/>
      </xsl:message>
      -->
      <xsl:copy-of select="$output"/>
    </xsl:template>

   <xsl:template match="equal[child::sum[1]/output/@spec=child::sum[2]/output/@spec]" 
		 mode="sum-equal-sum-same">
      <xsl:variable name="output">
	<equal>
	  <sum>
	    <xsl:copy-of select="child::sum[1]/child::output[
				 @spec=preceding-sibling::output/@spec or
				 not(@spec=ancestor::equal/sum[2]/child::output/@spec)]"/>
	    <xsl:copy-of select="child::sum[1]/child::*[not(self::output)]"/>
	  </sum>
	  <sum>
	    <xsl:copy-of select="child::sum[2]/child::output[
				 @spec=preceding-sibling::output/@spec or
				 not(@spec=ancestor::equal/sum[1]/child::output/@spec)]"/>
	    <xsl:copy-of select="child::sum[2]/child::*[not(self::output)]"/>
	  </sum>
	</equal>
      </xsl:variable>
      <!--
      <xsl:message>
	sum-equal-sum-same-output 

           <xsl:copy-of select="."/>    
	to <xsl:copy-of select="$output"/>
      </xsl:message>
      -->
      <xsl:copy-of select="$output"/>
    </xsl:template>

    <!-- KKKKKKK -->
   <xsl:template match="binary-minus[child::sum[1]/output/@spec=child::sum[2]/output/@spec]
			[not(child::sum[1]/speed/@reaction=child::sum[2]/speed/@reaction)]
			[not(child::sum[1]/input/@spec=child::sum[2]/input/@spec)]"
		 mode="sum-equal-sum-same">
       <xsl:variable name="output">
	<binary-minus>
	  <sum>
	    <xsl:copy-of select="child::sum[1]/child::output[
				 @spec=preceding-sibling::output/@spec or
				 not(@spec=ancestor::binary-minus/sum[2]/child::output/@spec)]"/>
	    <xsl:copy-of select="child::sum[1]/child::*[not(self::output)]"/>
	  </sum>
	  <sum>
	    <xsl:copy-of select="child::sum[2]/child::output[
				 @spec=preceding-sibling::output/@spec or
				 not(@spec=ancestor::binary-minus/sum[1]/child::output/@spec)]"/>
	    <xsl:copy-of select="child::sum[2]/child::*[not(self::output)]"/>
	  </sum>
	</binary-minus>
       </xsl:variable>
       <!--
      <xsl:message>
	binary-minus-same-output 

           <xsl:copy-of select="."/>    
	to <xsl:copy-of select="$output"/>
      </xsl:message>
      -->
      <xsl:copy-of select="$output"/>
    </xsl:template>

    <xsl:template match="*" mode="sum-equal-sum-same">
      <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="child::*|@*" mode="sum-equal-sum-same"/>
      </xsl:element>
     </xsl:template>

     <xsl:template match="@*" mode="sum-equal-sum-same">
      <xsl:copy-of select="."/>
    </xsl:template>
    

     <!-- sum-equal-sum -->
    <xsl:template match="steady-state" mode="sum-equal-sum">
      <xsl:choose>
	<xsl:when test="equal[count(child::sum)=2 and child::sum[binary-minus or sum]]">
	  <steady-state change="yes">
	    <xsl:apply-templates select="child::*" mode="sum-equal-sum"/>
	  </steady-state>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy-of select="."/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <xsl:template match="equal[count(child::sum)=2 and child::sum[binary-minus or sum]]" 
		  mode="sum-equal-sum">
      <xsl:variable name="output">
	<equal>
	  <sum>
	    <xsl:copy-of select="child::sum[1]/child::*[not(self::binary-minus) 
				 and not(self::sum)]"/>
	    <xsl:copy-of select="child::sum[1]/child::sum/child::*"/>
	    <xsl:copy-of select="child::sum[1]/child::binary-minus/child::*[1]"/>
	    <xsl:copy-of select="child::sum[2]/child::binary-minus/child::*[2]"/>
	  </sum>
	  <sum>
	    <xsl:copy-of select="child::sum[2]/child::*[not(self::binary-minus)
				 and not(self::sum)]"/>
	    <xsl:copy-of select="child::sum[2]/child::sum/child::*"/>
	    <xsl:copy-of select="child::sum[2]/child::binary-minus/child::*[1]"/>
	    <xsl:copy-of select="child::sum[1]/child::binary-minus/child::*[2]"/>
	  </sum>
	</equal>
      </xsl:variable>
      <!--
      <xsl:message>
	sum-equal-sum <xsl:copy-of select="."/>    
	to <xsl:copy-of select="$output"/>
      </xsl:message>
      -->
      <xsl:copy-of select="$output"/>
    </xsl:template>
     
    <xsl:template match="*|@*" mode="sum-equal-sum">
      <xsl:copy-of select="."/>
    </xsl:template>

     <!-- recursive simplify-->

     <xsl:template match="steady-state" mode="elim-equal">
        <xsl:choose>
	 <xsl:when 
	     test="equal[(speed[not(@reaction=following-sibling::speed/@reaction)   
		   and  not(@reaction=preceding-sibling::speed/@reaction)])
		   or
		   (conc[not(@spec=following-sibling::conc/@spec) 
		   and not(@spec=preceding-sibling::conc/@spec)]) 
		   or 
		   (var[not(@id=following-sibling::var/@id) 
		   and     not(@id=preceding-sibling::var/@id)])]">
	   <xsl:if test="$debug='yes'">
	     <xsl:message>
	       elim-equal B start with <xsl:copy-of select="."/>
	     </xsl:message>
	   </xsl:if>
	   <xsl:for-each 
	       select="equal[(speed[not(@reaction=following-sibling::speed/@reaction)   
		       and  not(@reaction=preceding-sibling::speed/@reaction)])
		       or
		       (conc[not(@spec=following-sibling::conc/@spec) 
		       and not(@spec=preceding-sibling::conc/@spec)]) 
		       or 
		       (var[not(@id=following-sibling::var/@id) 
		       and not(@id=preceding-sibling::var/@id)])][1]">
	     <!--
	     <xsl:message>
	       elim-equal B <xsl:copy-of select="."/>
	     </xsl:message>
	     -->
	     <xsl:variable name="rest-equations">
	       <steady-state change="yes">
		 <xsl:copy-of select="preceding-sibling::*"/>
		 <xsl:copy-of select="following-sibling::*"/>
	       </steady-state>
	     </xsl:variable>
	     <xsl:if test="$debug='yes'">
	       <xsl:message>
		 rest-equations B <xsl:copy-of select="$rest-equations"/>
	       </xsl:message>
	     </xsl:if>
	     <xsl:variable name="next-equations">
	       <xsl:apply-templates select="$rest-equations/steady-state" 
				    mode="query-replace">
		 <xsl:with-param name="query"   
				 select="(speed[not(@reaction=following-sibling::speed/@reaction) 
					 and  not(@reaction=preceding-sibling::speed/@reaction)]
					 | conc[not(@spec=following-sibling::conc/@spec) 
					 and not(@spec=preceding-sibling::conc/@spec)] 
					 | var[not(@id=following-sibling::var/@id) 
					 and not(@id=preceding-sibling::var/@id)])[1]"/>
		 <xsl:with-param name="replace" 
				 select="* except
					 (speed[not(@reaction=following-sibling::speed/@reaction) 
					   and  not(@reaction=preceding-sibling::speed/@reaction)]
					   | conc[not(@spec=following-sibling::conc/@spec) 
					   and not(@spec=preceding-sibling::conc/@spec)] 
					   | var[not(@id=following-sibling::var/@id) 
					   and not(@id=preceding-sibling::var/@id)])[1]
					   "/>
	       </xsl:apply-templates>
	     </xsl:variable>
	     <xsl:variable name="output">
		 <xsl:apply-templates select="$next-equations" mode="elim-equal"/>
	       </xsl:variable>
	       <xsl:if test="$debug='yes'">
		 <xsl:message>
		   elim-equal B again <xsl:copy-of select="$output"/>
		 </xsl:message>
	       </xsl:if>
	       <xsl:copy-of select="$output"/>
	     </xsl:for-each>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
	</xsl:choose>
     </xsl:template>

     <!-- solve linear equations -->
     

     <xsl:template match="steady-state" mode="linear-equations">
        <xsl:choose>
	 <xsl:when 
	     test="equal/sum/speed[(for $reaction in @reaction 
		   return count(ancestor::equal//speed[@reaction=$reaction]))=1]">
	   <xsl:for-each 
	       select="(equal/sum/speed[for $reaction in @reaction return 
		       .[count(ancestor::equal//speed[@reaction=$reaction])=1]])[1]">
	     <!--
	     <xsl:message>
	       solve linear equation <xsl:copy-of select="ancestor::equal"/>
	     </xsl:message>
	     -->
	     <!--
	     <xsl:message>
	       number of equations <xsl:copy-of 
	       select="count(ancestor::steady-state/child::equal)"/>
	     </xsl:message>
	     -->
	     <xsl:variable name="rest-equations">
	       <steady-state change="yes">
		 <xsl:copy-of select="ancestor::equal/(preceding-sibling::*|following-sibling::*)"/>
	       </steady-state>
	     </xsl:variable>
	     <!--
	     <xsl:message>
	       number of rest equations <xsl:copy-of 
	       select="count($rest-equations/steady-state/child::equal)"/>
	     </xsl:message>
	     -->
	     <xsl:variable name="next-equations">
	       <xsl:apply-templates select="$rest-equations/steady-state" 
				    mode="query-replace">
		 <xsl:with-param name="query" select="."/>
		 <xsl:with-param name="replace">
		   <binary-minus>
		     <xsl:copy-of select="parent::sum/(preceding-sibling::*|following-sibling::*)"/>
		     <sum>
		       <xsl:copy-of select="preceding-sibling::*|following-sibling::*"/>
		     </sum>
		   </binary-minus>
		 </xsl:with-param>
	       </xsl:apply-templates>
	     </xsl:variable>
	     <!--
	     <xsl:message>
	       number of next equations <xsl:copy-of 
	       select="count($next-equations/steady-state/child::equal)"/>
	     </xsl:message>
	     -->

	     <xsl:variable name="output">
	       <xsl:apply-templates select="$next-equations" mode="elim-equal"/>
	     </xsl:variable>
	     <xsl:if test="$debug='yes'">
	       <xsl:message>
		 linear-equations <xsl:copy-of select="$output"/>
	       </xsl:message>
	     </xsl:if>
	     <xsl:copy-of select="$output"/>
	   </xsl:for-each>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
	</xsl:choose>
     </xsl:template>


     <!-- query-replace -->

     <xsl:template match="@*" mode="query-replace">
       <xsl:copy-of select="."/>
     </xsl:template>

     <xsl:template match="*[not(self::speed) and
			  not(self::conc) and 
			  not(self::var)] " mode="query-replace">
       <xsl:param name="query"/>
       <xsl:param name="replace"/>
       
       <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
       
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="child::*|@*" mode="query-replace">
	  <xsl:with-param name="query"   select="$query"/>
	  <xsl:with-param name="replace" select="$replace"/>
	</xsl:apply-templates>
      </xsl:element>

     </xsl:template>

     <xsl:template match="speed|conc|var" mode="query-replace">
       <xsl:param name="query"/>
       <xsl:param name="replace"/>
       <xsl:choose>
	 <xsl:when test="self::speed[@reaction=$query/@reaction] or
 			 self::conc[@spec=$query/@spec] or
			 self::var[@id=$query/@id]">
	   <xsl:message>
	     <equal>
	       <xsl:copy-of select="$query"/> 
	       <xsl:copy-of select="$replace"/>
	     </equal>
	    </xsl:message>
	   <xsl:copy-of select="$replace"/>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>



</xsl:stylesheet>
