<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>

    <xsl:template match="//network"> 
      <xsl:apply-templates select="." mode="stoichiometry"/>
    </xsl:template>
    
    <!-- function stoichiometry -->
    
    <xsl:template match="network" mode="stoichiometry">
      <network>
	<xsl:copy-of select="@*"/>
	<xsl:copy-of select="*[not(self::reaction or self::equal)]"/>
	<xsl:apply-templates select="reaction|equal" mode="stoichiometry" />
      </network>
    </xsl:template>
    
    <xsl:template match="reaction" mode="stoichiometry">
      <reaction>
	<xsl:copy-of select="@*"/>
	<xsl:copy-of select="*[
			     not(name()='product'
			       or name()='reactant'
			       or name()=product-inh)]"/>
	<xsl:apply-templates select="product|reactant|product-inh"
			     mode="stoichiometry"/>
      </reaction>
    </xsl:template>
    
    <xsl:template match="product|reactant|product-inh"
		  mode="stoichiometry">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:variable name="spec">
	<xsl:value-of select="@spec"/>
      </xsl:variable>
      <xsl:choose>
	<xsl:when test="not(@spec=preceding-sibling::*[self::product 
			or self::reactant or self::product-inh]/@spec)">
	  <xsl:comment> first </xsl:comment>
	  <xsl:element name="{$tag}">     
	    <xsl:copy-of select="@*"/>
	    <xsl:attribute name="stoichiometry">
	      <!--xs:integer(floor(-->
	      <xsl:value-of
		  select="
			  count(parent::*/product[@spec=$spec][not(@stoichiometry)])
			  +
			  sum(parent::*/product[@spec=$spec]/@stoichiometry)
			  +
			  count(parent::*/product-inh[@spec=$spec])
			  -
			  count(parent::*/reactant[@spec=$spec][not(@stoichiometry)]  )
			  -
			  sum(parent::*/reactant[@spec=$spec]/@stoichiometry)"/>
	    </xsl:attribute>
	  </xsl:element>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:comment> keep copies without stoichiometries </xsl:comment>
  	  <xsl:element name="{$tag}">     
	    <copy-of select="@spec"/>
	  </xsl:element>
	</xsl:otherwise>
    </xsl:choose>
    </xsl:template>
    
    <xsl:template match="*|@*" mode="stoichiometry">
      <xsl:copy-of select="."/>
    </xsl:template>

</xsl:stylesheet>
