<xsl:stylesheet 
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="text" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:param name="OUTPUT_DIR"                           required="yes"/>
    <xsl:param name="NETWORK_FILE" select="network"        required="no"/>
    <xsl:param name="NETWORK"      select="$NETWORK_FILE"  required="no"/>
    <xsl:param name="VERSION"                              required="yes"/>
    <xsl:param name="ANALYSIS"                             required="no"/>
      
    <xsl:variable name="OUTPUT_DIR-LATEX-FIX">
      <xsl:value-of select="replace(OUTPUT_DIR,'_','\\_')"/>
    </xsl:variable>
  
    <xsl:variable name="NETWORK_FILE-LATEX-FIX">
      <xsl:value-of select="replace($NETWORK_FILE,'_','\\_')"/>
    </xsl:variable>
    
    <xsl:variable name="NETWORK-LATEX-FIX">
      <xsl:value-of select="replace($NETWORK,'_','\\_')"/>
    </xsl:variable>
    
   <xsl:variable name="ANALYSIS-LATEX-FIX">
      <xsl:value-of select="replace($ANALYSIS,'_','\\_')"/>
    </xsl:variable>
  
    <xsl:template match="/">

      <xsl:variable name="full-id">
	<xsl:value-of select="network/@id"/>
	<xsl:if test="network/@kind">
	  <text>-</text>
	  <xsl:value-of select="network/@kind"/>
	</xsl:if>
      </xsl:variable>


      <text xml:space="preserve">      
	\documentclass{Latex/llncs}
	

\usepackage[a0paper]{geometry}
\usepackage[colorlinks]{hyperref}

\usepackage{longtable}
\usepackage{capt-of}

\usepackage[utf8]{inputenc}
\usepackage{textalpha}

\usepackage{amsmath}
\usepackage{amssymb}
\usepackage{natbib}
\usepackage{stmaryrd}
\usepackage{mathrsfs}
\usepackage{bussproofs}
\usepackage{graphicx}
\usepackage{color}
\usepackage{multirow}
\usepackage{array,multirow,makecell}
\usepackage[T1]{fontenc}
\usepackage{Latex/inference}
\usepackage{xspace}
\usepackage{listings}
\usepackage[version=3]{mhchem}
\usepackage{pdftexcmds} % for switch-macros
\usepackage{Latex/macros}
      </text>
      
      <xsl:if test="not(//*[@latex-look])">
	<text>
	  \usepackage{Latex/macros-species}
	  \usepackage{Latex/switch-macros}
	</text>
      </xsl:if>

      <xsl:if test="//*[@latex-look]">
	<text>
	\usepackage{</text>
	<xsl:value-of select="$OUTPUT_DIR"/><text>/latex-macros} 
	</text>
      </xsl:if>

	
  <text>\usepackage[dvipsnames]{xcolor}
  \usepackage{tikz}
  \usetikzlibrary{math} %needed tikz library
  \definecolor{lightblue}{HTML}{92dcec}

\usepackage{Latex/tikz-macros}

\usepackage{mathtools}
\DeclarePairedDelimiter{\ceiling}{\lceil}{\rceil}
\DeclarePairedDelimiter\floor{\lfloor}{\rfloor}
\usetikzlibrary{decorations.pathmorphing}


\EnableBpAbbreviations

\renewcommand\TIKZID[1]{</text>
      <xsl:value-of select="$full-id"/>
      <xsl:text xml:space="preserve">}

\begin{document}

      \title{</xsl:text>
      <xsl:if test="network/@kind">
	<xsl:value-of select="network/@kind"/><text>\\ </text>
      </xsl:if>
<xsl:value-of select="$NETWORK_FILE-LATEX-FIX"/>
<xsl:if test="network/@name">
  <text>: </text>
  <xsl:value-of select="replace(network/@name,'_','\\_')"/>
</xsl:if>
<text>}

\subtitle{Presented by BioComputing's Network-Graph Tool }
\author{Joachim Niehren and Cristian Versari}
\institute{BioComputing, Inria and Université de Lille, France}


\maketitle

\section{Reaction Network}

\paragraph{Source File:}

\url{</text>
<xsl:copy-of select="$NETWORK_FILE-LATEX-FIX"/>
<text>} </text>
<xsl:if test="network/@source">
  <text> from
  \href{</text>
  <xsl:value-of select="network/@source"/>
  <text>}
  {</text>
	<xsl:value-of select="network/@kind"/><text>}. </text>
      </xsl:if>

<text>
\paragraph{Tool:}
      \href{http://researchers.lille.inria.fr/~niehren/BioComputing/Network-Graph/doc.html}{BioComputing's Network-Graph Tool} Version </text>
      <xsl:value-of select="$VERSION"/>

\paragraph{Application Date:}
<text> \today
      </text>

      <xsl:variable name="statistics">
  <statistics>
    <entry name="\# Species">
      <xsl:value-of
	  select="count(//network/metabolite|
		  //network/protein | //network/species)"/>
    </entry>
    <entry name="\# Reactions">
      <xsl:value-of select="count(//network/reaction)"/>
    </entry>
    <entry name="\# Macros">
	<xsl:value-of select="count(//network/expression)"/>
    </entry>
      <entry name="\# Elements">
	<xsl:value-of select="count(//network//*)"/>
      </entry>
  </statistics>
</xsl:variable>

<xsl:variable name="latex-statistics">
  
\begin{tabular}{||l|r|}
   \hline\hline
   
   <xsl:apply-templates select="$statistics/*:statistics/*:entry"
			mode="latex-statistics"/>
   
   \hline\hline
   \end{tabular}
</xsl:variable>

<xsl:message>
  --- Statistics <xsl:copy-of select="$NETWORK_FILE-LATEX-FIX"/> -----------
</xsl:message>

<xsl:for-each select="$statistics//*:entry">
    
<xsl:message>
  <xsl:value-of select="replace(@name,'\\','')"/> : <xsl:value-of select="."/>
  </xsl:message>
</xsl:for-each>

<xsl:message>
</xsl:message>
<text xml:space="preserve">


\paragraph{Statistics.} 

</text>

  <xsl:value-of select="$latex-statistics"/>
  
  <text>
    \section{Network Graph}
%See Figure \ref{fig:</text><xsl:value-of select="$NETWORK-LATEX-FIX"/><text>}.

%\paragraph{Analysis}

% \url{</text> <xsl:copy-of select="$ANALYSIS-LATEX-FIX"/> <text xml:space="preserve">}



%\begin{center}
\input{</text>
<xsl:value-of select="$OUTPUT_DIR"/>
<text>/graph-tikz.tex}

%\captionof{figure}{\label{fig:</text><xsl:value-of select="$NETWORK_FILE"/><text>}
%The </text><xsl:value-of select="$NETWORK_FILE-LATEX-FIX"/><text>.}
%\addtocounter{table}{-1}
%\end{center}


\section{Initial values}


%\begin{figure}
%\begin{center}
\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/species.tex}
%\end{center}
%\caption{Species of </text><xsl:value-of select="$NETWORK-LATEX-FIX"/><text>.}
%\end{figure}
</text>

<xsl:if test="//parameter">
 <text>
%\begin{figure}
%\begin{center}
\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/parameters.tex}
%\end{center}
%\caption{Control parameters of </text><xsl:value-of select="$NETWORK-LATEX-FIX"/><text>.}
%\end{figure}
</text>
</xsl:if>

<text>
\section{Equations over time}

\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/expressions.tex}

\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/reactions.tex}

\renewcommand\w[1]{\expr{#1}}

\begin{longtable}[l]{||l|}
\multicolumn{1}{l}{Differential equations of species}\\
\hline\hline
\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/ODEs/ODEs.tex}
\\\hline\hline
\end{longtable}

</text>

<xsl:if test="//event">

  <text>
\subsection{Events}
%\begin{figure}
%\begin{center}
\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/events.tex}
%\end{center}
%\caption{Events of </text><xsl:value-of select="$NETWORK-LATEX-FIX"/><text>.}
%\end{figure}
</text>
</xsl:if>


<xsl:if test="//function">

  <text>
\section{Functions}

%\begin{center}
\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/functions.tex}
%\end{center}
</text>
</xsl:if>


<xsl:if test="//network/comment">
<text>

  
\subsection{What Else}

\input{</text>
<xsl:value-of select="$OUTPUT_DIR"/>
<text>/comments.tex}
</text>
</xsl:if>
<text>
\end{document}</text>

</xsl:template>

<xsl:template match="*:entry" mode="latex-statistics">
  <xsl:value-of select="@name"/>
  <xsl:text>&amp; </xsl:text>
  <xsl:value-of select="."/>
  <xsl:text>\\
  </xsl:text>
</xsl:template>


</xsl:stylesheet>
