<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns=""
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    version="2.0"
    exclude-result-prefixes="xs">

  <xsl:output   
      method="text" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>

  <xsl:variable name="DEBUG">false</xsl:variable>
  
  <!-- default parameters -->
  <xsl:variable name="EMPTY-KO">
    <xsl:text>Knockouts/empty.xml</xsl:text>    
  </xsl:variable>
  <xsl:variable name="EMPTY-SOLUTIONS">
    <xsl:text>Solutions/empty.xml</xsl:text>    
  </xsl:variable>
  <xsl:variable name="EMPTY-FLUXES">
    <xsl:text>Fluxes/empty.xml</xsl:text>
  </xsl:variable>
  <xsl:variable name="EMPTY-CHARTS">
    <xsl:text>Charts/empty.xml</xsl:text>    
  </xsl:variable>
  <xsl:variable name="NO">no</xsl:variable>


  <!-- parameters -->

  <xsl:param name="VERSION"                                        required="yes"/>
  <xsl:param name="KNOCKOUTS_FILE"      select="$EMPTY-KO"         required="no"/>
  <xsl:param name="SAFE_KNOCKOUTS_FILE" select="$EMPTY-KO"         required="no"/>
  <xsl:param name="SOLUTIONS_FILE"      select="$EMPTY-SOLUTIONS"  required="no"/>
  <xsl:param name="FLUXES_FILE"         select="$EMPTY-FLUXES"     required="no"/>
  <xsl:param name="CHARTS_FILE"         select="$EMPTY-CHARTS"     required="no"/>
  <xsl:param name="MAX-SOLUTION-GRAPHS" select="10000000"          required="no"/>
  <xsl:param name="GLOBAL_SCALE"        select="$NO"               required="no"/>


  <!-- chose parameters file names or else the defaults -->
  
  <xsl:variable name="KNOCKOUTS_FILE_RES">
    <xsl:choose>
      <xsl:when test="doc-available(concat('../',$KNOCKOUTS_FILE))">
	<xsl:copy-of select="normalize-space($KNOCKOUTS_FILE)"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text>Knockouts/empty.xml</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="CHARTS_FILE_RES">
    <xsl:choose>
      <xsl:when test="doc-available(concat('../',$CHARTS_FILE))">
	<xsl:copy-of select="normalize-space($CHARTS_FILE)"/>
      </xsl:when>
      <xsl:otherwise>
	<text>Charts/empty.xml</text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="SAFE_KNOCKOUTS_FILE_RES">
    <xsl:choose>
      <xsl:when test="doc-available(concat('../',$SAFE_KNOCKOUTS_FILE))">
	<xsl:copy-of select="normalize-space($SAFE_KNOCKOUTS_FILE)"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text>Knockouts/empty.xml</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="SOLUTIONS_FILE_RES">
    <xsl:choose>
      <xsl:when test="doc-available(concat('../',$SOLUTIONS_FILE))">
	<xsl:copy-of select="normalize-space($SOLUTIONS_FILE)"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text>Solutions/empty.xml</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="FLUXES_FILE_RES">
    <xsl:choose>
      <xsl:when test="doc-available(concat('../',$FLUXES_FILE))">
	<xsl:copy-of select="normalize-space($FLUXES_FILE)"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text>Fluxes/empty.xml</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- access parameter files -->
  
  <xsl:variable name="ko">
    <xsl:copy-of select="document(concat('../',$KNOCKOUTS_FILE_RES))"/>
  </xsl:variable>
  
  <xsl:variable name="safe-ko">
    <xsl:copy-of select="document(concat('../',$SAFE_KNOCKOUTS_FILE_RES))"/>
  </xsl:variable>
  <xsl:variable name="solutions">
    <xsl:copy-of select="document(concat('../',$SOLUTIONS_FILE_RES))"/>
  </xsl:variable>

  <xsl:variable name="fluxes">
    <xsl:copy-of select="document(concat('../',$FLUXES_FILE_RES))"/>
  </xsl:variable>
  <xsl:variable name="charts">
    <xsl:copy-of select="document(concat('../',$CHARTS_FILE_RES))"/>
  </xsl:variable>


  <xsl:variable name="SOLUTIONS_AVAILABLE">
    <xsl:choose>
      <xsl:when test="doc-available(concat('../',$SOLUTIONS_FILE)) and  
		      not(ends-with($SOLUTIONS_FILE,'Solutions/empty.xml'))">yes</xsl:when>
      <xsl:otherwise>no</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="KNOCKOUTS_AVAILABLE">
    <xsl:choose>
      <xsl:when test="doc-available(concat('../',$KNOCKOUTS_FILE)) and  
		      not(ends-with($KNOCKOUTS_FILE,'Knockouts/empty.xml'))">yes</xsl:when>
      <xsl:otherwise>no</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
    
  <xsl:variable name="solution-variables">
    <xsl:copy-of select="$solutions/results/variables"/>
  </xsl:variable>


  <!-- start compilation of the network to a latex graph -->

  <xsl:template match="network">

    <xsl:if test="$DEBUG='true'">
      <xsl:message>
      SOLUTIONS_FILE = <xsl:copy-of select="$SOLUTIONS_FILE"/>
      KNOCKOUTS_FILE = <xsl:copy-of select="$KNOCKOUTS_FILE"/>
      SOLUTIONS_FILE_RES = <xsl:copy-of select="$SOLUTIONS_FILE_RES"/>
      KNOCKOUTS_FILE_RES = <xsl:copy-of select="$KNOCKOUTS_FILE_RES"/>
      FLUXES_FILE_RES= <xsl:copy-of select="$FLUXES_FILE_RES"/>
      ko = <xsl:copy-of select="$ko"/>
      solution-variables = <xsl:copy-of select="$solution-variables"/>
      <!--solutions <xsl:copy-of select="$solutions"/>-->
    </xsl:message></xsl:if>
    
 
    <xsl:variable name="network">
      <xsl:apply-templates select="." mode="kinetics"/>
    </xsl:variable>

    <xsl:if test="$DEBUG='true'">
      <xsl:message>
      kinetic-network <xsl:copy-of select="$network"/>
    </xsl:message>

      <xsl:message>resolution of default parameters:</xsl:message>
      <xsl:if test="not(contains($KNOCKOUTS_FILE_RES,'empty.xml'))">
	<xsl:message>  KNOCKOUTS_FILE: <xsl:value-of select="$KNOCKOUTS_FILE_RES"/>  </xsl:message>
      </xsl:if>
      <xsl:if test="not(contains($SAFE_KNOCKOUTS_FILE_RES,'empty.xml'))">
	<xsl:message>  SAFE_KNOCKOUTS_FILE: <xsl:value-of select="$SAFE_KNOCKOUTS_FILE_RES"/> </xsl:message>
      </xsl:if>
      <xsl:if test="not(contains($SOLUTIONS_FILE_RES,'empty.xml'))">
	<xsl:message>  SOLUTIONS_FILE: <xsl:value-of select="$SOLUTIONS_FILE_RES"/> </xsl:message>
      </xsl:if>
      <xsl:if test="not(contains($FLUXES_FILE_RES,'empty.xml'))">
	<xsl:message>  FLUXES_<xsl:value-of select="$FLUXES_FILE_RES"/>  </xsl:message>
      </xsl:if>
      <xsl:if test="not(contains($CHARTS_FILE_RES,'empty.xml'))">
	<xsl:message>  CHARTS_FILE: <xsl:value-of select="$CHARTS_FILE_RES"/>  </xsl:message>
      </xsl:if>
    </xsl:if>
    
    <xsl:if test="$DEBUG='true'"><xsl:message>
      +++++++++++++ start iteration over solutions
      -- number of solutions <xsl:value-of select="count(($solutions//solution
			  [not(@b_target='false')])
			  [position() &lt; $MAX-SOLUTION-GRAPHS+1])"/>
    </xsl:message>
    <xsl:text>
      % +++++++++++++ start iteration over solutions
      % -- number of solutions    </xsl:text>
    <xsl:value-of select="count(($solutions//solution
			  [not(@b_target='false')])
			  [position() &lt; $MAX-SOLUTION-GRAPHS+1])"/>
    <xsl:text>
</xsl:text>
    </xsl:if>
    <xsl:for-each select="($solutions//solution
			  [not(@b_target='false')])
			  [position() &lt; $MAX-SOLUTION-GRAPHS+1]">
      <xsl:apply-templates select="$network" mode="compile-one">
	<xsl:with-param name="solution" select="."/>
	<xsl:with-param name="position" select="position()"/>
      </xsl:apply-templates>
      <xsl:text>
	% ++++++++++ next solution
      </xsl:text>
    </xsl:for-each>
  </xsl:template>


  <!-- function compile
       maps a network with a solution to a latex graph-->
  
  <xsl:template match="network" mode="compile-one">
    <xsl:param name="solution"/>
    <xsl:param name="position"/>

    <!--
    <xsl:if test="$DEBUG='true'"><xsl:message>
      latex-graph: compile-one
      - solution=<xsl:value-of select="$solution"/>
      - position=<xsl:value-of select="$position"/>
    </xsl:message></xsl:if>
    -->
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
      <xsl:if test="@kind">
	<xsl:text>-</xsl:text>
	<xsl:value-of select="@kind"/>
      </xsl:if>
    </xsl:variable>

    <xsl:variable name="full-id">
      <xsl:value-of select="$id"/>
      <xsl:if test="not($FLUXES_FILE='Solutions/empty.xml')">
	<xsl:if test="$SOLUTIONS_AVAILABLE='yes'">
	  <xsl:text>---SOLUTION-</xsl:text>
	  <xsl:value-of select="$position"/>
	</xsl:if>
      </xsl:if>
    </xsl:variable>

    <xsl:if test="$DEBUG='true'">
    <xsl:message>
      +++++++++++++ end iteration over solutions
    </xsl:message>
      <xsl:text>
      % +++++++++++++ end iteration over solutions
    </xsl:text>
    <xsl:message>      -- SOLUTIONS_FILE <xsl:copy-of select="$SOLUTIONS_FILE"/> 
      -- SOLUTIONS_FILE_RES <xsl:copy-of select="$SOLUTIONS_FILE_RES"/> 
      -- full-id <xsl:copy-of select="$full-id"/>
    </xsl:message> </xsl:if>

    
    <text xml:space="preserve">\renewcommand\TIKZID[1]{</text>
    <xsl:value-of select="$full-id"/>
    <text xml:space="preserve">}
    \renewcommand\TIKZREFID[1]{</text>
    <xsl:value-of select="$id"/>
    <text>}

    <text xml:space="preserve">\pgfdeclaredecoration{simple line}{initial}{
    \state{initial}[width=\pgfdecoratedpathlength-1sp]{\pgfmoveto{\pgfpointorigin}}
    \state{final}{\pgflineto{\pgfpointorigin}}
    }
    </text><text xml:space="preserve">\tikzset{
    shift left/.style={decorate,decoration={simple line,raise=#1}},
    shift right/.style={decorate,decoration={simple line,raise=-1*#1}},
    }	  </text>

    \begin{tikzpicture}[concept color = gray!15</text>
    <xsl:if test="network/@scale">
      <text>,scale=</text>
      <xsl:value-of select="network/@scale"/>
    </xsl:if>
    <text>] 
    </text>

    <xsl:apply-templates select="." mode="compile">
      <xsl:with-param name="solution" select="$solution"/>
    </xsl:apply-templates>
    <text xml:space="preserve">
      \end{tikzpicture}\\[2em]
    </text>

    <xsl:if test="$DEBUG='true'">  <xsl:message>
      SOLUTIONS_FILE: <xsl:value-of select="$SOLUTIONS_FILE"/>
      SOLUTIONS_AVAILABLE:  <xsl:value-of select="$SOLUTIONS_AVAILABLE"/>
      <xsl:if test ="$SOLUTIONS_AVAILABLE='yes'">
	number of solutions  <xsl:value-of select="count($solutions//solution[not(@b_target='false')])"/>
	maximal number of displayed solutions <xsl:value-of select="$MAX-SOLUTION-GRAPHS"/>
    </xsl:if>
    </xsl:message></xsl:if> 
    
    <xsl:if test="$SOLUTIONS_AVAILABLE='yes'">
      <text>
      \begin{center} S</text>
      <xsl:value-of select="$position"/>
      <text>. Solution </text>
      <xsl:value-of select="$solution/position"/>
      <text>\end{center}
      </text>
    </xsl:if>
    <xsl:if test="$position&gt;1">
      <xsl:message>
	graph of solution at position <xsl:value-of select="$position"/>
      </xsl:message>
    </xsl:if>
    
  </xsl:template>

  <!-- function compile
           compiles a network with a solution to a latex graph
     -->

  <xsl:template match="network" mode="compile">
    <xsl:param name="solution"/>
    <xsl:variable name="graph">
      
      <xsl:text xml:space="preserve">%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% species, their copies, split points, and connections 
      %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      </xsl:text>
      <xsl:apply-templates select="metabolite|protein|actor|species|context-node" mode="compile">
	<xsl:with-param name="solution" select="$solution"/>
      </xsl:apply-templates>

      <xsl:text xml:space="preserve">%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% nodes
      %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      </xsl:text>
      <xsl:apply-templates select="node" mode="compile">
	<xsl:with-param name="solution" select="$solution"/>
      </xsl:apply-templates>

       <xsl:text xml:space="preserve">%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% context-nodes
      %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      </xsl:text>
      <xsl:apply-templates select="context-node" mode="node">
	<xsl:with-param name="solution" select="$solution"/>
      </xsl:apply-templates>
      <xsl:apply-templates select="context-node" mode="edge">
	<xsl:with-param name="solution" select="$solution"/>
      </xsl:apply-templates>

      
      <xsl:text xml:space="preserve">%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% context inflows/outflows
      %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      </xsl:text>
      <xsl:apply-templates select="context" mode="node">
	<xsl:with-param name="solution" select="$solution"/>
      </xsl:apply-templates>
      <xsl:apply-templates select="context" mode="edge"/>
 
      <xsl:text xml:space="preserve">%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% edgeclusters
      %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      </xsl:text>
      <xsl:apply-templates select="edgecluster" mode="node"/>
      <xsl:apply-templates select="edgecluster" mode="edge"/>

      <xsl:text xml:space="preserve">%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% reactions
      %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      </xsl:text>
      <xsl:apply-templates select="reaction" mode="node">
	<xsl:with-param name="solution" select="$solution"/>
      </xsl:apply-templates>
      <xsl:variable name="FLUXES_AVAILABLE">
	<xsl:choose>
	  <xsl:when test="doc-available(concat('../',$FLUXES_FILE)) and  
			  not($FLUXES_FILE='Fluxes/empty.xml')">yes</xsl:when>
	  <xsl:otherwise>no</xsl:otherwise>
	</xsl:choose>
      </xsl:variable>

      <xsl:if test="$DEBUG='true'"> <xsl:message>
	FLUXES_FILE <xsl:copy-of select="$FLUXES_FILE"/>
	FLUXES_AVAILABLE <xsl:copy-of select="$FLUXES_AVAILABLE"/>
      </xsl:message></xsl:if>
      <xsl:if test="$FLUXES_AVAILABLE='no'">
	<xsl:apply-templates select="reaction" mode="edge"/>
      </xsl:if>
      <xsl:text xml:space="preserve">%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% edges
      %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      </xsl:text>
      <xsl:apply-templates select="edge"/>

      <xsl:text xml:space="preserve">%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% fluxes
      %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      </xsl:text>
      <xsl:apply-templates select='$fluxes//colors/*' mode="fluxes">
	<xsl:with-param name="network" select="."/>
      </xsl:apply-templates>
    </xsl:variable> <!-- end of $graph -->

    <xsl:value-of select="$graph"/> 

    <!-- rest is compilation of charts -->
    <xsl:text xml:space="preserve">%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% charts
    %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    </xsl:text>
    <!--
	<xsl:message>
	Charts
	<xsl:copy-of select="$charts"/>
	</xsl:message>
    -->
    <xsl:choose>
      <xsl:when test="doc-available(concat('../',$CHARTS_FILE))">
	<xsl:variable name="ymaxglobal">
	  <xsl:value-of select="max($charts//charts/metabolite/chart/@*[not(.='unit')])*0.35"/>
	</xsl:variable>
	<xsl:variable name="yminglobal">
	  <xsl:value-of select="min($charts//charts/metabolite/chart/@*[not(.='unit')])"/>
	</xsl:variable>
	
	<xsl:message>
	  global ymax <xsl:value-of select="$ymaxglobal"/>
	  global ymin <xsl:value-of select="$yminglobal"/>
	</xsl:message>
	
	<xsl:apply-templates select='$charts//charts/metabolite' mode="charts">
	  <xsl:with-param name="network" select="."/>
	  <xsl:with-param name="ymaxglobal" select="$ymaxglobal"/>
	  <xsl:with-param name="yminglobal" select="$yminglobal"/>
	  <xsl:with-param name="xsize" select="0.7"/>
	  <xsl:with-param name="ysize" select="0.85"/>
	  <xsl:with-param name="legend">no</xsl:with-param>
	</xsl:apply-templates>
	
	<xsl:variable name="network" select="."/>
	<xsl:message>
	  
	  Measured metabolites with charts in the model:	
	  <xsl:for-each select="$charts//charts/metabolite[@name=$network//metabolite/@id]">
	    <xsl:value-of select="@id"/>:  <xsl:value-of select="@name"/>
	    <xsl:text>
	    </xsl:text>
	  </xsl:for-each>

	  Measured metabolites outside the model:
	  <xsl:for-each select="$charts//charts/metabolite[not(@name=$network//metabolite/@id)]">
	    <xsl:value-of select="@id"/>:  <xsl:value-of select="@name"/>
	    <xsl:text>
	    </xsl:text>
	  </xsl:for-each>

	</xsl:message>

	<xsl:variable name="legend-charts">
	  <charts>
	    <xsl:copy-of select="$charts//souches"/>
	    <metabolite id="legend-chart">
	      <chart>
		<xsl:for-each select="$charts//charts/metabolite[1]/chart/@*">
		  <xsl:attribute name="{name()}">
		    <xsl:value-of select="$ymaxglobal"/>
		  </xsl:attribute>
		</xsl:for-each>
	      </chart>
	    </metabolite>
	  </charts>
	</xsl:variable>
	
	<xsl:text> % legend chart
	</xsl:text>
	
	<xsl:apply-templates select='$legend-charts//metabolite' mode="charts">
	  <xsl:with-param name="network" select="."/>
	  <xsl:with-param name="ymaxglobal" select="$ymaxglobal"/>
	  <xsl:with-param name="yminglobal" select="0"/>
	  <xsl:with-param name="xsize" select="1.4"/>
	  <xsl:with-param name="ysize" select="2.0"/>
	  <xsl:with-param name="legend">yes</xsl:with-param>
	</xsl:apply-templates>
      </xsl:when>
      <xsl:otherwise>
	<xsl:if test="$DEBUG='true'">
	<xsl:message>
	  CHARTS_FILE <xsl:copy-of select="$CHARTS_FILE"/>
	  Charts available: no
	</xsl:message></xsl:if>
      </xsl:otherwise>
    </xsl:choose>
    <!-- end of charts -->
    

  </xsl:template>
  

  <!-- function patch-name -->

  <xsl:template match="@*" mode="patch-name">
    <!--
	<xsl:message>
	patch-name <xsl:value-of select='.'/>
	</xsl:message>
    -->
    <xsl:value-of select="replace(replace(.,'\+','Plus'),'-','_')"/>
  </xsl:template>

  <!-- function variable -->
  <xsl:template match="metabolite|actor|species|protein" mode="variable"
		><xsl:apply-templates select="@id" mode="patch-name"/></xsl:template>

  
  <xsl:template match="input|output" mode="variable">
    <xsl:apply-templates select="parent::context" mode="variable"/>
  </xsl:template>


  
  <xsl:template match="boolvar" mode="variable">
    <xsl:text>b_</xsl:text>
    <xsl:apply-templates select="@id" mode="patch-name"/>
  </xsl:template>

   <xsl:template match="reaction" mode="variable">
    <xsl:param name="solution"/>
    <xsl:variable name="patch-id">
      <xsl:apply-templates select="@id"  mode="patch-name"/>
    </xsl:variable>
    <xsl:variable name="reaction-variable">
      <xsl:value-of select="concat('r_',$patch-id)"/>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$solution-variables/variables/var=$reaction-variable">
	<xsl:value-of select="$reaction-variable"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:value-of select="concat('o_',$patch-id)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

   <xsl:template match="context" mode="variable">r_<xsl:apply-templates select="@id" mode="patch-name"/></xsl:template>

  <!-- function ko-variable -->
  <xsl:template match="reaction" mode="ko-variable"
		>o_<xsl:apply-templates select="@id" mode="patch-name"/></xsl:template>

  <!-- function annotate -->
  <xsl:template match="@*" mode="annotate">
    <xsl:param name="conc"/>
    <xsl:param name="prefix"/>
    <xsl:if test="not($conc='')">
      <!--
	  <xsl:message>
	  annotate <xsl:value-of select="."/> '<xsl:value-of select="$conc"/>'
	  </xsl:message>
      -->
      <text>\node [right] at (</text>
      <xsl:value-of select="$prefix"/>
      <xsl:value-of select="."/>
      <text>.north east) {$\</text>
      <xsl:value-of select="$conc"/>
      <text>$};
      </text>
    </xsl:if>
  </xsl:template>	 

  <!-- function value-of-molecule -->
  <xsl:template match="solution" mode="value-of-molecule">
    <xsl:param name="molecule"/>
    
    <xsl:variable name="variable">
      <xsl:apply-templates select="$molecule" mode="variable"/>
    </xsl:variable>

    <xsl:variable name="value">
      <xsl:apply-templates select="." mode="value-of-variable">
	<xsl:with-param name="variable" select="$variable"/>
      </xsl:apply-templates>
    </xsl:variable>
    <!--
    <xsl:message>
      ++ value-of-molecule:
          variable=<xsl:copy-of select="$variable"/>
	  solution=<xsl:copy-of select="."/>
	  molecule=<xsl:copy-of select="$molecule"/>
	  value=<xsl:copy-of select="$value"/>
   </xsl:message>
-->
   <xsl:copy-of select="$value"/>
   
   
  </xsl:template>

  <!-- function value-of-context -->

  <xsl:template match="solution" mode="value-of-context">
    <xsl:param name="context"/>
    
    <xsl:variable name="variable">
      <xsl:apply-templates select="$context" mode="variable"/>
    </xsl:variable>

   <xsl:variable name="value">
      <xsl:apply-templates select="." mode="value-of-variable">
	<xsl:with-param name="variable" select="$variable"/>
      </xsl:apply-templates>
   </xsl:variable>

   <!--
   <xsl:message>
      ++ value-of-context:
          variable=<xsl:copy-of select="$variable"/>
	  solution=<xsl:copy-of select="."/>
	  value=<xsl:copy-of select="$value"/>
   </xsl:message>
   -->
   <xsl:copy-of select="$value"/>

 
  </xsl:template>
  <!-- function unit2zero -->

  <xsl:template match="chart" mode="unit2zero">
    <chart>
      <xsl:apply-templates select="@*" mode="unit2zero"/>
    </chart>
  </xsl:template>
  <xsl:template match="@*" mode="unit2zero">
    <xsl:attribute name="{name()}">
      <xsl:choose>
	<xsl:when test=".='unit'">0 </xsl:when>
	<xsl:otherwise>
	  <xsl:value-of select="."/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
  </xsl:template>


  <!-- function charts -->
  <xsl:template match="metabolite" mode="charts">
    <xsl:param name="network"/>
    <xsl:param name="ymaxglobal"/>
    <xsl:param name="yminglobal"/>
    <xsl:param name="xsize"/>
    <xsl:param name="ysize"/>
    <xsl:param name="legend"/>
    <!--
	<xsl:message>
	legend: <xsl:value-of select="$legend"/>
	</xsl:message>
    -->

    <xsl:variable name="chart_unit2zero">
      <xsl:apply-templates select="." mode="unit2zero"/>
    </xsl:variable>

    <!--
	<xsl:message>

chart           <xsl:copy-of select="."/>
chart_unit2zero <xsl:copy-of select="$chart_unit2zero"/>

</xsl:message>
    -->
    <xsl:variable name="ymaxlocal">
      <xsl:value-of select="max(($chart_unit2zero//chart/@*,0))"/>
    </xsl:variable>
    <xsl:variable name="yminlocal">
      <xsl:value-of select="min(($chart_unit2zero//chart/@*,0))"/>
    </xsl:variable>

    <!--
	<xsl:message>
	metabolite: <xsl:value-of select="@id"/>
	local ymax: <xsl:value-of select="$ymaxlocal"/>
	local ymin: <xsl:value-of select="$yminlocal"/>
	</xsl:message>
    -->
    
    <xsl:variable name="ymax">
      <xsl:choose>
	<xsl:when test="$GLOBAL_SCALE='no' and not($yminlocal=0 and $ymaxlocal=0)">
	  <xsl:value-of select="$ymaxlocal"/>
	</xsl:when>
	<xsl:when test="$GLOBAL_SCALE='no' and $yminlocal=0 and $ymaxlocal=0">
	  <xsl:value-of select="1"/>
	</xsl:when>
	
	<xsl:otherwise>
	  <xsl:value-of select="$ymaxglobal"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="ymin">
      <xsl:choose>
	<xsl:when test="$GLOBAL_SCALE='no' and not($yminlocal=$ymaxlocal)">
	  <xsl:value-of select="$yminlocal"/>
	</xsl:when>
	<xsl:when test="$GLOBAL_SCALE='no' and $yminlocal=$ymaxlocal">
	  <xsl:value-of select="0"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:value-of select="$yminglobal"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>


    <xsl:message>
      metabolite:  <xsl:value-of select="@id"/>
      ymax: <xsl:value-of select="$ymax"/>
      ymin: <xsl:value-of select="$ymin"/>
    </xsl:message>

    <xsl:variable name="self" select="."/>
    <xsl:variable name="metabolite">
      <xsl:choose>
	<xsl:when test="not(@id='legend-chart')">
	  <xsl:copy-of select="$network//metabolite[@id=$self/@name]"/>
	</xsl:when>
	<xsl:otherwise>
	  <metabolite x="20" y="0"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="souches">
      <xsl:copy-of select="preceding-sibling::souches"/>
    </xsl:variable>
    
    <xsl:text>\tikzmath{
    % parameters
    </xsl:text>
    <xsl:text>\ymin=</xsl:text>
    <xsl:value-of select="$ymin"/>
    <xsl:text>;         % minimal value on y-axis
    \ymax=</xsl:text>
    <xsl:value-of select="$ymax"/>
    <xsl:text>;         % maximal value on y-axis
    \saulenzahl=</xsl:text>
    <!--
	<xsl:message>
	saulenzahl <xsl:copy-of select="count(chart/@*)"/>
	</xsl:message>
    -->
    <xsl:copy-of select="count(chart/@*)"/>
    <xsl:text>; % number of columns
    </xsl:text> 
    <!-- are to be fixed by the parameters of the chart? --> 
    <!--
	<xsl:message>
	Metabolite <xsl:value-of select="@id"/>:
	<xsl:copy-of select="$metabolite"/>
	x: <xsl:value-of select="$metabolite//metabolite/@x"/>
	</xsl:message>
    -->
    <xsl:text>    \ysize=</xsl:text>
    <xsl:value-of select="$ysize"/>
    <xsl:text>;          % size of y-axis 
    \xsize=</xsl:text>
    <xsl:value-of select="$xsize"/>
    <xsl:text>;          % size of x-axis = maximal value on x-axis
    \xorigin=</xsl:text>
    <xsl:value-of select="$metabolite//metabolite/@x"/>
    <xsl:text>-1.0;      % x-coordinate of origin (left down corner)
    \yorigin=</xsl:text>
    <xsl:value-of select="$metabolite//metabolite/@y"/>    
    <xsl:text> + 0.2;    % y-coordinate of origin (left down corner)    
    % 
    % derived values
    \ydelta = \ymax-\ymin; 
    \yfactor= \ysize/\ydelta ;
    \yfacmax=\ymax*\yfactor;
    \yfacmin=\ymin*\yfactor;
    \yfacsize=\yfacmax-\yfacmin;
    \yfacdelta=\ydelta*\yfactor;   
    \saulenbreite = \xsize/\saulenzahl;
    \xzero = \xorigin;
    \yzero = \yorigin - \yfacmin;
    \xaxisstart= \xzero - \xsize * 0.1;
    \xaxisend  = \xzero + \xsize * 1.1;
    \yaxisend = \yorigin + \yfacsize;
    }
    
    \draw (\xaxisstart cm, \yzero cm) -- (\xaxisend   cm, \yzero cm);  % x-Axis
    \draw (\xzero cm,   \yorigin  cm) -- (\xzero cm,   \yaxisend cm);  % y-Axis
    
    % auxillary lines parallel to x-Axis
    \foreach \ind in {0,...,4}  {
    \tikzmath{
    \yrename = \ind/4 * \yfacdelta ;
    \yval=\ymin+\ind/4*\ydelta;
    \yhead=int(\yval);
    \yrest=int(abs(round(\yval*10-\yhead*10)));
    }
    \draw[gray!50, text=black] (\xorigin cm, \yorigin cm + \yrename cm) --
    (\xorigin cm + \xsize cm, \yorigin cm +\yrename cm)
    </xsl:text>
    <!--      <xsl:if test="$legend='yes'"> -->
    <xsl:text>node[left] at (\xorigin cm,\yorigin cm + \yrename cm) {\tiny \yhead.\yrest}  % Values of auxillary lines
    %	 </xsl:text>
    <!--       </xsl:if> -->
    <xsl:value-of select="@id"/>
    <xsl:text>
      ;};
      
      \foreach \x/\y/\souche/\col in {                 %\x ist x-Start der Säulen
    </xsl:text>
    <!--
	0/19.44//myblue,             %\y ist y-Value der Säulen
	1/-2.265/bcd/red,           % \souche ist Säulenbeschriftung 
	2/-0.9717/codY/violet,   % \col ist Säulenfarbe
	3/-2.6817/lpdv/green,
	4/-0.275/tnra/yellow,
	5/1.5/yvwb/orange
    -->
    <xsl:apply-templates select="$souches//souche" mode="charts">
      <xsl:with-param name="metabolite" select="."/>
    </xsl:apply-templates>
    <xsl:text>}
    {
    \tikzmath{\yy=\yfactor*\y;}
    \draw[fill=\col] (\xzero cm + \x*\saulenbreite cm,\yzero cm) 
    rectangle ++(\saulenbreite cm,\yy cm) %die Säulen
    </xsl:text>
    <xsl:if test="$legend='yes'"> 
      <xsl:text>node[rotate=110, left] at (\xorigin cm+ 0.4*\saulenbreite cm
      +\x*\saulenbreite cm,\yorigin cm+\yfacmin cm) {\tiny\souche} %Säulenbeschriftung
      </xsl:text>
    </xsl:if>
    <xsl:text>;};
    </xsl:text>
  </xsl:template>

  <xsl:template match="souche" mode="charts">
    <xsl:param name="metabolite"/>
    <xsl:variable name="self">
      <xsl:copy-of select="."/>
    </xsl:variable>
    <!--
	<xsl:message>
	souche:
	<xsl:copy-of select="."/>
	</xsl:message>
    -->
    <xsl:copy-of select="position()-1"/>
    <xsl:text>/</xsl:text>
    <!--      <xsl:message>
	 id <xsl:value-of select="@id"/>
	 chart names <xsl:copy-of select="$metabolite//chart/@*/name()"/>
	 self         <xsl:copy-of select="$self//souche/@id"/>
	 </xsl:message>
    -->
    <xsl:variable name="value" select="$metabolite//chart/@*[name()=$self//souche/@id]"/>
    <xsl:choose>
      <xsl:when test="$value='unit'">0</xsl:when>
      <xsl:otherwise> <xsl:value-of select="$value"/> </xsl:otherwise>
    </xsl:choose>
    <xsl:text>/</xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:text>: </xsl:text>

    <xsl:value-of select="@id"/>
    <xsl:text>/</xsl:text>
    <xsl:value-of select="@color"/>
    <xsl:if test="following-sibling::souche">
      <xsl:text>,
      </xsl:text>
    </xsl:if>
  </xsl:template>
  
  <!-- function fluxes -->
  
  <xsl:template match="reaction" mode="fluxes">
    <xsl:param name="network"/>
    <xsl:variable name="number">
      <xsl:value-of select="count(*)"/>
    </xsl:variable>
    
    <xsl:variable name="thickness">
      <xsl:value-of select="@thickness"/>
    </xsl:variable>
    
    <xsl:variable name="shift">
      <xsl:value-of select="($number*$thickness) div 2"/>
    </xsl:variable>
    
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    
    <xsl:for-each select="color">
      <xsl:apply-templates select="." mode="annotate-reaction">
	<xsl:with-param name="thickness">
	  <xsl:value-of select="$thickness"/>
	</xsl:with-param>
	<xsl:with-param name="number">
	  <xsl:value-of select="$number"/>
	</xsl:with-param>
	<xsl:with-param name="shift">
	  <xsl:value-of select="(position() * $thickness div 2 + $thickness div 2) + (1 - $number mod 2) * $thickness div 4 - $thickness*$number div 2 "/>
	</xsl:with-param>
	<xsl:with-param name="id">
	  <xsl:value-of select="$id"/>
	</xsl:with-param>
	<xsl:with-param name="reactions">
	  <xsl:copy-of select="$network/reaction"/>
	</xsl:with-param>
	<xsl:with-param name="context">
	  <xsl:copy-of select="$network/context"/>
	</xsl:with-param>
	<xsl:with-param name="edgeclusters">
	  <xsl:copy-of select="$network/edgecluster"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="*" mode="annotate-reaction">
    <xsl:param name="thickness"/>
    <xsl:param name="number"/>
    <xsl:param name="shift"/>
    <xsl:param name="id"/>
    <xsl:param name="reactions"/>
    <xsl:param name="context"/>
    <xsl:param name="edgeclusters"/>
    <xsl:variable name="color" select="@id"/>
    
    <!-- case : product spec -->
    <xsl:for-each select="$reactions/*[@id=$id and product]/product[@spec]">
      <text xml:space="preserve">\path [</text>
      <xsl:choose>
	<xsl:when test="parent::reaction/@reversible">
	  <text>rev</text>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:text>-></xsl:text>
	</xsl:otherwise>
      </xsl:choose>
      <text>, line width=</text><xsl:value-of select="$thickness"/><text>pt, </text><xsl:value-of select="$color"/><text>] (r_</text><xsl:value-of select="$id"/><text>) edge[shift left=</text><xsl:value-of select="$shift"/><text>mm] (</text><xsl:value-of select="@spec"/><text xml:space="preserve">);
    </text>
    </xsl:for-each>
    
    <!-- case : product edgecluster -->
    <xsl:for-each select="$reactions/*[@id=$id and product]/product[@edgecluster]">
      <xsl:apply-templates select="@edgecluster" mode="annotate-edgecluster">
	<xsl:with-param name="thickness">
	  <xsl:value-of select="$thickness"/>
	</xsl:with-param>
	<xsl:with-param name="number">
	  <xsl:value-of select="$number"/>
	</xsl:with-param>
	<xsl:with-param name="shift">
	  <xsl:value-of select="$shift"/>
	</xsl:with-param>
	<xsl:with-param name="id">
	  <xsl:value-of select="$id"/>
	</xsl:with-param>
	<xsl:with-param name="reactions">
	  <xsl:value-of select="$reactions"/>
	</xsl:with-param>
	<xsl:with-param name="context">
	  <xsl:value-of select="$context"/>
	</xsl:with-param>
	<xsl:with-param name="edgeclusters">
	  <xsl:value-of select="$edgeclusters"/>
	</xsl:with-param>
	<xsl:with-param name="color">
	  <xsl:value-of select="$color"/>
	</xsl:with-param>
	<xsl:with-param name="cluster">
	  <xsl:value-of select="."/>
	</xsl:with-param>
      </xsl:apply-templates>
    </xsl:for-each>
    
    <!-- case : reactant -->
    <xsl:for-each select="$reactions/*[@id=$id and reactant]/reactant[@spec]"> 
      <text xml:space="preserve">\path [</text>
      <xsl:choose>
	<xsl:when test="parent::reaction/@reversible">
	  <text>rev</text>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:text>-></xsl:text>
	</xsl:otherwise>
      </xsl:choose>
      
      <text>, line width=</text><xsl:value-of select="$thickness"/><text>pt, </text><xsl:value-of select="$color"/><text>] (</text><xsl:value-of select="@spec"/><text>) edge[shift left=</text><xsl:value-of select="$shift"/><text>mm] (r_</text><xsl:value-of select="$id"/><text xml:space="preserve">);      </text> 
    </xsl:for-each>	
    
    <!-- case : reactant edgecluster -->
    <xsl:for-each select="$reactions/*[@id=$id and reactant]/reactant/@edgecluster">
      <xsl:variable name="cluster" select="."/>
      <text xml:space="preserve">\path [->, line width=</text><xsl:value-of select="$thickness"/><text>pt, </text><xsl:value-of select="$color"/><text>] (</text><xsl:value-of select="$edgeclusters/*[@id=$cluster]/@x"/><text>, </text><xsl:value-of select="$edgeclusters/*[@id=$cluster]/@y"/><text>) edge[shift left=</text><xsl:value-of select="$shift"/><text>mm] (</text><text>r_</text><xsl:value-of select="$id"/><text xml:space="preserve">);
    </text>
    <xsl:message> EDGECLUSTER <xsl:value-of select="$edgeclusters/*[@id=$cluster]/@id"/></xsl:message>

    </xsl:for-each>
    
    <!-- case : product-inh -->	
    <!-- <xsl:for-each select="$reactions/*[@id=$id and product-inh]/product-inh/@spec|@edgecluster">
	 <xsl:message>Warning product-inh in fluxes : <xsl:value-of select="$id"/> </xsl:message> 
	 </xsl:for-each>	-->
    
  </xsl:template>

  
  <xsl:template match="context" mode="fluxes">
    <xsl:param name="network"/>
    <xsl:variable name="number">
      <xsl:value-of select="count(*)"/>
    </xsl:variable>
    
    <xsl:variable name="thickness">
      <xsl:value-of select="@thickness"/>
    </xsl:variable>
    
    <xsl:variable name="shift">
      <xsl:value-of select="($number*$thickness) div 2"/>
    </xsl:variable>
    
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    
    <xsl:for-each select="color">
      <xsl:apply-templates select="." mode="annotate-context">
	<xsl:with-param name="thickness">
	  <xsl:value-of select="$thickness"/>
	</xsl:with-param>
	<xsl:with-param name="number">
	  <xsl:value-of select="$number"/>
	</xsl:with-param>
	<xsl:with-param name="shift">
	  <xsl:value-of select="(position() * $thickness div 2 + $thickness div 2) + (1 - $number mod 2) * $thickness div 4 - $thickness*$number div 2 "/>
	</xsl:with-param>
	<xsl:with-param name="id">
	  <xsl:value-of select="$id"/>
	</xsl:with-param>
	<xsl:with-param name="context">
	  <xsl:copy-of select="$network/context"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="*" mode="annotate-context">
    <xsl:param name="thickness"/>
    <xsl:param name="number"/>
    <xsl:param name="shift"/>
    <xsl:param name="id"/>
    <xsl:param name="context"/>
    <xsl:param name="edgeclusters"/>
    <xsl:param name="cluster"/>
    <xsl:variable name="color" select="@id"/>
    
    <!-- case : input -->
    <xsl:for-each select="$context/*[@id=$id and input]/input">
      <text xml:space="preserve">\path [->, line width=</text><xsl:value-of select="$thickness"/><text>pt, </text><xsl:value-of select    ="$color"/><text>] (<xsl:value-of select="$context/*[@id=$id]/@x"/><text>,</text><xsl:value-of select="$context/*[@id=$id]/@y"/></text><text>) edge[shift left=</text><xsl:value-of select="$shift"/><text>mm] (</text><xsl:value-of select="@spec"/><text xml:space="preserve">);
    </text>
    </xsl:for-each>
    
    <xsl:for-each select="$context/*[@id=$id and output]/output">
      <text xml:space="preserve">\path [->, line width=</text><xsl:value-of select="$thickness"/><text>pt, </text><xsl:value-of select    ="$color"/><text>] (</text><xsl:value-of select="@spec"/><text>) edge[shift left=</text><xsl:value-of select="$shift"/><text>mm] (<xsl:value-of select="$context/*[@id=$id]/@x"/><text>,</text><xsl:value-of select="$context/*[@id=$id]/@y"/></text><text xml:space="preserve">);
    </text>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="*" mode="annotate-edgecluster">
    <xsl:param name="thickness"/>
    <xsl:param name="number"/>
    <xsl:param name="shift"/>
    <xsl:param name="id"/>
    <xsl:param name="context"/>
    <xsl:param name="edgeclusters"/>
    <xsl:param name="cluster"/>
    <xsl:param name="color"/>
    <text xml:space="preserve">\path [->, line width=</text><xsl:value-of select="$thickness"/><text>pt, </text><xsl:value-of select="$color"/><text>] (r_</text><xsl:value-of select="$id"/><text>) edge[shift left=</text><xsl:value-of select="$shift"/><text>mm] (</text><xsl:value-of select="$edgeclusters/*[@id=$cluster]/@x"/><text>, </text><xsl:value-of select="$edgeclusters/*[@id=$cluster]/@y"/><text xml:space="preserve">);
  </text>
  <xsl:message> EDGECLUSTER Prod <xsl:value-of select="$edgeclusters/*[@id=$cluster]/@id"/></xsl:message>
  </xsl:template>
  
  <!-- function value-of-knockout -->
  
  <xsl:template match="solution" mode="value-of-knockout">
    <xsl:param name="reaction"/>
    <xsl:variable name="variable">
      <xsl:apply-templates select="$reaction" mode="ko-variable"/>
    </xsl:variable>

    <xsl:variable name="value">
      <xsl:apply-templates select="." mode="value-of-variable">
	<xsl:with-param name="variable" select="$variable"/>
      </xsl:apply-templates>
    </xsl:variable>

    <!--
    <xsl:message>
    +++++ value-of-knockout:
          variable=<xsl:copy-of select="$variable"/>
          solution=<xsl:copy-of select="."/>
	  value=<xsl:copy-of select="$value"/>
    </xsl:message>
    -->
    <xsl:copy-of select="$value"/>
  </xsl:template>
  
  <!-- function value-of-variable -->
  
  <xsl:template match="solution" mode="value-of-variable">
    <xsl:param name="variable"/>
    
    <xsl:variable name="value">
      <xsl:choose>
	<xsl:when test="attribute::*[name()=$variable]">
	  <xsl:value-of select="attribute::*[name()=$variable]"/>
	</xsl:when>
	<xsl:when test="ancestor::results/variables/var
			[@kind='global']=$variable">No</xsl:when>
      </xsl:choose>
    </xsl:variable>

    <!-- debug
	 <xsl:message>
	 latex-graph
	 variable <xsl:copy-of select="$variable"/>
	 value    <xsl:copy-of select="$value"/>
	 </xsl:message>
    -->
    <xsl:copy-of select="$value"/>
    
  </xsl:template>
  
  <xsl:template match="metabolite|protein|actor|species" mode="compile">
    <xsl:param name="solution"/>

    <xsl:variable name="tag">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>

    <xsl:variable name="conc">
      <xsl:apply-templates select="$solution" mode="value-of-molecule">
	<xsl:with-param name="molecule" select="."/>
      </xsl:apply-templates>
    </xsl:variable>

    <xsl:variable name="style">
      <xsl:value-of select="$tag"/>
      <xsl:if test="@cluster">-cluster</xsl:if> 
      <xsl:if test="@essential">-essential</xsl:if> 
    </xsl:variable>

    <!-- grey backround of main node if there are copies -->
    <xsl:if test="copy">
      <text xml:space="preserve">\node (fake--</text>
      <xsl:value-of select="@id"/>
      <text>)   at (</text>
      <xsl:value-of select="@x"/>
      <text>,</text>
      <xsl:value-of select="@y"/>
      <text>) [gn] {};
      </text>
    </xsl:if>

    <!-- main node -->
    <text xml:space="preserve">\node (</text>
    <xsl:value-of select="@id"/>
    <text>)   at (</text>
    <xsl:value-of select="@x"/>
    <text>,</text>
    <xsl:value-of select="@y"/>
    <text>) [</text>
    <xsl:value-of select="$style"/>

<text xml:space="preserve">, label={[label distance=-0.1cm]</text>
    <xsl:value-of select="kinetics/@angle"/>
    <text xml:space="preserve">:\idk{</text>
    <xsl:if test="    not(kinetics/@id='exp') 
		  and not(kinetics/@id='ma')
		  and not(kinetics/@id='deac')">
      <xsl:value-of select="kinetics/@id"/>
      <xsl:value-of select="kinetics/@expr"/>
    </xsl:if>
    <xsl:if test="kinetics/@id='ma'">
      <text>ma_{</text>
      <xsl:value-of select="kinetics/@para"/>
      <text>}</text>
    </xsl:if>
    <text xml:space="preserve">}} </text>


    
    <text>] {\idg{</text>
    <xsl:value-of select="@id"/>
    <text>}};
    </text>
    <xsl:apply-templates select="@id" mode="annotate">
      <xsl:with-param name="conc" select="$conc"/>
    </xsl:apply-templates>


    <!-- splitpoint -->
    <xsl:if test="splitpoint">
      <text xml:space="preserve">\node (splitpoint--</text>
      <xsl:value-of select="@id"/>
      <text>)   at (</text>
      <xsl:value-of select="splitpoint/@x"/>
      <text>,</text>
      <xsl:value-of select="splitpoint/@y"/>
      <text>) [gc] {};
      </text> 
      <!-- connect splitpoint to main species node -->
      <text xml:space="preserve">\path (splitpoint--</text>
      <xsl:value-of select="@id"/>
      <text>) to[circle connection bar] (fake--</text>
      <xsl:value-of select="@id"/>
      <text>);     
      </text>
    </xsl:if>

    <!-- copies --> 
    <xsl:for-each select="copy">

      <text xml:space="preserve">\node (copy--</text>
      <xsl:value-of select="parent::*/@id"/>
      <text>--</text>
      <xsl:value-of select="@id"/>
      <text>)  at (</text>
      <xsl:value-of select="@x"/>
      <text>,</text>
      <xsl:value-of select="@y"/>
      <text>) [gn] {};
      </text>    
      <text xml:space="preserve">\node (copy--</text>
      <xsl:value-of select="parent::*/@id"/>
      <text>--</text>
      <xsl:value-of select="@id"/>
      <text>)  at (</text>
      <xsl:value-of select="@x"/>
      <text>,</text>
      <xsl:value-of select="@y"/>
      <text>) [</text>
      <xsl:value-of select="$style"/>
      <text>-copy] {\idgnotarget{</text>
      <xsl:value-of select="parent::*/@id"/>
      <text>}};
      </text>

      <!--annotate copy -->
      <xsl:if test="not($conc='')">
	<text>
	\node [right] at (copy--</text>
	<xsl:value-of select="parent::*/@id"/>--<xsl:value-of select="@id"/>
	<text>.north east) {$\</text>
	<xsl:value-of select="$conc"/>
	<text>$};
	</text>
      </xsl:if>
      
      <!-- connect copy to splitpoint -->
      <text>\path (splitpoint--</text>
      <xsl:value-of select="parent::*/@id"/>
      <text>) to[circle connection bar] (copy--</text>
      <xsl:value-of select="parent::*/@id"/>
      <text>--</text>
      <xsl:value-of select="@id"/>
      <text>);
      </text> 
    </xsl:for-each>

  </xsl:template>


  <!-- edgecluster points -->
  <xsl:template match="edgecluster" mode="node">
    <text xml:space="preserve">\node (edgecluster--<xsl:value-of select="@id"/>)   at (<xsl:value-of select="@x"/>,<xsl:value-of select="@y"/>) [cpt] {};
    </text> 
  </xsl:template>

  <xsl:template match="edgecluster" mode="edge">
    <xsl:variable name="edgetype">
      <xsl:choose>
	<xsl:when test="@type='activator'">cid</xsl:when>
	<xsl:when test="@type='accelerator'">cid</xsl:when>
	<xsl:when test="@type='inhibitor'">cid</xsl:when>
	<xsl:when test="@type='modifier'">mod</xsl:when>
	<xsl:when test="@type='product'">i</xsl:when>
	<xsl:when test="@type='reactant'">ci</xsl:when>
	<xsl:when test="@type='product-inh'">io</xsl:when>
      </xsl:choose>
    </xsl:variable>

    <xsl:for-each select="source">
      <xsl:for-each select=".[@spec and not(@copy)]">
	<text xml:space="preserve">\draw [<xsl:value-of select="$edgetype"/>] (<xsl:value-of select="@spec"/>) -- (edgecluster--<xsl:value-of select="parent::*/@id"/>) {};
	</text>
      </xsl:for-each>
      <xsl:for-each select=".[@spec and @copy]">
	<text xml:space="preserve">\draw [<xsl:value-of select="$edgetype"/>] (copy--<xsl:value-of select="@spec"/>--<xsl:value-of select="@copy"/>) -- (edgecluster--<xsl:value-of select="parent::*/@id"/>) {};
	</text>
      </xsl:for-each>

      <xsl:for-each select=".[@edgecluster]">
	<text xml:space="preserve">\draw [<xsl:value-of select="$edgetype"/>] (edgecluster--<xsl:value-of select="@edgecluster"/>) -- (edgecluster--<xsl:value-of select="parent::*/@id"/>) {};
	</text>
      </xsl:for-each>
      
    </xsl:for-each>
  </xsl:template>


  <xsl:template match="edge">
    <xsl:variable name="edgetype"> transition  <xsl:if test="@curvedepth"> , curvedepth=<xsl:value-of select="@curvedepth"/>
  </xsl:if>
    </xsl:variable>
    
    <text xml:space="preserve">\draw [<xsl:value-of select="$edgetype"/>] (<xsl:value-of select="@from"/>)--(<xsl:value-of select="@to"/>) {<xsl:value-of select="@annotation"/>};
    </text>

  </xsl:template>

  <!-- compile reaction to graph node -->
  <xsl:template match="reaction" mode="node">
    <xsl:param name="solution"/>

    <xsl:variable name="variable">
      <xsl:apply-templates select="." mode="variable">
      <xsl:with-param name="solution">
	<xsl:copy-of select="$solution"/>
      </xsl:with-param></xsl:apply-templates>
    </xsl:variable>
    
    <xsl:variable name="speed">
      <xsl:choose>
	<xsl:when test="$SOLUTIONS_AVAILABLE='yes'">
	  <xsl:choose>
	    <xsl:when test="$solution/(child::solution|self::*)/
			    @*[name()=$variable]">
	      <xsl:value-of select="($solution/(child::solution |self::*)/@*
				    [name()=$variable])"/>
	    </xsl:when>
	    <xsl:when test="$solution-variables/variables/var=$variable">
	      <xsl:text>No</xsl:text>
	    </xsl:when>
	  </xsl:choose>	    
	</xsl:when>
	<xsl:when test="$KNOCKOUTS_AVAILABLE='yes'">
	  <xsl:apply-templates select="." mode="diffs"/>
	</xsl:when>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$DEBUG='true'">
    <xsl:if test="$SOLUTIONS_AVAILABLE='yes'">
      <xsl:message> compile-node changes solutions available
      - reaction id= <xsl:value-of select="@id"/>
      - variable=<xsl:value-of select="$variable"/>
      - speed=<xsl:copy-of select="$speed"/>
      - solution=<xsl:copy-of select="$solution"/>
     </xsl:message>
    </xsl:if> 
    
    <xsl:if test="not($SOLUTIONS_AVAILABLE='yes') and $KNOCKOUTS_AVAILABLE='yes'">
      <xsl:message>
      compile-node changes 
      -- reaction <xsl:value-of select="@id"/>
      -- speed   <xsl:copy-of select="$speed"/>
     </xsl:message></xsl:if>
    </xsl:if>
    
    <xsl:variable name="knockout">
      <xsl:apply-templates select="$solution" mode="value-of-knockout">
	<xsl:with-param name="reaction" select="."/>
      </xsl:apply-templates>
    </xsl:variable>

    <xsl:variable name="type">
      <xsl:choose>
	<xsl:when test=".[@id=$safe-ko//single/knockout/@reaction]">
	  <text>rct-kops</text>
	</xsl:when>
	<xsl:when test=".[   @id=$ko/changes/single/*/@reaction
			  or @id=$ko/changes/changes/change/@reaction
			  or @id=$ko/changes/changes/knockout/@reaction
			]
			or $knockout='DoN' or $knockout='Up'">
	  <text>rct-kop</text>
	</xsl:when>
	<xsl:when test=".[@candidate]">
	  <text>rct-ko</text>
	</xsl:when>
	<xsl:otherwise>
	  <text>rct</text>
	</xsl:otherwise>
      </xsl:choose>
      <xsl:if test=".[@reversible]">
	<text>,reversible</text>
      </xsl:if>
    </xsl:variable>
    
    <text xml:space="preserve">\node (r_</text>
    <xsl:value-of select="@id"/>
    <text xml:space="preserve">) at (<xsl:value-of select="@x"/>,<xsl:value-of select="@y"/>) [</text>
    <xsl:value-of select="$type"/>
    <text xml:space="preserve">, label={[label distance=-0.1cm]</text>
    <xsl:value-of select="kinetics/@angle"/>
    <text xml:space="preserve">:\idk{</text>
    <xsl:if test="    not(kinetics/@id='exp') 
		  and not(kinetics/@id='ma')
		  and not(kinetics/@id='deac')">
      <xsl:value-of select="kinetics/@id"/>
      <xsl:value-of select="kinetics/@expr"/>
    </xsl:if>
    <xsl:if test="kinetics/@id='ma'">
      <text>ma_{</text>
      <xsl:value-of select="kinetics/@para"/>
      <text>}</text>
    </xsl:if>
    <text xml:space="preserve">}}] {\idR{<xsl:value-of select="@id"/>}};
    </text>

    <xsl:apply-templates select="@id" mode="annotate">
      <xsl:with-param name="conc" select="$speed"/>
      <xsl:with-param name="prefix" select="'r_'"/>
    </xsl:apply-templates>

  </xsl:template>

  <xsl:template match="*" mode="annotate-stoichiometry">
    <xsl:param name="sign"/>
    <xsl:if test="@stoichiometry and not(@stoichiometry=$sign*1)">
      <text> node [rotate=0,midway,above] {$_</text>
      <xsl:copy-of select="concat('{',$sign*@stoichiometry,'}')"/>
      <text>$}</text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="reaction" mode="edge">

    <xsl:for-each select="activator[@spec and not(@copy)]">
      <text xml:space="preserve">\draw [act] (<xsl:value-of select="@spec"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>
    <xsl:for-each select="activator[@spec and @copy]">
      <text xml:space="preserve">\draw [act] (copy--<xsl:value-of select="@spec"/>--<xsl:value-of select="@copy"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>
    <xsl:for-each select="activator[@edgecluster]">
      <text xml:space="preserve">\draw [act] (edgecluster--<xsl:value-of select="@edgecluster"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>

    <xsl:for-each select="accelerator[@spec and not(@copy)]">
      <text xml:space="preserve">\draw [acc] (<xsl:value-of select="@spec"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>
    <xsl:for-each select="accelerator[@spec and @copy]">
      <text xml:space="preserve">\draw [acc] (copy--<xsl:value-of select="@spec"/>--<xsl:value-of select="@copy"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>
    <xsl:for-each select="accelerator[@edgecluster]">
      <text xml:space="preserve">\draw [acc] (edgecluster--<xsl:value-of select="@edgecluster"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>

    <xsl:for-each select="reactant[@spec]">
      <text xml:space="preserve">\draw [</text>
      <xsl:choose>
	<xsl:when test="parent::reaction/@reversible">
	  <text>rev</text>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:text>io</xsl:text>
	</xsl:otherwise>
      </xsl:choose>
      <text>] (</text>
      <xsl:if test="@copy">
	<text>copy--</text>
      </xsl:if>
      <xsl:value-of select="@spec"/>
      <xsl:if test="@copy">
	<text>--</text>
	<xsl:value-of select="@copy"/>
      </xsl:if>
      <text>) -- (r_<xsl:value-of select="parent::*/@id"/>) </text>
      <xsl:apply-templates select="." mode="annotate-stoichiometry">
	<xsl:with-param name="sign" select="-1" />
      </xsl:apply-templates>
      <text xml:space="preserve">;
      </text>
    </xsl:for-each>

    <xsl:for-each select="reactant[@edgecluster]">
      <text xml:space="preserve">\draw [io] (edgecluster--<xsl:value-of select="@edgecluster"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>

    <xsl:for-each select="inhibitor[@spec and not(@copy)]">
      <text xml:space="preserve">\draw [inh] (<xsl:value-of select="@spec"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>
    <xsl:for-each select="inhibitor[@spec and @copy]">
      <text xml:space="preserve">\draw [inh] (copy--<xsl:value-of select="@spec"/>--<xsl:value-of select="@copy"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>
    <xsl:for-each select="inhibitor[@edgecluster]">
      <text xml:space="preserve">\draw [inh] (edgecluster--<xsl:value-of select="@edgecluster"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>

    <xsl:for-each select="modifier[@spec and not(@copy)]">
      <text xml:space="preserve">\draw [mod] (<xsl:value-of select="@spec"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>
    <xsl:for-each select="modifier[@spec and @copy]">
      <text xml:space="preserve">\draw [mod] (copy--<xsl:value-of select="@spec"/>--<xsl:value-of select="@copy"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>
    <xsl:for-each select="modifier[@edgecluster]">
      <text xml:space="preserve">\draw [mod] (edgecluster--<xsl:value-of select="@edgecluster"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) {};
      </text>
    </xsl:for-each>

    <xsl:for-each select="product[@spec]">
      <text xml:space="preserve">\draw [</text>
      <xsl:choose>
	<xsl:when test="parent::reaction/@reversible">
	  <text>rev</text>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:text>io</xsl:text>
	</xsl:otherwise>
      </xsl:choose>
      <text>]  (r_<xsl:value-of select="parent::*/@id"/>) -- (</text>
      <xsl:if test="@copy">
	<text>copy--</text>
      </xsl:if>
      <xsl:value-of select="@spec"/>
      <xsl:if test="@copy">
	<text>--</text>
	<xsl:value-of select="@copy"/>
      </xsl:if>
      <text>)</text>
      <xsl:apply-templates select="." mode="annotate-stoichiometry">
	<xsl:with-param name="sign" select="1" />
      </xsl:apply-templates>
      <text>;
      </text>
    </xsl:for-each>


    <xsl:for-each select="product[@edgecluster]">
      <text xml:space="preserve">\draw [ci]  (r_<xsl:value-of select="parent::*/@id"/>) -- (edgecluster--<xsl:value-of select="@edgecluster"/>);
      </text>
    </xsl:for-each>

    <xsl:for-each select="product-inh[@spec and not(@copy)]">
      <text xml:space="preserve">\draw [oinh] (r_<xsl:value-of select="parent::*/@id"/>) -- (<xsl:value-of select="@spec"/>) </text>
      <xsl:apply-templates select="." mode="annotate-stoichiometry">
	<xsl:with-param name="sign" select="1" />
      </xsl:apply-templates>
      <text>;
      </text>
    </xsl:for-each>
    <xsl:for-each select="product-inh[@spec and @copy]">
      <text xml:space="preserve">\draw [oinh]  (r_<xsl:value-of select="parent::*/@id"/>) --  (copy--<xsl:value-of select="@spec"/>--<xsl:value-of select="@copy"/>) </text>
      <xsl:if test="@stoichiometry and not(@stoichiometry=1)">
	<text>node [rotate=0,above,midway] {$_</text>
	<xsl:copy-of select="concat(@stoichiometry,'')"/>
	<text>$}</text>
      </xsl:if>
      <text>;
      </text>
    </xsl:for-each>
    <xsl:for-each select="product-inh[@edgecluster]">
      <text xml:space="preserve">\draw [oinh] (r_<xsl:value-of select="parent::*/@id"/>) -- (edgecluster--<xsl:value-of select="@edgecluster"/>)  {};
      </text>
    </xsl:for-each>
  </xsl:template>

  <!-- inputs and outputs-->

  <xsl:template match="context-node" mode="node">
    <text xml:space="preserve">\node (r_<xsl:value-of select="@id"/>) at (<xsl:value-of select="@x"/>,<xsl:value-of select="@y"/>) [inout, label={[label distance=-0.1cm]-30:\idk{}}] {\IOR{<xsl:value-of select="@id"/>}};
    </text>
  </xsl:template>

  
  
  <xsl:template match="context" mode="node">
    <xsl:param name="solution"/>
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    <xsl:variable name="var">
      <xsl:value-of select="concat('r_',@id)"/>
    </xsl:variable>
    <xsl:variable name="type">
      <xsl:choose>
	<xsl:when test="$SOLUTIONS_AVAILABLE='yes'">
	  <xsl:choose>
	    <xsl:when test="input and 
			    $solution/(self::*|child::solution)
			    [@*[name()=$var]='Up' or @*[name()=$var]='Do']">
	      <xsl:text>inout-change</xsl:text>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:text>inout</xsl:text>
	    </xsl:otherwise>
	  </xsl:choose>
	</xsl:when>
	<xsl:when test="$KNOCKOUTS_AVAILABLE='yes'">
	  <xsl:choose>
	   <xsl:when test="input and 
			   $ko/changes/single/speed[@reaction=$id]">
	      <xsl:text>inout-change</xsl:text>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:text>inout</xsl:text>
	    </xsl:otherwise>
	  </xsl:choose>
	</xsl:when>
	<xsl:otherwise>
	      <xsl:text>inout</xsl:text>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

      <xsl:variable name="speed">
      <xsl:choose>
	<xsl:when test="$SOLUTIONS_AVAILABLE='yes'">
	  <xsl:choose>
	    <xsl:when test="$solution/(child::solution|self::*)/
			    @*[name()=$var]">
	      <xsl:value-of select="($solution/(child::solution |self::*)/@*
				    [name()=$var])"/>
	    </xsl:when>
	    <xsl:when test="$solution-variables/variables/var=$var">
	      <xsl:text>No</xsl:text>
	    </xsl:when>
	  </xsl:choose>	    
	</xsl:when>
	<xsl:when test="$KNOCKOUTS_AVAILABLE='yes'">
	  <xsl:apply-templates select="." mode="diffs"/>
	</xsl:when>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$DEBUG='true'">
      <xsl:message>       context
      - var=<xsl:copy-of select="$var"/>
      - solution=<xsl:copy-of select="$solution"/>
      - type=<xsl:copy-of select="$type"/>
      - speed=<xsl:value-of select="$speed"/>.
    </xsl:message>
    </xsl:if>

    <text xml:space="preserve">\node (r_<xsl:value-of select="@id"/>) at (<xsl:value-of
    select="@x"/>,<xsl:value-of select="@y"/>) [</text>
    <xsl:value-of select="$type"/>
    <text>, label={[label distance=-0.1cm]-30:\idk{}}] {\IOR{<xsl:value-of select="@id"/>}};
    </text>

    <xsl:variable name="influx">
       <xsl:apply-templates select="$solution" mode="value-of-context">
	<xsl:with-param name="context" select="."/>
      </xsl:apply-templates>
    </xsl:variable>
    <!--
    <xsl:message>
      CONTEXT <xsl:value-of select="@id"/>
      INFLUX  <xsl:value-of select="$influx"/>
      SOLUTION
      <xsl:copy-of select="$solution"/>
    </xsl:message>
    -->
    <xsl:apply-templates select="@id" mode="annotate">
            <xsl:with-param name="conc" select="$speed"/>
	    <!--<xsl:with-param name="conc" select="$influx"/>-->
      <xsl:with-param name="prefix" select="'r_'"/>
    </xsl:apply-templates>
  </xsl:template>
  
  <xsl:template match="context" mode="edge">
    <xsl:apply-templates select="child::*" mode="edge">
      <xsl:with-param name="solution"/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="input[@spec and @copy]" mode="edge">
    <text xml:space="preserve">\draw [In] (copy--<xsl:value-of select="@spec"/>--<xsl:value-of select="@copy"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) </text>


  </xsl:template>

  <xsl:template match="input[@spec and not(@copy)]" mode="edge">

    <text xml:space="preserve">\draw [In] (<xsl:value-of select="@spec"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) </text>
    <text>;
    </text>
  </xsl:template>


  <xsl:template match="output[@spec and @copy]" mode="edge">
    <text xml:space="preserve">\draw [Out] (copy--<xsl:value-of select="@spec"/>--<xsl:value-of select="@copy"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) 
    </text>
    <text>;
    </text>
  </xsl:template>

  <xsl:template match="output[@spec and not(@copy)]" mode="edge">
     <text xml:space="preserve">\draw [Out] (<xsl:value-of select="@spec"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) </text>
    <text>;
    </text>
  </xsl:template>

  <xsl:template match="in-out[@spec and @copy]" mode="edge">
 
    <text xml:space="preserve">\draw [InOut] (copy--<xsl:value-of select="@spec"/>--<xsl:value-of select="@copy"/>) -- (r_<xsl:value-of select="parent::*/@id"/>)
    </text>
    <text>;
    </text>
    
  </xsl:template>

  <xsl:template match="in-out[@spec and not(@copy)]" mode="edge">

    <text xml:space="preserve">\draw [InOut] (<xsl:value-of select="@spec"/>) -- (r_<xsl:value-of select="parent::*/@id"/>) 
    </text>
    <text>;
    </text>
    
  </xsl:template>

  <xsl:template match="reaction|context" mode="diffs">
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    <xsl:variable name="sorted-diffs">
      <diffs>
	<xsl:for-each select="$ko/changes/single/*[@reaction=$id]">
	  <xsl:sort select="@diff"/>
	  <diff>
	    <xsl:value-of select="@diff"/>
	  </diff>
	</xsl:for-each>
      </diffs>
    </xsl:variable>
    <xsl:variable name="diffs">
      <diffs>
	<xsl:copy-of select="$sorted-diffs/diffs/diff[
			     not(.=preceding-sibling::diff[1])]"/>
      </diffs>
    </xsl:variable>
    <!--<xsl:message>
      diffs <xsl:copy-of select="$diffs"/>
    </xsl:message>-->
    <xsl:variable name="out">
      <xsl:for-each select="$diffs//diff">
	<xsl:if test="preceding-sibling::*">
	  <xsl:text>\</xsl:text>
	</xsl:if>
	<xsl:copy-of select="text()"/>
      </xsl:for-each>
    </xsl:variable>
    <!--<xsl:message>
      out=<xsl:copy-of select="$out"/>s
    </xsl:message>-->
    <xsl:copy-of select="$out"/>
  </xsl:template>

  <!-- function kinetics -->

  <xsl:template match="kinetic-expression" mode="kinetics">
    <xsl:if test="$DEBUG='true'">
      <xsl:message>
     +++++ kinetic-expression <xsl:copy-of select="*/name()"/>
      </xsl:message>
    </xsl:if>
    <kinetics>
      <xsl:copy-of select="@angle"/>
      <xsl:attribute name="expr">
	<xsl:apply-templates select="*" mode="kinetic-expression"/>
      </xsl:attribute>
    </kinetics>
  </xsl:template>

   <xsl:template match="mult" mode="kinetic-expression">
     <xsl:for-each select="*">
       <xsl:apply-templates select="." mode="kinetic-expression"/>
       <xsl:if test="not(position()=last()) and
		     (self::conc or not(following-sibling::*[1][name()='conc']))">
	   <xsl:text>\;</xsl:text>
       </xsl:if>
     </xsl:for-each>
   </xsl:template>

 <xsl:template   match="apply[@fun]" mode="kinetic-expression">
      <xsl:text>\Look{</xsl:text><xsl:value-of select="@fun"/><xsl:text>}</xsl:text>
	<xsl:text>(</xsl:text>
	<xsl:for-each select="*">
	  <xsl:apply-templates select="." mode="kinetic-expression"/>
	  <xsl:if test="not(position()=last())">
	    <text xml:space="preserve">,</text>
	  </xsl:if>
     </xsl:for-each>
	  <xsl:text>)</xsl:text>
   </xsl:template>   

   <xsl:template match="sum" mode="kinetic-expression">
     <xsl:if test="parent::mult">
       <text> ( </text>
     </xsl:if>
     <xsl:for-each select="*">
       <xsl:apply-templates select="." mode="kinetic-expression"/>
       <xsl:if test="not(position()=last())">
	 <xsl:text> + </xsl:text>
       </xsl:if>
     </xsl:for-each>
     <xsl:if test="parent::mult">
       <text> ) </text>
     </xsl:if>
  </xsl:template>

   <xsl:template match="divide" mode="kinetic-expression">
     <xsl:text>\frac{ </xsl:text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
     <xsl:text>}{ </xsl:text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <xsl:text>} </xsl:text>
  </xsl:template>

  <xsl:template match="minus[count(*)=2]" mode="kinetic-expression">
     <xsl:text> (</xsl:text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
     <xsl:text>- </xsl:text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <xsl:text> )</xsl:text>
  </xsl:template>

    <xsl:template match="time" mode="kinetic-expression">
    <xsl:text>{\color{blue}\textrm{time}}</xsl:text>
    </xsl:template>
    
  <xsl:template   match="ifthenelse" mode="kinetic-expression">
      <xsl:text>\textbf{if }</xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 
     <xsl:text xml:space="preserve">\textbf{ then }</xsl:text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <xsl:text xml:space="preserve">\textbf{ else }</xsl:text>
     <xsl:apply-templates select="*[3]" mode="kinetic-expression"/>
     <xsl:text xml:space="preserve"> \textbf{ end}</xsl:text>
  </xsl:template>

  <xsl:template   match="lt" mode="kinetic-expression">
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 
    <xsl:text xml:space="preserve"> &lt; </xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
  </xsl:template>
  
  
  <xsl:template   match="power" mode="kinetic-expression">
    <xsl:choose>
      <xsl:when test="*[2][name()='constant'][@value='0.5']">
	<text> \sqrt{</text>
	<xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 	<text xml:space="preserve"> } </text>
      </xsl:when>
      <xsl:otherwise>
	<text> {(</text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 
     <text xml:space="preserve"> )}^{ </text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <text xml:space="preserve"> } </text>
      </xsl:otherwise>
    </xsl:choose>
   </xsl:template>

    <xsl:template match="minus[count(*)=1]" mode="kinetic-expression">
      <xsl:text>- </xsl:text>
      <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
      <xsl:text> </xsl:text>
    </xsl:template>

    <xsl:template match="inh" mode="kinetic-expression">
      <xsl:text> inh(</xsl:text>
      <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
      <xsl:text>)</xsl:text>
    </xsl:template>
    
      <xsl:template match="knockout" mode="kinetic-expression">
    <xsl:text>\ko{</xsl:text> 
    <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="conc" mode="kinetic-expression">
    <xsl:text>\z{</xsl:text> <xsl:value-of select="@spec"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="expr" mode="kinetic-expression">
    <xsl:text>\expr{</xsl:text> <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>
  
  <xsl:template match="var" mode="kinetic-expression">
    <xsl:text>\w{</xsl:text> <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>
  
    <xsl:template match="constant" mode="kinetic-expression">
      <xsl:value-of select="@value"/>
    </xsl:template>

    <xsl:template match="param" mode="kinetic-expression">
      <xsl:value-of select="@id"/>
    </xsl:template>

     <xsl:template match="*" mode="kinetic-expression">
       <xsl:message>
	 °°°°° otherwise of kinetic-expression with <xsl:copy-of select="."/>
       </xsl:message>
    </xsl:template>

    <xsl:template match="*" mode="kinetics">
      <xsl:variable name="name">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$name}">
	<xsl:apply-templates select="@*|*|comment()|text()" mode="kinetics"/>
      </xsl:element>
    </xsl:template>
    
    <xsl:template match="@*|text()|comment()" mode="kinetics">
      <xsl:copy-of select="."/>
    </xsl:template>
</xsl:stylesheet>
