<xsl:stylesheet 
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="text" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:strip-space elements="*" />
      
    <xsl:template match="/">

      <xsl:variable name="full-id">
	<xsl:value-of select="network/@id"/>
	<xsl:if test="network/@kind">
	  <text>-</text>
	  <xsl:value-of select="network/@kind"/>
	</xsl:if>
      </xsl:variable>
      
      <text xml:space="preserve">\renewcommand\TIKZID[1]{</text>
      <xsl:value-of select="$full-id"/>
      <text xml:space="preserve">}
\renewcommand\TIKZREFID[1]{</text>
      <xsl:value-of select="$full-id"/>
      <text>}

\begin{longtable}[l]{|l@{\ \ensuremath{\EQU}\ }l|m{.1\textwidth}|}
\multicolumn{3}{l}{Kinetic laws of reactions}
\\\hline\hline
</text>

      <xsl:apply-templates select="//network"/>

      <text xml:space="preserve">\hline\hline
\end{longtable}
</text>
      
    </xsl:template>
    
    <xsl:template match="//network">
      <xsl:variable name="sorted-reactions">
	<xsl:for-each select="reaction|context">
	  <xsl:sort select="@id" data-type="number"/>
	  <xsl:copy-of select="."/>
	</xsl:for-each>
      </xsl:variable>
      <xsl:variable name="reactions">
	<xsl:apply-templates select="$sorted-reactions//(reaction|context)"/>
      </xsl:variable>
      <!--
      <xsl:message>
	++ reactions <xsl:copy-of select="$sorted-reactions"/>
      </xsl:message>
      -->
      <xsl:value-of select="$reactions"/>
    </xsl:template>

    <xsl:template match="@experiment">
      <text>experiment: </text>
      <xsl:value-of select="."/>
    </xsl:template>
    <xsl:template match="@prediction">
      <text>prediction: </text>
      <xsl:value-of select="."/>
    </xsl:template>
    <xsl:template match="@*">
      <xsl:value-of select="."/>
    </xsl:template>

    <!-- reaction-->
   <xsl:template match="reaction" >
     <text xml:space="preserve">\Rtable{<xsl:value-of
     select="normalize-space(@id)"/>}&amp; \ensuremath{<xsl:apply-templates
     select="kinetic-expression" mode="kinetic-expression"/>
     <xsl:if test="expression">
       <xsl:text>\textrm{ where }</xsl:text>
       <xsl:if test="count(expression)&gt;1">
	 <xsl:text>\left\{</xsl:text>
       </xsl:if>
       <xsl:text>\begin{tabular}{l}</xsl:text>
       <xsl:for-each select="expression">
	 <xsl:apply-templates select="."/>
	 <xsl:if test="not(position()=last())">
	   <xsl:text>\\</xsl:text>
	 </xsl:if>
       </xsl:for-each>
       <xsl:text>\end{tabular} </xsl:text>
       <xsl:if test="count(expression)&gt;1">
	 <xsl:text>\right.</xsl:text>
       </xsl:if>
     </xsl:if>
     }&amp; <xsl:value-of select="@comment"/> 
\\
</text>
      <xsl:for-each select="comment">
	<text xml:space="preserve"> &amp; &amp;
	<xsl:apply-templates select="@*"/>
\\
</text>
     </xsl:for-each>
     <xsl:if test="product-inh">
       <text xml:space="preserve">\RtableDEG{<xsl:value-of
       select="normalize-space(@id)"/>}
       &amp;
       &amp; degradation of \M{</text>
	<xsl:value-of select="product-inh/@spec"/>
	<text xml:space="preserve">}\\
</text>
     </xsl:if>
   </xsl:template>

   <xsl:template match="expression">
     <xsl:text> \expr{</xsl:text>
     <xsl:value-of select="@id"/>
     <xsl:text>} \ensuremath{\ \EQU\ } </xsl:text>
     <xsl:apply-templates select="*" mode="kinetic-expression"/>
   </xsl:template>

   <!-- inflows and outflow -->
   <xsl:template match="context" >
     <text xml:space="preserve">\Rtable{<xsl:value-of
     select="normalize-space(@id)"/>}
     &amp;
     &amp; <xsl:apply-templates select="input|output"/>

     <xsl:value-of select="@comment"/>
\\
</text>
      <xsl:for-each select="comment">
	<text xml:space="preserve">
	  &amp;
	  &amp; <xsl:apply-templates select="@*"/>
\\
</text>
     </xsl:for-each>
     <xsl:if test="product-inh">
        <text xml:space="preserve">\RtableDEG{<xsl:value-of select="normalize-space(@id)"/>}&amp; degradation of \M{</text>
	<xsl:value-of select="product-inh/@spec"/>
	<text xml:space="preserve">}\\
</text>
     </xsl:if>
   </xsl:template>

   <xsl:template match="input">
     <xsl:text> inflow of \M{</xsl:text>
     <xsl:value-of select="@spec"/>
     <xsl:text>}</xsl:text>
   </xsl:template>


   <xsl:template match="output">
     <xsl:text> outflow of \M{</xsl:text>
     <xsl:value-of select="@spec"/>
     <xsl:text>}</xsl:text>
   </xsl:template>


<!-- function kinetics -->

  <xsl:template match="kinetic-expression" mode="kinetics">
    <kinetics>
      <xsl:copy-of select="@angle"/>
      <xsl:attribute name="expr">
	<xsl:apply-templates select="*" mode="kinetic-expression"/>
      </xsl:attribute>
    </kinetics>
  </xsl:template>

   <xsl:template match="mult" mode="kinetic-expression">
     <xsl:for-each select="*">
       <xsl:apply-templates select="." mode="kinetic-expression"/>
       <xsl:if test="not(position()=last()) and
		     (self::conc or not(following-sibling::*[1][name()='conc']))">
	   <xsl:text>\;</xsl:text>
       </xsl:if>
     </xsl:for-each>
   </xsl:template>

   
     <xsl:template match="delay" mode="kinetic-expression">
    <xsl:text> delay(</xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
   <xsl:text>,</xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
    <xsl:text>)</xsl:text>
     </xsl:template>
     
 <xsl:template   match="apply[@fun]" mode="kinetic-expression">
   <xsl:text>\expr{</xsl:text>
   <xsl:value-of select="@fun"/>
   <xsl:text>}</xsl:text>
   <xsl:text>(</xsl:text>
   <xsl:for-each select="*">
     <xsl:apply-templates select="." mode="kinetic-expression"/>
	  <xsl:if test="not(position()=last())">
	    <text xml:space="preserve">,</text>
	  </xsl:if>
   </xsl:for-each>
   <xsl:text>)</xsl:text>
 </xsl:template>   
 
   <xsl:template match="sum" mode="kinetic-expression">
     <xsl:if test="parent::mult">
       <text> ( </text>
     </xsl:if>
     <xsl:for-each select="*">
       <xsl:apply-templates select="." mode="kinetic-expression"/>
       <xsl:if test="not(position()=last())">
	 <xsl:text> + </xsl:text>
       </xsl:if>
     </xsl:for-each>
     <xsl:if test="parent::mult">
       <text> ) </text>
     </xsl:if>
  </xsl:template>

   <xsl:template match="divide" mode="kinetic-expression">
     <xsl:text>\frac{ </xsl:text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
     <xsl:text>}{ </xsl:text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <xsl:text>} </xsl:text>
  </xsl:template>

  <xsl:template match="minus[count(*)=2]" mode="kinetic-expression">
     <xsl:text> (</xsl:text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
     <xsl:text>- </xsl:text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <xsl:text> )</xsl:text>
  </xsl:template>

  <xsl:template   match="power" mode="kinetic-expression">
    <xsl:choose>
      <xsl:when test="*[2][name()='constant'][@value='0.5']">
	<text> \sqrt{</text>
	<xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 	<text xml:space="preserve"> } </text>
      </xsl:when>
      <xsl:otherwise>
	<text> {(</text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 
     <text xml:space="preserve"> )}^{ </text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <text xml:space="preserve"> } </text>
      </xsl:otherwise>
    </xsl:choose>
   </xsl:template>

    <xsl:template match="minus[count(*)=1]" mode="kinetic-expression">
      <xsl:text>- </xsl:text>
      <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
      <xsl:text> </xsl:text>
    </xsl:template>

     <xsl:template match="ceiling" mode="kinetic-expression">
     <xsl:text>\ceiling{ </xsl:text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
     <xsl:text>} </xsl:text>
    </xsl:template>

    <xsl:template match="floor" mode="kinetic-expression">
     <xsl:text>\floor{ </xsl:text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
     <xsl:text>} </xsl:text>
    </xsl:template>

    <xsl:template match="log|sin|cos|tan|cot|sinh|cosh|tanh|coth"
		mode="kinetic-expression">
    <xsl:variable name="name">
      <xsl:value-of select="name()"/>
    </xsl:variable> 
    <xsl:value-of select="$name"/>
    <xsl:text>(</xsl:text> 
    <xsl:apply-templates select="*" mode="kinetic-expression"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

    
      <xsl:template match="knockout" mode="kinetic-expression">
    <xsl:text>\ko{</xsl:text> 
    <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="conc" mode="kinetic-expression">
    <xsl:text>\z{</xsl:text> <xsl:value-of select="@spec"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="var" mode="kinetic-expression">
    <xsl:text>\w{</xsl:text> <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="expr" mode="kinetic-expression">
    <xsl:text>\expr{</xsl:text> <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

   <xsl:template match="time" mode="kinetic-expression">
    <xsl:text>{\color{blue}\textrm{time}}</xsl:text>
  </xsl:template>

  <xsl:template   match="ifthenelse" mode="kinetic-expression">
      <xsl:text>\textbf{if }</xsl:text>
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 
     <xsl:text xml:space="preserve">\textbf{ then }</xsl:text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <xsl:text xml:space="preserve">\textbf{ else }</xsl:text>
     <xsl:apply-templates select="*[3]" mode="kinetic-expression"/>
     <xsl:text xml:space="preserve"> \textbf{ end}</xsl:text>
  </xsl:template>

  <xsl:template   match="lt" mode="kinetic-expression">
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 
    <xsl:text xml:space="preserve"> &lt; </xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
  </xsl:template>
  
    <xsl:template match="constant" mode="kinetic-expression">
      <xsl:value-of select="@value"/>
    </xsl:template>

    <xsl:template match="*" mode="kinetics">
      <xsl:variable name="name">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$name}">
	<xsl:apply-templates select="@*|*|comment()|text()" mode="kinetics"/>
      </xsl:element>
    </xsl:template>
    
    <xsl:template match="@*|text()|comment()" mode="kinetics">
      <xsl:copy-of select="."/>
    </xsl:template>

   
</xsl:stylesheet>
