<xsl:stylesheet 
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="text" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:strip-space elements="*" />
      
    <xsl:template match="/">

      
      <xsl:variable name="full-id">
	<xsl:value-of select="network/@id"/>
	<xsl:if test="network/@kind">
	  <text>-</text>
	  <xsl:value-of select="network/@kind"/>
	</xsl:if>
      </xsl:variable>

      <text xml:space="preserve">\renewcommand\TIKZID[1]{</text>
      <xsl:value-of select="$full-id"/>
      <text xml:space="preserve">}
\renewcommand\TIKZREFID[1]{</text>
      <xsl:value-of select="$full-id"/>
      <text>}

\begin{longtable}[l]{||l@{\ensuremath{\ \EQU\ }}l|l|}
\multicolumn{3}{l}{Initial values of species} \\\hline\hline</text>

      <xsl:apply-templates select="//network"/>

      <text xml:space="preserve">\hline\hline
\end{longtable}
</text>
      
    </xsl:template>
    
    <xsl:template match="//network"> 
      <xsl:variable name="molecules">
	<xsl:apply-templates select="metabolite[1]"/>
	<xsl:apply-templates select="metabolite[position() > 1]"/>
	<xsl:apply-templates select="protein[1]"/>
	<xsl:apply-templates select="protein[position() > 1]"/>
	<xsl:apply-templates select="actor[1]"/>
	<xsl:apply-templates select="actor[position() > 1]"/>
	<xsl:apply-templates select="species[not(@type='algebraic')][1]"/>	
	<xsl:apply-templates select="species[not(@type='algebraic')]
				     [position() > 1]"/>
      </xsl:variable>

      <xsl:value-of select="$molecules"/>
    </xsl:template>
    
   <xsl:template match="metabolite[1]">

       <text xml:space="preserve"> 
       \Mtable{<xsl:value-of select="normalize-space(@id)"/>} </text>
       <text> &amp; $</text>

       <xsl:choose>
       <xsl:when test="@initial-expression">
	 <text xml:space="preserve">  \expr{</text>
	 <xsl:value-of select="@initial-expression"/>  <text>} </text>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:value-of select="@initial |  @initialConcentration"/>
       </xsl:otherwise>
       </xsl:choose>
       <text>$    &amp; </text>
       <xsl:value-of select="@comment"/>
<text>\\
</text>


<xsl:for-each select="comment">
     <text xml:space="preserve"> &amp; </text>
     <xsl:value-of select="@latex-look"/>
     <text> &amp; </text>
     <xsl:value-of select="@initial| @initialConcentration"/>
     <text>\\
</text>
   </xsl:for-each>
   </xsl:template>

   <xsl:template match="metabolite[position() > 1]">
     <text xml:space="preserve">  \Mtable{</text>
     <xsl:value-of select="normalize-space(@id)"/>  <text>} </text>
     <text>&amp; </text>
     <xsl:choose>
       <xsl:when test="@initial-expression">
	 <text xml:space="preserve">  \expr{</text>
	 <xsl:value-of select="@initial-expression"/>  <text>} </text>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:value-of select="@initial | @initialConcentration"/>
       </xsl:otherwise>
     </xsl:choose>
     <text>&amp; </text>
     <xsl:value-of  select="@comment"/>
     <text>\\
</text>

   <xsl:for-each select="comment">
     <text xml:space="preserve"> &amp;  <xsl:value-of
     select="@latex"/>&amp;  &amp;
     
\\
</text>
   </xsl:for-each>
   </xsl:template>


   <xsl:template match="protein[1]">
     <text> \Mtable{</text>
     <xsl:value-of select="normalize-space(@id)"/>
     <text>}  &amp; </text>
     <xsl:value-of select="@initial| @initialConcentration"/>
     <text> &amp; </text>
     <xsl:value-of select="@comment"/>
     <text>\\
</text>

   <xsl:for-each select="comment">
     <xsl:value-of  select="@latex"/>
     <text>\\
</text>
   </xsl:for-each>
 </xsl:template>

   <xsl:template match="protein[position() > 1]">

     <text xml:space="preserve">
       \Mtable{<xsl:value-of select="normalize-space(@id)"/>}
       &amp; <xsl:value-of select="@initial|@initialConcentration"/>
       &amp; <xsl:value-of select="@comment"/>
\\
</text>

   <xsl:for-each select="comment">
     <text xml:space="preserve">
       <xsl:value-of  select="@latex-look"/>
       &amp; <xsl:value-of select="@initial|@initialConcentration"/>
       &amp; <xsl:value-of select="@comment"/> 
\\
\\
</text>
   </xsl:for-each>
   </xsl:template>

 <xsl:template match="actor[1]">

   <text xml:space="preserve">
       \Mtable{<xsl:value-of select="normalize-space(@id)"/>}
           &amp; <xsl:value-of
	   select="@initial|@initialConcentration"/>
	   &amp; <xsl:value-of select="@comment"/>
\\
</text>

   <xsl:for-each select="comment">
     <text xml:space="preserve"> &amp; &amp; <xsl:value-of select="@latex"/>
\\
</text>
   </xsl:for-each>
 </xsl:template>

 <xsl:template match="species[1]">

   <text xml:space="preserve">
     \Mtable{<xsl:value-of select="normalize-space(@id)"/>} 
   &amp; $ <xsl:apply-templates select="kinetic-expression"   mode="kinetic-expression"/>  $
&amp; <xsl:value-of select="@comment"/>
\\
</text>
   <xsl:for-each select="comment">
     <text xml:space="preserve">  &amp; &amp;<xsl:value-of select="@latex"/> 
\\
</text>
   </xsl:for-each>
   </xsl:template>

   <xsl:template match="actor[position() > 1]">

     <text xml:space="preserve">
        \Mtable{<xsl:value-of select="normalize-space(@id)"/>}
        &amp; <xsl:value-of select="@initial|@initialConcentration"/>
       &amp; <xsl:value-of select="@comment"/>
\\
</text>

   <xsl:for-each select="comment">
     <text xml:space="preserve"> &amp;
     &amp; <xsl:value-of select="@latex"/>
     \\
     </text>
   </xsl:for-each>
   </xsl:template>


   <xsl:template match="species[position() > 1]">
     <text xml:space="preserve">
       \Mtable{<xsl:value-of select="normalize-space(@id)"/>}
       &amp; $ <xsl:apply-templates select="kinetic-expression"
                    mode="kinetic-expression"/>	  $
       &amp; <xsl:value-of select="@comment"/>
\\
</text>


   <xsl:for-each select="comment">
     <text xml:space="preserve"> &amp; &amp; <xsl:value-of select="@latex"/>
\\
</text>
   </xsl:for-each>
   </xsl:template>

   <!-- function kinetics -->

  <xsl:template match="kinetic-expression" mode="kinetics">
    <kinetics>
      <xsl:copy-of select="@angle"/>
      <xsl:attribute name="expr">
	<xsl:apply-templates select="*" mode="kinetic-expression"/>
      </xsl:attribute>
    </kinetics>
  </xsl:template>

   <xsl:template match="mult" mode="kinetic-expression">
     <xsl:for-each select="*">
       <xsl:apply-templates select="." mode="kinetic-expression"/>
       <xsl:if test="not(position()=last()) and
		     (self::conc or not(following-sibling::*[1][name()='conc']))">
	   <xsl:text>\;</xsl:text>
       </xsl:if>
     </xsl:for-each>
   </xsl:template>

 <xsl:template   match="apply[@fun]" mode="kinetic-expression">
   <xsl:text>\expr{</xsl:text>
   <xsl:choose>
     <xsl:when test="@latex-look">
       <xsl:value-of select="@latex-look"/>
     </xsl:when>
     <xsl:otherwise>
       <xsl:value-of select="@fun"/>
     </xsl:otherwise>
   </xsl:choose>
   <xsl:text>}(</xsl:text>
   <xsl:for-each select="*">
     <xsl:apply-templates select="." mode="kinetic-expression"/>
     <xsl:if test="not(position()=last())">
       <text xml:space="preserve">,</text>
     </xsl:if>
   </xsl:for-each>
   <xsl:text>)</xsl:text>
 </xsl:template>   

   <xsl:template match="sum" mode="kinetic-expression">
     <xsl:if test="parent::mult">
       <text> ( </text>
     </xsl:if>
     <xsl:for-each select="*">
       <xsl:apply-templates select="." mode="kinetic-expression"/>
       <xsl:if test="not(position()=last())">
	 <xsl:text> + </xsl:text>
       </xsl:if>
     </xsl:for-each>
     <xsl:if test="parent::mult">
       <text> ) </text>
     </xsl:if>
  </xsl:template>

   <xsl:template match="divide" mode="kinetic-expression">
     <xsl:text>\frac{ </xsl:text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
     <xsl:text>}{ </xsl:text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <xsl:text>} </xsl:text>
  </xsl:template>

  <xsl:template match="minus[count(*)=2]" mode="kinetic-expression">
     <xsl:text> (</xsl:text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
     <xsl:text>- </xsl:text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <xsl:text> )</xsl:text>
  </xsl:template>

  <xsl:template   match="power" mode="kinetic-expression">
    <xsl:choose>
      <xsl:when test="*[2][name()='constant'][@value='0.5']">
	<text> \sqrt{</text>
	<xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 	<text xml:space="preserve"> } </text>
      </xsl:when>
      <xsl:otherwise>
	<text> {(</text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 
     <text xml:space="preserve"> )}^{ </text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <text xml:space="preserve"> } </text>
      </xsl:otherwise>
    </xsl:choose>
   </xsl:template>

    <xsl:template match="minus[count(*)=1]" mode="kinetic-expression">
      <xsl:text>- </xsl:text>
      <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
      <xsl:text> </xsl:text>
    </xsl:template>

      <xsl:template match="knockout" mode="kinetic-expression">
    <xsl:text>\ko{</xsl:text> 
    <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="conc" mode="kinetic-expression">
    <xsl:text>\z{</xsl:text> <xsl:value-of select="@spec"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  <xsl:template match="expr" mode="kinetic-expression">
    <xsl:text>\expr{</xsl:text> <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>
  
  <xsl:template match="var" mode="kinetic-expression">
    <xsl:text>\w{</xsl:text> <xsl:value-of select="@id"/>
    <xsl:text>}</xsl:text>
    <xsl:apply-templates select="." mode="komma"/>
  </xsl:template>

  
   <xsl:template match="time" mode="kinetic-expression">
    <xsl:text>{\color{blue}\textrm{time}}</xsl:text>
  </xsl:template>

  <xsl:template   match="ifthenelse" mode="kinetic-expression">
      <xsl:text>\textbf{if }</xsl:text>
    <xsl:apply-templates select="*[1]" mode="condition"/> 
     <xsl:text xml:space="preserve">\textbf{ then }</xsl:text>
     <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <xsl:text xml:space="preserve">\textbf{ else }</xsl:text>
     <xsl:apply-templates select="*[3]" mode="kinetic-expression"/>
     <xsl:text xml:space="preserve"> \textbf{ end}</xsl:text>
  </xsl:template>

  <xsl:template   match="lt" mode="kinetic-expression">
    <xsl:apply-templates select="*[1]" mode="kinetic-expression"/> 
    <xsl:text xml:space="preserve"> &lt; </xsl:text>
    <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
  </xsl:template>
  
    <xsl:template match="constant" mode="kinetic-expression">
      <xsl:value-of select="@value"/>
    </xsl:template>

    <!-- function nice-id-->
    <xsl:template match="@id" mode="nice-id">
      <!--<xsl:message>	nice for <xsl:value-of select="."/> is <xsl:value-of select="replace(.,'_','\\_')"/>   </xsl:message>-->
      <xsl:value-of select="replace(.,'_','\\_')"/>
    </xsl:template>

       
   <!-- function condition -->

   <xsl:template match="condition" mode="condition">
     <xsl:message>
       event condition <xsl:copy-of select="."/>
     </xsl:message>
     <xsl:apply-templates select="*" mode="condition"/>
   </xsl:template>

   <xsl:template match="and" mode="condition">
     <!--<xsl:message>
       event condition <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:apply-templates select="*[1]" mode="condition"/>
       <xsl:text>  \wedge </xsl:text>
       <xsl:apply-templates select="*[2]" mode="condition"/>
   </xsl:template>

   <xsl:template match="leq" mode="condition">
     <!--<xsl:message>
       event condition <xsl:copy-of select="."/>
     </xsl:message>-->
          <xsl:text>  </xsl:text>
       <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
       <xsl:text> \leq </xsl:text>
       <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <xsl:text>  </xsl:text>
   </xsl:template>

    <xsl:template match="lt" mode="condition">
     <!--<xsl:message>
       event condition <xsl:copy-of select="."/>
     </xsl:message>-->
     <xsl:text>  </xsl:text>
       <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
       <xsl:text> &lt; </xsl:text>
       <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
     <xsl:text>  </xsl:text>
    </xsl:template>

    <xsl:template match="eq" mode="condition">
     <xsl:message>
       event condition <xsl:copy-of select="."/>
     </xsl:message>
     <xsl:text>  </xsl:text>
     <xsl:apply-templates select="*[1]" mode="kinetic-expression"/>
       <xsl:text> = </xsl:text>
       <xsl:apply-templates select="*[2]" mode="kinetic-expression"/>
       <xsl:text>  </xsl:text>
     </xsl:template>

  </xsl:stylesheet>
