<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>

    <!-- define a network from species and reactions -->
    
    <xsl:template match="network">
      <xsl:copy-of select="."/>
    </xsl:template>

    <!-- import a network descriptor from a file -->

    <xsl:template match="network[@file]">
      <xsl:variable name="file-name">
	<xsl:copy-of select="concat('../',@file,'.xml')"/>
      </xsl:variable>
      <!--      <xsl:message>
	import network from file <xsl:copy-of select="$file-name"/>
      </xsl:message> -->
      <xsl:variable name="file-network">
	<xsl:copy-of select="document($file-name)"/>
      </xsl:variable>
      <xsl:apply-templates select="$file-network"/>
    </xsl:template>

    <!-- generate a network from existing networks by using commands -->

    <xsl:template match="generate-network[@id]">
      <xsl:variable name="start-network">
	<network kind="expanded">
	  <xsl:copy-of select="@*"/>
	</network>
      </xsl:variable>
      <xsl:choose>
	<xsl:when test="child::*[1]">
	  <xsl:apply-templates select="child::*[1]" mode="apply-commands">
	    <xsl:with-param name="network" select="$start-network"/>
	  </xsl:apply-templates>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy-of select="$start-network"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template> 

    <!-- function apply-commands -->
    
    <xsl:template match="*" mode="apply-commands">
      <xsl:param name="network"/>
      <!--
      <xsl:message>
	fun apply-commands
	command: <xsl:copy-of select="."/>
      </xsl:message>
      -->
      <xsl:variable name="next-network">
	<xsl:apply-templates select="." mode="apply-command">
	  <xsl:with-param name="network" select="$network"/>
	</xsl:apply-templates>
      </xsl:variable>
      <!-- <xsl:message>
	next network:<xsl:copy-of select="$next-network"/>
      </xsl:message>
      -->
      <xsl:choose> 
	<xsl:when test="following-sibling::*">
	  <xsl:apply-templates select="following-sibling::*[1]"
			       mode="apply-commands">
	    <xsl:with-param name="network" select="$next-network"/>
	  </xsl:apply-templates>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy-of select="$next-network"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <!-- function apply-command -->

    <xsl:template match="add-network" mode="apply-command">
      <xsl:param name="network"/>
      <!-- b) this command is decomposed into by the core-commands
	   add-network-without-commands applied to a core-network
	   constructed by apply-commands -->
      <xsl:variable name="core-command">
	<add-network-no-commands>
	  <xsl:copy-of select="@*"/>
	  <generate-network>
	    <xsl:attribute name="id">core-network-for-add-networks</xsl:attribute>
	    <add-network-no-commands>
	      <xsl:copy-of select="child::*[1]"/>
	    </add-network-no-commands>
	    <xsl:copy-of select="child::*[not(position()=1)]"/>	    
	  </generate-network>
	</add-network-no-commands>
      </xsl:variable>
      <!--
      <xsl:message>
	fun apply-command
	core-command: <xsl:copy-of select="$core-command"/>
      </xsl:message>
      -->
      <xsl:apply-templates select="$core-command" mode="apply-command">
	<xsl:with-param name="network" select="$network"/>
      </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="add-network-no-commands" mode="apply-command">
      <xsl:param name="network"/>

      <xsl:variable name="additional-network">
	<xsl:apply-templates select="child::*[1]"/>
      </xsl:variable>
<!--      <xsl:message>
	fun apply-command
	command: <xsl:copy-of select="."/>

	additional network <xsl:copy-of select="$additional-network"/>
	old-network <xsl:copy-of select="$network"/>
	integrate additional network
	prefix="<xsl:value-of select="@prefix"/>"

      </xsl:message>
-->
<network>
	<xsl:copy-of select="$network//network/@* |
		    $network//network/child::*"/>
	
	<xsl:apply-templates
	    select="$additional-network//network/child::*" mode="add-network">
	  <xsl:with-param name="x">
	    <xsl:choose>
	      <xsl:when test="@x"> <xsl:value-of select="@x"/> </xsl:when>
	      <xsl:otherwise> 0</xsl:otherwise>
	    </xsl:choose>
	  </xsl:with-param>
	  <xsl:with-param name="y">
	    <xsl:choose>
	      <xsl:when test="@y"> <xsl:value-of select="@y"/> </xsl:when>
	      <xsl:otherwise> 0 </xsl:otherwise>
	    </xsl:choose>
	  </xsl:with-param>   
	  <xsl:with-param name="scale_x">
	    <xsl:choose>
	      <xsl:when test="@scale_x"> <xsl:value-of select="@scale_x"/> </xsl:when>
	      <xsl:otherwise> 1 </xsl:otherwise>
	    </xsl:choose>
	  </xsl:with-param>
	  <xsl:with-param name="scale_y">
	    <xsl:choose>
	      <xsl:when test="@scale_y"> <xsl:value-of select="@scale_y"/> </xsl:when>
	      <xsl:otherwise> 1 </xsl:otherwise>
	    </xsl:choose>
	  </xsl:with-param>
	  <xsl:with-param name="prefix">
	    <xsl:choose>
	      <xsl:when test="@prefix">
		<xsl:value-of select="@prefix"/>
	      </xsl:when>
	      <xsl:otherwise/>
	    </xsl:choose>
	  </xsl:with-param>
	  <xsl:with-param name="old-network">
	    <xsl:copy-of select="$network"/>
	  </xsl:with-param>
	</xsl:apply-templates>
      </network>
    </xsl:template>

    <xsl:template match="add-reaction[@id]" mode="apply-command">
      <xsl:param name="network"/>
      <network>
	<xsl:copy-of select="$network//network/@* |
		    $network//network/child::*"/>
	<!--
	<xsl:message>
	  fun apply-command
	  command: <xsl:copy-of select="."/>
	</xsl:message>
	-->
	<reaction>
	  <xsl:copy-of select="@*|child::*"/>
	</reaction>
      </network>
    </xsl:template>

    <xsl:template match="remove[@id]" mode="apply-command">
      <xsl:param name="network"/>
      <xsl:variable name="id">
	<xsl:value-of select="@id"/>
      </xsl:variable>
      <!--
      <xsl:message>
	 fun apply-command
	 command: <xsl:copy-of select="."/>
      </xsl:message>
      -->
      <xsl:if test="network//network/child::*[@id=$id]">
	<xsl:message>
	  WARNING: noting to remove for command <xsl:copy-of select="."/>
	</xsl:message>
      </xsl:if>
      <network>
	<xsl:copy-of select="$network//network/@id |
		    $network//network/child::*[not(@id=$id)]"/>
      </network>
    </xsl:template>

    <xsl:template match="fusion" mode="apply-command">
      <xsl:param name="network"/>
      <network>
	<xsl:copy-of select="$network/network/@id"/>
	<!-- <xsl:message>
	  <xsl:copy-of select="."/>
	  id1 <xsl:value-of select="child::*[1]/@id"/>
	  id2 <xsl:value-of select="child::*[2]/@id"/>
	</xsl:message>
	-->
	<xsl:apply-templates select="$network//network/*" mode="fusion">
	  <xsl:with-param name="id1">
	    <xsl:value-of select="child::*[1]/@id"/>
	  </xsl:with-param>   
	  <xsl:with-param name="id2">
	    <xsl:value-of select="child::*[2]/@id"/>
	  </xsl:with-param>
	  <xsl:with-param name="newid">
	    <xsl:choose>
	      <xsl:when test="@id">
		<xsl:value-of select="@id"/>
	      </xsl:when>
	      <xsl:otherwise>
		<xsl:value-of select="child::*[1]/@id"/>
	      </xsl:otherwise>
	    </xsl:choose>
	  </xsl:with-param>
	  
	  
	</xsl:apply-templates>
      </network>
    </xsl:template>
    
    <xsl:template match="*" mode="apply-command">
      <xsl:param name="network"/>
      <xsl:message>
	ERROR in apply-command
	Wrong command: <xsl:copy-of select="."/>
	Last network: <xsl:copy-of select="$network"/>
      </xsl:message>
    </xsl:template>
    
    <!-- function add-network -->
    
    <xsl:template match="*"  mode="add-network">
      <xsl:param name="x"/>
      <xsl:param name="y"/>
      <xsl:param name="scale_x"/>
      <xsl:param name="scale_y"/>
      <xsl:param name="prefix"/>
      <xsl:param name="old-network"/>
      <!--
      <xsl:message>
        fun add-network:
        command: <xsl:copy-of select="."/>
	param x: <xsl:copy-of select="$x"/>
	param y: <xsl:copy-of select="$y"/>
	param prefix: <xsl:copy-of select="$prefix"/>
	param old-network: <xsl:copy-of select="$old-network"/>
      </xsl:message>
      -->
      <xsl:variable name="tag">
	<xsl:value-of select="name()"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="@*|child::*" mode="add-network">
	  <xsl:with-param name="x">        <xsl:copy-of select="$x"/> </xsl:with-param>
	  <xsl:with-param name="y">        <xsl:copy-of select="$y"/> </xsl:with-param>
	  <xsl:with-param name="scale_x">  <xsl:copy-of select="$scale_x"/> </xsl:with-param>
	  <xsl:with-param name="scale_y">  <xsl:copy-of select="$scale_y"/> </xsl:with-param>
	  <xsl:with-param name="prefix">   <xsl:copy-of select="$prefix"/> </xsl:with-param>
	  <xsl:with-param name="old-network"> <xsl:copy-of select="$old-network"/> </xsl:with-param>
	</xsl:apply-templates>
      </xsl:element>
    </xsl:template>

    <xsl:template match="@x" mode="add-network">
      <xsl:param name="x"/>
      <xsl:param name="scale_x"/>
      <xsl:variable name="x1">
	<xsl:value-of select="."/>
      </xsl:variable>
      <xsl:attribute name="x">
	<xsl:copy-of select="$x1 * $scale_x + $x"/>
      </xsl:attribute>
    </xsl:template>

    <xsl:template match="@y" mode="add-network">
      <xsl:param name="y"/>
      <xsl:param name="scale_y"/>
      <!--
	  <xsl:message>
	scale_y <xsl:value-of select="$scale_y"/>
      </xsl:message>
      -->
      <xsl:variable name="y1">
	<xsl:value-of select="."/>
      </xsl:variable>
      <xsl:attribute name="y">
	<xsl:copy-of select="$y1 * $scale_y - $y"/>
      </xsl:attribute>
    </xsl:template>

    <xsl:template match="@*[name()='id' or name()='spec' or name()='edgecluster']
			 [not(parent::kinetics or parent::copy)]" mode="add-network">
      <xsl:param name="prefix"/>
      <xsl:param name="old-network"/>
      <xsl:variable name="val"><xsl:value-of select="."/></xsl:variable>
      <xsl:variable name="name"><xsl:value-of select="name(.)"/></xsl:variable>
      <!--
      <xsl:message>
	val="<xsl:copy-of select="$val"/>"
      </xsl:message>
      -->
      <xsl:attribute name="{$name}">
	<xsl:choose>
	  <xsl:when test="$prefix='' or not($old-network//*/@id=$val)">
	    <xsl:value-of select="$val"/>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:copy-of select="concat($prefix,$val)"/>
	    <xsl:message>
	      val <xsl:copy-of select="$val"/> conflict resolved
	    </xsl:message>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:attribute>
    </xsl:template>
   
    <xsl:template match="@*"  mode="add-network">
      <xsl:copy-of select="."/>
    </xsl:template>

    <!-- function fusion -->
    
    <xsl:template match="metabolite|protein|actor"  mode="fusion">
      <xsl:param name="id1"/>
      <xsl:param name="id2"/>
      <xsl:param name="newid"/>
      <xsl:variable name="tag">
	<xsl:copy-of select="name(.)"/>
      </xsl:variable>
      <xsl:variable name="id">
	<xsl:value-of select="@id"/>
      </xsl:variable>
      <xsl:choose>
	<xsl:when test="@id=$id2"/> <!-- $id2 is removed into a copy of $id1 -->	
	<xsl:when test="@id=$id1"> 
	  <xsl:element name="{$tag}">
	    <xsl:attribute name="id">
	      <xsl:value-of select="$newid"/>
	    </xsl:attribute>
	    <xsl:copy-of select="@*[not(name()='id')] | 
				 child::*[not(name()='splitpoint')]"/>
	    <xsl:variable name="spec2">
	      <xsl:copy-of select="preceding-sibling::*[@id=$id2] | 
				   following-sibling::*[@id=$id2]  "/>
	    </xsl:variable>
	    <copy id="{$id2}">
	      <xsl:copy-of select="$spec2/child::*/@x"/>
	      <xsl:copy-of select="$spec2/child::*/@y"/>
	    </copy>
	    
	    <xsl:apply-templates select="$spec2/child::*/child::*[not(name()='splitpoint')] "
				 mode="fusion">
	      <xsl:with-param name="newid">
		  <xsl:value-of select="$newid"/>
	      </xsl:with-param>
	      <xsl:with-param name="id1">
		<xsl:value-of select="$id1"/>
	      </xsl:with-param>
	      <xsl:with-param name="id2">
		<xsl:value-of select="$id2"/>
	      </xsl:with-param>
	    </xsl:apply-templates>
	    
	    <!-- copy or create a splitpoint -->
	    <xsl:choose>
	      <xsl:when test="child::splitpoint">
		<xsl:copy-of select="child::splitpoint"/>
	      </xsl:when>
	      <xsl:when test="$spec2/child::*/child::splitpoint">
		<xsl:copy-of select="$spec2/child::*/child::splitpoint"/>
	      </xsl:when>
	      <xsl:otherwise> 
		<splitpoint>
		  <xsl:attribute name="x">
		    <xsl:value-of select="($spec2/child::*/@x + @x) * 0.5"/>
		  </xsl:attribute>
		  <xsl:attribute name="y">
		    <xsl:value-of select="(($spec2/child::*/@y) + @y) * 0.5"/>
		  </xsl:attribute>
		</splitpoint>
		<xsl:message>
		  ==== splitpoint created for <xsl:value-of select="$newid"/> at =======
		  @x=<xsl:value-of select="($spec2/child::*/@x + @x) * 0.5"/>
		  @y=<xsl:value-of select="(($spec2/child::*/@y) + @y) * 0.5"/>
		  ===========
		</xsl:message>
	      </xsl:otherwise>
	    </xsl:choose>
	  </xsl:element>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy-of select="."/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>
    
    <xsl:template match="reaction|context|input|output|comment|
			 kinetics|comment|
			 edgecluster|reactant|product|activator|
			 accelerator|inhibitor|product-inh|
			 source|copy |splitpoint"  mode="fusion">
      <xsl:param name="id1"/>
      <xsl:param name="id2"/>
      <xsl:param name="newid"/>
      <xsl:variable name="tag">
	<xsl:copy-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="@* | child::*" mode="fusion">
	  <xsl:with-param name="id1">
	    <xsl:copy-of select="$id1"/>
	  </xsl:with-param>
	  <xsl:with-param name="id2">
	    <xsl:copy-of select="$id2"/>
	  </xsl:with-param>
	  <xsl:with-param name="newid">
	    <xsl:copy-of select="$newid"/>
	  </xsl:with-param>
	</xsl:apply-templates>
      </xsl:element>
    </xsl:template>

    <xsl:template match="@spec" mode="fusion">
     <xsl:param name="id1"/>
     <xsl:param name="id2"/>
     <xsl:param name="newid"/>
     <xsl:choose>
	<xsl:when test=".=$id1">
	  <xsl:attribute name="spec">
	    <xsl:value-of select="$newid"/>
	  </xsl:attribute>
	</xsl:when>
	<xsl:when test=".=$id2">
	  <xsl:attribute name="spec">
	    <xsl:value-of select="$newid"/>
	  </xsl:attribute>
	  <xsl:attribute name="copy">
	    <xsl:value-of select="$id2"/>
	  </xsl:attribute>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:attribute name="spec">
	    <xsl:value-of select="."/>
	  </xsl:attribute>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <xsl:template match="@*[not(name()='spec')]" mode="fusion">
      <xsl:copy-of select="."/>
    </xsl:template>
</xsl:stylesheet>
