<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:sbml2="http://www.sbml.org/sbml/level2"
    xmlns:sbml23="http://www.sbml.org/sbml/level2/version3"
    xmlns:sbml24="http://www.sbml.org/sbml/level2/version4"
    xmlns:sbml31="http://www.sbml.org/sbml/level3/version1/core"
    xmlns:sbml32="http://www.sbml.org/sbml/level3/version2/core"
    xmlns:html="http://www.w3.org/1999/xhtml"
    xmlns:mathml="http://www.w3.org/1998/Math/MathML"
    xmlns:bqbiol="http://biomodels.net/biology-qualifiers"
    version="2.0"
    exclude-result-prefixes="xs bqbiol mathml rdf sbml2 sbml23 sbml24
			     sbml31 sbml32 html"
    >

  <xsl:output   
      method="xml" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>
  <xsl:param name="NETWORK"      required="no"/>
  <xsl:param name="CSBML-VERSION"       required="no"/>

  <xsl:variable name="model">
    <xsl:copy-of select="/"/>
  </xsl:variable>

  <xsl:variable name="amount">
    <xsl:text></xsl:text>
  </xsl:variable>
  <xsl:variable name="conc">
    <xsl:text>conc_</xsl:text>
  </xsl:variable>
  <xsl:variable name="DEBUG">false</xsl:variable>

  
  <xsl:template match="/">
    <xsl:message>°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°
°°°° SBML2Core.xsl</xsl:message>

    <xsl:variable name="network">
      <xsl:apply-templates select="*" mode="network"/>
    </xsl:variable>

    <xsl:variable name="clean-network">
    <xsl:message> +++ clean network </xsl:message>
    <!--<xsl:comment> +++ clean network </xsl:comment>-->
      <!-- remove reactions with arithmetic and differential species -->
      <xsl:apply-templates select="$network" mode="clean-network"/>
    </xsl:variable>

    <xsl:message> +++ add modifiers </xsl:message>

    <xsl:variable name="clean-network-modifier">
      <xsl:apply-templates select="$clean-network" mode="modifier"/>
    </xsl:variable>

   <xsl:variable name="clean-network-aux">
      <xsl:apply-templates select="$clean-network-modifier" mode="aux"/>
    </xsl:variable>
    <!--<xsl:message>
	network-aux <xsl:copy-of select="$clean-network-aux"/>
	</xsl:message>
    -->
    <xsl:result-document  method="xml" href="core-aux.xml">
      <xsl:copy-of select="$clean-network-aux"/>
    </xsl:result-document>
    <xsl:copy-of select="$clean-network-modifier"/>
  </xsl:template>

  <!-- global variable for input -->
  <xsl:variable name="sbml">
    <xsl:copy-of select="/"/>
  </xsl:variable>

  <!-- start -->
  <xsl:template match="*:sbml" mode="network">
    <xsl:if test="*:model/@id">
      <xsl:message>°°°° model.id=<xsl:value-of select="(*:model/@id) |
      (*:model/@id)"/> </xsl:message>
    </xsl:if>
    <xsl:if test="*:model/@name">
      <xsl:message>°°°° model.name=<xsl:value-of select="*:model/@name"/> </xsl:message>
    </xsl:if>
    <xsl:message>°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°
    </xsl:message>

    <xsl:if test="//*[not(ancestor::*:annotation or ancestor::*:notes)]
	      [not(self::sbml31:* or self::sbml32:* or self::sbml23:* or self::sbml24:* or self::sbml2:* or self::sbml32:*
	      or self::mathml:* or self::rdf:*)]">
  <xsl:message>  Error in SBML2Core: model contains elements with unsupported namespaces
      <xsl:copy-of select="//*[not(ancestor::*:annotation or ancestor::*:notes)]
	      [not(self::sbml32:* or self::sbml24:* or self::sbml2:* or self::sbml32:*
	      or self::mathml:* or self::rdf:*)]/name()"/>
  </xsl:message>
    </xsl:if>
    <xsl:variable name="network">
      <xsl:apply-templates select="*:model" mode="network"/>
    </xsl:variable>
    <xsl:copy-of select="$network"/>
   <!-- <xsl:apply-templates select="$network" mode="simplify"/>-->
  </xsl:template>
  
  <xsl:template match="*:model" mode="network">
    <network>
      <xsl:copy-of select="@id|@name"/>
      <xsl:attribute name="biomodels">
	<xsl:copy-of select="$NETWORK"/>
      </xsl:attribute>
      <xsl:attribute name="csbml-version">
	<xsl:copy-of select="$CSBML-VERSION"/>
      </xsl:attribute>
      
      <xsl:message> +++ start species +++++
      </xsl:message>
      <xsl:comment>
	+++ start species +++++
      </xsl:comment>
    <xsl:text/> 
      <xsl:apply-templates select="*:listOfSpecies/*:species"
			   mode="network"/>
      <xsl:message> +++ start reactions +++++</xsl:message>
      <xsl:comment> +++ start reactions +++++</xsl:comment>
     <xsl:apply-templates select="*:listOfReactions/*:reaction"   mode="network"/>
      <xsl:message> +++ start rateRules +++++</xsl:message>
      <!--<xsl:comment> +++ start rateRules +++++</xsl:comment>-->
      <xsl:apply-templates select="*:listOfRules/*:rateRule"         mode="network"/>
      <xsl:message> +++ start assignmentRules +++++</xsl:message>
      <!--<xsl:comment> +++ start assignmentRules +++++</xsl:comment>-->
      <xsl:apply-templates select="*:listOfRules/*:assignmentRule"
			                                                                              mode="network"/>
      <xsl:message> +++ start initialAssignments </xsl:message>
      <!--<xsl:comment> +++ start initialAssignments +++++ </xsl:comment>-->
      <xsl:apply-templates select="*:listOfInitialAssignments/*:initialAssignment"
      			                                                                             mode="network"/>
      <xsl:if test="*:listOfFunctionDefinitions/*:functionDefinition">
      <xsl:message> +++ start functionDefinitions </xsl:message>
      <xsl:comment> +++ start functionDefinitions +++++ </xsl:comment>
      </xsl:if>
      <xsl:apply-templates
	  select="*:listOfFunctionDefinitions/*:functionDefinition"  mode="network"/>
      <xsl:message> +++ start parameters +++++</xsl:message>
      <xsl:comment> +++ start parameters +++++ </xsl:comment>
      <xsl:apply-templates
	  select="*:listOfParameters/*:parameter"              mode="network"/>
      <xsl:message> +++ start compartments </xsl:message>
      <xsl:comment> +++ start compartments </xsl:comment>
      <xsl:apply-templates
	  select="*:listOfCompartments/*:compartment"  mode="network"/>
      <xsl:if test="*:listOfEvents/*:event">
	<xsl:message> +++ start events </xsl:message>
	<xsl:comment> +++ start events </xsl:comment>
      </xsl:if>
      <xsl:apply-templates
	  select="*:listOfEvents/*:event"                            mode="network"/>
 
     <xsl:if test="*[not(local-name()='listOfSpecies' or
		    local-name()='listOfReactions' or
		    local-name()='listOfRules' or
		    local-name()='listOfFunctionDefinitions' or
		    local-name()='listOfParameters' or
		    local-name()='listOfCompartments' or
		    local-name()='listOfUnitDefinitions' or
		    local-name()='listOfInitialAssignments' or
		    local-name()='notes' or
		    local-name()='annotation' or
		    local-name()='listOfEvents'
		    )]
		    ">
	<xsl:message>
	  Warning: top-level element not treated ++++++
	  <xsl:copy-of select="*[not(
			       local-name()='listOfSpecies' or
		    local-name()='listOfReactions' or
		    local-name()='listOfRules' or
		    local-name()='listOfFunctionDefinitions' or
		    local-name()='listOfParameters' or
		    local-name()='listOfCompartments' or
		    local-name()='listOfUnitDefinitions' or
		    local-name()='notes' or
		    local-name()='annotation' or
		    local-name()='listOfEvents'
		    )]/local-name()  "/>
	</xsl:message>
      </xsl:if>
      <xsl:if test="$DEBUG='true'"><xsl:message>
	+++ number of events <xsl:copy-of select="count(*:listOfEvents/*:event)"/>
      </xsl:message></xsl:if>
    </network>
  </xsl:template>

  <xsl:template match="text()|@*" mode="initial">
      <xsl:value-of select="concat('initial_',.)"/>
  </xsl:template>
  
  <xsl:template match="*:species" mode="network">
    <xsl:if test="$DEBUG='true'">
      <xsl:message> +++ start species <xsl:value-of select="@id"/> </xsl:message>
    </xsl:if>
    
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    <xsl:variable name="amount_id">
      <xsl:value-of select="concat($amount,@id)"/>
    </xsl:variable>
       <xsl:variable name="conc_id">
      <xsl:value-of select="concat('conc_',@id)"/>
    </xsl:variable>
    <xsl:variable name="look_amount_id">
      <xsl:choose>
	<xsl:when test="@name">
	  <xsl:apply-templates select="concat($amount,@name)"
			       mode="look"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:apply-templates select="$amount_id"      mode="look"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!--<xsl:variable name="pre_look_amount_id">
      <xsl:choose>
	<xsl:when test="@name">
	  <xsl:value-of select="concat($amount,@name)"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:value-of select="$amount_id"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="look_amount_id">
      <xsl:apply-templates  select="$pre_look_amount_id"  mode="look"/>
    </xsl:variable>-->

    <xsl:variable name="latex-amount">
      <xsl:if test="@name and not(@name=@id)">
	<look>
	  <xsl:attribute name="latex-look">
	    <xsl:apply-templates
		select="concat($amount,@name)"  mode="look"/>
	  </xsl:attribute>
	</look>
      </xsl:if>
    </xsl:variable>
    
    <xsl:variable name="pre_look_conc_id">
      <xsl:choose>
	<xsl:when test="@name">
	  <xsl:value-of select="concat('\CONC[',@name,']')"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:value-of select="concat('\CONC[',@id,']')"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="look_conc_id">
      <xsl:apply-templates  select="$pre_look_conc_id"  mode="look"/>
    </xsl:variable>

    <xsl:variable name="latex-conc">
      <xsl:choose>
	<xsl:when test="@name and not(@name=@id)">
	  <look>
	    <xsl:attribute name="latex-look">
	      <xsl:apply-templates
		  select="concat('\CONC[',@name,']')"  mode="look"/>
	    </xsl:attribute>
	  </look>
	</xsl:when>
	<xsl:otherwise>
	  <look>
	    <xsl:attribute name="latex-look">
	      <xsl:apply-templates
		  select="concat('\CONC[',@id,']')"  mode="look"/>
	    </xsl:attribute>
	  </look>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="compartment-id">
      <xsl:value-of select="@*:compartment"/>
    </xsl:variable>
    <xsl:variable name="type">
      <xsl:apply-templates select="." mode="type"/>
     </xsl:variable>
     <xsl:message>----- species=<xsl:value-of
     select="$id"/>  type=<xsl:value-of select="$type"/> 
    </xsl:message>
    
    <!-- for each species A make an equation for its amount variable A -->

    <xsl:variable name="compartment"> <!-- volume -->
      <xsl:copy-of select="$model//*:model/*:listOfCompartments
			   /*:compartment[@id=$compartment-id]" />
    </xsl:variable>

    <xsl:if test="count($compartment/*) &gt; 1">
      <xsl:message>  °°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°
	Error: compartment not unique <xsl:value-of
	select="$compartment-id"/>
	
	<xsl:copy-of select="$compartment"/>
      </xsl:message>
    </xsl:if>
    
    <xsl:variable name="initial_id">
      <xsl:apply-templates select="$id" mode="initial"/>
    </xsl:variable>

    <xsl:variable name="initass">
      <xsl:copy-of select="$model//*:listOfInitialAssignments/
			   *:initialAssignment[@symbol=$id]"/>
    </xsl:variable>
    
    <!-- if the current species A has an initialAssignment, then we store it
  	     in an expression with id=initial-A and refer to it in  @initial  -->

    <xsl:if test="$initass/*">
      <xsl:message> ° species <xsl:value-of select="$id"/> with initialAssignment       </xsl:message>
    </xsl:if>

    <xsl:variable name="initial_attribute">
      <xsl:choose>
	<xsl:when test="$model//*:listOfInitialAssignments/
			*:initialAssignment[@symbol=$id]">
	  <xsl:text>initial-expression</xsl:text>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:text>initial</xsl:text>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
     
    <xsl:variable name="initialAmount">  
      <xsl:choose>
	<xsl:when test="$model//*:listOfInitialAssignments/
			*:initialAssignment[@symbol=$id]">
	  <xsl:value-of select="$initial_id"/>
	</xsl:when>
	<xsl:when test="@*:initialConcentration">
	  <xsl:variable name="size">
	    <!-- Joachim: case with initialAssignment not yet treated -->
	    <xsl:choose>
	      <xsl:when test="$compartment/*:compartment/@*:size">
		<xsl:value-of select="$compartment/*:compartment/@*:size"/>
	      </xsl:when>
	      <xsl:otherwise>
		<xsl:text>1</xsl:text>
		<xsl:message> Warning: compartment without @size:
		- compartment <xsl:value-of select="$compartment-id"/>	for species <xsl:value-of select="$id"/>.
		- compartment=<xsl:copy-of select="$compartment"/>
		- take default @size=1.
		- this is equally done by Copasi and SBML2Latex.
		- but it seems unjustified by the SBML specification of
		neither on level 2 nor level 3.
		
		Warning: The reason may be that initial assignment of compartments are not yet treated.
		
		</xsl:message>
	      </xsl:otherwise>
	    </xsl:choose>
	  </xsl:variable>
	  
	  <xsl:variable name="iA">
	    <xsl:value-of select="@*:initialConcentration
				  *
				  $size"/>
	  </xsl:variable>
	  <xsl:copy-of select="$iA"/>
	  <xsl:if test="$DEBUG='true'">
	    <xsl:message> °°°°° initialAmount from initialConcentration °°°°°°
	    - species <xsl:copy-of select="$id"/>
	    - compartment-id=<xsl:copy-of select="$compartment-id"/>
	    - volume= <xsl:value-of select="$compartment/*:compartment/@*:size"/>
	    - initialConcentration = <xsl:value-of select="@*:initialConcentration"/>
	    - initialAmount= <xsl:value-of select="$iA"/>
	    </xsl:message>
	  </xsl:if>
	</xsl:when>
	
	<xsl:when test="@*:initialAmount">
	  <xsl:value-of select="@*:initialAmount" />
	  <xsl:if test="$DEBUG='true'">
	    <xsl:message> °°°°° initialAmount °°°°°°
	    - species <xsl:copy-of select="$id"/>
	    - initialAmount= <xsl:value-of select="@*:initialAmount"/>
	    </xsl:message>
	  </xsl:if>
	</xsl:when>
	
	<xsl:when test="@boundaryCondition='true'"/>
	<xsl:otherwise/>
      </xsl:choose>
    </xsl:variable>

    <xsl:choose>
      <!-- type=species:  no rule, but initialAssignment -->  
      <xsl:when  test="
		       not($id=$model//*:listOfRules/*:assignmentRule/@variable)
		       and not($id=$model//*:listOfRules/*:rateRule/@variable)
		       and $initass/*">

	<xsl:comment> species with initial assignment </xsl:comment>  
	<variable type="species" id="{$amount_id}">
		 <xsl:copy-of select="$latex-amount/look/@latex-look"/>
	  <xsl:attribute name="{$initial_attribute}">
  	    <xsl:value-of select="$initialAmount"/>
	  </xsl:attribute>
	  <xsl:attribute name="concentration">
  	    <xsl:value-of select="$conc_id"/>
	  </xsl:attribute>
	  <xsl:copy-of select="@compartment"/>
	</variable>
      </xsl:when>

      <!-- type=species:  no rule, but initial value -->
      <xsl:when  test="not(@boundaryCondition='true')
		       and not($id=$model//*:listOfRules/*:assignmentRule/@variable)
		       and not($id=$model//*:listOfRules/*:rateRule/@variable)
		       and (@*:initialAmount  or  @*:initialConcentration)">

	<xsl:comment> species without initial assignment </xsl:comment>  
	<variable type="species" id="{$amount_id}">
	  <xsl:copy-of select="$latex-amount/look/@latex-look"/>
	  <xsl:attribute name="{$initial_attribute}">
  	    <xsl:value-of select="$initialAmount"/>
	  </xsl:attribute>
	  <xsl:attribute name="concentration">
  	    <xsl:value-of select="$conc_id"/>
	  </xsl:attribute>
	  <xsl:copy-of select="@compartment"/>
	</variable>
      </xsl:when>

 
      <!-- type=algebraic: with assignmentRule  -->
      <xsl:when
	  test="(not(@*:boundaryCondition) or @*:boundaryCondition='true')
		and $id=$model//*:listOfRules/*:assignmentRule/@variable">
	<xsl:comment> species with assignmentRule </xsl:comment>
	<variable type="algebraic"  id="{$amount_id}">
	  <xsl:copy-of select="$latex-amount/look/@latex-look"/>
	  <xsl:attribute name="concentration">
  	    <xsl:value-of select="$conc_id"/>
	  </xsl:attribute>
	  <xsl:copy-of select="@compartment"/>
	  <kinetic-expression>
	    <xsl:apply-templates
		select="$model//*:listOfRules/*:assignmentRule[$id=@variable]/*:math/*"
		mode="math">
	      <xsl:with-param name="env"/>
	    </xsl:apply-templates>
	  </kinetic-expression>
	</variable>
      </xsl:when>

      <!-- type differential species without any rule -->
      <xsl:when test="@*:boundaryCondition='true'
		      and  not(@*:constant='true')
		      and  not(@id=$model//*:listOfRules/*:assignmentRule/@variable)
		      and  not(@id=$model//*:listOfRules/*:rateRule/@variable)" >
	
	<xsl:message> Comment in SBML2Core: species with boundaryCondition=true but no rule:
	- <xsl:value-of
	select="$id"/> we treat it as a differential species with constant rate.	  </xsl:message>

	<xsl:variable name="rateRule">
	  <rateRule comment="invented">
	    <xsl:attribute name="variable">
	      <xsl:value-of select="@id"/>
	    </xsl:attribute>
	    <math xmlns="http://www.w3.org/1998/Math/MathML">
	      <cn type="integer"> 0 </cn>
	    </math>
	  </rateRule>
	</xsl:variable>

	<xsl:if test="$initialAmount=''">
	  <xsl:message> ERROR: No initial value set for  species for species <xsl:value-of
	  select="$id"/>:  $initialAmount=''
	  </xsl:message>
	</xsl:if>

	<xsl:apply-templates select="$rateRule/rateRule"
			     mode="rateRule">
	  <xsl:with-param name="initial">
	    <xsl:value-of select="$initialAmount"/>
	  </xsl:with-param>
	  <xsl:with-param name="initial_attribute">
	    <xsl:value-of select="$initial_attribute"/>
	  </xsl:with-param>
	</xsl:apply-templates>
      </xsl:when>

      <!-- constant species without boundaryCondition="true" is really
	   constant -->
      <xsl:when test="@*:boundaryCondition='true' and  @*:constant='true'">
	<xsl:comment> constant variable without boundaryCondition="true" </xsl:comment>
	<variable type="differential" id="{$amount_id}">
	  <xsl:copy-of select="$latex-amount/look/@latex-look"/>
	  <xsl:attribute name="{$initial_attribute}">
  	    <xsl:value-of select="$initialAmount"/>
	  </xsl:attribute>
	  <xsl:attribute name="concentration">
  	    <xsl:value-of select="$conc_id"/>
	  </xsl:attribute>
	  <xsl:copy-of select="@compartment"/>
	  <kinetic-expression>
	    <constant value="0"/>
	  </kinetic-expression>
	</variable>
      </xsl:when>
      
      <!-- type differential species with rateRule -->
      <xsl:when test="not(@*:constant='true')
		      and  $id=$model//*:listOfRules/*:rateRule/@variable">

	<xsl:message> species with rateRule <xsl:value-of select="$id"/> </xsl:message>
	<xsl:comment> species with rateRule </xsl:comment>
	<variable type="differential"  id="{$amount_id}">
	  <xsl:copy-of select="$latex-amount/look/@latex-look"/>
	  <xsl:attribute name="{$initial_attribute}">
	    <xsl:value-of select="$initialAmount"/>
	  </xsl:attribute>
	  <xsl:attribute name="concentration">
  	    <xsl:value-of select="$conc_id"/>
	  </xsl:attribute>
	  <xsl:copy-of select="@compartment"/>
	  <kinetic-expression>
	    <xsl:apply-templates
		select="$model//*:listOfRules/*:rateRule[@variable=$id]/*:math/*"
		mode="math">
	      <xsl:with-param name="env"/>
	    </xsl:apply-templates>
	  </kinetic-expression>
	</variable>
      </xsl:when>

      <xsl:otherwise>
	<xsl:message>
	  Error in SBML2Core: Unexpected type of SBML species: no initial value nor any rule: @id=<xsl:value-of select="@id"/>
	</xsl:message>
	<xsl:comment> no initial concentration </xsl:comment>
	<var type="species">
	  <xsl:copy-of select="@id"/>
	   <xsl:attribute name="concentration">
  	    <xsl:value-of select="$conc_id"/>
	   </xsl:attribute>
	   <xsl:copy-of select="@compartment"/>
	</var>
      </xsl:otherwise>
    </xsl:choose>

    <expression id="{$conc_id}">
      <xsl:copy-of select="$latex-conc/look/@latex-look"/>
      <divide>
	<var type="{$type}" id="{$amount_id}"/>
	<xsl:choose>
	  <xsl:when test="$model//*:model/*:listOfRules
			  /*:rateRule[@variable=$compartment-id]">
	    <!-- <xsl:comment>  compartment with rateRule </xsl:comment>-->
		<var type="differential" id="{$compartment-id}"/>
	  </xsl:when>
	      <xsl:when
		  test="$model//*:listOfEventAssignments/
			*:eventAssignment[@*:variable=$compartment-id] ">
		<var type="control" id="{$compartment-id}"/>
	      </xsl:when>
    	      <xsl:otherwise>
		<!--<xsl:comment>  compartment without rateRule nor event </xsl:comment> -->
		<expr  id="{$compartment-id}"/>
	      </xsl:otherwise>
	</xsl:choose>
      </divide>
    </expression>
  </xsl:template>
  
  <!--compartment of constant size-->
  <xsl:template match="*:compartment[not(@*:constant='false')
		       or not(@id=$model//*:listOfRules/*/@id)]" mode="network">

    <xsl:variable name="id">
      <xsl:value-of select="@*:id"/>
    </xsl:variable>

     <xsl:variable name="latex">
      <xsl:choose>
	<xsl:when test="@name and not(@name=@id)">
	  <look>
	    <xsl:attribute name="latex-look">
	    <xsl:apply-templates select="@name" mode="look"/>
	    </xsl:attribute>
	  </look>
	</xsl:when>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="initial_id">
      <xsl:apply-templates select="$id" mode="initial"/>
    </xsl:variable> 

    <xsl:if test="$model//*:listOfInitialAssignments/*:initialAssignment[@symbol=$id]">
      <xsl:if test="$DEBUG='true'">
	<xsl:message> +++++ compartment of constant size with intialAssignment <xsl:value-of
	  select="$id"/>
	</xsl:message>
	<xsl:comment> compartment of constant size with
	initialAssignment (done before)<xsl:value-of
	  select="$id"/>
	</xsl:comment>
      </xsl:if>
    </xsl:if>

    <xsl:variable name="initial">  
      <xsl:choose>
	<xsl:when test="$model//*:listOfInitialAssignments/
			*:initialAssignment[@symbol=$id]">
	  <xsl:value-of select="$initial_id"/>
	</xsl:when>
	<xsl:when test="@*:initial">
	  <xsl:value-of select="@*:initial"/>
	</xsl:when>
	
	<xsl:otherwise/>
      </xsl:choose>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="$model//*:listOfRules/*[@variable=$id]">
	<xsl:message> ° dynamic volume by equations <xsl:value-of select="$id"/>
	</xsl:message>
      </xsl:when>
      <xsl:when
	  test="$model//*:listOfEventAssignments/*:eventAssignment
		[@*:variable=$id]">
      	<xsl:message> ° dynamic volume controlled by events <xsl:value-of
	select="$id"/> </xsl:message>
	<xsl:message> dynamic volume controlled by events </xsl:message>
	<xsl:comment> dynamic volume controlled by events </xsl:comment>
	<xsl:choose>
	  <xsl:when test="$model//*:listOfInitialAssignments/
				       *:initialAssignment[@symbol=$id]">
	    <variable type="control"
		      id="{$id}"
		      initial-expression="{$initial_id}">
	      <xsl:copy-of select="$latex/look/@latex-look"/>
	      </variable>
	      <xsl:comment> take 2 </xsl:comment>
	      <expression>
		<xsl:attribute name="id">
		  <xsl:value-of select="$initial_id"/>
		</xsl:attribute>
	      <xsl:apply-templates select="$model//*:listOfInitialAssignments/
					   *:initialAssignment[@symbol=$id]/*:math/*" mode="math"/>
	    </expression>
	  </xsl:when>
	  <xsl:otherwise>
	    <variable type="control" id="{$id}">
	      <xsl:copy-of select="$latex/look/@latex-look"/>
	      <xsl:attribute name="initial">
		<xsl:value-of select="@size"/>
	      </xsl:attribute>
	    </variable>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:when>
      <xsl:otherwise>
	<xsl:message> ° compartment <xsl:value-of select="$id"/>
	</xsl:message>
	<xsl:comment>constant compartment </xsl:comment>
	
	<xsl:choose>
	  <xsl:when test="@*:size">
	    <expression id="{$id}">
	      <xsl:copy-of select="$latex/look/@latex-look"/>
	      <constant> 
		<xsl:attribute name="value">
		  <xsl:value-of select="@*:size"/>
		</xsl:attribute>
	      </constant>
	    </expression>
	  </xsl:when>
	  <xsl:when test="$model//*:listOfInitialAssignments/*[@symbol=$id]">
	    <expression id="{$id}">
	      <xsl:copy-of select="$latex/look/@latex_look"/>
	      <expr id="{$initial_id}"/>
	    </expression>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:comment>default compartment volume 1</xsl:comment>
	    <expression id="{$id}">
	      <xsl:copy-of select="$latex/look/@latex-look"/>
	      <constant value="1"/>
	    </expression>
	    <xsl:message>Warning in SBML2Core:
	    - constant compartment missing size <xsl:value-of
	    select="@id"/>
	    - apply default size=1
	    </xsl:message>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:otherwise>
	
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*:reaction" mode="network">
    <xsl:if test="$DEBUG='true'">
      <xsl:message> +++ start reaction <xsl:value-of select="@id"/>
      </xsl:message>
    </xsl:if>
    <reaction>
      <xsl:copy-of select="@id"/>
      <xsl:if test="not(*:kineticLaw)">
	<xsl:message> Warning: kineticLaw missing in reaction <xsl:value-of
	    select="@id"/></xsl:message>
      </xsl:if>
     <!-- <xsl:comment> start kineticLaw </xsl:comment>-->
      <xsl:apply-templates select="*:kineticLaw"
			   mode="network"/>
      <!--<xsl:comment> start reactants </xsl:comment>-->
      <xsl:apply-templates select="*:listOfReactants" mode="network"/> 
      <!--<xsl:comment> start products </xsl:comment>-->
      <xsl:apply-templates select="*:listOfProducts"  mode="network"/>
      <xsl:apply-templates select="*:listOfModifiers" mode="network"/>
    </reaction>
  </xsl:template>

  <xsl:template match="*:functionDefinition" mode="network">
    <!--<xsl:message>
	+++ functionDefintion <xsl:value-of select="@*:id"/>
	</xsl:message>-->
    
    
    <function>
      <xsl:copy-of select="@*:id"/>
      <xsl:apply-templates select="*:math/*" mode="math"/>
    </function>
  </xsl:template>

  <xsl:template match="*:initialAssignment" mode="network">
    <xsl:variable name="id">
      <xsl:value-of select="@*:symbol"/>
    </xsl:variable>
    <xsl:message>  ° initialAssignment for <xsl:value-of  select="$id"/></xsl:message>
    <xsl:choose>
	<xsl:when test="$model//*:listOfSpecies/*:species[@id=$id]">
	  <xsl:if test="$DEBUG='true'">
	    <xsl:message> initialAssignment for species <xsl:value-of
	    select="$id"/>
	    </xsl:message>
	  </xsl:if>
	  <xsl:comment> initialAssignment for species <xsl:value-of  select="$id"/>
	  </xsl:comment>
	</xsl:when>

	<xsl:when test="$model//*:listOfCompartments/*:compartment[@id=$id]">
	 <!-- <xsl:message> initialAssignment for compartment <xsl:value-of
	  select="$id"/>
	  </xsl:message>-->
	  <xsl:comment> initialAssignment for compartment <xsl:value-of
	  select="$id"/>
	  </xsl:comment>
	</xsl:when>

	<xsl:when test="$model//*:listOfParameters/*:parameter[@id=$id]">
	  <xsl:if test="$DEBUG='true'">
	    <xsl:message> initialAssignment for parameter <xsl:value-of
	    select="$id"/>
	    </xsl:message>
	  </xsl:if>
	  <xsl:comment> initialAssignment for parameter <xsl:value-of
	    select="$id"/>
	    </xsl:comment>
	</xsl:when>

	<xsl:otherwise>
	  <xsl:message> Warning in SBML2Core: initialAssignment for object with is
	  neither a species, a compartment nor a parameter:
	  - id=<xsl:value-of  select="$id"/>
	  </xsl:message>
	   <xsl:comment> Warning in SBML2Core: initialAssignment for object with is
	  neither a species, a compartment nor a parameter:
	  - id=<xsl:value-of  select="$id"/>
	  </xsl:comment>
	</xsl:otherwise>
    </xsl:choose>

    <xsl:variable name="result">
      <expression>
	<xsl:attribute name="id">
	  <xsl:apply-templates select="$id" mode="initial"/>
	</xsl:attribute>
	<xsl:attribute name="latex-look">
	  <xsl:apply-templates select="concat('initial_',$id)" mode="look"/>
	</xsl:attribute>
	<xsl:apply-templates select="*:math/*" mode="math"/>
      </expression>
    </xsl:variable>
    <!--<xsl:message>
      ++++ math=<xsl:copy-of select="*:math"/>
      ++++ result=<xsl:copy-of select="$result"/>
    </xsl:message>-->
    <xsl:copy-of select="$result"/>
  </xsl:template>
  
  <xsl:template match="*:parameter|*:localParameter" mode="network">
    <!--<xsl:comment> ° parameter <xsl:value-of select="@id"/></xsl:comment>-->
    <xsl:variable name="id">
      <xsl:value-of select="@*:id"/>
    </xsl:variable>
    <xsl:variable name="conc_id">
      <xsl:value-of select="concat($conc,$id)"/>
    </xsl:variable>
    
    <xsl:variable name="initial_id">
      <xsl:value-of select="concat('initial_',$id)"/>
    </xsl:variable>
    <xsl:variable name="initass">
      <xsl:copy-of select="$model//*:listOfInitialAssignments/*:initialAssignment[@symbol=$id]"/>
    </xsl:variable>
     <xsl:variable name="assrule">
      <xsl:copy-of select="$model//*:listOfRules/*:assignmentRule[@variable=$id]"/>
    </xsl:variable>
    
    <xsl:if test="$initass/*">
     <xsl:if test="$DEBUG='true'">
       <xsl:message> parameter with initialAssignment <xsl:value-of
       select="$id"/>
       </xsl:message>
     </xsl:if>
      <xsl:comment>   with initialAssignment
      </xsl:comment>
      <!--<expression>
	<xsl:attribute name="id">
	  <xsl:value-of select="$initial_id"/>
	</xsl:attribute>
	<at_zero>
	  <xsl:apply-templates
	      select="$initass/*:initialAssignment/*:math/*"
	      mode="math"/>
	</at_zero>
      </expression>-->
    </xsl:if>
    
    <xsl:variable name="initial">  
      <xsl:choose>
	<xsl:when test="$assrule/*">
	  <xsl:comment> parameter has assignmentRule, so no initial value is needed </xsl:comment>
	</xsl:when>
	<xsl:when test="$initass/*">
	  <xsl:value-of select="$initial_id"/>
	</xsl:when>
	<xsl:when test="@value">
	  <xsl:value-of select="@value"/>
	</xsl:when>	
	<xsl:otherwise>
	  <xsl:message> Error in SBML2Core:
	  - parameter without initial value and assignmentRule
	  <xsl:value-of select="$id"/>
	  <!--
	     $assrule=<xsl:value-of select="$assrule"/>
	     $initass=<xsl:value-of select="$initass"/>
	     -->
	  </xsl:message>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:choose>
      <xsl:when test="
		      $model//*:listOfEventAssignments/*:eventAssignment[@*:variable=$id]
		      and not($model//*:listOfSpecies/*:species[@*:id=$id])
		      and not($model//*:listOfCompartments/*:compartment[@*:id=$id])
		      and not($model//*:listOfRules/*:rateRule[@*:variable=$id])">

      <!-- control variable:
	   a variable that is updated by some event,
	   but not a species nor any rule -->
      
	<variable type="control" id="{$id}">
	  <xsl:attribute name="initial">
	    <xsl:value-of select="$initial"/>
	  </xsl:attribute>
	</variable>
    </xsl:when>

    <!-- nonconstant parameter with some rule-->
    <xsl:when test="not(@*:constant='true')"> 
      <!--<xsl:comment>   nonconstant </xsl:comment>-->

      <xsl:variable name="rule">
	<xsl:copy-of select="$model//*:listOfRules/*[@variable=$id]"/>
      </xsl:variable>

      <!--<xsl:if test="$rule/*">
	<xsl:message>	++++ parameter rule(<xsl:value-of select="$id"/>) =
	<xsl:copy-of select="$rule"/> </xsl:message>
      </xsl:if>-->
      <xsl:choose>
	  <!-- assignment rule -->
	  <xsl:when test="$rule/*:assignmentRule">
	    <xsl:comment>   with assignment rule </xsl:comment>
	    <expression id="{$id}">  
	      <xsl:if test="$DEBUG='true'">
		<xsl:message>  assignment rule
		  - id=<xsl:value-of select="@id"/>
		  - rule=<xsl:copy-of select="$rule/*:assignmentRule/*:math"/>
		</xsl:message>
	      </xsl:if>
	      <xsl:apply-templates
		  select="$rule/*:assignmentRule/*:math/*"  mode="math">
		<xsl:with-param name="env"/>
	      </xsl:apply-templates>
	    </expression>
	  </xsl:when>

	  <!-- rate rule -->
	  <xsl:when test="$rule/*:rateRule">
	    <xsl:comment>   with rateRule but no species (so will be translated with rateRules)  </xsl:comment>
	  </xsl:when>

	  <!-- no rule (hidden zero rate rule) -->
 	  <xsl:when test="not(ancestor::*:kineticLaw)">
	    <!--<xsl:comment> nonconstant nonlocal parameters with
		no rule  are constant</xsl:comment>-->

	    <expression id="{$id}">
	      <xsl:choose>
		<xsl:when test="$initass/*">
		  <expr id="{$initial_id}"/>
		</xsl:when>
		<xsl:when test="@value">
		  <constant>
		    <xsl:attribute name="value">
		    <xsl:value-of select="@value"/>
		    </xsl:attribute>
		  </constant>
		</xsl:when>	
		<xsl:otherwise>
		  <xsl:message> Error in SBML2Core:
		  - nonconstant parameter without initial value and assignmentRule
		  <xsl:value-of select="$id"/>
		  </xsl:message>
		</xsl:otherwise>
	      </xsl:choose>
	    </expression>

	    <!-- JJJ 17 June 2023
	    <variable type="differential" id="{$id}">
	      <xsl:choose>
		<xsl:when test="$initass/*">
		  <xsl:attribute name="initial-expression">
		    <xsl:value-of select="$initial_id"/>
		  </xsl:attribute>
		</xsl:when>
		<xsl:when test="@value">
		  <xsl:attribute name="initial">
		    <xsl:value-of select="@value"/>
		  </xsl:attribute>
		</xsl:when>	
		<xsl:otherwise>
		  <xsl:message> Error in SBML2Core:
		  - nonconstant parameter without initial value and assignmentRule
		  <xsl:value-of select="$id"/>
		  </xsl:message>
		</xsl:otherwise>
	      </xsl:choose>
	      <kinetic-expression>
		<constant value="0"/>
	      </kinetic-expression>
	    </variable>
	    -->
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:comment> local parameter </xsl:comment>
	    <expression id="{$id}"> 
	      <constant>
		<xsl:copy-of select="@value"/>
	      </constant>
	    </expression>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:when>
      
      <!-- constant parameter -->
      <xsl:otherwise>
	<xsl:variable name="id">
	  <xsl:value-of select="@*:id"/>
	</xsl:variable>
	<xsl:comment> - constant </xsl:comment>
	<xsl:choose>
	  <xsl:when test="$assrule/*">
	    <xsl:comment> parameter has assignmentRule,
	    so no initial value is needed </xsl:comment>
	  </xsl:when>
	  <xsl:when test="$initass/*">
	    <expression id="{$id}"> 
	      <xsl:comment> constant parameter with initialAssigment </xsl:comment>
	      <expr id="{$initial_id}"/>
	    </expression>
	  </xsl:when>
	  <xsl:when test="@value">
	    <xsl:comment> constant parameter without intialAssigment </xsl:comment>
	      <expression id="{$id}"> 
	      <constant>
		<xsl:copy-of select="@value"/>
	      </constant>
	    </expression>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:message> Error in SBML2Core:
	    - constant parameter without initial value and  assignmentRule <xsl:value-of select="$id"/>
	  <!--
	     $assrule=<xsl:value-of select="$assrule"/>
	     $initass=<xsl:value-of select="$initass"/>
	     -->
	  </xsl:message>
	</xsl:otherwise>
	</xsl:choose>
      </xsl:otherwise>

    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*:event" mode="network">
    
    <event>
      <xsl:copy-of select="@id"/>
      <xsl:apply-templates select="*:trigger" mode="network"/> 
      <xsl:apply-templates select="*:listOfEventAssignments"
			   mode="network_aux"/>
    <xsl:if test="*:delay">
      <xsl:variable name="delay">
	<xsl:apply-templates select="*:delay" mode="network"/> 
      </xsl:variable>
      <xsl:choose>
	<xsl:when test="$delay/delay/constant[@value='0']">
	  <xsl:message>Info: delay 0 in event: ignored. </xsl:message>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:message>Warning in SBML2Core:
	  - nonzero delay in SBML event
	  </xsl:message>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:if>
    </event>
    <xsl:variable name="else">
      <elements>
	<xsl:copy-of select="*[not(
			     local-name()='trigger' or
			     local-name()='listOfEventAssignments' or
			     local-name()='annotation' or
			     local-name()='delay' or
			     local-name()='notes')]"/>
      </elements>
    </xsl:variable>
    <xsl:if test="not(count($else/elements/*)=0)">
      <xsl:message>Error in SBML2Core:
	-  element of event not treated <xsl:copy-of select="$else/elements/*/local-name()"/>
      </xsl:message>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="*:trigger" mode="network">
    <condition>
      <xsl:apply-templates select="*:math/*"  mode="math"/>
    </condition>
  </xsl:template>
  
  <xsl:template match="*:delay" mode="network">
    <delay>
      <xsl:apply-templates select="*:math/*"  mode="math"/>
    </delay>
  </xsl:template>

  <xsl:template match="*:listOfEventAssignments" mode="network_aux">
    <xsl:apply-templates select="*:eventAssignment"  mode="network_aux"/>
  </xsl:template>

  <xsl:template match="*:eventAssignment" mode="network_aux">
    <update>
      <xsl:variable name="variable">
	<xsl:value-of select="@*:variable"/>
      </xsl:variable>
      <xsl:if test="$DEBUG='true'">
	<xsl:message>
	  ++++ update ++++++++
	  variable=<xsl:value-of select="@*:variable"/>
	  species=<xsl:copy-of select="$model//*:listOfSpecies
	  /*:species[@id=$variable]"/>
	</xsl:message>
      </xsl:if>
      <xsl:choose>
	<xsl:when
	    test="$model//*:listOfSpecies/*:species[@id=$variable]
		  [@boundaryCondition='true']
	          and not($model//*:listOfRules/*[@id=@variable])">
	  <xsl:comment> species with boundaryCondition but no rule </xsl:comment>
	  <var type="differential">
	    <xsl:attribute name="id">
	      <xsl:value-of select="@variable"/>
	    </xsl:attribute>
	  </var>
	</xsl:when>
	<xsl:when test="$model//*:listOfRules/*:rateRule[@variable=$variable]">
	  <xsl:comment> rateRule </xsl:comment>
	  <var type="differential">
	    <xsl:attribute name="id">
	      <xsl:value-of select="@variable"/>
	    </xsl:attribute>
	  </var>
	</xsl:when>
	<xsl:when test="$model//*:listOfRules/*:assignmentRule[@variable=$variable]">
	  <xsl:comment> assignmentRule for compartment</xsl:comment>
	  <expr>
	    <xsl:attribute name="id">
	      <xsl:value-of select="@variable"/>
	    </xsl:attribute>
	  </expr>
	</xsl:when>
	
	<xsl:when test="$model//*:listOfEventAssignments
			/*:eventAssignment[@variable=$variable] and
			not ($model//*:listOfSpecies/*:species[@id=$variable])">
	  <xsl:comment> no rule but event </xsl:comment>
	      <var type="control">
	    <xsl:attribute name="id">
	      <xsl:value-of select="@variable"/>
	    </xsl:attribute>
	  </var>
	</xsl:when>
	<xsl:when
	    test="$model//*:listOfSpecies/*:species[@id=$variable]
		  		  [not(@boundaryCondition='true')]">
	  <xsl:comment> species without boundaryCondition</xsl:comment>
	  <var type="species">
	    <xsl:attribute name="id">
	      <xsl:value-of select="@variable"/>
	    </xsl:attribute>
	  </var>
	</xsl:when>	<xsl:when test="$model//*:listOfCompartments
			/*:compartment[@id=$variable]">
	  <xsl:comment> compartment without rule nor event</xsl:comment>
	  <var type='differential'>
	    <xsl:attribute name="id">
	      <xsl:value-of select="@variable"/>
	    </xsl:attribute>
	  </var>
	</xsl:when>

	
	<xsl:when test="$model//*:listOfCompartments/*:compartment[@id=$variable]">
	  <xsl:comment> constant compartment </xsl:comment>	  
	  <var type='differential'>
	   <xsl:attribute name="id">
	     <xsl:value-of select="@variable"/>
	   </xsl:attribute>
	 </var>
	</xsl:when>	<xsl:otherwise>
	  <expr>
	    <xsl:attribute name="id">
	      <xsl:value-of select="@variable"/>
	    </xsl:attribute>
	  </expr>
	</xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*:math/*"  mode="math"/>
    </update>
  </xsl:template>
  
  <xsl:template  match="*:rateRule[@*:variable=
			$model//*:parameter/@*:id]" mode="network">
    <xsl:variable name="variable">
      <xsl:value-of select="@variable"/>
    </xsl:variable>
    <xsl:message>  ° rateRule for parameter <xsl:value-of select="@variable"/>
    </xsl:message>

   <xsl:variable name="parameter">
     <xsl:copy-of  select="$model//*:parameter[@*:id=$variable]"/>
   </xsl:variable>
   
   <xsl:variable name="initass">
     <xsl:copy-of  select="$model//*:initialAssignment[@*:symbol=$variable]"/>
   </xsl:variable>
   
    <xsl:variable name="initial">
    	<xsl:choose>
	<xsl:when test="$initass/*:initialAssignment">
	  <xsl:apply-templates
	      select="$initass/*:initialAssignment/@symbol"
	      mode="initial"/>
	</xsl:when>
	<xsl:when test="$parameter/*:parameter/@value">
	  <xsl:value-of select="$parameter/*:parameter/@value"/>
	</xsl:when>
	<xsl:otherwise>
	    <xsl:message>  Error in Core2SBML: parameter with rateRule
	    but no value nor initialAssignment  <xsl:value-of
	    select="@variable"/>
	    </xsl:message>
	</xsl:otherwise>
	</xsl:choose>
    </xsl:variable>


    <xsl:variable name="initial_attribute">
    	<xsl:choose>
	<xsl:when test="$initass/*:initialAssignment">
	  <xsl:text>initial-expression</xsl:text>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:text>initial</xsl:text>	    
	</xsl:otherwise>
	</xsl:choose>
    </xsl:variable>

    <xsl:if test="$initial=''">
      <xsl:message> ERROR in SBML2Core:
           initial for <xsl:value-of select="$variable"/> empty $initial='' 
      </xsl:message>
    </xsl:if>
    
    <xsl:apply-templates select="." mode="rateRule">
      <xsl:with-param name="initial">
	<xsl:value-of select="$initial"/>
      </xsl:with-param>
      <xsl:with-param name="initial_attribute">
	<xsl:value-of select="$initial_attribute"/>
      </xsl:with-param>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="*:rateRule[@*:variable=
		       $model//*:compartment[@*:constant='false']/@*:id]" mode="network">
    <xsl:variable name="variable">
      <xsl:value-of select="@variable"/>
    </xsl:variable>
      <xsl:message> ° rateRule for compartment <xsl:value-of select="@variable"/>
      </xsl:message>
    <xsl:variable name="size">
      <xsl:choose>
	<xsl:when
	    test="$model//*:initialAssignment[@*:symbol=$variable]">
	  <xsl:message>   with initialAssignment  </xsl:message>
	  <xsl:value-of select="concat('initial_',$variable)"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:value-of select="$model//*:compartment[@*:id=$variable]/@size"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="initial_attribute">
      <xsl:choose>
	<xsl:when test="$model//*:initialAssignment[@*:symbol=$variable]">
	  <xsl:text>initial-expression</xsl:text>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:text>initial</xsl:text>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$size=''">
      <xsl:message> ERROR:  size for <xsl:value-of
      select="$variable"/> is empty $size=<xsl:value-of select="$size"/> 
      </xsl:message>
    </xsl:if>
      
  
    <xsl:apply-templates select="." mode="rateRule">
      <xsl:with-param name="initial">
	<xsl:value-of select="$size"/>
      </xsl:with-param>
       <xsl:with-param name="initial_attribute">
	<xsl:value-of select="$initial_attribute"/>
      </xsl:with-param>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="*:rateRule[@*:variable=
		       $model//*:species/@*:id]" mode="network">
    <xsl:message> rate rules for species <xsl:value-of
    select="@*:variable"/> already treated, so ignore here </xsl:message>
  <xsl:comment> rate rules for species <xsl:value-of
    select="@*:variable"/> already treated, so ignore here </xsl:comment>
  </xsl:template>
  
  <xsl:template match="*:rateRule" mode="network">
       <xsl:message> Error in SBML2Core:
    - rateRule for nonspecies, nonparameter,  and noncompartment:  <xsl:copy-of
	select="."/>
    </xsl:message>
  </xsl:template>

  <!-- mode rateRule -->
  <xsl:template match="*:rateRule" mode="rateRule">
    <xsl:param name="initial"/>
    <xsl:param name="initial_attribute"/>
    <xsl:if test="$initial=''">
      <xsl:message>  Error in SBML2Core
      -  mode rateRule for <xsl:value-of select="@variable"/> has param  $initial = ""
      <xsl:if test="$DEBUG='true'">
	- <xsl:copy-of select="."/>
      </xsl:if>
      </xsl:message>
    </xsl:if>
    <xsl:if test="$initial_attribute=''">
      <xsl:message>  Error in SBML2Core
      -  mode rateRule for <xsl:value-of select="@variable"/> has param  $initial_attribute= ""
      <xsl:if test="$DEBUG='true'">
	- <xsl:copy-of select="."/>
      </xsl:if>
      </xsl:message>
    </xsl:if>
    <xsl:variable name="id">
      <xsl:value-of select="@variable"/>
    </xsl:variable>
     <xsl:variable name="conc_id">
      <xsl:value-of select="concat($conc,$id)"/>
    </xsl:variable>
    <xsl:if test="$DEBUG='true'">
      <xsl:message> translate rateRule for<xsl:value-of
      select="$id"/> to differential variable
      <xsl:copy-of select="."/></xsl:message>
    </xsl:if>
    <xsl:message> +++++++++++compartment ? +++++
    <xsl:copy-of select="."/>
    compartment=<xsl:value-of select="$model//*:listOfSpecies/*:species[@id=$id]/@compartment"/>
    </xsl:message>
    <xsl:choose>
      <xsl:when test="$model//*:listOfCompartments/*:compartment[@id=$id]">
	<xsl:comment> rateRule for compartment <xsl:value-of select="$id"/></xsl:comment>
      </xsl:when>
       <xsl:when test="$model//*:listOfParameters/*:parameter[@id=$id]">
	 <xsl:comment> rateRule for parameter <xsl:value-of select="$id"/> </xsl:comment>
       </xsl:when>
       <xsl:when test="$model//*:listOfSpecies/*:species[@id=$id]">
	       <xsl:comment> species with rateRule </xsl:comment>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:comment> rateRule for object <xsl:value-of
	 select="$id"/> which is neither a species, compartment nor parameter</xsl:comment>
       </xsl:otherwise>
    </xsl:choose>
    <variable type="differential">
      <xsl:attribute name="id">
	<xsl:value-of select="concat($amount,@variable)"/>
      </xsl:attribute>
      <xsl:copy-of select="$model//*:listOfSpecies/*:species[@id=$id]/@compartment"/>
      <xsl:attribute name="latex-look">
	<xsl:value-of select="@variable"/>
      </xsl:attribute>
      <xsl:attribute name="{$initial_attribute}">
	<xsl:value-of select="$initial"/>
      </xsl:attribute>
      <xsl:if test="$model//*:listOfSpecies/*:species[@id=$id]">
	<xsl:attribute name="concentration">
  	  <xsl:value-of select="$conc_id"/>
	</xsl:attribute>
	<xsl:copy-of select="@compartment"/>
      </xsl:if>
      <kinetic-expression>
	<xsl:apply-templates select="*:math/*" mode="math"/>
      </kinetic-expression>
    </variable>
  </xsl:template>
  
  <xsl:template match="*:kineticLaw" mode="network">
    <kinetic-expression>
      <xsl:apply-templates select="*:math/*" mode="math"/>
    </kinetic-expression>
    <xsl:apply-templates select="*:listOfParameters/*:parameter
				 |*:listOfLocalParameters/*:localParameter"
			 mode="network"/>
    <xsl:if test="*[not(
		  local-name()='math' or
		  local-name()='annotation' or
		  local-name()='listOfParameters' or
		  local-name()='listOfLocalParameters' or
		  local-name()='notes')]">
      <xsl:message>
	kineticLaw else <xsl:copy-of select="*[not(
	local-name()='math' or
	local-name()='annotation' or
	local-name()='listOfParameters' or
	local-name()='listOfLocalParameters'  )]"/>
      </xsl:message>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="*:listOfReactants" mode="network">
    <xsl:for-each select="*:speciesReference">
      <reactant>
	<xsl:attribute name="spec">
	  <xsl:value-of select="concat($amount,@species)"/>
	</xsl:attribute>
	<xsl:apply-templates select="." mode="stoichiometry"/>
      </reactant>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="*:speciesReference" mode="stoichiometry">
    <xsl:if test="@*:stoichiometry and not(@*:stoichiometry='1')">
      <xsl:message>  ° stoichiometry(<xsl:value-of
      select="@species"/>)=<xsl:value-of select="@stoichiometry"/> </xsl:message>
      <xsl:attribute name="stoichiometry">
	<xsl:value-of select="@stoichiometry"/>
      </xsl:attribute>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="*:listOfProducts" mode="network">
    <xsl:for-each select="*[local-name()='speciesReference']">
      <product>
	<xsl:attribute name="spec">
	  <xsl:value-of select="concat($amount,@*:species)"/>
	</xsl:attribute>
	<xsl:apply-templates select="." mode="stoichiometry"/>
      </product>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="*:listOfModifiers" mode="network">
    <xsl:for-each select="*:modifierSpeciesReference">
      <modifier>
	<xsl:attribute name="spec">
	  <xsl:value-of select="concat($amount,@species)"/>
	</xsl:attribute>
      </modifier>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="*:assignmentRule" mode="network">
     
     <xsl:message>  ° assignmentRule for <xsl:value-of select="@variable"/>
     </xsl:message>
    
     <xsl:choose>
       <xsl:when test="@variable=//*:listOfSpecies/*:species/@id">
	<!-- <xsl:comment>  ° assignmentRule for species <xsl:value-of
     select="@variable"/> </xsl:comment>-->
	 <!--done with species
	     <xsl:comment> assignmentRule </xsl:comment>
	     <expression>
	     <xsl:attribute name="spec">
	     <xsl:value-of select="@variable"/>
	     </xsl:attribute>
	     </expression>-->
       </xsl:when>
       <xsl:when
	   test="@variable=//*:listOfParameters/*:parameter[@constant='false']/@id">
	 <!-- <xsl:comment> ° assignmentRule for parameter <xsl:value-of
     select="@variable"/> </xsl:comment>-->
	 <!-- done with parameter
	      <expression>
	    <xsl:attribute name="id">
	      <xsl:value-of select="@variable"/>
	    </xsl:attribute>
	  </expr>-->
	</xsl:when>
	<xsl:when
	    test="@variable=//*:listOfCompartments/*:compartment[@constant='false']/@id">
	  <xsl:comment> ° assignmentRule for compartment <xsl:value-of
     select="@variable"/> </xsl:comment>
	  <!--<xsl:message>
	     +++++ assignment rule for compartment <xsl:value-of select="@variable"/> 
	  </xsl:message>-->
	  <expression>
	    <xsl:attribute name="id">
	      <xsl:value-of select="@variable"/>
	    </xsl:attribute>
	    <xsl:apply-templates select="*:math/*" mode="math"/>
	  </expression>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:message>
	   ++++++++  Error in SBML2Core:
	    - assignment rule for <xsl:value-of select="@variable"/>
	    - but <xsl:value-of select="@variable"/> is neither a
	    species, a compartment, nor a parameter
	  </xsl:message>
	</xsl:otherwise>
      </xsl:choose>
  </xsl:template>

  <xsl:template match="rdf:* | *:notes | *:annotation |
		       *:listOfUnitDefinitions " mode="network">
    <xsl:if test="$DEBUG='true'">
      <xsl:message> - remove "<xsl:value-of select="name()"/>"
    </xsl:message></xsl:if>
  </xsl:template>

  <xsl:template match="*:piecewise[count(*:piece)&gt;1]" mode="math">
        <xsl:param name="env"/>

    <!--<xsl:message>
	+++ piecwise +++++
	- #piece=<xsl:copy-of select="count(*:piece)"/>
	</xsl:message>-->
    <xsl:variable name="next-piecewise">
      <mathml:piecewise>
	<xsl:copy-of select="*[position()&gt;1]" />
      </mathml:piecewise>
    </xsl:variable>
    <!--<xsl:message>
	+++  next-piecewise
	<xsl:copy-of select="$next-piecewise"/>
	+++ #piece=<xsl:copy-of select="count($next-piecewise/*:piecewise/*:piece)"/>
    </xsl:message>-->
    <ifthenelse>
      <xsl:apply-templates select="*:piece[1]/*[2]" mode="math">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="*:piece[1]/*[1]" mode="math">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="$next-piecewise/*:piecewise"
			   mode="math">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </ifthenelse>
  </xsl:template>
  
  <xsl:template match="*:piecewise[count(*:piece)=1]" mode="math">
    <xsl:param name="env"/>

    <!--<xsl:message>
      piecewise with enviroment:
      <xsl:copy-of select="$env"/>
    </xsl:message>-->
    <ifthenelse>
      <xsl:apply-templates select="*:piece/*[2]" mode="math">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="*:piece/*[1]" mode="math">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
      <xsl:choose>
	<xsl:when test="*:otherwise">
	  <xsl:apply-templates select="*:otherwise/*[1]"
			       mode="math">
	    <xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
	</xsl:when>
	<xsl:otherwise>
	  <divide>
	    <constant value="1"/>
	    <constant value="0"/>
	  </divide>
	</xsl:otherwise>
      </xsl:choose>
    </ifthenelse>
  </xsl:template>


  <xsl:template match="*:csymbol" mode="math">
    <xsl:param name="env"/>
    <xsl:choose>
      <xsl:when
	  test="@*:definitionURL='http://www.sbml.org/sbml/symbols/time'">
	<time/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:message>SBML2Core Warning:
	- csymbol not treated: <xsl:copy-of select="."/>
	- replaced by dummy constant
	</xsl:message>
	<constant id="csymbol-dull-one" value="1"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*
		       [not(starts-with(local-name(),'listOf'))]
		       [not(local-name()='species')]
		       [not(local-name()='reaction')]
		       [not(local-name()='parameter')]
		       [not(local-name()='localParameter')]
		       [not(local-name()='event')]
		       [not(local-name()='delay')]
		       [not(local-name()='initialAssignment')]
		       [not(local-name()='eventAssignment')]
		       [not(local-name()='trigger')]
		       [not(local-name()='sbml')]
		       [not(local-name()='model')]
		       [not(local-name()='compartment')]
		       [not(local-name()='functionDefinition')]
		       [not(local-name()='kineticLaw')]
		       [not(local-name()='rateRule')]
		       [not(local-name()='assignmentRule')]
		       [not(local-name()='csymbol')]
		       [not(local-name()='notes')]
		       [not(local-name()='annotation')]
		       [not(self::rdf:*)]		       "
		mode="network">
    <xsl:message>SBML2Core Warning:
    - network else: "<xsl:copy-of  select="name()"/>" 
    </xsl:message>
    <xsl:apply-templates select="*" mode="network"/>
  </xsl:template>

  

  <!-- function aux -->
  
  <xsl:template match="@*|text()|comment()" mode="aux">
    <xsl:copy-of select="."/>
  </xsl:template>
  
  <xsl:template match="*
		       [not(local-name()='kinetic-expression')]
		       [not(local-name()='event')]
		       [not(local-name()='function')]
		       [not(local-name()='expression')]
		       [not(local-name()='reaction')]
		       [not(local-name()='modifier')]
		       [not(local-name()='parameter')]
		       [not(local-name()='species')]
		       [not(local-name()='variable')]" mode="aux">
    <!--<xsl:message> aux <xsl:value-of select="name()"/>
	</xsl:message>-->
    <xsl:variable name="name">
      <xsl:value-of select="name()"/>
    </xsl:variable>
    <xsl:element name="{$name}">
      <xsl:apply-templates
	  select="*|@*| text()|comment()" mode="aux"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="*:variable[not(@type='differential')]" mode="aux">
    <metabolite>
      <xsl:apply-templates select="." mode="positions"/>
      <xsl:apply-templates select="@*[not(name()='type')]"             mode="aux"/>
    </metabolite>
  </xsl:template>

  <xsl:template match="*"  mode="positions">
    <xsl:if test="not(@x)">
      <xsl:attribute name="x">0</xsl:attribute>
    </xsl:if>
    <xsl:if test="not(@y)">
      <xsl:attribute name="y">0</xsl:attribute>
    </xsl:if>
  </xsl:template>

  <xsl:template match="*:variable[@type='differential']" mode="aux">
    <metabolite>
      <xsl:apply-templates select="." mode="positions"/>
      <xsl:apply-templates select="@*[not(name()='type')]"             mode="aux"/>
    </metabolite>
    <reaction>
      <xsl:apply-templates select="." mode="positions"/>
      <xsl:attribute name="id">
	<xsl:value-of select="concat('reaction_',@id)"/>
      </xsl:attribute>
      <product>
	<xsl:attribute name="spec">
	<xsl:value-of select="@id"/>
      </xsl:attribute>
      </product>
    </reaction>
  </xsl:template>

<!--  <xsl:template match="*:reactant| *:product" mode="aux">
    <xsl:copy-of select="."/>
  </xsl:template>-->
  
  <xsl:template match="*:modifier" mode="aux">
    <activator>
      <xsl:apply-templates select="@*"             mode="aux"/>
    </activator>
  </xsl:template>

  <xsl:template match="*:reaction" mode="aux">
    <reaction>
      <xsl:apply-templates select="." mode="positions"/>
      <xsl:apply-templates select="@*|*|text()|comment()"             mode="aux"/>
    </reaction>
  </xsl:template>
  
  <xsl:template match="*:kinetic-expression|function|expression|event" mode="aux"/>

  <xsl:template match="@*:initialConcentration" mode="aux"/>
  
  <!-- function apply-->

  <xsl:template match="*:lambda[*[1][local-name()='bvar']]"  mode="apply">
    <xsl:param name="args"/>
    <xsl:if test="$DEBUG='true'">
      <xsl:message>   mode apply
      - function = <xsl:copy-of select="."/>
      - args = <xsl:copy-of select="$args/*"/>
      - firstarg = <xsl:copy-of select="$args/*:args/*[1]"/>
      </xsl:message>
    </xsl:if>
    <xsl:variable name="argument">
      <xsl:copy-of select="$args/*:args/*[1]"/>
    </xsl:variable>
    <xsl:variable name="nextfun">
      <mathml:lambda>
	<xsl:apply-templates select="*[position()&gt;1]"  mode="substitute">
	  <xsl:with-param name="variable">
	    <xsl:copy-of select="*[1]"/>
	  </xsl:with-param>
	  <xsl:with-param name="argument">
	    <xsl:copy-of select="$argument"/>
	  </xsl:with-param>
	</xsl:apply-templates>
      </mathml:lambda>
    </xsl:variable>
    <xsl:variable name="nextargs">
      <args>
	<xsl:copy-of select="$args/*:args/*[position() &gt; 1]"/>
      </args>
    </xsl:variable>
    <xsl:if test="$DEBUG='true'">
      <xsl:message>   - nextargs = <xsl:copy-of select="$nextargs"/>
      </xsl:message>
    </xsl:if>
    <xsl:apply-templates select="$nextfun"    mode="apply">
      <xsl:with-param name="args">
	<xsl:copy-of select="$nextargs"/>
      </xsl:with-param>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="*:lambda[*[1][not(local-name()='bvar')]]"
		mode="apply">
    <!--<xsl:message>
	end-of-apply: <xsl:copy-of select="*"/>
	</xsl:message>-->
    <xsl:copy-of select="*"/>    
  </xsl:template>

  <xsl:template match="*:ci"  mode="substitute">
    <xsl:param name="variable"/>
    <xsl:param name="argument"/>
    <!--   <xsl:message> substitute <xsl:value-of
	 select="normalize-space($variable)"/> in "<xsl:value-of
	 select="normalize-space(.)"/>"
	 </xsl:message>-->
    <xsl:choose>
      <xsl:when test="normalize-space($variable)=normalize-space(.)">
	<xsl:if test="$DEBUG='true'">
	  <xsl:message> successful [<xsl:value-of
	  select="normalize-space($variable)"/>/<xsl:copy-of select="$argument"/>]
	  </xsl:message>
	</xsl:if>
	<xsl:copy-of select="$argument"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:copy-of select="."/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="*[not(local-name()='ci')]"  mode="substitute">
    <xsl:param name="variable"/>
    <xsl:param name="argument"/>
    <!--
	<xsl:message> substitute <xsl:value-of
	select="normalize-space($variable)"/> in <xsl:value-of select="name()"/> 
	</xsl:message>
    -->
    <xsl:variable name="name">
      <xsl:value-of select="name()"/>
    </xsl:variable>
    
    <xsl:element name="{$name}">
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates select="*|text()" mode="substitute">
	<xsl:with-param name="variable">
	  <xsl:value-of select="$variable"/>
	</xsl:with-param>
	<xsl:with-param name="argument">
	  <xsl:copy-of select="$argument"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </xsl:element>
  </xsl:template>

  <xsl:template match="text()"  mode="substitute">
    <xsl:copy-of select="."/>
  </xsl:template>

  <!--
      function math
  -->
  
 
  <xsl:template match="*:apply[*:xor]" mode="math">
    <xsl:param name="env"/>
    <and>
      <or>
	<xsl:apply-templates select="*[position()&gt;1]"
			     mode="showmath">
	  <xsl:with-param name="env">
	    <xsl:copy-of select="$env"/>
	  </xsl:with-param>
	</xsl:apply-templates>
      </or>
      <not>
	<and>
	  <xsl:apply-templates select="*[position()&gt;1]"
			       mode="showmath">
	    <xsl:with-param name="env">
	      <xsl:copy-of select="$env"/>
	    </xsl:with-param>
	  </xsl:apply-templates>
	</and>
      </not>
    </and>
  </xsl:template>
  
  <xsl:template match="*:apply[*:ln]" mode="math">
    <xsl:param name="env"/>
    <log>
      <constant id="e" value="2.71828"/>
      <xsl:apply-templates select="*[position()&gt;1]"
			   mode="showmath">
	<xsl:with-param name="env">
	    <xsl:copy-of select="$env"/>
	  </xsl:with-param>
	</xsl:apply-templates>
    </log>
  </xsl:template>

  <xsl:template match="*:apply[*:log]" mode="math">
    <xsl:param name="env"/>
    <log>
      <xsl:apply-templates select="*:logbase/*" mode="showmath">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="*[position()&gt;2]"
			   mode="showmath">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </log>
  </xsl:template>
   
  <xsl:template match="*:apply[*[1]
		       [
		       local-name()='sin' or
		       local-name()='cos' or
		       local-name()='tan' or
		       local-name()='sinh' or
		       local-name()='cosh' or 
		       local-name()='tanh' or
		       local-name()='ceiling' or
		       local-name()='floor' or
		       local-name()='leq' or
		       local-name()='lt' or
		       local-name()='eq' or
		       local-name()='plus' or
		       local-name()='minus' or
		       local-name()='times' or
		       local-name()='divide' or
		       local-name()='power' or 
		       local-name()='and' or
		       local-name()='or'
		       ]]" mode="math">
    <xsl:param name="env"/>
    <xsl:variable name="name">
      <xsl:copy-of select="*[1]/local-name()"/>
    </xsl:variable>
    <xsl:element name="{$name}">
      <xsl:apply-templates select="*[position()&gt;1]"
			   mode="showmath">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="*:apply[*:abs]" mode="math">
    <xsl:param name="env"/>
    <xsl:comment> apply function abs </xsl:comment>
    <ifthenelse>
      <leq>
	<xsl:apply-templates select="*[2]" mode="showmath">
	  <xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
	<constant id="0" value="0"/>
      </leq>
      <times>
	<constant id="-1" value="-1"/>
	<xsl:apply-templates select="*[2]" mode="showmath">
	  <xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
      </times>
      <xsl:apply-templates select="*[2]" mode="showmath">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </ifthenelse>
  </xsl:template>
  
  <xsl:template match="*:pi" mode="math">
    <constant id="pi" value="3.141592653"/>
  </xsl:template>
 

  <xsl:template match="*:apply[*:geq]" mode="math">
    <xsl:param name="env"/>
    <leq>
      <xsl:apply-templates select="*[3]" mode="showmath">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="*[2]" mode="showmath">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </leq>
  </xsl:template>
  
  <xsl:template match="*:apply[*:gt]" mode="math">
    <xsl:param name="env"/>
    <lt>
      <xsl:apply-templates select="*[3]" mode="showmath">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="*[2]" mode="showmath">
	<xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </lt>
  </xsl:template>

  <xsl:template match="*:apply[*:neq]" mode="math">
    <xsl:param name="env"/>
    <not>
      <eq>
	<xsl:apply-templates select="*[position()&gt;1]"
			     mode="showmath">
	  <xsl:with-param name="env">
	    <xsl:copy-of select="$env"/>
	  </xsl:with-param>
      </xsl:apply-templates>
      </eq>
    </not>
  </xsl:template>

  <xsl:template match="*:true" mode="math">
    <xsl:param name="env"/>
    <true/>
  </xsl:template>

  <xsl:template match="*:apply[*:exp]" mode="math">
    <xsl:param name="env"/>	
    <power>
      <constant id="e" value="2.71828"/>
      <xsl:apply-templates select="*[position()&gt;1]"
			   mode="showmath">
	<xsl:with-param name="env">
	    <xsl:copy-of select="$env"/>
	  </xsl:with-param>
      </xsl:apply-templates>
    </power>
  </xsl:template>
  
  <xsl:template match="*:apply[*:root][*:degree[*:cn]]"   mode="math">
    <xsl:param name="env"/>	
    <xsl:variable name="div-degree">
      <xsl:value-of select="1 div *:degree/*:cn/text()"/>
    </xsl:variable>
    <xsl:if test="$DEBUG='true'">
      <xsl:message>
	power div-degree <xsl:copy-of select="$div-degree"/>
    </xsl:message></xsl:if>
    <xsl:variable name="out">
      <power>
	<xsl:apply-templates select="*[3]" mode="math">
	  <xsl:with-param name="env">
	    <xsl:copy-of select="$env"/>
	  </xsl:with-param>
      </xsl:apply-templates>
	<constant id="div-degree">
	  <xsl:attribute name="value">
	    <xsl:value-of select="$div-degree"/>
	  </xsl:attribute>
	</constant>
      </power>
    </xsl:variable>
    <!-- <xsl:message> - out=<xsl:copy-of select="$out"/>
	 </xsl:message>-->
    <xsl:copy-of select="$out"/>
  </xsl:template>

  <xsl:template match="*" mode="showmath">
    <xsl:param name="env"/>
    <xsl:if test="$DEBUG='true'">
      <xsl:message> showmath <xsl:value-of
      select="name()"/>.<xsl:value-of select="normalize-space()"/>.<xsl:value-of select="*[1]/name()"/>
    </xsl:message></xsl:if>
    <xsl:apply-templates select="." mode="math">
      <xsl:with-param name="env">
	  <xsl:copy-of select="$env"/>
	</xsl:with-param>
      </xsl:apply-templates>
  </xsl:template>
  

  <!-- apply a function definition -->
  <xsl:template match="*:apply[*[1][local-name()='ci']]" mode="math">
    <xsl:param name="env"/>
    <xsl:variable name="id">
      <xsl:value-of select="normalize-space(*[1])"/>
    </xsl:variable>
    <!--<xsl:comment> apply function "<xsl:value-of select="$id"/> </xsl:comment>-->
    <xsl:if test="$DEBUG='true'">
      <xsl:message>  apply function "<xsl:value-of select="normalize-space(*[1])"/>"
      <xsl:copy-of select="$model//*:listOfFunctionDefinitions/
			   *:functionDefinition[normalize-space(@id)=$id]"/>
      </xsl:message>
   </xsl:if>
    <xsl:variable name="formal-arguments">
      <args>
	<xsl:copy-of select="$model//*:listOfFunctionDefinitions/
			     *:functionDefinition[normalize-space(@id)=$id]
			     /*:math/*:lambda/*:bvar"/>
      </args>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="count($formal-arguments/args/*:bvar)
		      =count(*[position()&gt; 1])">
	<xsl:if test="$DEBUG='true'">
	  <xsl:message> function type correct </xsl:message>
	</xsl:if>
      </xsl:when>
      <xsl:otherwise>
	<xsl:message> type error in function application
	<xsl:copy-of select="."/>
	- number of formal arguments = <xsl:value-of
	select="count($formal-arguments/args/*:bvar)"/>
	- number of passed arguments = <xsl:value-of
	select="count(*[position()&gt; 1])"/>
	function name <xsl:value-of select="$id"/>
	function definition: jo
	<xsl:copy-of select="$model//*:listOfFunctionDefinitions/
			     *:functionDefinition[normalize-space(@id)=$id]"/>-->
	<xsl:copy-of select="$model//*:listOfFunctionDefinitions/
			     *:functionDefinition[normalize-space(@id)=$id]"/>
	</xsl:message>
      </xsl:otherwise>
    </xsl:choose>
    <!--<xsl:variable name="apply-result">
	<xsl:apply-templates select="$model//*:listOfFunctionDefinitions/
	*:functionDefinition[normalize-space(@id)=$id]/
				   *:math/*:lambda" mode="apply">
				   <xsl:with-param name="args">
				   <args>
				   <xsl:copy-of select="*[position()&gt; 1]"/>
				   </args>
				   </xsl:with-param>
				   </xsl:apply-templates>
				   </xsl:variable>
				   <xsl:if test="$DEBUG='true'">
				   <xsl:message>
				   apply-result=<xsl:copy-of select="$apply-result"/>
				   </xsl:message>
				   </xsl:if>
				   <xsl:variable name="translation">
				   <xsl:copy-of select="$apply-result"/>
				   </xsl:variable>
				   <xsl:if test="$DEBUG='true'">
				   <xsl:message>
				   translation=<xsl:copy-of select="$translation"/>
				   </xsl:message></xsl:if>
				   <xsl:copy-of select="$translation"/>-->
    <apply>
      <xsl:attribute name="fun">
	  <xsl:value-of select="$id"/>
	</xsl:attribute>
	<xsl:apply-templates select="*[position()&gt; 1]" mode="math">
	  <xsl:with-param name="env">
	    <xsl:copy-of select="$env"/>
	  </xsl:with-param> 
	</xsl:apply-templates>
      </apply>
  </xsl:template>

  <xsl:template match="*:apply" mode="math">
    <xsl:param name="env"/>
    <xsl:choose>
      <xsl:when
	  test="*[1][local-name()='csymbol'      and
		@definitionURL='http://www.sbml.org/sbml/symbols/delay']">
	<!--<xsl:message>Error in SBML2Core:	  <xsl:if test="$DEBUG='true'">- &lt; csymbol
	  definitionURL=http://www.sbml.org/sbml/symbols/delay'</xsl:if>
	- delays in differential equations are not allowed in Core
	SBML
	</xsl:message>-->
	<delay>
	  <xsl:apply-templates select="*[position()&gt; 1]" mode="math">
	    <xsl:with-param name="env">
	      <xsl:copy-of select="$env"/>
	    </xsl:with-param> 
	  </xsl:apply-templates>
	</delay>
      </xsl:when>
      <xsl:otherwise>
	<xsl:message>SMBL2Core error:
	- application of unknown function <xsl:copy-of
	select="."/></xsl:message>

	<xsl:comment> unknown function <xsl:value-of select="normalize-space(*[1])"/>
	</xsl:comment>
	<constant value="unknown-function-dummy">
	  <xsl:attribute name="id">
	    <xsl:value-of select="normalize-space(*[1])"/>
	  </xsl:attribute>
	</constant>
	
      </xsl:otherwise>
    </xsl:choose>
   
  </xsl:template>

  <xsl:template match="text()" mode="e-notation">
    <xsl:value-of select="replace(normalize-space(.),' ','')"/>
  </xsl:template>
  <xsl:template match="*:sep" mode="e-notation">
    <xsl:text>e</xsl:text>
  </xsl:template>
  
  <xsl:template match="*:cn" mode="math">
   <!-- <xsl:message>
	cn <xsl:copy-of select="normalize-space(.)"/>
    </xsl:message>-->
    <xsl:choose>
      <xsl:when test="@type='integer' or not(@type)">
	<constant>
	  <xsl:attribute name="value">
	    <xsl:value-of select="normalize-space(.)"/>
	  </xsl:attribute>
	</constant>
      </xsl:when>
      <xsl:when test="@type='e-notation'">
	<xsl:comment> e-notation, may be to be improved </xsl:comment>
	<constant>
	  <xsl:attribute name="value">
	    <xsl:apply-templates select="text()|*" mode="e-notation"/>
	  </xsl:attribute>
	</constant>
      </xsl:when>
      
      <xsl:otherwise>
	<xsl:message> Error in SBML2Core ?
	- element cn  may be not properly treated:
	<xsl:copy-of select="."/></xsl:message>
	<xsl:value-of select="."/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="*:lambda" mode="math">
    <lambda>
      <xsl:apply-templates select="*" mode="math">
	<xsl:with-param name="env">
	  <xsl:apply-templates select="*:bvar" mode="math"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </lambda>
  </xsl:template>

  <xsl:template match="*:bvar" mode="math">
    <xsl:param name="env"/>
    <bvar>
      <xsl:attribute name="id">
	<xsl:value-of select="normalize-space(*:ci/text())"/>
      </xsl:attribute>
    </bvar>
  </xsl:template>
  
  <xsl:template match="*:ci" mode="math">
    <xsl:param name="env"/>
    <xsl:variable name="id">
      <xsl:value-of select="normalize-space(string(.))"/>
    </xsl:variable>
    <!-- JN -->
    <xsl:variable name="conc_id">
      <xsl:value-of select="concat($conc,$id)"/>
    </xsl:variable>
 
    <xsl:variable name="bvar">
      <xsl:if test="not($env instance of xs:string)">
	<xsl:copy-of select="$env/bvar[@id=$id]"/>
      </xsl:if>
    </xsl:variable>
    <!--<xsl:message>
      environment in ci <xsl:copy-of select="text()"/> 
      <xsl:copy-of select="$env"/>
    </xsl:message>-->
    <xsl:variable name="event">
      <xsl:copy-of select="$sbml//*:listOfEventAssignments
			   /*:eventAssignment[@variable=$id]"/>
    </xsl:variable>
    <xsl:variable name="local-parameter">
      <xsl:copy-of
	  select="ancestor::*:kineticLaw/((*:listOfParameters/*:parameter)
		  | (*:listOfLocalParameters/*:localParameter) )[@id=$id]
		  "/>
    </xsl:variable>
    <xsl:variable name="global-parameter">
      <xsl:copy-of  select="$sbml//*:listOfParameters/*:parameter[@id=$id]"/>
    </xsl:variable>
    
    <xsl:variable name="species">
      <xsl:copy-of select="$sbml//*:listOfSpecies/*:species[@id=$id]"/>
    </xsl:variable>

    <xsl:variable name="type">
      <xsl:apply-templates select="$species" mode="type"/>
     </xsl:variable>
     
    <xsl:variable name="rule">
      <xsl:copy-of select="$sbml//*:listOfRules/*[@variable=$id]"/>
    </xsl:variable>

    <xsl:variable name="reaction">
      <xsl:copy-of select="$sbml//*:listOfReactions/*:reaction[@id=$id]"/>
    </xsl:variable>

    <xsl:variable name="compartment">
      <xsl:copy-of select="$sbml//*:listOfCompartments/*:compartment[@id=$id]"/>
    </xsl:variable>
    
    <xsl:if test="$DEBUG='true'">
    <xsl:message>°°° ci °°°
      - id="<xsl:value-of select="$id"/>"
      - local parameter <xsl:copy-of
      select="$local-parameter/(*:parameter|*:localParameter)"/>
      - global parameter <xsl:copy-of select="$global-parameter/*:parameter"/>
      - species <xsl:copy-of select="$species/*:species"/>
      - reaction <xsl:copy-of select="$reaction/*:reaction"/>
      - assignmentRule <xsl:copy-of select="$rule/*:assignmentRule"/>
      - rateRule <xsl:copy-of select="$rule/*:rateRule"/>
      - compartment <xsl:copy-of select="$compartment/*:compartment"/>
      </xsl:message>
    </xsl:if>
    
    <xsl:choose>
      <xsl:when test="$bvar/*:bvar[@id=$id]">
	<!-- <xsl:message>  variable bound in environment
	     </xsl:message> 
	<xsl:comment>   variable bound in environment   </xsl:comment>-->
	<var>
	  <xsl:attribute name="id">
	    <xsl:value-of select="normalize-space(.)"/>
	  </xsl:attribute>
	</var>
      </xsl:when>
      
      <xsl:when test="$species/*:species[@id=$id]">
	<xsl:choose>
	  <!-- not finished. right place? -->
	  <xsl:when test="$species/*:species[@id=$id]/@hasOnlySubstanceUnits='true'">
	    <var type="{$type}" id="{$id}"/>
	  </xsl:when>	
	  <xsl:otherwise>
	    <expr id="{$conc_id}"/>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:when>
      <xsl:when test="$reaction/*:reaction[@id=$id]">
	<!--<xsl:comment>  reaction speed (kinetic expression) </xsl:comment>-->
	<speed>
	  <xsl:attribute name="reaction">
	    <xsl:value-of select="normalize-space(.)"/>
	  </xsl:attribute>
	</speed>
	<!--<xsl:comment> end  reaction speed </xsl:comment>-->
      </xsl:when>

      <xsl:when test="$rule/*:rateRule[@variable=$id]">
	<xsl:comment>  parameter with rateRule </xsl:comment>
	<var type="differential">
	  <xsl:attribute name="id">
	    <xsl:value-of select="normalize-space(.)"/>
	  </xsl:attribute>
	</var>
      </xsl:when>

      <xsl:when
	  test="$global-parameter/*:parameter[@id=$id][@constant='false']
		and not($rule/*:rateRule[@variable=$id])
		and not($event/*:eventAssignment[@variable=$id])">
	<xsl:comment>  nonconstant parameter without rule nor event</xsl:comment>
	<expr>
	  <xsl:attribute name="id">
	    <xsl:value-of select="normalize-space(.)"/>
	  </xsl:attribute>
	</expr>
      </xsl:when>
      
      <xsl:when test="$event/*:eventAssignment[@variable=$id]">
	<xsl:comment>  control parameter  </xsl:comment>
	<var type="control">
	  <xsl:attribute name="id">
	    <xsl:value-of select="normalize-space(.)"/>
	  </xsl:attribute>
	</var>
      </xsl:when>

      <xsl:when test="ancestor::*:lambda/*:bvar/*:ci/
		      normalize-space()=$id">
	<var id="{$id}"/>
      </xsl:when>
 
      <xsl:when  test="$local-parameter/(*:parameter|*:localParameter)">
	<xsl:comment> constant local expression <xsl:value-of
	select="$id"/></xsl:comment>
	<expr>
	  <xsl:attribute name="id">
	    <xsl:value-of select="normalize-space($id)"/>
	  </xsl:attribute>
	</expr>
      </xsl:when>


      <xsl:when  test="$global-parameter/*:parameter[@*:constant='true']">
	<xsl:comment> constant global expression </xsl:comment>
	<expr>
	  <xsl:attribute name="id">
	    <xsl:value-of select="normalize-space($id)"/>
	  </xsl:attribute>
	</expr>
      </xsl:when>

      <xsl:when
	  test="$global-parameter/*:parameter[not(@*:constant='true')]
		[not($rule/*)]">
	<!--<xsl:comment> nonconstant parameter without rule
	    is constant</xsl:comment>-->
	<expr>
	  <xsl:attribute name="id">
	    <xsl:value-of select="normalize-space($id)"/>
	  </xsl:attribute>
	</expr>
      </xsl:when>

    <xsl:when  test="$global-parameter/*:parameter[@*:constant='false']">
	<xsl:choose>
	  <xsl:when test="$sbml//*:listOfRules
			  /*:rateRule[@variable=$id]">
	    <xsl:comment>  global parameter with rate rule </xsl:comment>
	    <var type="differential">
	      <xsl:attribute name="id">
		<xsl:value-of select="normalize-space(.)"/>
	      </xsl:attribute>
	    </var>
	  </xsl:when>
	  <xsl:when test="$sbml//*:listOfRules
			  /*:assignmentRule[@variable=$id] and
			  $sbml//*:listOfSpecies/*:species[@id=$id] ">
	    <xsl:comment> algebraic variable </xsl:comment>
	    <var type="algebraic">
	      <xsl:attribute name="id">
		<xsl:value-of select="normalize-space(.)"/>
	      </xsl:attribute>
	    </var>
	  </xsl:when>
	  
	  <xsl:when test="$sbml//*:listOfRules
			  /*:assignmentRule[@variable=$id] and
			  not($sbml//*:listOfSpecies/*:species[@id=$id]) ">
	    <xsl:comment> with assignment rule, but not a species</xsl:comment>
	    <expr>
	      <xsl:attribute name="id">
		<xsl:value-of select="normalize-space(.)"/>
	      </xsl:attribute>
	    </expr>
	  </xsl:when>
	  
	  <xsl:when test="$sbml//*:listOfEventAssignments
			  /*:eventAssignment[@variable=$id]">
	    <!--<xsl:message>  control parameter <xsl:value-of
		select="$id"/> </xsl:message>-->
	    <var type="control">
	      <xsl:attribute name="id">
		<xsl:value-of select="normalize-space(.)"/>
	      </xsl:attribute>
	    </var>
	  </xsl:when>
	  
	  <xsl:otherwise>
	    <!--
		nonconstant parameter with no rule is constant
		(differential species with rate 0)
	    -->
	    <xsl:message>    Warning: nonconstant parameter with no rule
	      <xsl:value-of select="$id"/>
	      - is constant (differential species with rate 0)
	    </xsl:message>
	    <xsl:comment> nonconstant parameter with no rule </xsl:comment>
	    <var type="differential" id="{$id}"/>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:when> <!-- end of parameter -->
      
      <xsl:when test="$compartment/*:compartment[@id=$id]">
	<xsl:choose>
	  <xsl:when test="$sbml//*:listOfRules
			  /*:assignmentRule[@variable=$id]">
	    <!--<xsl:comment> compartment with assignmentRule </xsl:comment>-->
	    <expr id="{$id}"/>
	  </xsl:when>
	  <xsl:when test="$sbml//*:listOfRules
			  /*:rateRule[@variable=$id]">
	    <xsl:comment> compartment with rateRule </xsl:comment>
	    <var type="differential" id="{$id}"/>
	  </xsl:when>
	  <xsl:otherwise>
	    <!--<xsl:comment> compartment without assignmentRule </xsl:comment>-->
	    <expr id="{$id}"/>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:when>
      
      <xsl:otherwise>
	<xsl:message> +++++ SBML2Core Warning +++++
	- constant without value: <xsl:value-of
	select="normalize-space(.)"/>
	---- compiled without
	</xsl:message>
	<constant id="otherwise">
	  <xsl:attribute name="value">
	    <xsl:value-of select="normalize-space(.)"/>
	  </xsl:attribute>
	</constant>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>
  
  <xsl:template match="*
		       [not(local-name()='and')]
		       [not(local-name()='lt')]
		       [not(local-name()='gt')]
		       [not(local-name()='leq')]
		       [not(local-name()='eq')]
		       [not(local-name()='true')]
		       [not(local-name()='plus')]
		       [not(local-name()='times')]
		       [not(local-name()='minus')]
		       [not(local-name()='divide')]
		       [not(local-name()='lambda')]
		       [not(local-name()='csymbol')]
		       [not(local-name()='power')]
		       [not(local-name()='ci')]
		       [not(local-name()='cn')]
		       [not(local-name()='pi')]
		       [not(local-name()='apply')]
		       [not(local-name()='bvar')]
		       [not(local-name()='piecewise')] "  mode="math">
    <xsl:message>SBML2Core warning
    - math else:  "<xsl:copy-of  select="name()"/>" 
    </xsl:message>
    <xsl:apply-templates select="*" mode="math"/>
  </xsl:template>

  <!-- function simplify -->
  <xsl:template match="*:constant[@value='1']
		       [parent::*:times]" mode="simplify">
    <xsl:comment> simplify constant=1 </xsl:comment>
  </xsl:template>

  <xsl:template match="*" mode="simplify">
    <xsl:variable name="tag">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>
    <xsl:element name="{$tag}">
      <xsl:apply-templates select="@*|*|comment()|text()" mode="simplify"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="@*|
		       comment()|text()" mode="simplify">
    <xsl:copy-of select="."/>
  </xsl:template>

  <!-- function clean-networks -->

  <xsl:template
      match="product  | modfier | reactant" mode="clean-network">
    <xsl:choose>
      <xsl:when test="@spec=//variable/@id and
		      not(@spec=//variable[@type='algebraic' or
		      @type='differential']/@id)">
	<xsl:copy-of select="."/>
      </xsl:when>
      <xsl:when test="not(@spec=//variable/@id)">
	<xsl:message>  Comment in SBML2Core: variable with boundaryCondition=true occurs in reaction
	   - remove <xsl:value-of select="@spec"/>  from reaction <xsl:value-of select="parent::reaction/@id"/>
	</xsl:message>
      </xsl:when>
      <xsl:otherwise>
	<xsl:message>  Comment in SBML2Core:  variable with boundaryCondition=true occurs in reaction
	  - remove <xsl:value-of select="@spec"/>  from reaction <xsl:value-of select="parent::reaction/@id"/>
	</xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*" mode="clean-network">
    <xsl:variable name="tag">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>
    <xsl:element name="{$tag}">
      <xsl:apply-templates select="@*|*|comment()|text()" mode="clean-network"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="@*|
		       comment()|text()" mode="clean-network">
    <xsl:copy-of select="."/>
  </xsl:template>

  <!-- function look -->
  <xsl:variable name="triple_prime">
    <xsl:text>'''</xsl:text>
  </xsl:variable>
  <xsl:variable name="double_prime">
    <xsl:text>''</xsl:text>
  </xsl:variable>
  <xsl:variable name="prime">
    <xsl:text>'</xsl:text>
  </xsl:variable>
  <xsl:variable name="sigma">
    <xsl:text>\\ensuremath{\\sigma}</xsl:text>
  </xsl:variable>
   <xsl:variable name="beta">
    <xsl:text>\\ensuremath{\\beta} </xsl:text>
   </xsl:variable>
    <xsl:variable name="alpha">
    <xsl:text>\\ensuremath{\\alpha} </xsl:text>
    </xsl:variable>
    <xsl:variable name="kappa">
    <xsl:text>\\ensuremath{\\kappa}  </xsl:text>
    </xsl:variable>
    <xsl:variable name="gamma">
    <xsl:text>\\ensuremath{\\gamma} </xsl:text>
    </xsl:variable>
  <xsl:variable name="underscore">
    <xsl:text>\\_</xsl:text>
  </xsl:variable>
 
  <xsl:template match="*| @*| text()" mode="look">
    <xsl:variable name="look">
      <xsl:value-of select="replace(replace(replace(replace(
			    replace(replace(replace(replace(replace(.,
			    '_double_prime',$double_prime),
			    '_triple_prime',$triple_prime),
			    '_prime',$prime),
			     'prime',$prime),
			    '_',$underscore),
			    'gramma',$gamma),
			    'M−phase promoting factor','M{-}phasePF'),
			    'kappa',$kappa),
			    'sigma',$sigma)
			    "/>
    </xsl:variable>
   <xsl:if test="$DEBUG='true'">
      <xsl:message> look(<xsl:value-of select="."/>)=<xsl:value-of
      select="$look"/></xsl:message>
    </xsl:if>
    <xsl:value-of select="$look"/>
  </xsl:template>

  <xsl:template match="*" mode="type">
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$model//*:model/*:listOfRules
		      /*:assignmentRule[@variable=$id]">
	<xsl:text>algebraic</xsl:text>
      </xsl:when>
      <xsl:when test="$model//*:model/*:listOfRules
		      /*:rateRule[@variable=$id]">
	<xsl:text>differential</xsl:text>
      </xsl:when>
      <xsl:when test="@*:boundaryCondition='true' and
		      not($model//*:model/*:listOfRules/*[@variable=$id])
		      ">
	<xsl:text>differential</xsl:text>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text>species</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <!-- function modifiers -->

  <xsl:template  match="reaction" mode="modifier">
    <xsl:variable name="reaction">
      <xsl:copy-of select="."/>
    </xsl:variable>
    <xsl:variable name="modifiers-with-dublicates">
      <xsl:for-each select="kinetic-expression//expr[@id]">
	<xsl:variable name='id'>
	  <xsl:value-of select="@id"/>
	</xsl:variable>
	<!--<xsl:message>
	  concentration candidate <xsl:value-of select="@id"/>
	</xsl:message>-->
	<xsl:for-each
	    select="ancestor::network/variable[@concentration=$id]
		   /@id">
	  <!--<xsl:message>
	    ++ selected candidate <xsl:value-of select="."/>
	</xsl:message>-->
	  <xsl:if test="not( . = $reaction/reaction/reactant/@spec)">
	      <modifier>
	      <xsl:attribute name="spec">
		<xsl:value-of select="."/>
	      </xsl:attribute>
	    </modifier>
	  </xsl:if>
	</xsl:for-each>
      </xsl:for-each>
    </xsl:variable>
    <!--<xsl:message>
      reaction <xsl:value-of select="@id"/>
    </xsl:message>-->
    <reaction>
      <xsl:copy-of select="@*|*[not(name()='modifier')] |comment()"/>
      <xsl:for-each select="$modifiers-with-dublicates
			    //modifier[not(@spec=preceding::modifier/@spec)]">
      	<!--<xsl:comment> code line 4711 </xsl:comment>-->
	<xsl:copy-of select="."/>
      </xsl:for-each>
    </reaction>
  </xsl:template>
  
  <xsl:template match="*" mode="modifier">
    <xsl:variable name="tag">
      <xsl:value-of select="local-name(.)"/>
    </xsl:variable>
    <xsl:element name="{$tag}">
      <xsl:apply-templates select="@*|*|comment()|text()" mode="modifier"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="@*|
		       comment()|text()" mode="modifier">
    <xsl:copy-of select="."/>
  </xsl:template>
  
</xsl:stylesheet>
