<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xpath-default-namespace="http://www.sbml.org/sbml/level2/version4"   
    version="2.0"
    exclude-result-prefixes="xs">

  <xsl:output   
      method="xml" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>

  <xsl:param name="NETWORK"          required="yes"/>
  <xsl:param name="CONC_DIR"      required="yes"/>

  <xsl:param name="DEBUG" select="false"  required="no"/>
    
  <xsl:variable name="NETWORK-CONCENTRATIONS">
    <xsl:copy-of select="document(concat($CONC_DIR,'/',$NETWORK,'-1.0.xml'))"/>
  </xsl:variable>

  <xsl:template match="/">
    <xsl:if test="$DEBUG='true'"><xsl:message>NETWORK-CONCENTRATIONS
      <xsl:copy-of select="$NETWORK-CONCENTRATIONS"/>
    </xsl:message></xsl:if>
    <xsl:apply-templates select="*"/>
  </xsl:template>
  
  <xsl:template match="text()|comment()|@*">
    <xsl:copy-of select="."/>
  </xsl:template>
  
  <xsl:template match="*[not(
		           local-name()='species'
		       or local-name()='metabolite'
		       or local-name()='actor'
		       or local-name()='protein')]">
   <xsl:variable name="name">
     <xsl:value-of select="name()"/>
   </xsl:variable>
   <xsl:message> zip <xsl:value-of select="name()"/>
   </xsl:message>
   <xsl:element name="{$name}">
     <xsl:apply-templates select="*|@*|text()|comment()"/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="*:species|*:metabolite|*:actor|*:protein">
    <xsl:variable name="name">
      <xsl:value-of select="name()"/>
    </xsl:variable>
    <xsl:variable name="id">
      <xsl:value-of select="@id"/>
    </xsl:variable>
    <xsl:message>concentration-zip ++++++++
    - species=<xsl:copy-of select="$id"/></xsl:message>
    <xsl:variable name="co-species">
      <xsl:copy-of
	  select="$NETWORK-CONCENTRATIONS//*
		  [name()=$name]
		  [@id=$id]"/>
    </xsl:variable>
    <xsl:variable name="initConc">
      <xsl:value-of select="$co-species/*/@initialConcentration"/>
    </xsl:variable>
    <xsl:element name="{$name}">
      <xsl:attribute name="initialConcentration">
	<xsl:value-of select="$initConc"/>
      </xsl:attribute>
     <xsl:apply-templates
	 select="*|text()|comment() |@*[not(name()='initialConcentrations')]"/>
   </xsl:element>
  </xsl:template>
</xsl:stylesheet>
