<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:mathml="http://www.w3.org/1998/Math/MathML"
    xmlns:math="http://www.w3.org/2005/xpath-functions/math"
    xmlns:op="https://www.w3.org/TR/xpath-functions-31"
    version="2.0"
    exclude-result-prefixes="xs rdf xsl mathml math op"
    >

  <xsl:output   
      method="xml" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>
  <xsl:param name="NETWORK"      required="no"/>
  <xsl:variable name="DEBUG">false</xsl:variable>

  <xsl:variable name="network">
    <xsl:copy-of select="network"/>
  </xsl:variable>

  <xsl:template match="network">
    <xsl:if test="$DEBUG='true'">
      <xsl:message>°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°
      °°°° core-elim-hyperbol:
      eliminate  hyperbolic trigonometric functions from Core SBML:
                 sinh cosh tanh coth </xsl:message>
    </xsl:if>
    <network>
      <xsl:apply-templates select="*|@*|text()|comment()"
			   mode="elim-hyperbol"/>

      <xsl:if test="//tanh | //coth | //sinh | //cosh">
	<xsl:comment>
	  +++ hyperbolic trigonimetric functions
	</xsl:comment>
	<function id="sinh">
	  <lambda>
            <bvar id="x"/>
	    <divide>
	      <minus>
	      <power>
		<constant id="e" value="2.71828"/>
		<var id="x"/>
	      </power>
	      <power>
		<constant id="e" value="2.71828"/>
		<minus>
		  <constant value="0"/>
		  <var id="x"/>
		</minus>
	      </power>
	    </minus>
	    <constant  value="2"/>
	  </divide>
	</lambda>
      </function>
      <function id="cosh">
	<lambda>
          <bvar id="x"/>
	  <divide>
	    <plus>
	      <power>
		<constant id="e" value="2.71828"/>
		<var id="x"/>
	      </power>
	      <power>
		<constant id="e" value="2.71828"/>
		<minus>
		  <constant value="0"/>
		  <var id="x"/>
		</minus>
	      </power>
	    </plus>
	    <constant  value="2"/>
	  </divide>
	</lambda>
      </function>
      <function id="tanh">
	<lambda>
          <bvar id="x"/>
	  <divide>
	    <apply fun="sinh">
	      <var id="x"/>
	    </apply>
	    <apply fun="cosh">
	      <var id="x"/>
	    </apply>
	  </divide>
	</lambda>
      </function>
      <function id="coth">
	<lambda>
          <bvar id="x"/>
	  <divide>
	    <apply fun="cosh">
	      <var id="x"/>
	    </apply>
	    <apply fun="sinh">
	      <var id="x"/>
	    </apply>
	  </divide>
	</lambda>
      </function>
      </xsl:if>
    </network>
  </xsl:template>
  
  <!-- function elim-hyperbol -->
  
  <xsl:template match="sinh" mode="elim-hyperbol">
    <xsl:message> ° elim sinh </xsl:message>
    <apply fun="sinh">
      <xsl:apply-templates select="*" mode="elim-hyperbol"/>
    </apply>
  </xsl:template>
  <xsl:template match="cosh" mode="elim-hyperbol">
    <xsl:message> ° elim cosh </xsl:message>
    <apply fun="cosh">
      <xsl:apply-templates select="*" mode="elim-hyperbol"/>
    </apply>
  </xsl:template>
  <xsl:template match="tanh" mode="elim-hyperbol">
    <xsl:message> ° elim tanh </xsl:message>
    <apply fun="tanh">
      <xsl:apply-templates select="*" mode="elim-hyperbol"/>
    </apply>
  </xsl:template>
  <xsl:template match="coth" mode="elim-hyperbol">
    <xsl:message> ° elim coth </xsl:message>
    <apply fun="coth">
      <xsl:apply-templates select="*" mode="elim-hyperbol"/>
    </apply>
  </xsl:template>
  
  <xsl:template match="*" mode="elim-hyperbol">
    <xsl:variable name="tag">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>
    <!--<xsl:if test="$DEBUG='true'">
	<xsl:message>
	  elim-hyperbol otherwise <xsl:value-of select="$tag"/>
	  </xsl:message>
	  </xsl:if>-->
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="@*|*|comment()|text()"
			     mode="elim-hyperbol"/>
      </xsl:element>
    </xsl:template>

   <xsl:template match="@*|comment()|text()" mode="elim-hyperbol">
     <xsl:copy-of select="."/>
   </xsl:template>


  
</xsl:stylesheet>
