<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:mathml="http://www.w3.org/1998/Math/MathML"
    xmlns:math="http://www.w3.org/2005/xpath-functions/math"
    xmlns:op="https://www.w3.org/TR/xpath-functions-31"
    version="2.0"
    exclude-result-prefixes="xs rdf xsl mathml"
    >

  <xsl:output   
      method="xml" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>
  <xsl:param name="NETWORK"      required="no"/>
  <xsl:variable name="DEBUG">false</xsl:variable>

  <xsl:variable name="network">
    <xsl:copy-of select="network"/>
  </xsl:variable>

  <xsl:template match="/">
    <xsl:if test="$DEBUG='true'">
      <xsl:message>°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°
°°°° Core SBML: eliminate initial-expression </xsl:message>
    </xsl:if>
    <xsl:apply-templates select="*" mode="elim-initial-expr"/>
  </xsl:template>
    
   <!-- function apply-->

  <xsl:template match="*:lambda[*[1][local-name()='bvar']]"  mode="apply">
    <xsl:param name="args"/>
    <xsl:if test="$DEBUG='true'">
    <xsl:message>   apply function: lambda
    - function = <xsl:copy-of select="."/>
    - args = <xsl:copy-of select="$args/*"/>
    - firstarg = <xsl:copy-of select="$args/*:args/*[1]"/>
    </xsl:message>
   </xsl:if> 
    <xsl:variable name="argument">
	 <xsl:copy-of select="$args/*:args/*[1]"/>
    </xsl:variable>
    <!--<xsl:message>
      variable =  <xsl:copy-of select="*[1]"/>
    </xsl:message>-->
     <xsl:variable name="nextfun">
       <lambda>
	 <xsl:apply-templates select="*[position()&gt;1]"  mode="substitute">
	   <xsl:with-param name="variable">
	      <xsl:copy-of select="*[1]"/>
	   </xsl:with-param>
	   <xsl:with-param name="argument">
	     <xsl:copy-of select="$argument"/>
	   </xsl:with-param>
	   <xsl:with-param name="reaction">
	     <xsl:copy-of select="ancestor::reaction"/>
	  </xsl:with-param>
	 </xsl:apply-templates>
       </lambda>
     </xsl:variable>
    <xsl:variable name="nextargs">
      <args>
	<xsl:copy-of select="$args/*:args/*[position() &gt; 1]"/>
      </args>
    </xsl:variable>
    <xsl:if test="$DEBUG='true'">
      <xsl:message>   - nextargs = <xsl:copy-of select="$nextargs"/>
    </xsl:message></xsl:if>
   <xsl:apply-templates select="$nextfun"    mode="apply">
      <xsl:with-param name="args">
	<xsl:copy-of select="$nextargs"/>
      </xsl:with-param>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="*:lambda[*[1][not(local-name()='bvar')]]"
		mode="apply">
    <!--<xsl:message>
      end-of-apply: <xsl:copy-of select="*"/>
    </xsl:message>-->
    <xsl:copy-of select="*"/>    
  </xsl:template>

  <!-- function substitute -->
  <xsl:template match="var"  mode="substitute">
    <xsl:param name="variable"/>
    <xsl:param name="argument"/>
    <xsl:param name="reaction"/>
   <!-- <xsl:message> substitute 
    var=<xsl:copy-of select="."/>
    variable=<xsl:copy-of select="$variable"/>
    argument=<xsl:copy-of select="$argument"/>
    </xsl:message>-->
    <xsl:choose>
      <xsl:when test="$variable/bvar/@id=@id">
	<xsl:if test="$DEBUG='true'">
	  <xsl:message> successful [<xsl:copy-of
	  select="$variable"/>/<xsl:copy-of select="$argument"/>]
	  </xsl:message>
	</xsl:if>
	<xsl:copy-of select="$argument"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:copy-of select="."/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="*"  mode="substitute">
    <xsl:param name="variable"/>
    <xsl:param name="argument"/>
    <xsl:param name="reaction"/>
    <xsl:variable name="name">
      <xsl:value-of select="name()"/>
    </xsl:variable>
      
    <xsl:element name="{$name}">
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates select="*|text()" mode="substitute">
	<xsl:with-param name="variable">
	  <xsl:copy-of select="$variable"/>
	</xsl:with-param>
	<xsl:with-param name="argument">
	  <xsl:copy-of select="$argument"/>
	</xsl:with-param>
	<xsl:with-param name="reaction">
	  <xsl:copy-of select="$reaction"/>
	</xsl:with-param>
      </xsl:apply-templates>
    </xsl:element>
  </xsl:template>

    <xsl:template match="text()"  mode="substitute">
      <xsl:copy-of select="."/>
    </xsl:template>
     
  

    <!-- function elim-initial-expr -->
    
    <xsl:template match="@initial-expression" mode="elim-initial-expr">
      <xsl:message> ° elim @initial-expression <xsl:value-of
      select="parent::*/@id"/></xsl:message>
      <xsl:attribute name="initial">
	<xsl:apply-templates select="." mode="eval-initial"/>
      </xsl:attribute>
    </xsl:template>

    <xsl:template match="apply" mode="eval-initial">
      <xsl:variable name="fun-id">
	<xsl:value-of select="@fun"/>
      </xsl:variable>
      <xsl:variable name="function">
	<xsl:copy-of select="$network//function[@id=$fun-id]/lambda"/>
      </xsl:variable>
      <xsl:variable name="apply-result">
	<xsl:apply-templates select="$function" mode="apply">
	  <xsl:with-param name="args">
	    <args>
	      <xsl:copy-of select="*"/>
	    </args>
	  </xsl:with-param>
	</xsl:apply-templates>
       </xsl:variable>
       <xsl:if test="$DEBUG='true'">
	 <xsl:message>
	   function=<xsl:value-of select="$fun-id"/>
	   <xsl:copy-of select="$function"/>
	   apply-result=<xsl:copy-of select="$apply-result"/>
	 </xsl:message>
       </xsl:if>
     <xsl:apply-templates select="$apply-result" mode="eval-initial"/>
    </xsl:template>
    
    <xsl:template match="*" mode="elim-initial-expr">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <!--<xsl:if test="$DEBUG='true'">
	<xsl:message>
	  elim-initial-expr otherwise <xsl:value-of select="$tag"/>
	</xsl:message>
      </xsl:if>-->
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="@*|*|comment()|text()"
			     mode="elim-initial-expr"/>
      </xsl:element>
    </xsl:template>

   <xsl:template match="@*|comment()|text()" mode="elim-initial-expr">
     <xsl:copy-of select="."/>
   </xsl:template>


    <!-- function eval-initial -->

    <xsl:template match="@initial-expression" mode="eval-initial">
      <xsl:if test="$DEBUG='true'">
	<xsl:message> eval-initial @initial-expression <xsl:value-of
	select="."/>
	</xsl:message>
	</xsl:if>
      <xsl:variable name="expr">
	<expr>
	  <xsl:attribute name="id">
	    <xsl:value-of select="."/>
	  </xsl:attribute>
	</expr>
      </xsl:variable>
      <xsl:apply-templates select="$expr" mode="eval-initial"/>
    </xsl:template>

    <xsl:template match="expr" mode="eval-initial">
      <xsl:variable name="id">
	<xsl:value-of select="@id"/>
      </xsl:variable>
      <xsl:if test="$DEBUG='true'">
	<xsl:message> eval-initial expr <xsl:value-of select="@id"/>
	<!-- expression=<xsl:copy-of select="$network//expression[@id=$id]/*"/>-->
	</xsl:message>
      </xsl:if>
      <xsl:apply-templates select="$network//expression[@id=$id]/*"
	 mode="eval-initial"/>
    </xsl:template>
 
   <xsl:template match="var" mode="eval-initial">
      <xsl:variable name="id">
	<xsl:value-of select="@id"/>
      </xsl:variable>
      <xsl:variable name="variable">
	<xsl:copy-of select="$network//variable[@id=$id]"/>
      </xsl:variable>
      <xsl:if test="$DEBUG='true'">
	<xsl:message> eval-initial var <xsl:value-of select="@id"/>
	<!--variable=<xsl:copy-of select="$network//variable[@id=$id]"/>-->
	</xsl:message>
      </xsl:if>
      <xsl:choose>
	<xsl:when test="$variable/variable/@initial">
	  <xsl:value-of select="$variable/variable/@initial"/>
	  <xsl:if test="$DEBUG='true'">
	    <xsl:message> eval-initial variable @initial=<xsl:value-of
	    select="$variable/variable/@initial"/>
	    </xsl:message>
	  </xsl:if>
	</xsl:when>
	<xsl:when test="$variable/variable/@initial-expression">
	  <xsl:if test="$DEBUG='true'">
	    <xsl:message> eval-initial variable @initial-expression=<xsl:value-of
	  select="$variable/variable/@initial-expression"/>	    </xsl:message>
	  </xsl:if>
	  <!--<xsl:message> +++ go eval-initial for @initial-expression <xsl:value-of select="$variable/variable/@initial-expression"/></xsl:message>-->
	  <xsl:apply-templates select="$variable/variable/@initial-expression"  mode="eval-initial"/>
	</xsl:when>
	<xsl:when test="$variable/variable/@type='algebraic'">
	  <xsl:if test="$DEBUG='true'">
	    <xsl:message> eval-initial variable algebraic var
	  <!--<xsl:value-of
	      select="@id"/>
	     kinetic-expression <xsl:copy-of
	     select="$variable/variable/kinetic-expression"/>-->
	    </xsl:message>
	  </xsl:if>
	  <xsl:apply-templates select="$variable/variable/kinetic-expression/*"
			       mode="eval-initial"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:message> var error in eval-initial
	  var =<xsl:copy-of select="."/>
	  <!--variable=<xsl:copy-of select="$variable"/>-->
	  </xsl:message>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <xsl:template match="constant" mode="eval-initial">
      <xsl:if test="$DEBUG='true'">
	<xsl:message> eval-initial constant @value=<xsl:value-of
	select="@value"/>
	</xsl:message>
      </xsl:if>
     <xsl:value-of select="@value"/>
   </xsl:template>
    
    <xsl:template match="plus" mode="eval-initial">
      <xsl:choose>
	<xsl:when test="count(*)=0">0</xsl:when>
	<xsl:when test="count(*)=1">
	    <xsl:apply-templates select="*[1]" mode="eval-initial"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:variable name="val1">
	    <xsl:apply-templates select="*[1]" mode="eval-initial"/>
	  </xsl:variable>
	  <xsl:variable name="next_plus">
	    <plus>
	      <xsl:copy-of select="*[position() &gt; 1]"/>
	    </plus>
	  </xsl:variable>
	  <xsl:variable name="val2">
	    <xsl:apply-templates select="$next_plus" mode="eval-initial"/>
	  </xsl:variable>
	 <xsl:if test="$DEBUG='true'">
	    <xsl:message> eval-initial plus +++++++++++++++++
	    - *[1]=<xsl:copy-of select="*[1]"/>
	    - val1="<xsl:value-of select="$val1"/>"
	    - *[2]=<xsl:copy-of select="*[2]"/>
	    - val2="<xsl:value-of select="$val2"/>"
	    </xsl:message>
	  </xsl:if>
	  <xsl:value-of select="$val1 + $val2"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <xsl:template match="times" mode="eval-initial">
      <xsl:choose>
	<xsl:when test="count(*)=0">1</xsl:when>
	<xsl:otherwise>
	  <xsl:variable name="val1">
	    <xsl:apply-templates select="*[1]" mode="eval-initial"/>
	  </xsl:variable>
	  <xsl:variable name="next_times">
	    <times>
	      <xsl:copy-of select="*[position() &gt; 1]"/>
	    </times>
	  </xsl:variable>
	  <xsl:variable name="val2">
	    <xsl:apply-templates select="$next_times" mode="eval-initial"/>
	  </xsl:variable>
	  <xsl:if test="$DEBUG='true'">
	    <xsl:message> eval-initial times
	    - val1="<xsl:value-of select="$val1"/>"
	    - val2="<xsl:value-of select="$val2"/>"
	    </xsl:message>
	  </xsl:if>
	  <xsl:value-of select="$val1 * $val2"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <xsl:template match="minus" mode="eval-initial">
      <xsl:variable name="val1">
	<xsl:apply-templates select="*[1]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:variable name="val2">
	<xsl:apply-templates select="*[2]" mode="eval-initial"/>
      </xsl:variable>
        <xsl:choose>
	  <xsl:when test="count(*)=1">
	    <xsl:if test="$DEBUG='true'">
	      <xsl:message> eval-initial minus 
	      $val1=	  <xsl:value-of select="$val1"/>
	      </xsl:message>
	    </xsl:if>
	    <xsl:value-of select="0 - $val1"/>
	  </xsl:when>
	  <xsl:when test="count(*)=2">
	    <xsl:if test="$DEBUG='true'">
	      <xsl:message> eval-initial minus
	      -  $val1=  <xsl:value-of select="$val1"/>
	      -  $val2=  <xsl:value-of select="$val2"/>
	      </xsl:message>
	    </xsl:if>
	    <xsl:value-of select="$val1 - $val2"/>
	  </xsl:when>
	</xsl:choose>
    </xsl:template>

    <xsl:template match="divide" mode="eval-initial">
      <xsl:if test="$DEBUG='true'">
	<xsl:message> eval-initial divide </xsl:message>
      </xsl:if>


      <xsl:variable name="val1">
	<xsl:apply-templates select="*[1]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:variable name="val2">
	<xsl:apply-templates select="*[2]" mode="eval-initial"/>
      </xsl:variable>
     <xsl:if test="$DEBUG='true'">
       <xsl:message> eval-initial divide
     <!--   *[1]=<xsl:copy-of select="*[1]"/> -->
	-  $val1="<xsl:value-of select="$val1"/>"
       <!--   *[2]=<xsl:copy-of select="*[2]"/> -->
	-  $val2="<xsl:value-of select="$val2"/>"
	</xsl:message>
    </xsl:if>
      <xsl:value-of select="$val1 div $val2"/>
    </xsl:template>

     <xsl:template match="power" mode="eval-initial">
      <xsl:variable name="val1">
	<xsl:apply-templates select="*[1]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:variable name="val2">
	<xsl:apply-templates select="*[2]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:if test="$DEBUG='true'">
	<xsl:message>  eval-initial power 
	-  $val1="<xsl:value-of select="$val1"/>"
	 -  $val2="<xsl:value-of select="$val2"/>"
	</xsl:message>
      </xsl:if>
	<xsl:value-of select="math:pow($val1,$val2)"/>
     </xsl:template>

     <xsl:template match="log" mode="eval-initial">
       <xsl:if test="$DEBUG='true'">
	 <xsl:message>eval-initial log using
	 log_b(e)=ln(e)/ln(b)</xsl:message>
       </xsl:if>
      <xsl:variable name="val1">
	<xsl:apply-templates select="*[1]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:variable name="val2">
	<xsl:apply-templates select="*[2]" mode="eval-initial"/>
      </xsl:variable>
	<xsl:value-of select="math:log($val1) div math:log($val2)"/>
    </xsl:template>


    <xsl:template match="ifthenelse" mode="eval-initial">
      <xsl:if test="$DEBUG='true'">
	<xsl:message> eval-initial ifthenelse </xsl:message>
      </xsl:if>
      <xsl:variable name="condition">
	<xsl:apply-templates select="*[1]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:variable name="then">
	<xsl:apply-templates select="*[2]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:variable name="else">
	<xsl:apply-templates select="*[3]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:if test="$DEBUG='true'">
	<xsl:message>  eval-initial ifthenelse      <!--  *[1]=<xsl:copy-of select="*[1]"/>-->
	 -  $condition="<xsl:value-of select="$condition"/>"      <!--  *[2]=<xsl:copy-of select="*[2]"/>-->
	 -  $then="<xsl:value-of select="$then"/>"	 <!--  *[3]=<xsl:copy-of select="*[3]"/>-->
	 -  $else="<xsl:value-of select="$else"/>"
	</xsl:message>
      </xsl:if>
      <xsl:choose>
	<xsl:when test="$condition='1'">
	  <xsl:value-of select="$then"/>
	</xsl:when>
	<xsl:when test="$condition='0'">
	  <xsl:value-of select="$else"/>
	</xsl:when>
	<xsl:otherwise>  ERROR in Core-elim-initial-exp
	   - condition in ifthenelse does not evaluate to 0 or 1. 
	   - $condition=<xsl:value-of select="$condition"/> 
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <xsl:template match="time" mode="eval-initial">
      <xsl:text>0</xsl:text>
    </xsl:template>	  
    <xsl:template match="child::and" mode="eval-initial">
      <xsl:if test="$DEBUG='true'">
	<xsl:message> eval-initial and </xsl:message>
      </xsl:if>
      <xsl:variable name="B1">
	<xsl:apply-templates select="*[1]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:variable name="B2">
	<xsl:apply-templates select="*[2]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:choose>
	<xsl:when test="$B1=1 and  $B2=2">1</xsl:when>
	<xsl:otherwise>0</xsl:otherwise>
      </xsl:choose>
    </xsl:template>


     <xsl:template match="eq" mode="eval-initial">
       <xsl:if test="$DEBUG='true'">
	 <xsl:message> eval-initial eq </xsl:message>
       </xsl:if>
       <xsl:variable name="V1">
	<xsl:apply-templates select="*[1]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:variable name="V2">
	<xsl:apply-templates select="*[2]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:choose>
	<xsl:when test="$V1=$V2">1</xsl:when>
	<xsl:otherwise>0</xsl:otherwise>
      </xsl:choose>
     </xsl:template>

     <xsl:template match="lt" mode="eval-initial">
       <xsl:if test="$DEBUG='true'">
	 <xsl:message> eval-initial lt </xsl:message>
       </xsl:if>
      <xsl:variable name="V1">
	<xsl:apply-templates select="*[1]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:variable name="V2">
	<xsl:apply-templates select="*[2]" mode="eval-initial"/>
      </xsl:variable>
      <xsl:choose>
	<xsl:when test="$V1 &lt; $V2">1</xsl:when>
	<xsl:otherwise>0</xsl:otherwise>
      </xsl:choose>
    </xsl:template>
    
    
    <xsl:template match="*" mode="eval-initial">
    <xsl:variable name="tag">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>
    <!--    <xsl:if test="$DEBUG='true'"> -->
    <xsl:message>

++++ ERROR eval-initial otherwise <xsl:value-of select="$tag"/> +++++++++++++++++++++++++++++++++++
      
    </xsl:message>
    <!--    </xsl:if>	-->
    <xsl:element name="{$tag}">
      <xsl:apply-templates select="@*|*|comment()|text()"
			   mode="eval-initial"/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="@*|comment()|text()" mode="eval-initial">
     <xsl:copy-of select="."/>
  </xsl:template>
  
</xsl:stylesheet>
