<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:mathml="http://www.w3.org/1998/Math/MathML"
    xmlns:sbml="http://www.sbml.org/sbml/level2/version5" 
    version="2.0"
    exclude-result-prefixes="sbml">


  <xsl:output   
      method="xml" 
      version="1.0"
      encoding="utf-8"
      indent="yes"/>

  <xsl:variable name="Core">
    <xsl:copy-of select="/"/>
  </xsl:variable>
  <xsl:variable name="DEBUG">false</xsl:variable>

  <xsl:variable name="network">
    <xsl:copy-of select="network"/>
  </xsl:variable>
		 
  <xsl:template match="network">
    <xsl:if test="$DEBUG='true'">
    <xsl:message>°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°
°°°° Core2SBML
    </xsl:message>
    </xsl:if>
   
    <sbml:sbml xmlns:sbml="http://www.sbml.org/sbml/level2/version5"
	       level="2" version="5">
    <model>
      <xsl:copy-of select="@id|@name"/>
      <listOfCompartments>
	<compartment id="dummy" name="dummy" size="1"/>
      </listOfCompartments>
      <listOfSpecies>
	<xsl:comment>
	  variable[@type=species]
	</xsl:comment> 
	<xsl:apply-templates select="variable[@type='species']"/>
	<xsl:comment>
	  variable[@type=algebraic]
	</xsl:comment> 
	<xsl:apply-templates select="variable[@type='algebraic']"/>
	<xsl:comment>
	  variable[@type=differential]
	</xsl:comment> 
	<xsl:apply-templates select="variable[@type='differential']"/>
      </listOfSpecies>
      
      <xsl:if test="variable[@type='control']|expression">
	<listOfParameters>
	  <xsl:comment>
	    control variables
	  </xsl:comment>
	  <xsl:apply-templates select="variable[@type='control']"/>
	  <xsl:comment>
	    macros for expressions with dummy zero values (overwritten by assignment rules)
	  </xsl:comment>
	  <xsl:apply-templates select="expression" mode="dummy-zero"/>
	</listOfParameters>
      </xsl:if>

      <xsl:if test="variable[@initial-expression]">
        <listOfInitialAssignments>
	<xsl:apply-templates select="variable[@initial-expression]" mode="initass"/>
      </listOfInitialAssignments>
      </xsl:if>
       
      <xsl:if test="variable[@type='algebraic'] | expression">
	<listOfRules>
	  <xsl:comment>
	    algebraic variables
	  </xsl:comment>
	  <xsl:apply-templates select="variable[@type='algebraic']"
			       mode="rule"/>
	  <xsl:comment>
	    expression macros
	  </xsl:comment>
	  <xsl:apply-templates select="expression"/>
	</listOfRules>
      </xsl:if>

      <xsl:if test="reaction">
	<listOfReactions>
	  <xsl:apply-templates select="reaction"/>
	</listOfReactions>
      </xsl:if>

      <xsl:if test="function">
	<listOfFunctionDefinitions>
	  <xsl:apply-templates select="function"/>
	</listOfFunctionDefinitions>
      </xsl:if>
    
      <xsl:variable name="nlp">
	<xsl:value-of select="count(reaction/expression)"/>
      </xsl:variable>
     <xsl:if test="$nlp&gt;0">
	<xsl:message>#local-parameters=<xsl:value-of select="$nlp"/>
	</xsl:message>
     </xsl:if>
     
     <xsl:if test="event">
       <listOfEvents>
	 <xsl:apply-templates select="event"/>
       </listOfEvents>
     </xsl:if>

    </model>
    </sbml:sbml>

    
  </xsl:template>

  <xsl:template match="variable[@initial-expression]" mode="initass">
    <initialAssignment>
      <xsl:attribute name="symbol">
	<xsl:value-of select="@id"/>
      </xsl:attribute>
      <math xmlns="http://www.w3.org/1998/Math/MathML">        
        <ci>
	  <xsl:value-of select="@initial-expression"/>
	</ci>
      </math>
    </initialAssignment>
  </xsl:template>
  
  <xsl:template match="reaction">
    <xsl:if test="reactant|product">
    <reaction>
      <xsl:attribute name="id">
	<xsl:value-of select="@id"/>
      </xsl:attribute>
      <xsl:if test="reactant">
	<listOfReactants>
	  <xsl:apply-templates select="reactant"/>
	</listOfReactants>
      </xsl:if>
      <xsl:if test="product">
	<listOfProducts>
	  <xsl:apply-templates select="product"/>
	</listOfProducts>
      </xsl:if>
      <xsl:apply-templates select="." mode="listOfModifiers"/>
      <xsl:apply-templates select="kinetic-expression">
	<xsl:with-param name="expressions">
	  <expressions>
	    <xsl:copy-of select="expression"/>
	  </expressions>
	</xsl:with-param>
      </xsl:apply-templates>
    </reaction>
  </xsl:if>
  </xsl:template>

  <xsl:template match="reaction" mode="listOfModifiers">
    <xsl:variable name="free-species-vars">
     <xsl:copy-of
	 select="kinetic-expression//var[@type][not(@type='control')]
		 [not(@id=$network//variable[@type='control']/@id)]"/>
    </xsl:variable>
    <xsl:if test="$DEBUG='true'">
      <xsl:message>
      +++ listOfModifiers
      - reaction   <xsl:value-of select="@id"/>
      - variables <xsl:value-of  select="kinetic-expression//var/@id"/>
      - species   <xsl:value-of   select="$free-species-vars/var/@id"/>
      - kinetic-expression-vars <xsl:copy-of  select="kinetic-expression//var"/>
    </xsl:message>
    </xsl:if>

    <xsl:if test="$free-species-vars/var or modifier">
      <listOfModifiers>
	<xsl:comment>
	  variables but of type control occuring in kinetic expression of Core SBML reaction
	</xsl:comment>
	<xsl:for-each select="$free-species-vars/var">
	  <modifierSpeciesReference>
	    <xsl:attribute name="species">
	      <xsl:value-of select="@id"/>
	    </xsl:attribute>
	  </modifierSpeciesReference>
	</xsl:for-each>
	<xsl:comment>
	  modifiers declared in Core SBML reaction
	</xsl:comment>
	<xsl:for-each select="modifier">
	  <modifierSpeciesReference>
	    <xsl:attribute name="species">
	      <xsl:value-of select="@spec"/>
	    </xsl:attribute>
	  </modifierSpeciesReference>
	</xsl:for-each>
      </listOfModifiers>
    </xsl:if>
    </xsl:template>

    <xsl:template match="kinetic-expression">
      <xsl:param name="expressions"/>
       <kineticLaw>
	<math xmlns="http://www.w3.org/1998/Math/MathML">
	  <xsl:apply-templates select="*"  mode="math"/>
	</math>
	<xsl:if test="$expressions//expression">
	  <listOfParameters>
	    <xsl:apply-templates select="$expressions/
					 expressions/expression" mode="parameter"/>
	  </listOfParameters>
	</xsl:if>
       </kineticLaw>
    </xsl:template>
  
  <xsl:template match="expression" mode="parameter">
    <!--<xsl:message>  local parameter <xsl:value-of select="@id"/>
    </xsl:message>-->
    <parameter>
      <xsl:attribute name="id">
	<xsl:value-of  select="@id"/>
      </xsl:attribute>
      <xsl:attribute name="value">
	<xsl:value-of  select="constant/@value"/>
      </xsl:attribute>      
    </parameter>
  </xsl:template>
  
  <xsl:template match="expression">
    <assignmentRule>
      <xsl:attribute name="variable">
	<xsl:value-of select="@id"/>
      </xsl:attribute>
      <mathml:math>
	<xsl:apply-templates select="*"  mode="math"/>
      </mathml:math>
    </assignmentRule>
  </xsl:template>


  
  <xsl:template match="expression" mode="dummy-zero">
    <xsl:comment> dummy value=0 </xsl:comment>
    <parameter constant="false" value="0">
      <xsl:attribute name="id">
	<xsl:value-of select="@id"/>
      </xsl:attribute>
    </parameter>
  </xsl:template>

  <xsl:template match="variable[@type='algebraic']" mode="rule">
    <assignmentRule>
      <xsl:attribute name="variable">
	<xsl:value-of select="@id"/>
      </xsl:attribute>
      <mathml:math>
	<xsl:apply-templates select="kinetic-expression/*"  mode="math"/>
      </mathml:math>
    </assignmentRule>
  </xsl:template>

  
  <xsl:template match="reactant | product">
    <speciesReference>
      <xsl:attribute name="species">
	<xsl:value-of select="@spec"/>
      </xsl:attribute>
      <xsl:if test="@stoichiometry">
	<xsl:attribute name="stoichiometry">
	  <xsl:value-of select="@stoichiometry"/>
	</xsl:attribute>
      </xsl:if>
    </speciesReference>
  </xsl:template>
  
  <xsl:template match="variable[
		       @type='species' or
		       @type='differential']">
    <species compartment="dummy">
      <xsl:attribute name="id">
	<xsl:value-of select="@id"/>
      </xsl:attribute>
      <xsl:if test="@initial-expression">
	<xsl:attribute name="boundaryCondition">true</xsl:attribute>
	<xsl:attribute name="constant">false</xsl:attribute>
      </xsl:if>
      <xsl:attribute name="initialConcentration">
	<xsl:choose>
	  <xsl:when test="@initial-expression">0</xsl:when>
	  <xsl:when test="@initial">
	    <xsl:value-of select="normalize-space(@initial)"/>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:message>
		Warning: differential variable without @initial and @initial-expression
	      </xsl:message>
	    </xsl:otherwise>
	</xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="@latex-look"/>
    </species>
    
  </xsl:template>

  <xsl:template match="variable[@type='algebraic' ]">
    <!--<xsl:message>++hi <xsl:value-of select="@type"/> </xsl:message>-->
    <species compartment="dummy">
      <xsl:attribute name="id">
	<xsl:value-of select="@id"/>
      </xsl:attribute> 
      <xsl:attribute name="initialConcentration">0</xsl:attribute>
      <xsl:apply-templates select="@latex-look"/>
    </species>
  </xsl:template>

  <xsl:template match="variable[@type='control']" >
    <parameter constant="false">
      <xsl:attribute name="id">
	<xsl:value-of select="@id"/>
      </xsl:attribute>
      <xsl:attribute name="value">
	<xsl:value-of select="@initial"/>
      </xsl:attribute>
      <xsl:apply-templates select="@latex-look"/>	    
    </parameter>
  </xsl:template>


  <xsl:template match="@latex-look" >
    <!--<xsl:message> @latex-look <xsl:value-of select="."/> </xsl:message>-->
    <xsl:attribute name="name">
	<xsl:value-of select="."/>
      </xsl:attribute>
  </xsl:template>
  
   <xsl:template match="constant" mode="math">
    <mathml:cn>
      <xsl:value-of select="@value"/>
    </mathml:cn>   
  </xsl:template>

  <xsl:template match="var" mode="math">
    <mathml:ci>
      <xsl:value-of select="@id"/>
    </mathml:ci>   
  </xsl:template>

   <xsl:template match="expr" mode="math">
    <mathml:ci>
      <xsl:value-of select="@id"/>
    </mathml:ci>   
   </xsl:template>

    <xsl:template match="speed" mode="math">
    <mathml:ci>
      <xsl:value-of select="@reaction"/>
    </mathml:ci>   
  </xsl:template>


  <xsl:template match="apply" mode="math">
      <mathml:apply>
	<mathml:ci>
	  <xsl:value-of select="@fun"/>
	</mathml:ci>
	<xsl:apply-templates select="*" mode="math"/>
      </mathml:apply>
  </xsl:template>
  
  
  <xsl:template match="time" mode="math">
    <mathml:csymbol encoding="text" definitionURL="http://www.sbml.org/sbml/symbols/time"> time </mathml:csymbol>
  </xsl:template>

  <xsl:template match="ifthenelse" mode="math">
    <mathml:piecewise>
      <mathml:piece>
	<xsl:apply-templates select="*[2]" mode="math"/>
	<xsl:apply-templates select="*[1]" mode="math"/>
      </mathml:piece>
      <mathml:otherwise>
	<xsl:apply-templates select="*[3]" mode="math"/>
      </mathml:otherwise>
      <xsl:apply-templates select="*[1]" mode="math"/>
    </mathml:piecewise>
  </xsl:template>

   <xsl:template match="log" mode="math">
     <mathml:apply>
       <mathml:log/>
       <mathml:logbase>
	 <xsl:apply-templates select="*[1]" mode="math"/>
       </mathml:logbase>
       <xsl:apply-templates select="*[2]" mode="math"/>
    </mathml:apply>
  </xsl:template>


   <xsl:template match="delay" mode="math">
     <mathml:apply>
       <mathml:csymbol encoding="text"
		definitionURL="http://www.sbml.org/sbml/symbols/delay"> delay </mathml:csymbol>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:apply-templates select="*[2]" mode="math"/>
     </mathml:apply>
  </xsl:template>

  
  <xsl:template match="*" mode="math">
    <xsl:variable name="tag">
      <xsl:value-of select="name()"/>
    </xsl:variable>
    <mathml:apply>
       <xsl:element name="mathml:{$tag}"/>
       <xsl:apply-templates select="*" mode="math"/>
    </mathml:apply>
   </xsl:template>

   

   <xsl:template match="variable[@type='parameter']">
     <parameter constant="false">
       <xsl:attribute name="id">
	 <xsl:value-of select="@id"/>
       </xsl:attribute>
       <xsl:attribute name="value">
	<xsl:value-of select="@initial"/>
      </xsl:attribute>
    </parameter>
  </xsl:template>
  
  <xsl:template match="event">
    <event>
      <xsl:attribute name="id">
	<xsl:value-of select="generate-id()"/>
      </xsl:attribute>
       <trigger>
	 <mathml:math>
	   <xsl:apply-templates select="condition/*" mode="math"/>
	 </mathml:math>
       </trigger>
       <listOfEventAssignments>
	 <xsl:apply-templates select="update"/>
       </listOfEventAssignments>
     </event>
   </xsl:template>

  <xsl:template match="update">
    <eventAssignment>
      <xsl:attribute name="variable">
	<xsl:value-of select="var/@id "/>
      </xsl:attribute>
      <mathml:math>
	<xsl:apply-templates select="*[2]" mode="math"/>
      </mathml:math>
    </eventAssignment>
  </xsl:template>


   <xsl:template match="function">
    <functionDefinition>
      <xsl:copy-of select="@id"/>
      <mathml:math xmlns:mathml="http://www.w3.org/1998/Math/MathML">
	<lambda>
	  <xsl:apply-templates select="lambda/bvar"/>	  
	  <xsl:apply-templates select="lambda/*[position()=last()]" mode="math"/>
	</lambda>
      </mathml:math>
    </functionDefinition>
   </xsl:template>

   <xsl:template match="bvar">
     <bvar>
       <ci>
	 <xsl:value-of select="@id"/>
       </ci>
     </bvar>
   </xsl:template>

</xsl:stylesheet>
   
