<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:mathml="http://www.w3.org/1998/Math/MathML"
    version="2.0"
    exclude-result-prefixes="xs rdf xsl mathml"
    >

  <!--xpath-default-namespace="http://www.sbml.org/sbml/level2/version4"-->


  <xsl:output   
      method="text" 
      encoding="utf-8"
      indent="yes"/>
  <xsl:param name="NETWORK"      required="no"/>
  <xsl:variable name="network">
    <xsl:copy-of select="/network"/>
  </xsl:variable>
  <xsl:variable name="DEBUG">false</xsl:variable>

 
   
  <xsl:template match="network">
    <xsl:text>%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Network: </xsl:text>
     <xsl:value-of select="@id"/>
<xsl:text>
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
              
%%%%  reaction rules %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

</xsl:text>   
<xsl:apply-templates select="reaction|variable[@type='differential']"/>
<xsl:text>
% bug fix for Time
1 for _ => RealTime.

%%%% initial values %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

</xsl:text>

<xsl:apply-templates select="variable[@type='species' or @type='differential']" mode="initial"/>

<xsl:text>
present(RealTime,0.0).
</xsl:text>
%%%%% macros  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
<xsl:if test="variable[kinetic-expression]">  
<xsl:text>
% for algebraic variables  %%%%%%%%%%%%%%%%%%%%%%%%%%
  
</xsl:text>
</xsl:if>

<xsl:apply-templates select="variable[@type='algebraic']" mode="macro"/>

<xsl:if test="expression">  
<xsl:text>
% for parameters and expressions  %%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
</xsl:text>
</xsl:if>

<xsl:apply-templates select="expression"/>

<xsl:if test="parameter">
<xsl:text>
% for control variables %%%%%%%%%%%%%%%%%%%%%%%%%%

</xsl:text>
</xsl:if>
<xsl:apply-templates select="variable[@type='control']"/>

<xsl:if test="event">
  <xsl:text>%%%% events  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
</xsl:text>
</xsl:if>
<xsl:apply-templates select="event"/>

<xsl:if test="function">
  <xsl:text>
%%%% functions  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
</xsl:text>
</xsl:if>
<xsl:apply-templates select="function"/>



  </xsl:template>


  <xsl:template match="variable[@type='differential']">
    <xsl:variable name="reaction">
      <reaction>
	<xsl:copy-of select="@id"/>
	<xsl:copy-of select="kinetic-expression"/>
	<product>
	  <xsl:attribute name="spec">
	    <xsl:value-of select="@id"/>
	  </xsl:attribute>
	</product>
      </reaction>
    </xsl:variable>
    <!--<xsl:message>
      reaction for differential variable:
      <xsl:copy-of select="$reaction"/>
    </xsl:message>-->
      <xsl:apply-templates select="$reaction/reaction"/>
  </xsl:template>

  
  <xsl:template match="reaction">
    <!--<xsl:message> reaction <xsl:value-of select="@id"/>
    </xsl:message>-->
    <xsl:text>% </xsl:text>
    <xsl:value-of select="@id"/>
    <xsl:text>
</xsl:text>
    <xsl:apply-templates select="kinetic-expression"/>
    <xsl:text> for </xsl:text>
    <xsl:choose>
      <xsl:when test="reactant">
	<xsl:apply-templates select="reactant"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text>_</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>=></xsl:text>
    <xsl:choose>
      <xsl:when test="product">
	<xsl:apply-templates select="product"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text>_</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>.
</xsl:text>
      <xsl:if test="expression">
	<xsl:text>% local parameters
</xsl:text>
      <xsl:apply-templates select="expression" mode="local-parameter"/>
    </xsl:if>
  </xsl:template>

  
   <xsl:template match="function">
    <xsl:message>function <xsl:value-of select="@id"/>
    </xsl:message>
    
    <xsl:text>function(</xsl:text>
    <xsl:value-of select="@id"/>
    <xsl:text>(</xsl:text>
    <xsl:for-each select="lambda/bvar">
      <xsl:value-of select="@id"/>
      <xsl:if test="not(position()=last())">
	<xsl:text>,</xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:text>) = </xsl:text>
    <xsl:apply-templates select="lambda/*[position()=last()]" mode="math"/>
    <xsl:text>).
</xsl:text>
    </xsl:template>


    <xsl:template match="@id" mode="parameter">
      <!--<xsl:message> local parameter
	- reaction <xsl:copy-of select="ancestor::*:reaction"/>
      </xsl:message>-->
      <xsl:value-of select="concat(.,'_local_',ancestor::*:reaction/@id)"/>
    </xsl:template>
    
    
  <xsl:template match="reactant">
    <xsl:apply-templates select="." mode="stoichiometry"/>
    <xsl:value-of select="@spec"/> 
    <xsl:if test="not(position()=last())">
      <xsl:text>+</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="reactant|product" mode="stoichiometry">
    <xsl:if test="@stoichiometry">
      <xsl:value-of select="@stoichiometry"/>
      <xsl:text>*</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="product">
    <xsl:apply-templates select="." mode="stoichiometry"/>
     <xsl:value-of select="@spec"/>
     <xsl:if test="not(position()=last())">
       <xsl:text>+</xsl:text>
     </xsl:if>
   </xsl:template>

   <xsl:template match="kinetic-expression">
     <!--<xsl:message>
       kinetic-expression <xsl:copy-of select="."/>
     </xsl:message>-->
     <xsl:apply-templates select="*" mode="math"/>
   </xsl:template>

   <xsl:template match="times" mode="math">
       <!--<xsl:message>
       ++ times <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
         <xsl:apply-templates select="." mode="math"/>
         <xsl:if test="not(position()=last())">
       <xsl:text> * </xsl:text>
     </xsl:if>
     </xsl:for-each>
       <xsl:text>)</xsl:text>
 </xsl:template>
  

   <xsl:template match="plus" mode="math">
       <!--<xsl:message>
       ++ plus <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
       <xsl:apply-templates select="." mode="math"/>
       <xsl:if test="not(position()=last())">
       <xsl:text> + </xsl:text>
     </xsl:if>
     </xsl:for-each>
       <xsl:text>)</xsl:text>
   </xsl:template>

   <xsl:template match="divide" mode="math">
     <!--<xsl:message>
       ++ divide <xsl:copy-of select="."/>
     </xsl:message>-->
     <xsl:text>(</xsl:text>
     <xsl:apply-templates select="*[1]" mode="math"/>
     <xsl:text> / </xsl:text>
     <xsl:apply-templates select="*[2]" mode="math"/>
     <xsl:text>)</xsl:text>
   </xsl:template>
   
   <xsl:template match="minus" mode="math">
       <!--<xsl:message>
	   ++ plus <xsl:copy-of select="."/>
	   </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
	 <xsl:apply-templates select="." mode="math"/>
	 <xsl:if test="not(position()=last())">
	   <xsl:text> - </xsl:text>
	 </xsl:if>
       </xsl:for-each>
       <xsl:text>)</xsl:text>
       
   </xsl:template>
   
   <xsl:template match="log" mode="math">
     <!--<xsl:message>
       ++ log <xsl:copy-of select="."/>
       </xsl:message>-->
     <xsl:comment> lg(b,​x​) = log(​x​) ÷ log(b).</xsl:comment>

       <xsl:text>(log(</xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>) / log(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>))</xsl:text>
   </xsl:template>

   
   
   <xsl:template match="
			sin | cos | tan | cot |
			ceiling | floor"           mode="math">
     <xsl:message> Warning in Core2BioCham: <xsl:value-of
     select="local-name()"/> used. Is this function supported by BioCham?</xsl:message>
     <xsl:text>(</xsl:text>
     <xsl:value-of select="local-name()"/>
     <xsl:text>(</xsl:text>
     <xsl:apply-templates select="*[1]" mode="math"/>
     <xsl:text>))</xsl:text>
   </xsl:template>
 

   
   
      <xsl:template match="power[*[2][self::constant[@value='0.5']]]" mode="math">
     <!--<xsl:message> sqrt  </xsl:message>-->
       <xsl:text>sqrt(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>)</xsl:text>
      </xsl:template>

      <xsl:template match="power[*[2][self::constant[@value='1']]]" mode="math">
	<xsl:apply-templates select="*[1]" mode="math"/>
      </xsl:template>

      <xsl:template match="power[*[2][self::constant[@value='-1']]]" mode="math">
     <xsl:message> power -1    </xsl:message>
     <xsl:text>( 1 /</xsl:text>
     <xsl:apply-templates select="*[1]" mode="math"/>
     <xsl:text>)</xsl:text>
   </xsl:template>
   
       <xsl:template
	   match="power[not(*[2][self::constant[@value='0.5' or
		  @value='-1' or @value='1']])]" mode="math">
	 <xsl:message> Comment in Core2BioCham: power with complex exponent.</xsl:message>
	 <xsl:if test="$DEBUG='true'">
	 <xsl:message> - exponent=<xsl:copy-of select="*[2]"/></xsl:message>
	 </xsl:if> 
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>^</xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>)</xsl:text>
   </xsl:template>
     <xsl:template match="lt" mode="math">
       <!--<xsl:message>
       ++ < <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>&lt;</xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>)</xsl:text>
     </xsl:template>

       <xsl:template match="child::true" mode="math">
       
       <xsl:text>true</xsl:text>
      
       </xsl:template>
       
     <xsl:template match="leq" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>&lt;=</xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>)</xsl:text>
     </xsl:template>

     
     <xsl:template match="eq" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>=</xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>)</xsl:text>
   </xsl:template>


    <xsl:template match="and" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
	 <xsl:apply-templates select="." mode="math"/>
	 <xsl:if test="not(position()=last())">
	   <xsl:text> and </xsl:text>
	 </xsl:if>
       </xsl:for-each>
       <xsl:text>)</xsl:text>
   </xsl:template>

    <xsl:template match="or" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text> or </xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>)</xsl:text>
    </xsl:template>

     <xsl:template match="not" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>not </xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text> </xsl:text>
   </xsl:template>

    <xsl:template match="ifthenelse" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(if </xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text> then </xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text> else </xsl:text>
       <xsl:apply-templates select="*[3]" mode="math"/>
       <xsl:text>)</xsl:text>
   </xsl:template>

   
   <xsl:template match="var"  mode="math">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:choose>
       <xsl:when test="$network/network/variables[@id=$id]
		       /kinetic-expression">
<!--	 <xsl:message> °°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°° macro <xsl:value-of select="@spec"/> </xsl:message>-->
	   <xsl:value-of select="@id"/>
       </xsl:when>
       <xsl:otherwise>
	 <!--<xsl:message> °°°°°°°°°°°°°°°°° species <xsl:value-of select="@spec"/> </xsl:message>
     <xsl:text></xsl:text>-->
     <xsl:value-of select="@id"/>
     <xsl:text></xsl:text>
     </xsl:otherwise>
     </xsl:choose>
   </xsl:template>

   <xsl:template match="constant"  mode="math">
     <xsl:value-of select="@value"/>
   </xsl:template>

    <xsl:template match="param"  mode="math">
     <xsl:value-of select="@id"/>
    </xsl:template>

    <xsl:template match="speed"  mode="math">
      <xsl:variable name="reaction_id">
	<xsl:value-of select="@reaction"/>
      </xsl:variable>
      <xsl:variable name="reaction">
	<xsl:copy-of select="$network//reaction[@id=$reaction_id]"/>
      </xsl:variable>
      <xsl:message>	Core2BioCham compiles speed of reaction with @id=<xsl:value-of select="$reaction_id"/> 
      </xsl:message>
      <xsl:apply-templates select="$reaction/reaction/kinetic-expression"/>
   </xsl:template>
   
   
   <xsl:template match="expr"  mode="math">
     <xsl:choose>
       <xsl:when test="@id=ancestor::*:reaction/expression/@id">
	 <xsl:apply-templates select="@id" mode="parameter"/>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:value-of select="@id"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:template>

   <xsl:template match="apply"  mode="math">
     <xsl:value-of select="@fun"/>
     <xsl:text>(</xsl:text>
     <xsl:for-each select="*">
       <!--<xsl:message> apply <xsl:value-of
       select="concat(@fun,':',position())"/>
       <xsl:copy-of select="."/>
       </xsl:message>-->
       <xsl:apply-templates select="." mode="math"/>
       <xsl:if test="not(position()=last())">
	      <xsl:text>,</xsl:text>
       </xsl:if>
     </xsl:for-each>
     <xsl:text>)</xsl:text>
   </xsl:template>

   <!--
     </variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:apply-templates select="$network//expression[@id=$id]/*"  mode="math"/>
   </xsl:template>-->

   <xsl:template match="*"  mode="math">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:message> Error in Core2BioCham: math otherwise: <xsl:value-of select="name()"/>
       </xsl:message>
     <xsl:apply-templates select="$network//expression[@id=$id]/*"  mode="math"/>
   </xsl:template>

   <!-- function initial --> 

     <xsl:template match="variable[@type='species' or
			  @type='differential']"  mode="initial">
         <xsl:if test="@initial_expression">
	 <xsl:message>Error in Core2BioCham:
	   - @initial_expression not yet implemented  <xsl:value-of select="@id"/>
	 </xsl:message>
       </xsl:if>
       <xsl:if test="not(@initial or @initial_expression) or @initial=''">
	 <xsl:message>Error in Core2BioCham:
	   - missing @initial in nonalgebraic variable  <xsl:value-of select="@id"/>
	 </xsl:message>
       </xsl:if>
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!--<xsl:message>species <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text>present(</xsl:text>
     <xsl:value-of select="@id"/>
     <xsl:text>,</xsl:text>
     <xsl:value-of select="@initial"/>
     <xsl:text>).
</xsl:text>
     </xsl:template>

      <!-- function macro --> 

     <xsl:template match="variable[kinetic-expression]"  mode="macro">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!--<xsl:message>arithmetic species <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text>function(</xsl:text>
     <xsl:value-of select="@id"/>
     <xsl:text>=</xsl:text>
     <xsl:apply-templates select="kinetic-expression"/>
     <xsl:text>).
</xsl:text>
     </xsl:template>


    


    <xsl:template match="variable[@type='control']">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!--<xsl:message>parameter <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text>parameter(</xsl:text>
     <xsl:value-of select="@id"/>
     <xsl:text>=</xsl:text>
     <xsl:value-of select="@initial"/>
     <xsl:text>).
</xsl:text>
    </xsl:template>


 <xsl:template match="expression" mode="local-parameter">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!--<xsl:message>arithmetic species <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text>function(</xsl:text>
     <xsl:apply-templates select="@id" mode="parameter"/>
     <xsl:text>=</xsl:text>
     <xsl:apply-templates select="*" mode="math"/>
     <xsl:text>).
</xsl:text>
 </xsl:template>

 
    
    <xsl:template match="expression">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!--<xsl:message>arithmetic species <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text>function(</xsl:text>
     <xsl:value-of select="@id"/>
     <xsl:text>=</xsl:text>
     <xsl:apply-templates select="*" mode="math"/>
     <xsl:text>).
</xsl:text>
    </xsl:template>
    
    <xsl:template match="event">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:message>event <xsl:value-of select="$id"/>
     </xsl:message>
     <xsl:text>add_event(</xsl:text>
     <xsl:apply-templates select="condition/*" mode="math"/>
     <xsl:text>,</xsl:text>
     <xsl:for-each select="update">
       <xsl:apply-templates select="."/>
       <xsl:if test="not(position()=last())">
	 <xsl:text>,</xsl:text>
       </xsl:if>
     </xsl:for-each>
     <xsl:text>).
</xsl:text>
   </xsl:template>

   <xsl:template match="update">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!--<xsl:message>update <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text></xsl:text>
     <xsl:apply-templates select="*[1]" mode="math"/>
     <xsl:text>=</xsl:text>
     <xsl:apply-templates select="*[2]" mode="math"/>
     <xsl:text></xsl:text>
   </xsl:template>

   <xsl:template match="time" mode="math">
      <xsl:text>RealTime</xsl:text>
   </xsl:template>

    
    
</xsl:stylesheet>
