<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:mathml="http://www.w3.org/1998/Math/MathML"
    version="2.0"
    exclude-result-prefixes="xs rdf xsl mathml"
    >

  <!--xpath-default-namespace="http://www.sbml.org/sbml/level2/version4"-->


  <xsl:output   
      method="text" 
      encoding="utf-8"
      indent="yes"/>
  <xsl:param name="NETWORK"      required="no"/>
  <xsl:variable name="network">
    <xsl:copy-of select="/network"/>
  </xsl:variable>
  <xsl:variable name="DEBUG">false</xsl:variable>

 
   
  <xsl:template match="network">

    
    <xsl:text>############################################
# Network:   </xsl:text>
     <xsl:value-of select="@id"/>
<xsl:text>
#####################################################

#=
  Start Catalyst by feeding the following commented line first.

using Catalyst
  
  Then feed the whole buffer for defining with catalyst a
  reaction network and simulating it

=#

begin

#### identity signal t for real time ########################

   @variables t

#### species with their initial values ######################

   @species begin
</xsl:text>

<xsl:variable name="initial">
  <xsl:apply-templates select="variable[@type='species' or
			       @type='differential']" mode="initial"/>
</xsl:variable>
<xsl:copy-of select="$initial"/>
  end

<xsl:if test="function">
  <xsl:text>
#### functions (may use species) #############################
    
</xsl:text>
</xsl:if>
<xsl:apply-templates select="function"/>


##### algebraic equations (may use functions and species)  ########
<xsl:if test="variable[kinetic-expression]|expression">  
<xsl:text>
  
</xsl:text>
</xsl:if>

<xsl:apply-templates select="variable[@type='algebraic']|expression " mode="macro"/>

<xsl:if test="reaction/expression">
#  local parameters of reactions   ##########################

<xsl:apply-templates select="reaction" mode="local"/>
</xsl:if>

<xsl:if test="variable[@type='control']">
<xsl:text>
#  control variables ##########################

</xsl:text>
</xsl:if>
<xsl:apply-templates select="variable[@type='control']"/>

<xsl:if test="event">
  <xsl:text>#### events  ###################################
</xsl:text>
</xsl:if>
<xsl:apply-templates select="event"/>


<xsl:text>

####  reaction network ###################################

    rn = @reaction_network begin
</xsl:text>   
<xsl:apply-templates select="reaction|variable[@type='differential']"/>
<xsl:text>
   end

###  numerical simulation ################################

   using DifferentialEquations

   odes = convert(ODESystem, rn)
   prob = ODEProblem(odes, [], (0.0, 100.0), [])
   

# tsidas_alg  = AutoTsit5(Rosenbrock23())
#  tsidas_alg = AutoTsit5(Rodas5())
#  sol = solve(prob, tsidas_alg)

  using LSODA
  sol = solve(prob, lsoda(), abstol=1e-12, reltol=1e-6)
   
  using Plots

  function plot_signals(reaction_network
  ::ReactionSystem{Catalyst.NetworkProperties{Int64, SymbolicUtils.BasicSymbolic{Real}}},
  solution::ODESolution)

    l_species = species(reaction_network)
    l_of_plots = []
    for i = 1:length(species(reaction_network))
        my_name = species(reaction_network)[i]
        X = solution.t
        Y = map(x -> x[i], solution.u)
        my_plot = plot(X, Y, label = "$my_name")
        push!(l_of_plots, my_plot)
    end
    return (plot(l_of_plots..., size = default(:size) .* (1, 12),
                        layout = (12, 1),
                        left_margin = 5Plots.mm))
   end
  
   plot(sol)
   plot_signals(rn,sol)

end
</xsl:text>
  </xsl:template>


  <xsl:template match="variable[@type='differential']">
    <xsl:variable name="reaction">
      <reaction>
	<xsl:copy-of select="@id"/>
	<xsl:copy-of select="kinetic-expression"/>
	<product>
	  <xsl:attribute name="spec">
	    <xsl:value-of select="@id"/>
	  </xsl:attribute>
	</product>
      </reaction>
    </xsl:variable>
    <!--<xsl:message>
      reaction for differential variable:
      <xsl:copy-of select="$reaction"/>
    </xsl:message>-->
      <xsl:apply-templates select="$reaction/reaction"/>
  </xsl:template>

  
  <xsl:template match="reaction">
    <!--<xsl:message> reaction <xsl:value-of select="@id"/>
    </xsl:message>-->
    <xsl:text>     # </xsl:text>
    <xsl:value-of select="@id"/>
    <xsl:text>
</xsl:text>
   <xsl:if test="product|reactant">
     <xsl:text>     </xsl:text>
      <xsl:apply-templates select="kinetic-expression" mode="dollar"/>
      <xsl:text> , </xsl:text>
      <xsl:choose>
	<xsl:when test="reactant">
	  <xsl:apply-templates select="reactant"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:text>0</xsl:text>
	</xsl:otherwise>
      </xsl:choose>
      <xsl:text>  =>  </xsl:text>
      <xsl:choose>
	<xsl:when test="product">
	  <xsl:apply-templates select="product"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:text>0</xsl:text>
	</xsl:otherwise>
      </xsl:choose>
   </xsl:if>
   <xsl:text>
</xsl:text>     
  </xsl:template>

<xsl:template match="reaction" mode="local">	
  <xsl:if test="expression">
    <xsl:apply-templates select="expression" mode="local-parameter"/>
  </xsl:if>
</xsl:template>
  
  
   <xsl:template match="function">
    <!--<xsl:message>function <xsl:value-of select="@id"/>
    </xsl:message>-->
    
    <xsl:text>   function </xsl:text>
    <xsl:value-of select="@id"/>
    <xsl:text>(</xsl:text>
    <xsl:for-each select="lambda/bvar">
      <xsl:value-of select="@id"/>
      <xsl:if test="not(position()=last())">
	<xsl:text>,</xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:text>)
     </xsl:text>
    <xsl:apply-templates select="lambda/*[position()=last()]" mode="math"/>
    <xsl:text>
   end
</xsl:text>
    </xsl:template>


    <xsl:template match="@id" mode="parameter">
      <!--<xsl:message> local parameter
	- reaction <xsl:copy-of select="ancestor::*:reaction"/>
      </xsl:message>-->
      <xsl:value-of select="concat(.,'_local_',ancestor::*:reaction/@id)"/>
    </xsl:template>

     <xsl:template match="@id" mode="parameterdollar">
      <!--<xsl:message> local parameter
	- reaction <xsl:copy-of select="ancestor::*:reaction"/>
      </xsl:message>-->
      <xsl:value-of select="concat('$',.,'_local_',ancestor::*:reaction/@id)"/>
    </xsl:template>
    
    
 
  <xsl:template match="reactant|product" mode="stoichiometry">
    <xsl:if test="@stoichiometry">
      <xsl:value-of select="@stoichiometry"/>
      <xsl:text>*</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="reactant">
    <xsl:apply-templates select="." mode="stoichiometry"/>
    <xsl:apply-templates select="@spec" mode="reaction"/>
    <xsl:if test="not(position()=last())">
      <xsl:text>+</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="product">
    <xsl:apply-templates select="." mode="stoichiometry"/>
     <xsl:apply-templates select="@spec" mode="reaction"/>
     <xsl:if test="not(position()=last())">
       <xsl:text>+</xsl:text>
     </xsl:if>
   </xsl:template>

   <xsl:template match="@spec" mode="reaction">
     <xsl:value-of select="concat('$',.)"/>
   </xsl:template>
   <xsl:template match="kinetic-expression">
     <!--<xsl:message>
       kinetic-expression <xsl:copy-of select="."/>
     </xsl:message>-->
     <xsl:apply-templates select="*" mode="math"/>
   </xsl:template>

    <xsl:template match="kinetic-expression" mode="dollar">
     <!--<xsl:message>
       kinetic-expression <xsl:copy-of select="."/>
     </xsl:message>-->
     <xsl:apply-templates select="*" mode="mathdollar"/>
   </xsl:template>

   <xsl:template match="times" mode="math">
       <!--<xsl:message>
       ++ times <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
         <xsl:apply-templates select="." mode="math"/>
         <xsl:if test="not(position()=last())">
       <xsl:text> * </xsl:text>
     </xsl:if>
     </xsl:for-each>
       <xsl:text>)</xsl:text>
 </xsl:template>
  

   <xsl:template match="plus" mode="math">
       <!--<xsl:message>
       ++ plus <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
       <xsl:apply-templates select="." mode="math"/>
       <xsl:if test="not(position()=last())">
       <xsl:text> + </xsl:text>
     </xsl:if>
     </xsl:for-each>
       <xsl:text>)</xsl:text>
   </xsl:template>

   <xsl:template match="divide" mode="math">
     <!--<xsl:message>
       ++ divide <xsl:copy-of select="."/>
     </xsl:message>-->
     <xsl:text>(</xsl:text>
     <xsl:apply-templates select="*[1]" mode="math"/>
     <xsl:text> / </xsl:text>
     <xsl:apply-templates select="*[2]" mode="math"/>
     <xsl:text>)</xsl:text>
   </xsl:template>
   
   <xsl:template match="minus" mode="math">
       <!--<xsl:message>
	   ++ plus <xsl:copy-of select="."/>
	   </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
	 <xsl:apply-templates select="." mode="math"/>
	 <xsl:if test="not(position()=last())">
	   <xsl:text> - </xsl:text>
	 </xsl:if>
       </xsl:for-each>
       <xsl:text>)</xsl:text>
       
   </xsl:template>
   
   <xsl:template match="log" mode="math">
     <!--<xsl:message>
       ++ log <xsl:copy-of select="."/>
       </xsl:message>-->
     <xsl:comment> lg(b,​x​) = log(​x​) ÷ log(b).</xsl:comment>

       <xsl:text>(log(</xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>) / log(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>))</xsl:text>
   </xsl:template>

   
   
   <xsl:template match="
			sin | cos | tan | cot |
			ceiling | floor"           mode="math">
       <xsl:text>(</xsl:text>
     <xsl:value-of select="local-name()"/>
     <xsl:text>(</xsl:text>
     <xsl:apply-templates select="*[1]" mode="math"/>
     <xsl:text>))</xsl:text>
   </xsl:template>
 

   
   
      <xsl:template match="power[*[2][self::constant[@value='0.5']]]" mode="math">
     <!--<xsl:message> sqrt  </xsl:message>-->
       <xsl:text>sqrt(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>)</xsl:text>
      </xsl:template>

      <xsl:template match="power[*[2][self::constant[@value='1']]]" mode="math">
	<xsl:apply-templates select="*[1]" mode="math"/>
      </xsl:template>

      <xsl:template match="power[*[2][self::constant[@value='-1']]]" mode="math">
     <xsl:message> power -1    </xsl:message>
     <xsl:text>( 1 /</xsl:text>
     <xsl:apply-templates select="*[1]" mode="math"/>
     <xsl:text>)</xsl:text>
   </xsl:template>
   
   <xsl:template
	   match="power[not(*[2][self::constant[@value='0.5' or
		  @value='-1' or @value='1']])]" mode="math">
	 <xsl:if test="$DEBUG='true'">
	 <xsl:message> - power with exponent=<xsl:copy-of select="*[2]"/></xsl:message>
	</xsl:if> 
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>^</xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>)</xsl:text>
   </xsl:template>
     <xsl:template match="lt" mode="math">
       <!--<xsl:message>
       ++ < <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>&lt;</xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>)</xsl:text>
     </xsl:template>

     <xsl:template match="child::true" mode="math">
              <xsl:text>true</xsl:text>
     </xsl:template>
     
     <xsl:template match="leq" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>&lt;=</xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>)</xsl:text>
     </xsl:template>

     
     <xsl:template match="eq" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>=</xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>)</xsl:text>
   </xsl:template>


    <xsl:template match="and" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
	 <xsl:apply-templates select="." mode="math"/>
	 <xsl:if test="not(position()=last())">
	   <xsl:text> and </xsl:text>
	 </xsl:if>
       </xsl:for-each>
       <xsl:text>)</xsl:text>
   </xsl:template>

    <xsl:template match="or" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text> or </xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text>)</xsl:text>
    </xsl:template>

     <xsl:template match="not" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>not </xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text> </xsl:text>
   </xsl:template>

    <xsl:template match="ifthenelse" mode="math">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(ifelse( </xsl:text>
       <xsl:apply-templates select="*[1]" mode="math"/>
       <xsl:text>, </xsl:text>
       <xsl:apply-templates select="*[2]" mode="math"/>
       <xsl:text> , </xsl:text>
       <xsl:apply-templates select="*[3]" mode="math"/>
       <xsl:text>))</xsl:text>
   </xsl:template>

   
   <xsl:template match="var"  mode="math">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:choose>
       <xsl:when test="$network/network/variables[@id=$id]
		       /kinetic-expression">
<!--	 <xsl:message> °°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°° macro <xsl:value-of select="@spec"/> </xsl:message>-->
	   <xsl:value-of select="@id"/>
       </xsl:when>
       <xsl:otherwise>
	 <!--<xsl:message> °°°°°°°°°°°°°°°°° species <xsl:value-of select="@spec"/> </xsl:message>
     <xsl:text></xsl:text>-->
     <xsl:value-of select="@id"/>
     <xsl:text></xsl:text>
     </xsl:otherwise>
     </xsl:choose>
   </xsl:template>

   <xsl:template match="constant"  mode="math">
     <xsl:value-of select="@value"/>
   </xsl:template>

    <xsl:template match="param"  mode="math">
     <xsl:value-of select="@id"/>
    </xsl:template>

    <xsl:template match="speed"  mode="math">
      <xsl:variable name="reaction_id">
	<xsl:value-of select="@reaction"/>
      </xsl:variable>
      <xsl:variable name="reaction">
	<xsl:copy-of select="$network//reaction[@id=$reaction_id]"/>
      </xsl:variable>
      <xsl:message>	Core2Catalyst compiles speed of reaction with @id=<xsl:value-of select="$reaction_id"/> 
      </xsl:message>
      <xsl:apply-templates select="$reaction/reaction/kinetic-expression"/>
   </xsl:template>
   
   
   <xsl:template match="expr"  mode="math">
     <xsl:choose>
       <xsl:when test="@id=ancestor::*:reaction/expression/@id">
	 <xsl:apply-templates select="@id" mode="parameter"/>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:value-of select="@id"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:template>

   <xsl:template match="apply"  mode="math">
     <xsl:value-of select="@fun"/>
     <xsl:text>(</xsl:text>
     <xsl:for-each select="*">
       <!--<xsl:message> apply <xsl:value-of
       select="concat(@fun,':',position())"/>
       <xsl:copy-of select="."/>
       </xsl:message>-->
       <xsl:apply-templates select="." mode="math"/>
       <xsl:if test="not(position()=last())">
	      <xsl:text>,</xsl:text>
       </xsl:if>
     </xsl:for-each>
     <xsl:text>)</xsl:text>
   </xsl:template>

   <!--
     </variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:apply-templates select="$network//expression[@id=$id]/*"  mode="math"/>
   </xsl:template>-->

   <xsl:template match="*"  mode="math">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:message> Error in Core2Catalyst: math otherwise: <xsl:value-of select="name()"/>
       </xsl:message>
     <xsl:apply-templates select="$network//expression[@id=$id]/*"  mode="math"/>
   </xsl:template>

   <!-- function mathdollar
	 this is the same code as for math, but with a dollar ($) in front of
	 expr identifiers
   -->
   
   <xsl:template match="times" mode="mathdollar">
      <!-- <xsl:message>
       ++ mathdollar times <xsl:copy-of select="."/>
       </xsl:message>
       -->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
         <xsl:apply-templates select="." mode="mathdollar"/>
         <xsl:if test="not(position()=last())">
       <xsl:text> * </xsl:text>
     </xsl:if>
     </xsl:for-each>
       <xsl:text>)</xsl:text>
 </xsl:template>
  

   <xsl:template match="plus" mode="mathdollar">
       <!--<xsl:message>
       ++ plus <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
       <xsl:apply-templates select="." mode="mathdollar"/>
       <xsl:if test="not(position()=last())">
       <xsl:text> + </xsl:text>
     </xsl:if>
     </xsl:for-each>
       <xsl:text>)</xsl:text>
   </xsl:template>

   <xsl:template match="divide" mode="mathdollar">
     <!--<xsl:message>
       ++ divide <xsl:copy-of select="."/>
     </xsl:message>-->
     <xsl:text>(</xsl:text>
     <xsl:apply-templates select="*[1]" mode="mathdollar"/>
     <xsl:text> / </xsl:text>
     <xsl:apply-templates select="*[2]" mode="mathdollar"/>
     <xsl:text>)</xsl:text>
   </xsl:template>
   
   <xsl:template match="minus" mode="mathdollar">
       <!--<xsl:message>
	   ++ plus <xsl:copy-of select="."/>
	   </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
	 <xsl:apply-templates select="." mode="mathdollar"/>
	 <xsl:if test="not(position()=last())">
	   <xsl:text> - </xsl:text>
	 </xsl:if>
       </xsl:for-each>
       <xsl:text>)</xsl:text>
       
   </xsl:template>
   
   <xsl:template match="log" mode="mathdollar">
     <!--<xsl:message>
       ++ log <xsl:copy-of select="."/>
       </xsl:message>-->
     <xsl:comment> lg(b,​x​) = log(​x​) ÷ log(b).</xsl:comment>

       <xsl:text>(log(</xsl:text>
       <xsl:apply-templates select="*[2]" mode="mathdollar"/>
       <xsl:text>) / log(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="mathdollar"/>
       <xsl:text>))</xsl:text>
   </xsl:template>

   
   
   <xsl:template match="
			sin | cos | tan | cot |
			ceiling | floor"           mode="mathdollar">
       <xsl:text>(</xsl:text>
     <xsl:value-of select="local-name()"/>
     <xsl:text>(</xsl:text>
     <xsl:apply-templates select="*[1]" mode="mathdollar"/>
     <xsl:text>))</xsl:text>
   </xsl:template>
 

   
   
      <xsl:template match="power[*[2][self::constant[@value='0.5']]]" mode="mathdollar">
     <!--<xsl:message> sqrt  </xsl:message>-->
       <xsl:text>sqrt(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="mathdollar"/>
       <xsl:text>)</xsl:text>
      </xsl:template>

      <xsl:template match="power[*[2][self::constant[@value='1']]]" mode="mathdollar">
	<xsl:apply-templates select="*[1]" mode="mathdollar"/>
      </xsl:template>

      <xsl:template match="power[*[2][self::constant[@value='-1']]]" mode="mathdollar">
     <xsl:message> power -1    </xsl:message>
     <xsl:text>( 1 /</xsl:text>
     <xsl:apply-templates select="*[1]" mode="mathdollar"/>
     <xsl:text>)</xsl:text>
   </xsl:template>
   
   <xsl:template
	   match="power[not(*[2][self::constant[@value='0.5' or
		  @value='-1' or @value='1']])]" mode="mathdollar">
	 <xsl:if test="$DEBUG='true'">
	 <xsl:message> mathdollar: power with exponent=<xsl:copy-of select="*[2]"/></xsl:message>
	 </xsl:if>
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="mathdollar"/>
       <xsl:text>^</xsl:text>
       <xsl:apply-templates select="*[2]" mode="mathdollar"/>
       <xsl:text>)</xsl:text>
   </xsl:template>
     <xsl:template match="lt" mode="mathdollar">
       <!--<xsl:message>
       ++ < <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="mathdollar"/>
       <xsl:text>&lt;</xsl:text>
       <xsl:apply-templates select="*[2]" mode="mathdollar"/>
       <xsl:text>)</xsl:text>
     </xsl:template>

     <xsl:template match="child::true" mode="mathdollar">
              <xsl:text>true</xsl:text>
     </xsl:template>
     
     <xsl:template match="leq" mode="mathdollar">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="mathdollar"/>
       <xsl:text>&lt;=</xsl:text>
       <xsl:apply-templates select="*[2]" mode="mathdollar"/>
       <xsl:text>)</xsl:text>
     </xsl:template>

     
     <xsl:template match="eq" mode="mathdollar">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="mathdollar"/>
       <xsl:text>=</xsl:text>
       <xsl:apply-templates select="*[2]" mode="mathdollar"/>
       <xsl:text>)</xsl:text>
   </xsl:template>


    <xsl:template match="and" mode="mathdollar">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:for-each select="*">
	 <xsl:apply-templates select="." mode="mathdollar"/>
	 <xsl:if test="not(position()=last())">
	   <xsl:text> and </xsl:text>
	 </xsl:if>
       </xsl:for-each>
       <xsl:text>)</xsl:text>
   </xsl:template>

    <xsl:template match="or" mode="mathdollar">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(</xsl:text>
       <xsl:apply-templates select="*[1]" mode="mathdollar"/>
       <xsl:text> or </xsl:text>
       <xsl:apply-templates select="*[2]" mode="mathdollar"/>
       <xsl:text>)</xsl:text>
    </xsl:template>

     <xsl:template match="not" mode="mathdollar">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>not </xsl:text>
       <xsl:apply-templates select="*[1]" mode="mathdollar"/>
       <xsl:text> </xsl:text>
   </xsl:template>

    <xsl:template match="ifthenelse" mode="mathdollar">
       <!--<xsl:message>
       ++ <= <xsl:copy-of select="."/>
     </xsl:message>-->
       <xsl:text>(if </xsl:text>
       <xsl:apply-templates select="*[1]" mode="mathdollar"/>
       <xsl:text> then </xsl:text>
       <xsl:apply-templates select="*[2]" mode="mathdollar"/>
       <xsl:text> else </xsl:text>
       <xsl:apply-templates select="*[3]" mode="mathdollar"/>
       <xsl:text>)</xsl:text>
   </xsl:template>

   
   <xsl:template match="var"  mode="mathdollar">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:choose>
       <xsl:when test="$network/network/variables[@id=$id]
		       /kinetic-expression">
<!--	 <xsl:message> °°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°° macro <xsl:value-of select="@spec"/> </xsl:message>-->
	   <xsl:value-of select="@id"/>
       </xsl:when>
       <xsl:otherwise>
	 <!--<xsl:message> °°°°°°°°°°°°°°°°° species <xsl:value-of select="@spec"/> </xsl:message>
     <xsl:text></xsl:text>-->
     <xsl:value-of select="concat('$',@id)"/>
     <xsl:text></xsl:text>
     </xsl:otherwise>
     </xsl:choose>
   </xsl:template>

   <xsl:template match="constant"  mode="mathdollar">
     <xsl:value-of select="@value"/>
   </xsl:template>

    <xsl:template match="param"  mode="mathdollar">
     <xsl:value-of select="@id"/>
    </xsl:template>

    <xsl:template match="speed"  mode="mathdollar">
      <xsl:variable name="reaction_id">
	<xsl:value-of select="@reaction"/>
      </xsl:variable>
      <xsl:variable name="reaction">
	<xsl:copy-of select="$network//reaction[@id=$reaction_id]"/>
      </xsl:variable>
      <xsl:message>	Core2Catalyst compiles speed of reaction with @id=<xsl:value-of select="$reaction_id"/> 
      </xsl:message>
      <xsl:apply-templates select="$reaction/reaction/kinetic-expression"/>
   </xsl:template>
   
   
   <xsl:template match="expr"  mode="mathdollar">
      <!-- <xsl:message>
       ++ mathdollar expr <xsl:copy-of select="."/>
       </xsl:message>
       -->
     <xsl:choose>
       <xsl:when test="@id=ancestor::*:reaction/expression/@id">
	 <xsl:apply-templates select="@id" mode="parameterdollar"/>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:value-of select="concat('$',@id)"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:template>

   <xsl:template match="apply"  mode="mathdollar">
     <xsl:value-of select="@fun"/>
     <xsl:text>(</xsl:text>
     <xsl:for-each select="*">
       <!--<xsl:message> apply <xsl:value-of
       select="concat(@fun,':',position())"/>
       <xsl:copy-of select="."/>
       </xsl:message>-->
       <xsl:apply-templates select="." mode="mathdollar"/>
       <xsl:if test="not(position()=last())">
	      <xsl:text>,</xsl:text>
       </xsl:if>
     </xsl:for-each>
     <xsl:text>)</xsl:text>
   </xsl:template>

   <!--
     </variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:apply-templates select="$network//expression[@id=$id]/*"  mode="mathdollar"/>
     </xsl:template>-->

   <xsl:template match="time" mode="mathdollar">
     <xsl:if test="$DEBUG='true'">
       <xsl:message>time mathdollar
       </xsl:message>
     </xsl:if>
      <xsl:text>t</xsl:text>
   </xsl:template>


   <xsl:template match="*"  mode="mathdollar">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:message> Error in Core2Catalyst: mathdollar otherwise: <xsl:value-of select="name()"/>
       </xsl:message>
     <xsl:apply-templates select="$network//expression[@id=$id]/*"  mode="mathdollar"/>
   </xsl:template>

   
   <!-- function initial --> 

     <xsl:template match="variable[@type='species' or
			  @type='differential']"  mode="initial">
         <xsl:if test="@initial_expression">
	 <xsl:message>Error in Core2Catalyst:
	   - @initial_expression not yet implemented  <xsl:value-of
	   select="@id"/>
	 </xsl:message>
       </xsl:if>
       <xsl:if test="not(@initial or @initial_expression) or @initial=''">
	 <xsl:message>Error in Core2Catalyst:
	   - missing @initial in nonalgebraic variable  <xsl:value-of select="@id"/>
	 </xsl:message>
       </xsl:if>
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!--<xsl:message>species <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text>     </xsl:text>
     <xsl:value-of select="@id"/>
     <xsl:text>(t)=</xsl:text>
     <xsl:value-of select="@initial" />
     <xsl:text>
 </xsl:text>
     </xsl:template>

     <xsl:template match="@*" mode="number-to-float">
       <!--<xsl:message>
	   casted to float   <xsl:value-of select="@initial cast as
	   xs:float"/>:      <xsl:apply-templates select="@initial" mode="number-to-float"/>
	   </xsl:message>-->
       <xsl:choose>
	 <xsl:when test=". castable as xs:integer">
	   <xsl:value-of select="concat(.,'.0')"/>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:value-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>
      <!-- function macro --> 

     <xsl:template match="variable[kinetic-expression]"  mode="macro">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!--<xsl:message>arithmetic species <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text>   </xsl:text>
     <xsl:value-of select="@id"/>
     <xsl:text> = </xsl:text>
     <xsl:apply-templates select="kinetic-expression"/>
     <xsl:text>
</xsl:text>
     </xsl:template>

     <xsl:template match="expression"  mode="macro">
       <xsl:apply-templates select="."/>
     </xsl:template>
    


    <xsl:template match="variable[@type='control']">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!---<xsl:message>parameter <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text>   </xsl:text>
     <xsl:value-of select="@id"/>
     <xsl:text>=</xsl:text>
     <xsl:value-of select="@initial"/>
     <xsl:text>
</xsl:text>

    </xsl:template>


 <xsl:template match="expression" mode="local-parameter">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!--<xsl:message>arithmetic species <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text>   </xsl:text>
     <xsl:apply-templates select="@id" mode="parameter"/>
     <xsl:text> = </xsl:text>
     <xsl:apply-templates select="*" mode="math"/>
     <xsl:text>
</xsl:text>
 </xsl:template>

 
    
    <xsl:template match="expression">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!--<xsl:message>arithmetic species <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text>   </xsl:text>
     <xsl:value-of select="@id"/>
     <xsl:text> = </xsl:text>
     <xsl:apply-templates select="*" mode="math"/>
     <xsl:text>
</xsl:text>
    </xsl:template>
    
    <xsl:template match="event">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <xsl:message>event <xsl:value-of select="$id"/>
     </xsl:message>
     <xsl:text>add_event(</xsl:text>
     <xsl:apply-templates select="condition/*" mode="math"/>
     <xsl:text>,</xsl:text>
     <xsl:for-each select="update">
       <xsl:apply-templates select="."/>
       <xsl:if test="not(position()=last())">
	 <xsl:text>,</xsl:text>
       </xsl:if>
     </xsl:for-each>
     <xsl:text>).
</xsl:text>
   </xsl:template>

   <xsl:template match="update">
     <xsl:variable name="id">
       <xsl:value-of select="@id"/>
     </xsl:variable>
     <!--<xsl:message>update <xsl:value-of select="$id"/>
     </xsl:message>-->
     <xsl:text></xsl:text>
     <xsl:apply-templates select="*[1]" mode="math"/>
     <xsl:text>=</xsl:text>
     <xsl:apply-templates select="*[2]" mode="math"/>
     <xsl:text></xsl:text>
   </xsl:template>

   <xsl:template match="time" mode="math">
     <xsl:message>time
     </xsl:message>
      <xsl:text>t</xsl:text>
   </xsl:template>

    
    
</xsl:stylesheet>
